@@ -56,7 +56,6 @@
 #include "nsIDOMWindowInternal.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDocShell.h"
-#include "nsIDocumentViewer.h"
 #include "nsIContentViewer.h"
 #include "nsIEventListenerManager.h"
 #include "nsIPresShell.h"
@@ -67,6 +66,7 @@
 #include "nsISelectionController.h"
 #include "nsPIDOMWindow.h"
 #include "nsGUIEvent.h"
+#include "nsIView.h"
 
 #include "nsContentCID.h"
 #include "nsComponentManagerUtils.h"
@@ -74,14 +74,84 @@
 
 static NS_DEFINE_IID(kRangeCID, NS_RANGE_CID);
 
+////////////////////////////////////////////////////////////////////////////////
+// nsCoreUtils
+////////////////////////////////////////////////////////////////////////////////
+
 PRBool
-nsCoreUtils::HasListener(nsIContent *aContent, const nsAString& aEventType)
+nsCoreUtils::HasClickListener(nsIContent *aContent)
 {
   NS_ENSURE_TRUE(aContent, PR_FALSE);
   nsIEventListenerManager* listenerManager =
     aContent->GetListenerManager(PR_FALSE);
 
-  return listenerManager && listenerManager->HasListenersFor(aEventType);  
+  return listenerManager &&
+    (listenerManager->HasListenersFor(NS_LITERAL_STRING("click")) ||
+     listenerManager->HasListenersFor(NS_LITERAL_STRING("mousedown")) ||
+     listenerManager->HasListenersFor(NS_LITERAL_STRING("mouseup")));
+}
+
+void
+nsCoreUtils::DispatchClickEvent(nsITreeBoxObject *aTreeBoxObj,
+                                PRInt32 aRowIndex, nsITreeColumn *aColumn,
+                                const nsCString& aPseudoElt)
+{
+  nsCOMPtr<nsIDOMElement> tcElm;
+  aTreeBoxObj->GetTreeBody(getter_AddRefs(tcElm));
+  if (!tcElm)
+    return;
+
+  nsCOMPtr<nsIContent> tcContent(do_QueryInterface(tcElm));
+  nsIDocument *document = tcContent->GetCurrentDoc();
+  if (!document)
+    return;
+
+  nsIPresShell *presShell = nsnull;
+  presShell = document->GetShell();
+  if (!presShell)
+    return;
+
+  // Ensure row is visible.
+  aTreeBoxObj->EnsureRowIsVisible(aRowIndex);
+
+  // Calculate x and y coordinates.
+  PRInt32 x = 0, y = 0, width = 0, height = 0;
+  nsresult rv = aTreeBoxObj->GetCoordsForCellItem(aRowIndex, aColumn,
+                                                  aPseudoElt,
+                                                  &x, &y, &width, &height);
+  if (NS_FAILED(rv))
+    return;
+
+  nsCOMPtr<nsIDOMXULElement> tcXULElm(do_QueryInterface(tcElm));
+  nsCOMPtr<nsIBoxObject> tcBoxObj;
+  tcXULElm->GetBoxObject(getter_AddRefs(tcBoxObj));
+
+  PRInt32 tcX = 0;
+  tcBoxObj->GetX(&tcX);
+
+  PRInt32 tcY = 0;
+  tcBoxObj->GetY(&tcY);
+
+  // Dispatch mouse events.
+  nsIFrame* tcFrame = tcContent->GetPrimaryFrame();
+  nsIFrame* rootFrame = presShell->GetRootFrame();
+
+  nsPoint offset;
+  nsIWidget *rootWidget =
+    rootFrame->GetViewExternal()->GetNearestWidget(&offset);
+
+  nsPresContext* presContext = presShell->GetPresContext();
+
+  PRInt32 cnvdX = presContext->CSSPixelsToDevPixels(tcX + x + 1) +
+    presContext->AppUnitsToDevPixels(offset.x);
+  PRInt32 cnvdY = presContext->CSSPixelsToDevPixels(tcY + y + 1) +
+    presContext->AppUnitsToDevPixels(offset.y);
+
+  DispatchMouseEvent(NS_MOUSE_BUTTON_DOWN, cnvdX, cnvdY,
+                     tcContent, tcFrame, presShell, rootWidget);
+
+  DispatchMouseEvent(NS_MOUSE_BUTTON_UP, cnvdX, cnvdY,
+                     tcContent, tcFrame, presShell, rootWidget);
 }
 
 PRBool
@@ -89,41 +159,44 @@ nsCoreUtils::DispatchMouseEvent(PRUint32 aEventType,
                                 nsIPresShell *aPresShell,
                                 nsIContent *aContent)
 {
-  nsIFrame *frame = aPresShell->GetPrimaryFrameFor(aContent);
+  nsIFrame *frame = aContent->GetPrimaryFrame();
   if (!frame)
     return PR_FALSE;
 
-  nsIFrame* rootFrame = aPresShell->GetRootFrame();
-  if (!rootFrame)
-    return PR_FALSE;
-
-  nsCOMPtr<nsIWidget> rootWidget = rootFrame->GetWindow();
-  if (!rootWidget)
-    return PR_FALSE;
-
   // Compute x and y coordinates.
-  nsPoint point = frame->GetOffsetToExternal(rootFrame);
+  nsPoint point;
+  nsCOMPtr<nsIWidget> widget = frame->GetNearestWidget(point);
+  if (!widget)
+    return PR_FALSE;
+
   nsSize size = frame->GetSize();
 
   nsPresContext* presContext = aPresShell->GetPresContext();
 
   PRInt32 x = presContext->AppUnitsToDevPixels(point.x + size.width / 2);
   PRInt32 y = presContext->AppUnitsToDevPixels(point.y + size.height / 2);
-  
+
   // Fire mouse event.
-  nsMouseEvent event(PR_TRUE, aEventType, rootWidget,
+  DispatchMouseEvent(aEventType, x, y, aContent, frame, aPresShell, widget);
+  return PR_TRUE;
+}
+
+void
+nsCoreUtils::DispatchMouseEvent(PRUint32 aEventType, PRInt32 aX, PRInt32 aY,
+                                nsIContent *aContent, nsIFrame *aFrame,
+                                nsIPresShell *aPresShell, nsIWidget *aRootWidget)
+{
+  nsMouseEvent event(PR_TRUE, aEventType, aRootWidget,
                      nsMouseEvent::eReal, nsMouseEvent::eNormal);
 
-  event.refPoint = nsIntPoint(x, y);
-  
+  event.refPoint = nsIntPoint(aX, aY);
+
   event.clickCount = 1;
   event.button = nsMouseEvent::eLeftButton;
   event.time = PR_IntervalNow();
-  
-  nsEventStatus status = nsEventStatus_eIgnore;
-  aPresShell->HandleEventWithTarget(&event, frame, aContent, &status);
 
-  return PR_TRUE;
+  nsEventStatus status = nsEventStatus_eIgnore;
+  aPresShell->HandleEventWithTarget(&event, aFrame, aContent, &status);
 }
 
 PRUint32
@@ -142,7 +215,7 @@ nsCoreUtils::GetAccessKeyFor(nsIContent *aContent)
   if (!doc)
     return 0;
 
-  nsCOMPtr<nsIPresShell> presShell = doc->GetPrimaryShell();
+  nsCOMPtr<nsIPresShell> presShell = doc->GetShell();
   if (!presShell)
     return 0;
 
@@ -159,73 +232,44 @@ nsCoreUtils::GetAccessKeyFor(nsIContent *aContent)
   return key;
 }
 
-already_AddRefed<nsIDOMElement>
-nsCoreUtils::GetDOMElementFor(nsIDOMNode *aNode)
+nsIContent *
+nsCoreUtils::GetDOMElementFor(nsIContent *aContent)
 {
-  nsCOMPtr<nsINode> node(do_QueryInterface(aNode));
-  nsIDOMElement *element = nsnull;
+  if (aContent->IsElement())
+    return aContent;
 
-  if (node->IsNodeOfType(nsINode::eELEMENT))
-    CallQueryInterface(node, &element);
+  if (aContent->IsNodeOfType(nsINode::eTEXT))
+    return aContent->GetParent();
 
-  else if (node->IsNodeOfType(nsINode::eTEXT)) {
-    nsCOMPtr<nsINode> nodeParent = node->GetNodeParent();
-    NS_ASSERTION(nodeParent, "Text node has no parent!");
-    if (nodeParent)
-      CallQueryInterface(nodeParent, &element);
-  }
-
-  else if (node->IsNodeOfType(nsINode::eDOCUMENT)) {
-    nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(node));
-    if (htmlDoc) {
-      nsCOMPtr<nsIDOMHTMLElement> bodyElement;
-      htmlDoc->GetBody(getter_AddRefs(bodyElement));
-      if (bodyElement) {
-        CallQueryInterface(bodyElement, &element);
-        return element;
-      }
-    }
-
-    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(node));
-    domDoc->GetDocumentElement(&element);
-  }
-
-  return element;
+  return nsnull;
 }
 
-already_AddRefed<nsIDOMNode>
-nsCoreUtils::GetDOMNodeFromDOMPoint(nsIDOMNode *aNode, PRUint32 aOffset)
+nsINode *
+nsCoreUtils::GetDOMNodeFromDOMPoint(nsINode *aNode, PRUint32 aOffset)
 {
-  nsIDOMNode *resultNode = nsnull;
-
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
-  if (content && content->IsNodeOfType(nsINode::eELEMENT)) {
-
-    PRUint32 childCount = content->GetChildCount();
+  if (aNode && aNode->IsElement()) {
+    PRUint32 childCount = aNode->GetChildCount();
     NS_ASSERTION(aOffset >= 0 && aOffset <= childCount,
                  "Wrong offset of the DOM point!");
 
     // The offset can be after last child of container node that means DOM point
     // is placed immediately after the last child. In this case use the DOM node
     // from the given DOM point is used as result node.
-    if (aOffset != childCount) {
-      CallQueryInterface(content->GetChildAt(aOffset), &resultNode);
-      return resultNode;
-    }
+    if (aOffset != childCount)
+      return aNode->GetChildAt(aOffset);
   }
 
-  NS_IF_ADDREF(resultNode = aNode);
-  return resultNode;
+  return aNode;
 }
 
 nsIContent*
-nsCoreUtils::GetRoleContent(nsIDOMNode *aDOMNode)
+nsCoreUtils::GetRoleContent(nsINode *aNode)
 {
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aDOMNode));
+  nsCOMPtr<nsIContent> content(do_QueryInterface(aNode));
   if (!content) {
-    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(aDOMNode));
+    nsCOMPtr<nsIDOMDocument> domDoc(do_QueryInterface(aNode));
     if (domDoc) {
-      nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(aDOMNode));
+      nsCOMPtr<nsIDOMHTMLDocument> htmlDoc(do_QueryInterface(aNode));
       if (htmlDoc) {
         nsCOMPtr<nsIDOMHTMLElement> bodyElement;
         htmlDoc->GetBody(getter_AddRefs(bodyElement));
@@ -243,35 +287,19 @@ nsCoreUtils::GetRoleContent(nsIDOMNode *aDOMNode)
 }
 
 PRBool
-nsCoreUtils::IsAncestorOf(nsIDOMNode *aPossibleAncestorNode,
-                          nsIDOMNode *aPossibleDescendantNode)
+nsCoreUtils::IsAncestorOf(nsINode *aPossibleAncestorNode,
+                          nsINode *aPossibleDescendantNode,
+                          nsINode *aRootNode)
 {
   NS_ENSURE_TRUE(aPossibleAncestorNode && aPossibleDescendantNode, PR_FALSE);
 
-  nsCOMPtr<nsIDOMNode> loopNode = aPossibleDescendantNode;
-  nsCOMPtr<nsIDOMNode> parentNode;
-  while (NS_SUCCEEDED(loopNode->GetParentNode(getter_AddRefs(parentNode))) &&
-         parentNode) {
-    if (parentNode == aPossibleAncestorNode) {
+  nsINode *parentNode = aPossibleDescendantNode;
+  while ((parentNode = parentNode->GetNodeParent()) &&
+         parentNode != aRootNode) {
+    if (parentNode == aPossibleAncestorNode)
       return PR_TRUE;
-    }
-    loopNode.swap(parentNode);
   }
-  return PR_FALSE;
-}
 
-PRBool
-nsCoreUtils::AreSiblings(nsIDOMNode *aDOMNode1,
-                        nsIDOMNode *aDOMNode2)
-{
-  NS_ENSURE_TRUE(aDOMNode1 && aDOMNode2, PR_FALSE);
-
-  nsCOMPtr<nsIDOMNode> parentNode1, parentNode2;
-  if (NS_SUCCEEDED(aDOMNode1->GetParentNode(getter_AddRefs(parentNode1))) &&
-      NS_SUCCEEDED(aDOMNode2->GetParentNode(getter_AddRefs(parentNode2))) &&
-      parentNode1 == parentNode2) {
-    return PR_TRUE;
-  }
   return PR_FALSE;
 }
 
@@ -349,7 +377,7 @@ nsCoreUtils::ScrollFrameToPoint(nsIFrame *aScrollableFrame,
   nsPoint scrollPoint = scrollableFrame->GetScrollPosition();
   scrollPoint -= deltaPoint;
 
-  scrollableFrame->ScrollTo(scrollPoint);
+  scrollableFrame->ScrollTo(scrollPoint, nsIScrollableFrame::INSTANT);
 }
 
 void
@@ -390,7 +418,7 @@ nsCoreUtils::ConvertScrollTypeToPercents(PRUint32 aScrollType,
 }
 
 nsIntPoint
-nsCoreUtils::GetScreenCoordsForWindow(nsIDOMNode *aNode)
+nsCoreUtils::GetScreenCoordsForWindow(nsINode *aNode)
 {
   nsIntPoint coords(0, 0);
   nsCOMPtr<nsIDocShellTreeItem> treeItem(GetDocShellTreeItemFor(aNode));
@@ -416,17 +444,12 @@ nsCoreUtils::GetScreenCoordsForWindow(nsIDOMNode *aNode)
 }
 
 already_AddRefed<nsIDocShellTreeItem>
-nsCoreUtils::GetDocShellTreeItemFor(nsIDOMNode *aNode)
+nsCoreUtils::GetDocShellTreeItemFor(nsINode *aNode)
 {
   if (!aNode)
     return nsnull;
 
-  nsCOMPtr<nsIDOMDocument> domDoc;
-  aNode->GetOwnerDocument(getter_AddRefs(domDoc));
-  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDoc));
-  if (!doc)
-    doc = do_QueryInterface(aNode);
-
+  nsIDocument *doc = aNode->GetOwnerDoc();
   NS_ASSERTION(doc, "No document for node passed in");
   NS_ENSURE_TRUE(doc, nsnull);
 
@@ -438,18 +461,64 @@ nsCoreUtils::GetDocShellTreeItemFor(nsIDOMNode *aNode)
   return docShellTreeItem;
 }
 
-nsIFrame*
-nsCoreUtils::GetFrameFor(nsIDOMElement *aElm)
+PRBool
+nsCoreUtils::IsDocumentBusy(nsIDocument *aDocument)
 {
-  nsCOMPtr<nsIPresShell> shell = GetPresShellFor(aElm);
-  if (!shell)
-    return nsnull;
-  
-  nsCOMPtr<nsIContent> content(do_QueryInterface(aElm));
-  if (!content)
-    return nsnull;
-  
-  return shell->GetPrimaryFrameFor(content);
+  nsCOMPtr<nsISupports> container = aDocument->GetContainer();
+  nsCOMPtr<nsIDocShell> docShell = do_QueryInterface(container);
+  if (!docShell)
+    return PR_TRUE;
+
+  PRUint32 busyFlags = 0;
+  docShell->GetBusyFlags(&busyFlags);
+  return (busyFlags != nsIDocShell::BUSY_FLAGS_NONE);
+}
+
+PRBool
+nsCoreUtils::IsRootDocument(nsIDocument *aDocument)
+{
+  nsCOMPtr<nsISupports> container = aDocument->GetContainer();
+  nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem =
+    do_QueryInterface(container);
+  NS_ASSERTION(docShellTreeItem, "No document shell for document!");
+
+  nsCOMPtr<nsIDocShellTreeItem> parentTreeItem;
+  docShellTreeItem->GetParent(getter_AddRefs(parentTreeItem));
+
+  return !parentTreeItem;
+}
+
+PRBool
+nsCoreUtils::IsContentDocument(nsIDocument *aDocument)
+{
+  nsCOMPtr<nsISupports> container = aDocument->GetContainer();
+  nsCOMPtr<nsIDocShellTreeItem> docShellTreeItem =
+    do_QueryInterface(container);
+  NS_ASSERTION(docShellTreeItem, "No document shell tree item for document!");
+
+  PRInt32 contentType;
+  docShellTreeItem->GetItemType(&contentType);
+  return (contentType == nsIDocShellTreeItem::typeContent);
+}
+
+PRBool
+nsCoreUtils::IsErrorPage(nsIDocument *aDocument)
+{
+  nsIURI *uri = aDocument->GetDocumentURI();
+  PRBool isAboutScheme = PR_FALSE;
+  uri->SchemeIs("about", &isAboutScheme);
+  if (!isAboutScheme)
+    return PR_FALSE;
+
+  nsCAutoString path;
+  uri->GetPath(path);
+
+  nsCAutoString::const_iterator start, end;
+  path.BeginReading(start);
+  path.EndReading(end);
+
+  NS_NAMED_LITERAL_CSTRING(neterror, "neterror");
+  return FindInReadable(neterror, start, end);
 }
 
 PRBool
@@ -463,25 +532,6 @@ nsCoreUtils::IsCorrectFrameType(nsIFrame *aFrame, nsIAtom *aAtom)
   return aFrame->GetType() == aAtom;
 }
 
-already_AddRefed<nsIPresShell>
-nsCoreUtils::GetPresShellFor(nsIDOMNode *aNode)
-{
-  nsCOMPtr<nsIDOMDocument> domDocument;
-  aNode->GetOwnerDocument(getter_AddRefs(domDocument));
-
-  nsCOMPtr<nsIDocument> doc(do_QueryInterface(domDocument));
-  if (!doc) // This is necessary when the node is the document node
-    doc = do_QueryInterface(aNode);
-
-  nsIPresShell *presShell = nsnull;
-  if (doc) {
-    presShell = doc->GetPrimaryShell();
-    NS_IF_ADDREF(presShell);
-  }
-
-  return presShell;
-}
-
 already_AddRefed<nsIDOMNode>
 nsCoreUtils::GetDOMNodeForContainer(nsIDocShellTreeItem *aContainer)
 {
@@ -493,17 +543,12 @@ nsCoreUtils::GetDOMNodeForContainer(nsIDocShellTreeItem *aContainer)
   if (!cv)
     return nsnull;
 
-  nsCOMPtr<nsIDocumentViewer> docv(do_QueryInterface(cv));
-  if (!docv)
-    return nsnull;
-
-  nsCOMPtr<nsIDocument> doc;
-  docv->GetDocument(getter_AddRefs(doc));
+  nsIDocument* doc = cv->GetDocument();
   if (!doc)
     return nsnull;
 
   nsIDOMNode* node = nsnull;
-  CallQueryInterface(doc.get(), &node);
+  CallQueryInterface(doc, &node);
   return node;
 }
 
@@ -514,6 +559,23 @@ nsCoreUtils::GetID(nsIContent *aContent, nsAString& aID)
   return idAttribute ? aContent->GetAttr(kNameSpaceID_None, idAttribute, aID) : PR_FALSE;
 }
 
+PRBool
+nsCoreUtils::GetUIntAttr(nsIContent *aContent, nsIAtom *aAttr, PRInt32 *aUInt)
+{
+  nsAutoString value;
+  aContent->GetAttr(kNameSpaceID_None, aAttr, value);
+  if (!value.IsEmpty()) {
+    PRInt32 error = NS_OK;
+    PRInt32 integer = value.ToInteger(&error);
+    if (NS_SUCCEEDED(error) && integer > 0) {
+      *aUInt = integer;
+      return PR_TRUE;
+    }
+  }
+
+  return PR_FALSE;
+}
+
 PRBool
 nsCoreUtils::IsXLink(nsIContent *aContent)
 {
@@ -690,7 +752,7 @@ nsCoreUtils::FindDescendantPointingToIDImpl(nsCString& aIdWithSpaces,
 nsIContent*
 nsCoreUtils::GetLabelContent(nsIContent *aForNode)
 {
-  if (aForNode->IsNodeOfType(nsINode::eXUL))
+  if (aForNode->IsXUL())
     return FindNeighbourPointingToNode(aForNode, nsAccessibilityAtoms::control,
                                        nsAccessibilityAtoms::label);
 
@@ -854,28 +916,27 @@ nsCoreUtils::GetElementsHavingIDRefsAttrImpl(nsIContent *aRootContent,
   }
 }
 
-void
+already_AddRefed<nsIDOMCSSStyleDeclaration>
 nsCoreUtils::GetComputedStyleDeclaration(const nsAString& aPseudoElt,
-                                         nsIDOMNode *aNode,
-                                         nsIDOMCSSStyleDeclaration **aCssDecl)
+                                         nsIContent *aContent)
 {
-  *aCssDecl = nsnull;
-
-  nsCOMPtr<nsIDOMElement> domElement = GetDOMElementFor(aNode);
-  if (!domElement)
-    return;
+  nsIContent* content = GetDOMElementFor(aContent);
+  if (!content)
+    return nsnull;
 
   // Returns number of items in style declaration
-  nsCOMPtr<nsIContent> content = do_QueryInterface(domElement);
-  nsCOMPtr<nsIDocument> doc = content->GetDocument();
-  if (!doc)
-    return;
+  nsIDocument* document = content->GetOwnerDoc();
+  if (!document)
+    return nsnull;
 
-  nsCOMPtr<nsIDOMViewCSS> viewCSS(do_QueryInterface(doc->GetWindow()));
+  nsCOMPtr<nsIDOMViewCSS> viewCSS(do_QueryInterface(document->GetWindow()));
   if (!viewCSS)
-    return;
+    return nsnull;
 
-  viewCSS->GetComputedStyle(domElement, aPseudoElt, aCssDecl);
+  nsIDOMCSSStyleDeclaration* cssDecl = nsnull;
+  nsCOMPtr<nsIDOMElement> domElement(do_QueryInterface(content));
+  viewCSS->GetComputedStyle(domElement, aPseudoElt, &cssDecl);
+  return cssDecl;
 }
 
 already_AddRefed<nsIBoxObject>
@@ -891,3 +952,216 @@ nsCoreUtils::GetTreeBodyBoxObject(nsITreeBoxObject *aTreeBoxObj)
   tcXULElm->GetBoxObject(&boxObj);
   return boxObj;
 }
+
+already_AddRefed<nsITreeBoxObject>
+nsCoreUtils::GetTreeBoxObject(nsIContent *aContent)
+{
+  // Find DOMNode's parents recursively until reach the <tree> tag
+  nsIContent* currentContent = aContent;
+  while (currentContent) {
+    if (currentContent->NodeInfo()->Equals(nsAccessibilityAtoms::tree,
+                                           kNameSpaceID_XUL)) {
+      // We will get the nsITreeBoxObject from the tree node
+      nsCOMPtr<nsIDOMXULElement> xulElement(do_QueryInterface(currentContent));
+      if (xulElement) {
+        nsCOMPtr<nsIBoxObject> box;
+        xulElement->GetBoxObject(getter_AddRefs(box));
+        nsCOMPtr<nsITreeBoxObject> treeBox(do_QueryInterface(box));
+        if (treeBox)
+          return treeBox.forget();
+      }
+    }
+    currentContent = currentContent->GetParent();
+  }
+
+  return nsnull;
+}
+
+already_AddRefed<nsITreeColumn>
+nsCoreUtils::GetFirstSensibleColumn(nsITreeBoxObject *aTree)
+{
+  nsCOMPtr<nsITreeColumns> cols;
+  aTree->GetColumns(getter_AddRefs(cols));
+  if (!cols)
+    return nsnull;
+
+  nsCOMPtr<nsITreeColumn> column;
+  cols->GetFirstColumn(getter_AddRefs(column));
+  if (column && IsColumnHidden(column))
+    return GetNextSensibleColumn(column);
+
+  return column.forget();
+}
+
+already_AddRefed<nsITreeColumn>
+nsCoreUtils::GetLastSensibleColumn(nsITreeBoxObject *aTree)
+{
+  nsCOMPtr<nsITreeColumns> cols;
+  aTree->GetColumns(getter_AddRefs(cols));
+  if (!cols)
+    return nsnull;
+
+  nsCOMPtr<nsITreeColumn> column;
+  cols->GetLastColumn(getter_AddRefs(column));
+  if (column && IsColumnHidden(column))
+    return GetPreviousSensibleColumn(column);
+
+  return column.forget();
+}
+
+PRUint32
+nsCoreUtils::GetSensibleColumnCount(nsITreeBoxObject *aTree)
+{
+  PRUint32 count = 0;
+
+  nsCOMPtr<nsITreeColumns> cols;
+  aTree->GetColumns(getter_AddRefs(cols));
+  if (!cols)
+    return count;
+
+  nsCOMPtr<nsITreeColumn> column;
+  cols->GetFirstColumn(getter_AddRefs(column));
+
+  while (column) {
+    if (!IsColumnHidden(column))
+      count++;
+
+    nsCOMPtr<nsITreeColumn> nextColumn;
+    column->GetNext(getter_AddRefs(nextColumn));
+    column.swap(nextColumn);
+  }
+
+  return count;
+}
+
+already_AddRefed<nsITreeColumn>
+nsCoreUtils::GetSensibleColumnAt(nsITreeBoxObject *aTree, PRUint32 aIndex)
+{
+  PRUint32 idx = aIndex;
+
+  nsCOMPtr<nsITreeColumn> column = GetFirstSensibleColumn(aTree);
+  while (column) {
+    if (idx == 0)
+      return column.forget();
+
+    idx--;
+    column = GetNextSensibleColumn(column);
+  }
+
+  return nsnull;
+}
+
+already_AddRefed<nsITreeColumn>
+nsCoreUtils::GetNextSensibleColumn(nsITreeColumn *aColumn)
+{
+  nsCOMPtr<nsITreeColumn> nextColumn;
+  aColumn->GetNext(getter_AddRefs(nextColumn));
+
+  while (nextColumn && IsColumnHidden(nextColumn)) {
+    nsCOMPtr<nsITreeColumn> tempColumn;
+    nextColumn->GetNext(getter_AddRefs(tempColumn));
+    nextColumn.swap(tempColumn);
+  }
+
+  return nextColumn.forget();
+}
+
+already_AddRefed<nsITreeColumn>
+nsCoreUtils::GetPreviousSensibleColumn(nsITreeColumn *aColumn)
+{
+  nsCOMPtr<nsITreeColumn> prevColumn;
+  aColumn->GetPrevious(getter_AddRefs(prevColumn));
+
+  while (prevColumn && IsColumnHidden(prevColumn)) {
+    nsCOMPtr<nsITreeColumn> tempColumn;
+    prevColumn->GetPrevious(getter_AddRefs(tempColumn));
+    prevColumn.swap(tempColumn);
+  }
+
+  return prevColumn.forget();
+}
+
+PRBool
+nsCoreUtils::IsColumnHidden(nsITreeColumn *aColumn)
+{
+  nsCOMPtr<nsIDOMElement> element;
+  aColumn->GetElement(getter_AddRefs(element));
+  nsCOMPtr<nsIContent> content = do_QueryInterface(element);
+  return content->AttrValueIs(kNameSpaceID_None, nsAccessibilityAtoms::hidden,
+                              nsAccessibilityAtoms::_true, eCaseMatters);
+}
+
+void
+nsCoreUtils::GeneratePopupTree(nsIContent *aContent, PRBool aIsAnon)
+{
+  // Set menugenerated="true" on the menupopup node to generate the sub-menu
+  // items if they have not been generated.
+
+  nsCOMPtr<nsIDOMNodeList> list;
+  if (aIsAnon) {    
+    nsIDocument* document = aContent->GetCurrentDoc();
+    if (document)
+      document->GetXBLChildNodesFor(aContent, getter_AddRefs(list));
+
+  } else {
+    list = aContent->GetChildNodesList();
+  }
+
+  PRUint32 length = 0;
+  if (!list || NS_FAILED(list->GetLength(&length)))
+    return;
+
+  for (PRUint32 idx = 0; idx < length; idx++) {
+    nsCOMPtr<nsIDOMNode> childNode;
+    list->Item(idx, getter_AddRefs(childNode));
+    nsCOMPtr<nsIContent> child(do_QueryInterface(childNode));
+
+    PRBool isPopup = child->NodeInfo()->Equals(nsAccessibilityAtoms::menupopup,
+                                               kNameSpaceID_XUL) ||
+                     child->NodeInfo()->Equals(nsAccessibilityAtoms::panel,
+                                               kNameSpaceID_XUL);
+    if (isPopup && !child->AttrValueIs(kNameSpaceID_None,
+                                       nsAccessibilityAtoms::menugenerated,
+                                       nsAccessibilityAtoms::_true,
+                                       eCaseMatters)) {
+
+      child->SetAttr(kNameSpaceID_None, nsAccessibilityAtoms::menugenerated,
+                     NS_LITERAL_STRING("true"), PR_TRUE);
+      return;
+    }
+  }
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+// nsAccessibleDOMStringList
+////////////////////////////////////////////////////////////////////////////////
+
+NS_IMPL_ISUPPORTS1(nsAccessibleDOMStringList, nsIDOMDOMStringList)
+
+NS_IMETHODIMP
+nsAccessibleDOMStringList::Item(PRUint32 aIndex, nsAString& aResult)
+{
+  if (aIndex >= mNames.Length())
+    SetDOMStringToNull(aResult);
+  else
+    aResult = mNames.ElementAt(aIndex);
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsAccessibleDOMStringList::GetLength(PRUint32 *aLength)
+{
+  *aLength = mNames.Length();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsAccessibleDOMStringList::Contains(const nsAString& aString, PRBool *aResult)
+{
+  *aResult = mNames.Contains(aString);
+
+  return NS_OK;
+}
