@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Drew Willcoxon <adw@mozilla.com> (Original Author)
+ *   Ehsan Akhgari <ehsan.akhgari@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -50,15 +51,13 @@
  */
 
 Cc["@mozilla.org/moz/jssubscript-loader;1"].
-  getService(Components.interfaces.mozIJSSubScriptLoader).
+  getService(Ci.mozIJSSubScriptLoader).
   loadSubScript("chrome://mochikit/content/MochiKit/packed.js");
 
 Cc["@mozilla.org/moz/jssubscript-loader;1"].
-  getService(Components.interfaces.mozIJSSubScriptLoader).
+  getService(Ci.mozIJSSubScriptLoader).
   loadSubScript("chrome://browser/content/sanitize.js");
 
-const winWatch = Cc["@mozilla.org/embedcomp/window-watcher;1"].
-                 getService(Ci.nsIWindowWatcher);
 const dm = Cc["@mozilla.org/download-manager;1"].
            getService(Ci.nsIDownloadManager);
 const bhist = Cc["@mozilla.org/browser/global-history;2"].
@@ -69,6 +68,22 @@ const formhist = Cc["@mozilla.org/satchel/form-history;1"].
 // Add tests here.  Each is a function that's called by doNextTest().
 var gAllTests = [
 
+  /**
+   * Initializes the dialog to its default state.
+   */
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Select "Last Hour"
+      this.selectDuration(Sanitizer.TIMESPAN_HOUR);
+      // Hide details
+      if (!this.getItemList().collapsed)
+        this.toggleDetails();
+      this.acceptDialog();
+    };
+    wh.open();
+  },
+
   /**
    * Cancels the dialog, makes sure history not cleared.
    */
@@ -230,16 +245,16 @@ var gAllTests = [
          "with a predefined timespan");
       this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
       this.checkPrefCheckbox("history", true);
-      this.checkDetails(false);
-
-      // Show details
-      this.toggleDetails();
       this.checkDetails(true);
 
       // Hide details
       this.toggleDetails();
       this.checkDetails(false);
 
+      // Show details
+      this.toggleDetails();
+      this.checkDetails(true);
+
       this.acceptDialog();
 
       intPrefIs("sanitize.timeSpan", Sanitizer.TIMESPAN_EVERYTHING,
@@ -280,15 +295,57 @@ var gAllTests = [
   },
 
   /**
-   * These next two tests together ensure that toggling details persists
+   * These next six tests together ensure that toggling details persists
    * across dialog openings.
    */
   function () {
     let wh = new WindowHelper();
     wh.onload = function () {
-      // Show details
+      // Check all items and select "Everything"
+      this.checkAllCheckboxes();
+      this.selectDuration(Sanitizer.TIMESPAN_EVERYTHING);
+
+      // Hide details
       this.toggleDetails();
+      this.checkDetails(false);
+      this.acceptDialog();
+    };
+    wh.open();
+  },
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Details should remain closed because all items are checked.
+      this.checkDetails(false);
+
+      // Uncheck history.
+      this.checkPrefCheckbox("history", false);
+      this.acceptDialog();
+    };
+    wh.open();
+  },
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Details should be open because not all items are checked.
       this.checkDetails(true);
+
+      // Modify the Site Preferences item state (bug 527820)
+      this.checkAllCheckboxes();
+      this.checkPrefCheckbox("siteSettings", false);
+      this.acceptDialog();
+    };
+    wh.open();
+  },
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Details should be open because not all items are checked.
+      this.checkDetails(true);
+
+      // Hide details
+      this.toggleDetails();
+      this.checkDetails(false);
       this.cancelDialog();
     };
     wh.open();
@@ -296,12 +353,37 @@ var gAllTests = [
   function () {
     let wh = new WindowHelper();
     wh.onload = function () {
-      // Details should have remained open
+      // Details should be open because not all items are checked.
       this.checkDetails(true);
-      
+
+      // Select another duration
+      this.selectDuration(Sanitizer.TIMESPAN_HOUR);
       // Hide details
       this.toggleDetails();
       this.checkDetails(false);
+      this.acceptDialog();
+    };
+    wh.open();
+  },
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Details should not be open because "Last Hour" is selected
+      this.checkDetails(false);
+
+      this.cancelDialog();
+    };
+    wh.open();
+  },
+  function () {
+    let wh = new WindowHelper();
+    wh.onload = function () {
+      // Details should have remained closed
+      this.checkDetails(false);
+
+      // Show details
+      this.toggleDetails();
+      this.checkDetails(true);
       this.cancelDialog();
     };
     wh.open();
@@ -368,8 +450,10 @@ WindowHelper.prototype = {
        "Details button should be " + dir + " because item list is " +
        (hidden ? "" : "not ") + "hidden");
     let height = 0;
-    if (!hidden)
+    if (!hidden) {
+      ok(list.boxObject.height > 30, "listbox has sufficient size")
       height += list.boxObject.height;
+    }
     if (this.isWarningPanelVisible())
       height += this.getWarningPanel().boxObject.height;
     ok(height < this.win.innerHeight,
@@ -394,6 +478,19 @@ WindowHelper.prototype = {
       cb[0].click();
   },
 
+  /**
+   * Makes sure all the checkboxes are checked.
+   */
+  checkAllCheckboxes: function () {
+    var cb = this.win.document.querySelectorAll("#itemList > [preference]");
+    ok(cb.length > 1, "found checkboxes for preferences");
+    for (var i = 0; i < cb.length; ++i) {
+      var pref = this.win.document.getElementById(cb[i].getAttribute("preference"));
+      if (!pref.value)
+        cb[i].click();
+    }
+  },
+
   /**
    * @return The details progressive disclosure button
    */
@@ -439,75 +536,73 @@ WindowHelper.prototype = {
   open: function () {
     let wh = this;
 
-    let windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic !== "domwindowopened")
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic != "domwindowopened")
+        return;
+
+      Services.ww.unregisterNotification(windowObserver);
+
+      var loaded = false;
+      let win = aSubject.QueryInterface(Ci.nsIDOMWindow);
+
+      win.addEventListener("load", function onload(event) {
+        win.removeEventListener("load", onload, false);
+
+        if (win.name !== "SanitizeDialog")
           return;
 
-        winWatch.unregisterNotification(this);
+        wh.win = win;
+        loaded = true;
 
-        var loaded = false;
-        let win = aSubject.QueryInterface(Ci.nsIDOMWindow);
-
-        win.addEventListener("load", function onload(event) {
-          win.removeEventListener("load", onload, false);
-
-          if (win.name !== "SanitizeDialog")
-            return;
-
-          wh.win = win;
-          loaded = true;
-
-          executeSoon(function () {
-            // Some exceptions that reach here don't reach the test harness, but
-            // ok()/is() do...
-            try {
-              wh.onload();
-            }
-            catch (exc) {
-              win.close();
-              ok(false, "Unexpected exception: " + exc + "\n" + exc.stack);
-              finish();
-            }
-          });
-        }, false);
-
-        win.addEventListener("unload", function onunload(event) {
-          if (win.name !== "SanitizeDialog") {
-            win.removeEventListener("unload", onunload, false);
-            return;
+        executeSoon(function () {
+          // Some exceptions that reach here don't reach the test harness, but
+          // ok()/is() do...
+          try {
+            wh.onload();
           }
+          catch (exc) {
+            win.close();
+            ok(false, "Unexpected exception: " + exc + "\n" + exc.stack);
+            finish();
+          }
+        });
+      }, false);
 
-          // Why is unload fired before load?
-          if (!loaded)
-            return;
-
+      win.addEventListener("unload", function onunload(event) {
+        if (win.name !== "SanitizeDialog") {
           win.removeEventListener("unload", onunload, false);
-          wh.win = win;
+          return;
+        }
 
-          executeSoon(function () {
-            // Some exceptions that reach here don't reach the test harness, but
-            // ok()/is() do...
-            try {
-              if (wh.onunload)
-                wh.onunload();
-              doNextTest();
-            }
-            catch (exc) {
-              win.close();
-              ok(false, "Unexpected exception: " + exc + "\n" + exc.stack);
-              finish();
-            }
-          });
-        }, false);
-      }
-    };
-    winWatch.registerNotification(windowObserver);
-    winWatch.openWindow(null,
-                        "chrome://browser/content/sanitize.xul",
-                        "SanitizeDialog",
-                        "chrome,titlebar,dialog,centerscreen,modal",
-                        null);
+        // Why is unload fired before load?
+        if (!loaded)
+          return;
+
+        win.removeEventListener("unload", onunload, false);
+        wh.win = win;
+
+        executeSoon(function () {
+          // Some exceptions that reach here don't reach the test harness, but
+          // ok()/is() do...
+          try {
+            if (wh.onunload)
+              wh.onunload();
+            doNextTest();
+          }
+          catch (exc) {
+            win.close();
+            ok(false, "Unexpected exception: " + exc + "\n" + exc.stack);
+            finish();
+          }
+        });
+      }, false);
+    }
+    Services.ww.registerNotification(windowObserver);
+    Services.ww.openWindow(null,
+                           "chrome://browser/content/sanitize.xul",
+                           "SanitizeDialog",
+                           "chrome,titlebar,dialog,centerscreen,modal",
+                           null);
   },
 
   /**
@@ -655,7 +750,7 @@ function downloadExists(aID)
     "WHERE id = :id"
   );
   stmt.params.id = aID;
-  let rows = stmt.step();
+  let rows = stmt.executeStep();
   stmt.finalize();
   return !!rows;
 }
