@@ -67,6 +67,9 @@ const TYPE_BOOKMARK = 1;
 
 const TEST_URL = "http://www.mozilla.org/";
 
+const DIALOG_URL = "chrome://browser/content/places/bookmarkProperties.xul";
+const DIALOG_URL_MINIMAL_UI = "chrome://browser/content/places/bookmarkProperties2.xul";
+
 var wm = Cc["@mozilla.org/appshell/window-mediator;1"].
          getService(Ci.nsIWindowMediator);
 var win = wm.getMostRecentWindow("navigator:browser");
@@ -208,7 +211,7 @@ gTests.push({
     // Add a tag to this bookmark.
     PlacesUtils.tagging.tagURI(PlacesUtils._uri(TEST_URL),
                                ["testTag"]);
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Correctly added a tag");
   },
 
@@ -222,16 +225,15 @@ gTests.push({
     var tagsField = this.window.document.getElementById("editBMPanel_tagsField");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed") {
-          ww.unregisterNotification(this);
-          tagsField.popup.removeEventListener("popuphidden", popupListener, true);
-          ok(false, "Dialog window should not be closed by pressing Enter on the autocomplete popup");
-          self.finish();
-        }
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic == "domwindowclosed" &&
+          aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL) {
+        ww.unregisterNotification(windowObserver);
+        tagsField.popup.removeEventListener("popuphidden", popupListener, true);
+        ok(false, "Dialog window should not be closed by pressing Enter on the autocomplete popup");
+        self.finish();
       }
-    };
+    }
 
     var popupListener = {
       handleEvent: function(aEvent) {
@@ -282,7 +284,7 @@ gTests.push({
 
   cleanup: function() {
     // Check tags have not changed.
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Tag on node has not changed");
 
     // Cleanup.
@@ -366,7 +368,7 @@ gTests.push({
     // Add a tag to this bookmark.
     PlacesUtils.tagging.tagURI(PlacesUtils._uri(TEST_URL),
                                ["testTag"]);
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Correctly added a tag");
   },
 
@@ -380,16 +382,15 @@ gTests.push({
     var tagsField = this.window.document.getElementById("editBMPanel_tagsField");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed") {
-          ww.unregisterNotification(this);
-          tagsField.popup.removeEventListener("popuphidden", popupListener, true);
-          ok(false, "Dialog window should not be closed by pressing Escape on the autocomplete popup");
-          self.finish();
-        }
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic == "domwindowclosed" &&
+          aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL) {
+        ww.unregisterNotification(windowObserver);
+        tagsField.popup.removeEventListener("popuphidden", popupListener, true);
+        ok(false, "Dialog window should not be closed by pressing Escape on the autocomplete popup");
+        self.finish();
       }
-    };
+    }
 
     var popupListener = {
       handleEvent: function(aEvent) {
@@ -440,7 +441,7 @@ gTests.push({
 
   cleanup: function() {
     // Check tags have not changed.
-    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL), {});
+    var tags = PlacesUtils.tagging.getTagsForURI(PlacesUtils._uri(TEST_URL));
     is(tags[0], "testTag", "Tag on node has not changed");
 
     // Cleanup.
@@ -481,16 +482,15 @@ gTests.push({
     var folderTree = this.window.document.getElementById("editBMPanel_folderTree");
     var self = this;
 
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowclosed") {
-          ww.unregisterNotification(this);
-          ok(self._cleanShutdown,
-             "Dialog window should not be closed by pressing ESC in folder name textbox");
-          self.finish();
-        }
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic == "domwindowclosed" &&
+          aSubject.QueryInterface(Ci.nsIDOMWindow).location == DIALOG_URL_MINIMAL_UI) {
+        ww.unregisterNotification(windowObserver);
+        ok(self._cleanShutdown,
+           "Dialog window should not be closed by pressing ESC in folder name textbox");
+        self.finish();
       }
-    };
+    }
     ww.registerNotification(windowObserver);
 
     folderTree.addEventListener("DOMAttrModified", function onDOMAttrModified(event) {
@@ -530,6 +530,10 @@ gTests.push({
 
 function test() {
   waitForExplicitFinish();
+  // This test can take some time, if we timeout too early it could run
+  // in the middle of other tests, or hang them.
+  requestLongerTimeout(2);
+
   // Sanity checks.
   ok(PlacesUtils, "PlacesUtils in context");
   ok(PlacesUIUtils, "PlacesUIUtils in context");
@@ -543,6 +547,9 @@ function runNextTest() {
   if (gCurrentTest) {
     gCurrentTest.cleanup();
     info("End of test: " + gCurrentTest.desc);
+    gCurrentTest = null;
+    executeSoon(runNextTest);
+    return;
   }
 
   if (gTests.length > 0) {
@@ -592,32 +599,30 @@ function open_properties_dialog() {
        "We have a places node selected: " + tree.selectedNode.title);
 
     // Wait for the Properties dialog.
-    var windowObserver = {
-      observe: function(aSubject, aTopic, aData) {
-        if (aTopic === "domwindowopened") {
-          ww.unregisterNotification(this);
-          var win = aSubject.QueryInterface(Ci.nsIDOMWindow);
-          win.addEventListener("focus", function(event) {
-            win.removeEventListener("focus", arguments.callee, false);
-            // Windows has been loaded, execute our test now.
-            executeSoon(function () {
-              // Ensure overlay is loaded
-              ok(win.gEditItemOverlay._initialized, "EditItemOverlay is initialized");
-              gCurrentTest.window = win;
-              try {
-                gCurrentTest.run();
-              } catch (ex) {
-                ok(false, "An error occured during test run: " + ex.message);
-              }
-            });
-          }, false);
-        }
-      }
-    };
+    function windowObserver(aSubject, aTopic, aData) {
+      if (aTopic != "domwindowopened")
+        return;
+      ww.unregisterNotification(windowObserver);
+      var win = aSubject.QueryInterface(Ci.nsIDOMWindow);
+      win.addEventListener("focus", function (event) {
+        win.removeEventListener("focus", arguments.callee, false);
+        // Windows has been loaded, execute our test now.
+        executeSoon(function () {
+          // Ensure overlay is loaded
+          ok(win.gEditItemOverlay._initialized, "EditItemOverlay is initialized");
+          gCurrentTest.window = win;
+          try {
+            gCurrentTest.run();
+          } catch (ex) {
+            ok(false, "An error occured during test run: " + ex.message);
+          }
+        });
+      }, false);
+    }
     ww.registerNotification(windowObserver);
 
     var command = null;
-    switch(gCurrentTest.action) {
+    switch (gCurrentTest.action) {
       case ACTION_EDIT:
         command = "placesCmd_show:info";
         break;
