@@ -1,7 +1,7 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et cindent: */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: ML 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Mozilla Public License Version
  * 1.1 (the "License"); you may not use this file except in compliance with
@@ -42,7 +42,6 @@
 #include "nsIDocument.h"
 #include "nsIFrame.h"
 #include "nsIObserver.h"
-#include "nsIObserverService.h"
 #include "nsISeekableStream.h"
 #include "nsAudioStream.h"
 #include "nsAutoLock.h"
@@ -50,10 +49,18 @@
 #include "nsNetUtil.h"
 #include "nsThreadUtils.h"
 #include "nsWaveDecoder.h"
+#include "nsHTMLTimeRanges.h"
 
 using mozilla::TimeDuration;
 using mozilla::TimeStamp;
 
+#ifdef PR_LOGGING
+static PRLogModuleInfo* gWaveDecoderLog;
+#define LOG(type, msg) PR_LOG(gWaveDecoderLog, type, msg)
+#else
+#define LOG(type, msg)
+#endif
+
 // Maximum number of seconds to wait when buffering.
 #define BUFFERING_TIMEOUT 3
 
@@ -155,7 +162,7 @@ public:
   // Called on the decoder thread
   void NotifyBytesConsumed(PRInt64 aBytes);
 
-  // Called by the main thread only
+  // Called by decoder and main thread.
   nsHTMLMediaElement::NextFrameStatus GetNextFrameStatus();
 
   // Clear the flag indicating that a playback position change event is
@@ -163,6 +170,8 @@ public:
   // main thread.
   float GetTimeForPositionChange();
 
+  nsresult GetBuffered(nsHTMLTimeRanges* aBuffered);
+
 private:
   // Returns PR_TRUE if we're in shutdown state. Threadsafe.
   PRBool IsShutdown();
@@ -172,6 +181,27 @@ private:
   // this.
   PRBool ReadAll(char* aBuf, PRInt64 aSize, PRInt64* aBytesRead);
 
+  void UpdateReadyState() {
+    PR_ASSERT_CURRENT_THREAD_IN_MONITOR(mMonitor);
+
+    nsCOMPtr<nsIRunnable> event;
+    switch (GetNextFrameStatus()) {
+      case nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE_BUFFERING:
+        event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::NextFrameUnavailableBuffering);
+        break;
+      case nsHTMLMediaElement::NEXT_FRAME_AVAILABLE:
+        event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::NextFrameAvailable);
+        break;
+      case nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE:
+        event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::NextFrameUnavailable);
+        break;
+      default:
+        PR_NOT_REACHED("unhandled frame state");
+    }
+
+    NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+  }
+
   // Change the current state and wake the playback thread if it is waiting
   // on mMonitor.  Used by public member functions called from both threads,
   // so must hold mMonitor.  Threadsafe.
@@ -226,7 +256,7 @@ private:
   PRInt64 TimeToBytes(float aTime) const
   {
     NS_ABORT_IF_FALSE(mMetadataValid, "Requires valid metadata");
-    NS_ABORT_IF_FALSE(aTime >= 0.0, "Must be >= 0");
+    NS_ABORT_IF_FALSE(aTime >= 0.0f, "Must be >= 0");
     return RoundDownToSample(PRInt64(aTime * mSampleRate * mSampleSize));
   }
 
@@ -353,7 +383,7 @@ nsWaveStateMachine::nsWaveStateMachine(nsWaveDecoder* aDecoder,
     mNextState(STATE_PAUSED),
     mPlaybackPosition(0),
     mInitialVolume(aInitialVolume),
-    mSeekTime(0.0),
+    mSeekTime(0.0f),
     mMetadataValid(PR_FALSE),
     mPositionChangeQueued(PR_FALSE),
     mPaused(mNextState == STATE_PAUSED)
@@ -416,8 +446,8 @@ nsWaveStateMachine::Seek(float aTime)
 {
   nsAutoMonitor monitor(mMonitor);
   mSeekTime = aTime;
-  if (mSeekTime < 0.0) {
-    mSeekTime = 0.0;
+  if (mSeekTime < 0.0f) {
+    mSeekTime = 0.0f;
   }
   if (mState == STATE_LOADING_METADATA) {
     mNextState = STATE_SEEKING;
@@ -461,7 +491,12 @@ nsWaveStateMachine::GetNextFrameStatus()
   nsAutoMonitor monitor(mMonitor);
   if (mState == STATE_BUFFERING)
     return nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE_BUFFERING;
-  if (mPlaybackPosition < mStream->GetCachedDataEnd(mPlaybackPosition))
+  // If mMetadataValid is false then we can't call GetDataLength because
+  // we haven't got the length from the Wave header yet. But we know that
+  // if we haven't read the metadata then we don't have playable data.
+  if (mMetadataValid &&
+      mPlaybackPosition < mStream->GetCachedDataEnd(mPlaybackPosition) &&
+      mPlaybackPosition < mWavePCMOffset + GetDataLength())
     return nsHTMLMediaElement::NEXT_FRAME_AVAILABLE;
   return nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE;
 }
@@ -490,25 +525,17 @@ nsWaveStateMachine::Run()
         PRBool loaded = LoadRIFFChunk() && LoadFormatChunk() && FindDataOffset();
         monitor.Enter();
 
+        if (!loaded) {
+          ChangeState(STATE_ERROR);
+        }
+
         if (mState == STATE_LOADING_METADATA) {
-          nsCOMPtr<nsIRunnable> event;
-          State newState;
-
-          if (loaded) {
-            mMetadataValid = PR_TRUE;
-            if (mNextState != STATE_SEEKING) {
-              event = NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, MetadataLoaded);
-            }
-            newState = mNextState;
-          } else {
-            event = NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, MediaErrorDecode);
-            newState = STATE_ERROR;
-          }
-
-          if (event) {
+          mMetadataValid = PR_TRUE;
+          if (mNextState != STATE_SEEKING) {
+            nsCOMPtr<nsIRunnable> event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::MetadataLoaded);
             NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
           }
-          ChangeState(newState);
+          ChangeState(mNextState);
         }
       }
       break;
@@ -526,9 +553,7 @@ nsWaveStateMachine::Run()
         monitor.Wait(PR_MillisecondsToInterval(1000));
       } else {
         ChangeState(mNextState);
-        nsCOMPtr<nsIRunnable> event =
-          NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, UpdateReadyStateForData);
-        NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+        UpdateReadyState();
       }
 
       break;
@@ -545,7 +570,7 @@ nsWaveStateMachine::Run()
 
       TimeStamp now = TimeStamp::Now();
       TimeStamp lastWakeup = now -
-          TimeDuration::FromMilliseconds(AUDIO_BUFFER_LENGTH);
+        TimeDuration::FromMilliseconds(AUDIO_BUFFER_LENGTH);
 
       do {
         TimeDuration sleepTime = now - lastWakeup;
@@ -563,7 +588,7 @@ nsWaveStateMachine::Run()
           targetTime = sleepTime;
         }
 
-        PRInt64 len = TimeToBytes(targetTime.ToSeconds());
+        PRInt64 len = TimeToBytes(float(targetTime.ToSeconds()));
 
         PRInt64 leftToPlay =
           GetDataLength() - (mPlaybackPosition - mWavePCMOffset);
@@ -580,17 +605,16 @@ nsWaveStateMachine::Run()
         if (mState != STATE_ENDED &&
             availableOffset < mPlaybackPosition + len &&
             !mStream->IsSuspendedByCache()) {
-            mBufferingStart = now;
-            mBufferingEndOffset = mPlaybackPosition +
-              TimeToBytes(mBufferingWait.ToSeconds());
-            mBufferingEndOffset = PR_MAX(mPlaybackPosition + len, mBufferingEndOffset);
-            mNextState = mState;
-            ChangeState(STATE_BUFFERING);
+          mBufferingStart = now;
+          mBufferingEndOffset = mPlaybackPosition +
+            TimeToBytes(float(mBufferingWait.ToSeconds()));
+          mBufferingEndOffset = PR_MAX(mPlaybackPosition + len,
+                                       mBufferingEndOffset);
+          mNextState = mState;
+          ChangeState(STATE_BUFFERING);
 
-            nsCOMPtr<nsIRunnable> event =
-              NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, UpdateReadyStateForData);
-            NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
-            break;
+          UpdateReadyState();
+          break;
         }
 
         if (len > 0) {
@@ -625,10 +649,10 @@ nsWaveStateMachine::Run()
 
           PRUint32 sampleSize = mSampleFormat == nsAudioStream::FORMAT_U8 ? 1 : 2;
           NS_ABORT_IF_FALSE(got % sampleSize == 0, "Must write complete samples");
-          PRUint32 lengthInSamples = got / sampleSize;
+          PRUint32 lengthInSamples = PRUint32(got / sampleSize);
 
           monitor.Exit();
-          mAudioStream->Write(buf.get(), lengthInSamples);
+          mAudioStream->Write(buf.get(), lengthInSamples, PR_FALSE);
           monitor.Enter();
 
           FirePositionChanged(PR_FALSE);
@@ -651,7 +675,7 @@ nsWaveStateMachine::Run()
 
         monitor.Exit();
         nsCOMPtr<nsIRunnable> startEvent =
-          NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, SeekingStarted);
+          NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::SeekingStarted);
         NS_DispatchToMainThread(startEvent, NS_DISPATCH_SYNC);
         monitor.Enter();
 
@@ -661,7 +685,8 @@ nsWaveStateMachine::Run()
 
         // Calculate relative offset within PCM data.
         PRInt64 position = RoundDownToSample(TimeToBytes(seekTime));
-        NS_ABORT_IF_FALSE(position >= 0 && position <= GetDataLength(), "Invalid seek position");
+        NS_ABORT_IF_FALSE(position >= 0 && position <= GetDataLength(),
+                          "Invalid seek position");
         // Convert to absolute offset within stream.
         position += mWavePCMOffset;
 
@@ -691,11 +716,15 @@ nsWaveStateMachine::Run()
           // event, playback has not ended as far as the user's
           // concerned--the state machine needs to return to the last
           // playback state.
+          // Special case #3: if seeking to the end of the media, transition
+          // directly into STATE_ENDED.
           State nextState = mNextState;
           if (nextState == STATE_SEEKING) {
             nextState = STATE_PAUSED;
           } else if (nextState == STATE_ENDED) {
             nextState = mPaused ? STATE_PAUSED : STATE_PLAYING;
+          } else if (GetDuration() == seekTime) {
+            nextState = STATE_ENDED;
           }
           ChangeState(nextState);
         }
@@ -704,7 +733,7 @@ nsWaveStateMachine::Run()
 
         monitor.Exit();
         nsCOMPtr<nsIRunnable> stopEvent =
-          NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, SeekingStopped);
+          NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::SeekingStopped);
         NS_DispatchToMainThread(stopEvent, NS_DISPATCH_SYNC);
         monitor.Enter();
       }
@@ -729,7 +758,7 @@ nsWaveStateMachine::Run()
 
       if (mState == STATE_ENDED) {
         nsCOMPtr<nsIRunnable> event =
-          NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, PlaybackEnded);
+          NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::PlaybackEnded);
         NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
 
         do {
@@ -739,10 +768,16 @@ nsWaveStateMachine::Run()
       break;
 
     case STATE_ERROR:
-      monitor.Wait();
-      if (mState != STATE_SHUTDOWN) {
-        NS_WARNING("Invalid state transition");
-        ChangeState(STATE_ERROR);
+      {
+        nsCOMPtr<nsIRunnable> event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::DecodeError);
+        NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
+
+        monitor.Wait();
+
+        if (mState != STATE_SHUTDOWN) {
+          NS_WARNING("Invalid state transition");
+          ChangeState(STATE_ERROR);
+        }
       }
       break;
 
@@ -785,7 +820,8 @@ IsValidStateTransition(State aStartState, State aEndState)
       return PR_TRUE;
     break;
   case STATE_SEEKING:
-    if (aEndState == STATE_PLAYING || aEndState == STATE_PAUSED)
+    if (aEndState == STATE_PLAYING || aEndState == STATE_PAUSED ||
+        aEndState == STATE_ENDED)
       return PR_TRUE;
     break;
   case STATE_PAUSED:
@@ -916,7 +952,7 @@ nsWaveStateMachine::ReadAll(char* aBuf, PRInt64 aSize, PRInt64* aBytesRead = nsn
   }
   do {
     PRUint32 read = 0;
-    if (NS_FAILED(mStream->Read(aBuf + got, aSize - got, &read))) {
+    if (NS_FAILED(mStream->Read(aBuf + got, PRUint32(aSize - got), &read))) {
       NS_WARNING("Stream read failed");
       return PR_FALSE;
     }
@@ -976,19 +1012,23 @@ nsWaveStateMachine::ScanForwardUntil(PRUint32 aWantedChunk, PRUint32* aChunkSize
     }
 
     PRUint32 magic = ReadUint32BE(&p);
-    PRUint32 size = ReadUint32LE(&p);
+    PRUint32 chunkSize = ReadUint32LE(&p);
 
     if (magic == aWantedChunk) {
-      *aChunkSize = size;
+      *aChunkSize = chunkSize;
       return PR_TRUE;
     }
 
     // RIFF chunks are two-byte aligned, so round up if necessary.
-    size += size % 2;
+    chunkSize += chunkSize % 2;
 
-    nsAutoArrayPtr<char> chunk(new char[size]);
-    if (!ReadAll(chunk.get(), size)) {
-      return PR_FALSE;
+    while (chunkSize > 0) {
+      PRUint32 size = PR_MIN(chunkSize, 1 << 16);
+      nsAutoArrayPtr<char> chunk(new char[size]);
+      if (!ReadAll(chunk.get(), size)) {
+        return PR_FALSE;
+      }
+      chunkSize -= size;
     }
   }
 }
@@ -1136,35 +1176,70 @@ nsWaveStateMachine::FirePositionChanged(PRBool aCoalesce)
   }
 
   mPositionChangeQueued = PR_TRUE;
-  nsCOMPtr<nsIRunnable> event = NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, mDecoder, PlaybackPositionChanged);
+  nsCOMPtr<nsIRunnable> event = NS_NewRunnableMethod(mDecoder, &nsWaveDecoder::PlaybackPositionChanged);
   NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
 }
 
+nsresult
+nsWaveStateMachine::GetBuffered(nsHTMLTimeRanges* aBuffered)
+{
+  PRInt64 startOffset = mStream->GetNextCachedData(mWavePCMOffset);
+  while (startOffset >= 0) {
+    PRInt64 endOffset = mStream->GetCachedDataEnd(startOffset);
+    // Bytes [startOffset..endOffset] are cached.
+    aBuffered->Add(BytesToTime(startOffset - mWavePCMOffset),
+                   BytesToTime(endOffset - mWavePCMOffset));
+    startOffset = mStream->GetNextCachedData(endOffset);
+  }
+  return NS_OK;
+}
+
 NS_IMPL_THREADSAFE_ISUPPORTS1(nsWaveDecoder, nsIObserver)
 
 nsWaveDecoder::nsWaveDecoder()
-  : mInitialVolume(1.0),
-    mCurrentTime(0.0),
+  : mInitialVolume(1.0f),
+    mCurrentTime(0.0f),
     mEndedDuration(std::numeric_limits<float>::quiet_NaN()),
     mEnded(PR_FALSE),
-    mNotifyOnShutdown(PR_FALSE),
     mSeekable(PR_TRUE),
     mResourceLoaded(PR_FALSE),
     mMetadataLoadedReported(PR_FALSE),
     mResourceLoadedReported(PR_FALSE)
 {
   MOZ_COUNT_CTOR(nsWaveDecoder);
+
+#ifdef PR_LOGGING
+  if (!gWaveDecoderLog) {
+    gWaveDecoderLog = PR_NewLogModule("nsWaveDecoder");
+  }
+#endif
 }
 
 nsWaveDecoder::~nsWaveDecoder()
 {
   MOZ_COUNT_DTOR(nsWaveDecoder);
+  UnpinForSeek();
 }
 
-void
-nsWaveDecoder::GetCurrentURI(nsIURI** aURI)
+PRBool
+nsWaveDecoder::Init(nsHTMLMediaElement* aElement)
 {
-  NS_IF_ADDREF(*aURI = mURI);
+  nsMediaDecoder::Init(aElement);
+
+  nsContentUtils::RegisterShutdownObserver(this);
+
+  mPlaybackStateMachine = new nsWaveStateMachine(this,
+    TimeDuration::FromMilliseconds(BUFFERING_TIMEOUT),
+    mInitialVolume);
+  NS_ENSURE_TRUE(mPlaybackStateMachine, PR_FALSE);
+
+  return PR_TRUE;
+}
+
+nsMediaStream*
+nsWaveDecoder::GetCurrentStream()
+{
+  return mStream;
 }
 
 already_AddRefed<nsIPrincipal>
@@ -1186,6 +1261,7 @@ nsresult
 nsWaveDecoder::Seek(float aTime)
 {
   if (mPlaybackStateMachine) {
+    PinForSeek();
     mPlaybackStateMachine->Seek(aTime);
     return NS_OK;
   }
@@ -1260,44 +1336,23 @@ nsWaveDecoder::Stop()
   mPlaybackStateMachine = nsnull;
   mStream = nsnull;
 
-  UnregisterShutdownObserver();
+  nsContentUtils::UnregisterShutdownObserver(this);
 }
 
 nsresult
-nsWaveDecoder::Load(nsIURI* aURI, nsIChannel* aChannel, nsIStreamListener** aStreamListener)
+nsWaveDecoder::Load(nsMediaStream* aStream, nsIStreamListener** aStreamListener)
 {
-  // Reset progress member variables
-  mResourceLoaded = PR_FALSE;
-  mResourceLoadedReported = PR_FALSE;
-  mMetadataLoadedReported = PR_FALSE;
+  NS_ASSERTION(aStream, "A stream should be provided");
 
   if (aStreamListener) {
     *aStreamListener = nsnull;
   }
 
-  if (aURI) {
-    NS_ASSERTION(!aStreamListener, "No listener should be requested here");
-    mURI = aURI;
-  } else {
-    NS_ASSERTION(aChannel, "Either a URI or a channel is required");
-    NS_ASSERTION(aStreamListener, "A listener should be requested here");
+  mStream = aStream;
 
-    nsresult rv = NS_GetFinalChannelURI(aChannel, getter_AddRefs(mURI));
-    NS_ENSURE_SUCCESS(rv, rv);
-  }
-
-  RegisterShutdownObserver();
-
-  mPlaybackStateMachine = new nsWaveStateMachine(this,
-    TimeDuration::FromMilliseconds(BUFFERING_TIMEOUT),
-    mInitialVolume);
-  NS_ENSURE_TRUE(mPlaybackStateMachine, NS_ERROR_OUT_OF_MEMORY);
-
-  // Open the stream *after* setting mPlaybackStateMachine, to ensure
-  // that callbacks (e.g. setting stream size) will actually work
-  nsresult rv = nsMediaStream::Open(this, mURI, aChannel, getter_Transfers(mStream),
-                                    aStreamListener);
+  nsresult rv = mStream->Open(aStreamListener);
   NS_ENSURE_SUCCESS(rv, rv);
+
   mPlaybackStateMachine->SetStream(mStream);
 
   rv = NS_NewThread(getter_AddRefs(mPlaybackThread));
@@ -1341,6 +1396,9 @@ nsWaveDecoder::PlaybackEnded()
     return;
   }
 
+  // Update ready state; now that we've finished playback, we should
+  // switch to HAVE_CURRENT_DATA.
+  UpdateReadyStateForData();
   if (mElement) {
     mElement->PlaybackEnded();
   }
@@ -1421,6 +1479,7 @@ void
 nsWaveDecoder::NotifyBytesDownloaded()
 {
   UpdateReadyStateForData();
+  Progress(PR_FALSE);
 }
 
 void
@@ -1451,7 +1510,7 @@ nsWaveDecoder::Shutdown()
   // this event is posted asynchronously to the main thread to perform the
   // shutdown.
   nsCOMPtr<nsIRunnable> event =
-    NS_NEW_RUNNABLE_METHOD(nsWaveDecoder, this, Stop);
+    NS_NewRunnableMethod(this, &nsWaveDecoder::Stop);
   NS_DispatchToMainThread(event, NS_DISPATCH_NORMAL);
 }
 
@@ -1464,9 +1523,44 @@ nsWaveDecoder::Observe(nsISupports* aSubject, const char* aTopic, const PRUnicha
   return NS_OK;
 }
 
+void
+nsWaveDecoder::NextFrameUnavailableBuffering()
+{
+  NS_ASSERTION(NS_IsMainThread(), "Should be called on main thread");
+  if (!mElement || mShuttingDown || !mPlaybackStateMachine)
+    return;
+
+  mElement->UpdateReadyStateForData(nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE_BUFFERING);
+}
+
+void
+nsWaveDecoder::NextFrameAvailable()
+{
+  NS_ASSERTION(NS_IsMainThread(), "Should be called on main thread");
+  if (!mElement || mShuttingDown || !mPlaybackStateMachine)
+    return;
+
+  if (!mMetadataLoadedReported) {
+    mElement->UpdateReadyStateForData(nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE);
+  } else {
+    mElement->UpdateReadyStateForData(nsHTMLMediaElement::NEXT_FRAME_AVAILABLE);
+  }
+}
+
+void
+nsWaveDecoder::NextFrameUnavailable()
+{
+  NS_ASSERTION(NS_IsMainThread(), "Should be called on main thread");
+  if (!mElement || mShuttingDown || !mPlaybackStateMachine)
+    return;
+
+  mElement->UpdateReadyStateForData(nsHTMLMediaElement::NEXT_FRAME_UNAVAILABLE);
+}
+
 void
 nsWaveDecoder::UpdateReadyStateForData()
 {
+  NS_ASSERTION(NS_IsMainThread(), "Should be called on main thread");
   if (!mElement || mShuttingDown || !mPlaybackStateMachine)
     return;
 
@@ -1495,6 +1589,7 @@ nsWaveDecoder::SeekingStarted()
 void
 nsWaveDecoder::SeekingStopped()
 {
+  UnpinForSeek();
   if (mShuttingDown) {
     return;
   }
@@ -1506,48 +1601,15 @@ nsWaveDecoder::SeekingStopped()
 }
 
 void
-nsWaveDecoder::RegisterShutdownObserver()
-{
-  if (!mNotifyOnShutdown) {
-    nsCOMPtr<nsIObserverService> observerService =
-      do_GetService("@mozilla.org/observer-service;1");
-    if (observerService) {
-      mNotifyOnShutdown =
-        NS_SUCCEEDED(observerService->AddObserver(this,
-                                                  NS_XPCOM_SHUTDOWN_OBSERVER_ID,
-                                                  PR_FALSE));
-    } else {
-      NS_WARNING("Could not get an observer service. Audio playback may not shutdown cleanly.");
-    }
-  }
-}
-
-void
-nsWaveDecoder::UnregisterShutdownObserver()
-{
-  if (mNotifyOnShutdown) {
-    nsCOMPtr<nsIObserverService> observerService =
-      do_GetService("@mozilla.org/observer-service;1");
-    if (observerService) {
-      mNotifyOnShutdown = PR_FALSE;
-      observerService->RemoveObserver(this, NS_XPCOM_SHUTDOWN_OBSERVER_ID);
-    }
-  }
-}
-
-void
-nsWaveDecoder::MediaErrorDecode()
+nsWaveDecoder::DecodeError()
 {
   if (mShuttingDown) {
     return;
   }
-#if 0
   if (mElement) {
-    mElement->MediaErrorDecode();
+    mElement->DecodeError();
   }
-#else
-  NS_WARNING("MediaErrorDecode fired, but not implemented.");
-#endif
+  Shutdown();
 }
 
 void
@@ -1597,7 +1659,7 @@ nsWaveDecoder::Suspend()
 }
 
 void
-nsWaveDecoder::Resume()
+nsWaveDecoder::Resume(PRBool aForceBuffering)
 {
   if (mStream) {
     mStream->Resume();
@@ -1611,3 +1673,10 @@ nsWaveDecoder::MoveLoadsToBackground()
     mStream->MoveLoadsToBackground();
   }
 }
+
+nsresult
+nsWaveDecoder::GetBuffered(nsHTMLTimeRanges* aBuffered)
+{
+  NS_ASSERTION(NS_IsMainThread(), "Only call on main thread");
+  return mPlaybackStateMachine->GetBuffered(aBuffered);
+}
