@@ -47,34 +47,17 @@
 #include "nsIServiceManager.h"
 #include "nsIPrefService.h"
 #include "nsCRT.h"
+#include "mozilla/Services.h"
 
 #include "nsThebesDeviceContext.h"
 #include "nsThebesRenderingContext.h"
 #include "gfxUserFontSet.h"
+#include "gfxPlatform.h"
 
 #include "nsIWidget.h"
 #include "nsIView.h"
 #include "nsILookAndFeel.h"
 
-#ifdef MOZ_ENABLE_GTK2
-// for getenv
-#include <cstdlib>
-// for round
-#include <cmath>
-
-#include <gtk/gtk.h>
-#include <gdk/gdk.h>
-
-#include "nsFont.h"
-
-#include <pango/pango.h>
-#ifdef MOZ_X11
-#include <gdk/gdkx.h>
-#include <pango/pangox.h>
-#endif /* MOZ_X11 */
-#include <pango/pango-fontmap.h>
-#endif /* GTK2 */
-
 #include "gfxImageSurface.h"
 
 #ifdef MOZ_ENABLE_GTK2
@@ -105,19 +88,13 @@ static nsSystemFontsMac *gSystemFonts = nsnull;
 #elif defined(MOZ_WIDGET_QT)
 #include "nsSystemFontsQt.h"
 static nsSystemFontsQt *gSystemFonts = nsnull;
+#elif defined(ANDROID)
+#include "nsSystemFontsAndroid.h"
+static nsSystemFontsAndroid *gSystemFonts = nsnull;
 #else
 #error Need to declare gSystemFonts!
 #endif
 
-#if defined(MOZ_ENABLE_GTK2) && defined(MOZ_X11)
-extern "C" {
-static int x11_error_handler (Display *dpy, XErrorEvent *err) {
-    NS_ASSERTION(PR_FALSE, "X Error");
-    return 0;
-}
-}
-#endif
-
 #ifdef PR_LOGGING
 PRLogModuleInfo* gThebesGFXLog = nsnull;
 #endif
@@ -129,7 +106,7 @@ public:
     ~nsFontCache();
 
     nsresult Init(nsIDeviceContext* aContext);
-    nsresult GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
+    nsresult GetMetricsFor(const nsFont& aFont, nsIAtom* aLanguage,
                            gfxUserFontSet* aUserFontSet,
                            nsIFontMetrics*& aMetrics);
 
@@ -167,7 +144,7 @@ nsFontCache::Init(nsIDeviceContext* aContext)
 }
 
 nsresult
-nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
+nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLanguage,
   gfxUserFontSet* aUserFontSet, nsIFontMetrics*& aMetrics)
 {
     // First check our cache
@@ -179,9 +156,9 @@ nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
         fm = mFontMetrics[i];
         nsIThebesFontMetrics* tfm = static_cast<nsIThebesFontMetrics*>(fm);
         if (fm->Font().Equals(aFont) && tfm->GetUserFontSet() == aUserFontSet) {
-            nsCOMPtr<nsIAtom> langGroup;
-            fm->GetLangGroup(getter_AddRefs(langGroup));
-            if (aLangGroup == langGroup.get()) {
+            nsCOMPtr<nsIAtom> language;
+            fm->GetLanguage(getter_AddRefs(language));
+            if (aLanguage == language.get()) {
                 if (i != n) {
                     // promote it to the end of the cache
                     mFontMetrics.RemoveElementAt(i);
@@ -199,7 +176,7 @@ nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
     aMetrics = nsnull;
     nsresult rv = CreateFontMetricsInstance(&fm);
     if (NS_FAILED(rv)) return rv;
-    rv = fm->Init(aFont, aLangGroup, mContext, aUserFontSet);
+    rv = fm->Init(aFont, aLanguage, mContext, aUserFontSet);
     if (NS_SUCCEEDED(rv)) {
         // the mFontMetrics list has the "head" at the end, because append
         // is cheaper than insert
@@ -218,7 +195,7 @@ nsFontCache::GetMetricsFor(const nsFont& aFont, nsIAtom* aLangGroup,
     Compact();
     rv = CreateFontMetricsInstance(&fm);
     if (NS_FAILED(rv)) return rv;
-    rv = fm->Init(aFont, aLangGroup, mContext, aUserFontSet);
+    rv = fm->Init(aFont, aLanguage, mContext, aUserFontSet);
     if (NS_SUCCEEDED(rv)) {
         mFontMetrics.AppendElement(fm);
         aMetrics = fm;
@@ -267,7 +244,7 @@ nsresult nsFontCache::Compact()
         NS_RELEASE(fm); // this will reset fm to nsnull
         // if the font is really gone, it would have called back in
         // FontMetricsDeleted() and would have removed itself
-        if (mFontMetrics.IndexOf(oldfm) >= 0) {
+        if (mFontMetrics.IndexOf(oldfm) != mFontMetrics.NoIndex) { 
             // nope, the font is still there, so let's hold onto it too
             NS_ADDREF(oldfm);
         }
@@ -334,7 +311,7 @@ static PRBool DeleteValue(nsHashKey* aKey, void* aValue, void* closure)
 
 nsThebesDeviceContext::~nsThebesDeviceContext()
 {
-    nsCOMPtr<nsIObserverService> obs(do_GetService("@mozilla.org/observer-service;1"));
+    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
     if (obs)
         obs->RemoveObserver(this, "memory-pressure");
 
@@ -376,22 +353,22 @@ NS_IMETHODIMP nsThebesDeviceContext::FontMetricsDeleted(const nsIFontMetrics* aF
 }
 
 void
-nsThebesDeviceContext::GetLocaleLangGroup(void)
+nsThebesDeviceContext::GetLocaleLanguage(void)
 {
-    if (!mLocaleLangGroup) {
+    if (!mLocaleLanguage) {
         nsCOMPtr<nsILanguageAtomService> langService;
         langService = do_GetService(NS_LANGUAGEATOMSERVICE_CONTRACTID);
         if (langService) {
-            mLocaleLangGroup = langService->GetLocaleLanguageGroup();
+            mLocaleLanguage = langService->GetLocaleLanguage();
         }
-        if (!mLocaleLangGroup) {
-            mLocaleLangGroup = do_GetAtom("x-western");
+        if (!mLocaleLanguage) {
+            mLocaleLanguage = do_GetAtom("x-western");
         }
     }
 }
 
 NS_IMETHODIMP nsThebesDeviceContext::GetMetricsFor(const nsFont& aFont,
-  nsIAtom* aLangGroup, gfxUserFontSet* aUserFontSet, nsIFontMetrics*& aMetrics)
+  nsIAtom* aLanguage, gfxUserFontSet* aUserFontSet, nsIFontMetrics*& aMetrics)
 {
     if (nsnull == mFontCache) {
         nsresult rv = CreateFontCache();
@@ -400,15 +377,16 @@ NS_IMETHODIMP nsThebesDeviceContext::GetMetricsFor(const nsFont& aFont,
             return rv;
         }
         // XXX temporary fix for performance problem -- erik
-        GetLocaleLangGroup();
+        GetLocaleLanguage();
     }
 
-    // XXX figure out why aLangGroup is NULL sometimes
-    if (!aLangGroup) {
-        aLangGroup = mLocaleLangGroup;
+    // XXX figure out why aLanguage is NULL sometimes
+    //      -> see nsPageFrame.cpp:511
+    if (!aLanguage) {
+        aLanguage = mLocaleLanguage;
     }
 
-    return mFontCache->GetMetricsFor(aFont, aLangGroup, aUserFontSet, aMetrics);
+    return mFontCache->GetMetricsFor(aFont, aLanguage, aUserFontSet, aMetrics);
 }
 
 NS_IMETHODIMP nsThebesDeviceContext::GetMetricsFor(const nsFont& aFont,
@@ -422,9 +400,9 @@ NS_IMETHODIMP nsThebesDeviceContext::GetMetricsFor(const nsFont& aFont,
             return rv;
         }
         // XXX temporary fix for performance problem -- erik
-        GetLocaleLangGroup();
+        GetLocaleLanguage();
     }
-    return mFontCache->GetMetricsFor(aFont, mLocaleLangGroup, aUserFontSet,
+    return mFontCache->GetMetricsFor(aFont, mLocaleLanguage, aUserFontSet,
                                      aMetrics);
 }
 
@@ -624,22 +602,46 @@ nsThebesDeviceContext::SetDPI()
     // The number of device pixels per CSS pixel. A value <= 0 means choose
     // automatically based on the DPI. A positive value is used as-is. This effectively
     // controls the size of a CSS "px".
-    PRInt32 prefDevPixelsPerCSSPixel = -1;
+    float prefDevPixelsPerCSSPixel = -1.0;
 
     nsCOMPtr<nsIPrefBranch> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
     if (prefs) {
-        nsresult rv = prefs->GetIntPref("layout.css.devPixelsPerPx", &prefDevPixelsPerCSSPixel);
-        if (NS_FAILED(rv)) {
-            prefDevPixelsPerCSSPixel = -1;
+        nsXPIDLCString prefString;
+        nsresult rv = prefs->GetCharPref("layout.css.devPixelsPerPx", getter_Copies(prefString));
+        if (NS_SUCCEEDED(rv) && !prefString.IsEmpty()) {
+            prefDevPixelsPerCSSPixel = static_cast<float>(atof(prefString));
         }
     }
 
     // PostScript, PDF and Mac (when printing) all use 72 dpi
-    if (mPrintingSurface &&
-        (mPrintingSurface->GetType() == gfxASurface::SurfaceTypePDF ||
-         mPrintingSurface->GetType() == gfxASurface::SurfaceTypePS ||
-         mPrintingSurface->GetType() == gfxASurface::SurfaceTypeQuartz)) {
-        dpi = 72;
+    // Use a printing DC to determine the other dpi values
+    if (mPrintingSurface) {
+        switch (mPrintingSurface->GetType()) {
+            case gfxASurface::SurfaceTypePDF:
+            case gfxASurface::SurfaceTypePS:
+            case gfxASurface::SurfaceTypeQuartz:
+                dpi = 72;
+                break;
+#ifdef XP_WIN
+            case gfxASurface::SurfaceTypeWin32:
+            case gfxASurface::SurfaceTypeWin32Printing: {
+                PRInt32 OSVal = GetDeviceCaps(GetPrintHDC(), LOGPIXELSY);
+                dpi = 144;
+                mPrintingScale = float(OSVal) / dpi;
+                break;
+            }
+#endif
+#ifdef XP_OS2
+            case gfxASurface::SurfaceTypeOS2:
+                LONG lDPI;
+                if (DevQueryCaps(GetPrintHDC(), CAPS_VERTICAL_FONT_RES, 1, &lDPI))
+                    dpi = lDPI;
+                break;
+#endif
+            default:
+                NS_NOTREACHED("Unexpected printing surface type");
+                break;
+        }
         dotsArePixels = PR_FALSE;
     } else {
         nsresult rv;
@@ -655,67 +657,13 @@ nsThebesDeviceContext::SetDPI()
             }
         }
 
-#if defined(MOZ_ENABLE_GTK2)
-        GdkScreen *screen = gdk_screen_get_default();
-        gtk_settings_get_for_screen(screen); // Make sure init is run so we have a resolution
-        PRInt32 OSVal = PRInt32(round(gdk_screen_get_resolution(screen)));
+        dpi = gfxPlatform::GetDPI();
 
-        if (prefDPI == 0) // Force the use of the OS dpi
-            dpi = OSVal;
-        else  // Otherwise, the minimum dpi is 96dpi
-            dpi = PR_MAX(OSVal, 96);
-
-#elif defined(XP_WIN)
-        // XXX we should really look at the widget if !dc but it is currently always null
-        HDC dc = GetPrintHDC();
-        if (dc) {
-            PRInt32 OSVal = GetDeviceCaps(dc, LOGPIXELSY);
-
-            dpi = 144;
-            mPrintingScale = float(OSVal)/dpi;
-            dotsArePixels = PR_FALSE;
-        } else {
-            dc = GetDC((HWND)nsnull);
-
-            PRInt32 OSVal = GetDeviceCaps(dc, LOGPIXELSY);
-
-            ReleaseDC((HWND)nsnull, dc);
-
-            if (OSVal != 0)
-                dpi = OSVal;
-        }
-
-#elif defined(XP_OS2)
-        // get a printer DC if available, otherwise create a new (memory) DC
-        HDC dc = GetPrintHDC();
-        PRBool doCloseDC = PR_FALSE;
-        if (dc <= 0) { // test for NULLHANDLE/DEV_ERROR or HDC_ERROR
-            // create DC compatible with the screen
-            dc = DevOpenDC((HAB)1, OD_MEMORY,"*",0L, NULL, NULLHANDLE);
-            doCloseDC = PR_TRUE;
-        }
-        if (dc > 0) {
-            // we do have a DC and we can query the DPI setting from it
-            LONG lDPI;
-            if (DevQueryCaps(dc, CAPS_VERTICAL_FONT_RES, 1, &lDPI))
-                dpi = lDPI;
-            if (doCloseDC)
-                DevCloseDC(dc);
-        }
-        if (dpi < 0) // something didn't work before, fall back to hardcoded DPI value
-            dpi = 96;
-#elif defined(XP_MACOSX)
-
-        // we probably want to actually get a real DPI here?
-        dpi = 96;
-
-#elif defined(MOZ_WIDGET_QT)
-        // TODO: get real DPI here with Qt methods
-        dpi = 96;
-#else
-#error undefined platform dpi
+#ifdef MOZ_ENABLE_GTK2
+        if (prefDPI < 0) // Clamp the minimum dpi to 96dpi
+            dpi = PR_MAX(dpi, 96);
 #endif
-
+ 
         if (prefDPI > 0 && !mPrintingSurface)
             dpi = prefDPI;
     }
@@ -724,21 +672,17 @@ nsThebesDeviceContext::SetDPI()
 
     if (dotsArePixels) {
         if (prefDevPixelsPerCSSPixel <= 0) {
-            // First figure out the closest multiple of 96, which is the number of
-            // dev pixels per CSS pixel.  Then, divide that into AppUnitsPerCSSPixel()
-            // to get the number of app units per dev pixel.  The PR_MAXes are to
-            // make sure we don't end up dividing by zero.
-            PRUint32 roundedDPIScaleFactor = (dpi + 48)/96;
-#ifdef MOZ_WIDGET_GTK2
-            // be more conservative about activating scaling on GTK2, since the dpi
-            // information is more likely to be wrong
-            roundedDPIScaleFactor = dpi/96;
-#endif
+            // Round down to multiple of 96, which is the number of dev pixels
+            // per CSS pixel.  Then, divide that into AppUnitsPerCSSPixel()
+            // to get the number of app units per dev pixel.  The PR_MAXes are
+            // to make sure we don't end up dividing by zero.
+            PRUint32 roundedDPIScaleFactor = dpi/96;
             mAppUnitsPerDevNotScaledPixel =
                 PR_MAX(1, AppUnitsPerCSSPixel() / PR_MAX(1, roundedDPIScaleFactor));
         } else {
             mAppUnitsPerDevNotScaledPixel =
-                PR_MAX(1, AppUnitsPerCSSPixel() / prefDevPixelsPerCSSPixel);
+                PR_MAX(1, static_cast<PRInt32>(AppUnitsPerCSSPixel() /
+                                               prefDevPixelsPerCSSPixel));
         }
     } else {
         /* set mAppUnitsPerDevPixel so we're using exactly 72 dpi, even
@@ -769,19 +713,10 @@ nsThebesDeviceContext::Init(nsIWidget *aWidget)
 
     // register as a memory-pressure observer to free font resources
     // in low-memory situations.
-    nsCOMPtr<nsIObserverService> obs(do_GetService("@mozilla.org/observer-service;1"));
+    nsCOMPtr<nsIObserverService> obs = mozilla::services::GetObserverService();
     if (obs)
         obs->AddObserver(this, "memory-pressure", PR_TRUE);
 
-#if defined(MOZ_ENABLE_GTK2) && defined(MOZ_X11)
-    if (getenv ("MOZ_X_SYNC")) {
-        PR_LOG (gThebesGFXLog, PR_LOG_DEBUG, ("+++ Enabling XSynchronize\n"));
-        XSynchronize (gdk_x11_get_default_xdisplay(), True);
-        XSetErrorHandler(x11_error_handler);
-    }
-
-#endif
-
     mScreenManager = do_GetService("@mozilla.org/gfx/screenmanager;1");
 
     return NS_OK;
@@ -897,6 +832,8 @@ nsThebesDeviceContext::GetSystemFont(nsSystemFontID aID, nsFont *aFont) const
         gSystemFonts = new nsSystemFontsMac();
 #elif defined(MOZ_WIDGET_QT)
         gSystemFonts = new nsSystemFontsQt();
+#elif defined(ANDROID)
+        gSystemFonts = new nsSystemFontsAndroid();
 #else
 #error Need to know how to create gSystemFonts, fix me!
 #endif
@@ -941,13 +878,6 @@ nsThebesDeviceContext::GetDepth(PRUint32& aDepth)
     return NS_OK;
 }
 
-NS_IMETHODIMP
-nsThebesDeviceContext::ConvertPixel(nscolor aColor, PRUint32 & aPixel)
-{
-    aPixel = aColor;
-    return NS_OK;
-}
-
 NS_IMETHODIMP
 nsThebesDeviceContext::GetDeviceSurfaceDimensions(nscoord &aWidth, nscoord &aHeight)
 {
@@ -1256,7 +1186,7 @@ nsThebesDeviceContext::CalcPrintingSize()
     }
 #endif
     default:
-        NS_ASSERTION(0, "trying to print to unknown surface type");
+        NS_ERROR("trying to print to unknown surface type");
     }
 
     if (inPoints) {
@@ -1294,7 +1224,8 @@ nsThebesDeviceContext::SetPixelScale(float aScale)
 void
 nsThebesDeviceContext::UpdateScaledAppUnits()
 {
-    mAppUnitsPerDevPixel = PR_MAX(1, PRInt32(float(mAppUnitsPerDevNotScaledPixel) / mPixelScale));
+    mAppUnitsPerDevPixel =
+        PR_MAX(1, NSToIntRound(float(mAppUnitsPerDevNotScaledPixel) / mPixelScale));
 }
 
 #if defined(XP_WIN) || defined(XP_OS2)
@@ -1315,7 +1246,7 @@ nsThebesDeviceContext::GetPrintHDC()
 #endif
 
             default:
-                NS_ASSERTION(0, "invalid surface type in GetPrintHDC");
+                NS_ERROR("invalid surface type in GetPrintHDC");
                 break;
         }
     }
