@@ -41,7 +41,6 @@
 /*
  * JS standard exception implementation.
  */
-
 #include <stdlib.h>
 #include <string.h>
 #include "jstypes.h"
@@ -63,9 +62,14 @@
 #include "jsscript.h"
 #include "jsstaticcheck.h"
 
+#include "jscntxtinlines.h"
+#include "jsobjinlines.h"
+
+using namespace js;
+
 /* Forward declarations for js_ErrorClass's initializer. */
 static JSBool
-Exception(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval);
+Exception(JSContext *cx, JSObject *obj, uintN argc, Value *argv, Value *rval);
 
 static void
 exn_trace(JSTracer *trc, JSObject *obj);
@@ -77,17 +81,28 @@ static JSBool
 exn_enumerate(JSContext *cx, JSObject *obj);
 
 static JSBool
-exn_resolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             JSObject **objp);
 
-JSClass js_ErrorClass = {
+Class js_ErrorClass = {
     js_Error_str,
     JSCLASS_HAS_PRIVATE | JSCLASS_NEW_RESOLVE | JSCLASS_MARK_IS_TRACE |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Error),
-    JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,
-    exn_enumerate,    (JSResolveOp)exn_resolve, JS_ConvertStub, exn_finalize,
-    NULL,             NULL,             NULL,             Exception,
-    NULL,             NULL,             JS_CLASS_TRACE(exn_trace), NULL
+    PropertyStub,   /* addProperty */
+    PropertyStub,   /* delProperty */
+    PropertyStub,   /* getProperty */
+    PropertyStub,   /* setProperty */
+    exn_enumerate,
+    (JSResolveOp)exn_resolve,
+    ConvertStub,
+    exn_finalize,
+    NULL,           /* reserved0   */
+    NULL,           /* checkAccess */
+    NULL,           /* call        */
+    Exception,      /* construct   */
+    NULL,           /* xdrObject   */
+    NULL,           /* hasInstance */
+    JS_CLASS_TRACE(exn_trace)
 };
 
 typedef struct JSStackTraceElem {
@@ -164,7 +179,7 @@ CopyErrorReport(JSContext *cx, JSErrorReport *report)
      */
     mallocSize = sizeof(JSErrorReport) + argsArraySize + argsCopySize +
                  ucmessageSize + uclinebufSize + linebufSize + filenameSize;
-    cursor = (uint8 *)JS_malloc(cx, mallocSize);
+    cursor = (uint8 *)cx->malloc(mallocSize);
     if (!cursor)
         return NULL;
 
@@ -247,10 +262,10 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
                JSString *filename, uintN lineno, JSErrorReport *report)
 {
     JSSecurityCallbacks *callbacks;
-    JSCheckAccessOp checkAccess;
+    CheckAccessOp checkAccess;
     JSErrorReporter older;
     JSExceptionState *state;
-    jsval callerid, v;
+    jsid callerid;
     JSStackFrame *fp, *fpstop;
     size_t stackDepth, valueCount, size;
     JSBool overflow;
@@ -258,7 +273,7 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     JSStackTraceElem *elem;
     jsval *values;
 
-    JS_ASSERT(OBJ_GET_CLASS(cx, exnObject) == &js_ErrorClass);
+    JS_ASSERT(exnObject->getClass() == &js_ErrorClass);
 
     /*
      * Prepare stack trace data.
@@ -269,19 +284,19 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
      */
     callbacks = JS_GetSecurityCallbacks(cx);
     checkAccess = callbacks
-                  ? callbacks->checkObjectAccess
+                  ? Valueify(callbacks->checkObjectAccess)
                   : NULL;
     older = JS_SetErrorReporter(cx, NULL);
     state = JS_SaveExceptionState(cx);
 
-    callerid = ATOM_KEY(cx->runtime->atomState.callerAtom);
+    callerid = ATOM_TO_JSID(cx->runtime->atomState.callerAtom);
     stackDepth = 0;
     valueCount = 0;
     for (fp = js_GetTopStackFrame(cx); fp; fp = fp->down) {
         if (fp->fun && fp->argv) {
-            v = JSVAL_NULL;
+            Value v = NullValue();
             if (checkAccess &&
-                !checkAccess(cx, fp->callee, callerid, JSACC_READ, &v)) {
+                !checkAccess(cx, fp->callee(), callerid, JSACC_READ, &v)) {
                 break;
             }
             valueCount += fp->argc;
@@ -301,7 +316,7 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
         js_ReportAllocationOverflow(cx);
         return JS_FALSE;
     }
-    priv = (JSExnPrivate *)JS_malloc(cx, size);
+    priv = (JSExnPrivate *)cx->malloc(size);
     if (!priv)
         return JS_FALSE;
 
@@ -334,7 +349,7 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
         elem->filename = NULL;
         if (fp->script) {
             elem->filename = fp->script->filename;
-            if (fp->regs)
+            if (fp->pc(cx))
                 elem->ulineno = js_FramePCToLineNumber(cx, fp);
         }
         ++elem;
@@ -342,14 +357,14 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     JS_ASSERT(priv->stackElems + stackDepth == elem);
     JS_ASSERT(GetStackTraceValueBuffer(priv) + valueCount == values);
 
-    STOBJ_SET_SLOT(exnObject, JSSLOT_PRIVATE, PRIVATE_TO_JSVAL(priv));
+    exnObject->setPrivate(priv);
 
     if (report) {
         /*
          * Construct a new copy of the error report struct. We can't use the
          * error report struct that was passed in, because it's allocated on
          * the stack, and also because it may point to transient data in the
-         * JSTokenStream.
+         * TokenStream.
          */
         priv->errorReport = CopyErrorReport(cx, report);
         if (!priv->errorReport) {
@@ -361,19 +376,10 @@ InitExnPrivate(JSContext *cx, JSObject *exnObject, JSString *message,
     return JS_TRUE;
 }
 
-static JSExnPrivate *
+static inline JSExnPrivate *
 GetExnPrivate(JSContext *cx, JSObject *obj)
 {
-    jsval privateValue;
-    JSExnPrivate *priv;
-
-    JS_ASSERT(OBJ_GET_CLASS(cx, obj) == &js_ErrorClass);
-    privateValue = OBJ_GET_SLOT(cx, obj, JSSLOT_PRIVATE);
-    if (JSVAL_IS_VOID(privateValue))
-        return NULL;
-    priv = (JSExnPrivate *)JSVAL_TO_PRIVATE(privateValue);
-    JS_ASSERT(priv);
-    return priv;
+    return (JSExnPrivate *) obj->getPrivate();
 }
 
 static void
@@ -417,8 +423,8 @@ exn_finalize(JSContext *cx, JSObject *obj)
     priv = GetExnPrivate(cx, obj);
     if (priv) {
         if (priv->errorReport)
-            JS_free(cx, priv->errorReport);
-        JS_free(cx, priv);
+            cx->free(priv->errorReport);
+        cx->free(priv);
     }
 }
 
@@ -445,13 +451,13 @@ exn_enumerate(JSContext *cx, JSObject *obj)
         if (!js_LookupProperty(cx, obj, ATOM_TO_JSID(atom), &pobj, &prop))
             return JS_FALSE;
         if (prop)
-            OBJ_DROP_PROPERTY(cx, pobj, prop);
+            pobj->dropProperty(cx, prop);
     }
     return JS_TRUE;
 }
 
 static JSBool
-exn_resolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+exn_resolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
             JSObject **objp)
 {
     JSExnPrivate *priv;
@@ -463,8 +469,8 @@ exn_resolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
 
     *objp = NULL;
     priv = GetExnPrivate(cx, obj);
-    if (priv && JSVAL_IS_STRING(id)) {
-        str = JSVAL_TO_STRING(id);
+    if (priv && JSID_IS_ATOM(id)) {
+        str = JSID_TO_STRING(id);
 
         atom = cx->runtime->atomState.messageAtom;
         if (str == ATOM_TO_STRING(atom)) {
@@ -518,7 +524,7 @@ js_ErrorFromException(JSContext *cx, jsval exn)
     if (JSVAL_IS_PRIMITIVE(exn))
         return NULL;
     obj = JSVAL_TO_OBJECT(exn);
-    if (OBJ_GET_CLASS(cx, obj) != &js_ErrorClass)
+    if (obj->getClass() != &js_ErrorClass)
         return NULL;
     priv = GetExnPrivate(cx, obj);
     if (!priv)
@@ -533,13 +539,13 @@ ValueToShortSource(JSContext *cx, jsval v)
 
     /* Avoid toSource bloat and fallibility for object types. */
     if (JSVAL_IS_PRIMITIVE(v)) {
-        str = js_ValueToSource(cx, v);
+        str = js_ValueToSource(cx, Valueify(v));
     } else if (VALUE_IS_FUNCTION(cx, v)) {
         /*
          * XXX Avoid function decompilation bloat for now.
          */
         str = JS_GetFunctionId(JS_ValueToFunction(cx, v));
-        if (!str && !(str = js_ValueToSource(cx, v))) {
+        if (!str && !(str = js_ValueToSource(cx, Valueify(v)))) {
             /*
              * Continue to soldier on if the function couldn't be
              * converted into a string.
@@ -554,7 +560,7 @@ ValueToShortSource(JSContext *cx, jsval v)
          */
         char buf[100];
         JS_snprintf(buf, sizeof buf, "[object %s]",
-                    OBJ_GET_CLASS(cx, JSVAL_TO_OBJECT(v))->name);
+                    JSVAL_TO_OBJECT(v)->getClass()->name);
         str = JS_NewStringCopyZ(cx, buf);
     }
     return str;
@@ -586,7 +592,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
             if (stackmax >= STACK_LENGTH_LIMIT)                               \
                 goto done;                                                    \
             stackmax = stackmax ? 2 * stackmax : 64;                          \
-            ptr_ = JS_realloc(cx, stackbuf, (stackmax+1) * sizeof(jschar));   \
+            ptr_ = cx->realloc(stackbuf, (stackmax+1) * sizeof(jschar));      \
             if (!ptr_)                                                        \
                 goto bad;                                                     \
             stackbuf = (jschar *) ptr_;                                       \
@@ -608,7 +614,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
                 goto done;                                                    \
             }                                                                 \
             stackmax = JS_BIT(JS_CeilingLog2(stacklen + length_));            \
-            ptr_ = JS_realloc(cx, stackbuf, (stackmax+1) * sizeof(jschar));   \
+            ptr_ = cx->realloc(stackbuf, (stackmax+1) * sizeof(jschar));      \
             if (!ptr_)                                                        \
                 goto bad;                                                     \
             stackbuf = (jschar *) ptr_;                                       \
@@ -659,7 +665,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
          * don't use JS_realloc here; simply let the oversized allocation
          * be owned by the string in that rare case.
          */
-        void *shrunk = JS_realloc(cx, stackbuf, (stacklen+1) * sizeof(jschar));
+        void *shrunk = cx->realloc(stackbuf, (stacklen+1) * sizeof(jschar));
         if (shrunk)
             stackbuf = (jschar *) shrunk;
     }
@@ -671,7 +677,7 @@ StackTraceToString(JSContext *cx, JSExnPrivate *priv)
 
   bad:
     if (stackbuf)
-        JS_free(cx, stackbuf);
+        cx->free(stackbuf);
     return NULL;
 }
 
@@ -690,9 +696,8 @@ StringToFilename(JSContext *cx, JSString *str)
 }
 
 static JSBool
-Exception(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
+Exception(JSContext *cx, JSObject *obj, uintN argc, Value *argv, Value *rval)
 {
-    uint32 lineno;
     JSString *message, *filename;
     JSStackFrame *fp;
 
@@ -701,33 +706,35 @@ Exception(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
          * ECMA ed. 3, 15.11.1 requires Error, etc., to construct even when
          * called as functions, without operator new.  But as we do not give
          * each constructor a distinct JSClass, whose .name member is used by
-         * js_NewObject to find the class prototype, we must get the class
-         * prototype ourselves.
+         * NewNativeClassInstance to find the class prototype, we must get the
+         * class prototype ourselves.
          */
-        if (!OBJ_GET_PROPERTY(cx, JSVAL_TO_OBJECT(argv[-2]),
-                              ATOM_TO_JSID(cx->runtime->atomState
-                                           .classPrototypeAtom),
-                              rval))
+        if (!argv[-2].toObject().getProperty(cx,
+                                             ATOM_TO_JSID(cx->runtime->atomState
+                                                          .classPrototypeAtom),
+                                             rval)) {
             return JS_FALSE;
-        obj = js_NewObject(cx, &js_ErrorClass, JSVAL_TO_OBJECT(*rval), NULL);
+        }
+        JSObject *errProto = &rval->toObject();
+        obj = NewNativeClassInstance(cx, &js_ErrorClass, errProto, errProto->getParent());
         if (!obj)
             return JS_FALSE;
-        *rval = OBJECT_TO_JSVAL(obj);
+        rval->setObject(*obj);
     }
 
     /*
      * If it's a new object of class Exception, then null out the private
      * data so that the finalizer doesn't attempt to free it.
      */
-    if (OBJ_GET_CLASS(cx, obj) == &js_ErrorClass)
-        STOBJ_SET_SLOT(obj, JSSLOT_PRIVATE, JSVAL_VOID);
+    if (obj->getClass() == &js_ErrorClass)
+        obj->setPrivate(NULL);
 
     /* Set the 'message' property. */
     if (argc != 0) {
         message = js_ValueToString(cx, argv[0]);
         if (!message)
             return JS_FALSE;
-        argv[0] = STRING_TO_JSVAL(message);
+        argv[0].setString(message);
     } else {
         message = cx->runtime->emptyString;
     }
@@ -737,7 +744,7 @@ Exception(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
         filename = js_ValueToString(cx, argv[1]);
         if (!filename)
             return JS_FALSE;
-        argv[1] = STRING_TO_JSVAL(filename);
+        argv[1].setString(filename);
         fp = NULL;
     } else {
         fp = js_GetScriptedCaller(cx, NULL);
@@ -751,17 +758,17 @@ Exception(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     }
 
     /* Set the 'lineNumber' property. */
+    uint32_t lineno;
     if (argc > 2) {
-        lineno = js_ValueToECMAUint32(cx, &argv[2]);
-        if (JSVAL_IS_NULL(argv[2]))
+        if (!ValueToECMAUint32(cx, argv[2], &lineno))
             return JS_FALSE;
     } else {
         if (!fp)
             fp = js_GetScriptedCaller(cx, NULL);
-        lineno = (fp && fp->regs) ? js_FramePCToLineNumber(cx, fp) : 0;
+        lineno = (fp && fp->pc(cx)) ? js_FramePCToLineNumber(cx, fp) : 0;
     }
 
-    return (OBJ_GET_CLASS(cx, obj) != &js_ErrorClass) ||
+    return (obj->getClass() != &js_ErrorClass) ||
             InitExnPrivate(cx, obj, message, filename, lineno, NULL);
 }
 
@@ -782,12 +789,8 @@ exn_toString(JSContext *cx, uintN argc, jsval *vp)
     size_t name_length, message_length, length;
 
     obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj ||
-        !OBJ_GET_PROPERTY(cx, obj,
-                          ATOM_TO_JSID(cx->runtime->atomState.nameAtom),
-                          &v)) {
+    if (!obj || !obj->getProperty(cx, ATOM_TO_JSID(cx->runtime->atomState.nameAtom), Valueify(&v)))
         return JS_FALSE;
-    }
     name = JSVAL_IS_STRING(v) ? JSVAL_TO_STRING(v) : cx->runtime->emptyString;
     *vp = STRING_TO_JSVAL(name);
 
@@ -800,7 +803,7 @@ exn_toString(JSContext *cx, uintN argc, jsval *vp)
         name_length = name->length();
         message_length = message->length();
         length = (name_length ? name_length + 2 : 0) + message_length;
-        cp = chars = (jschar *) JS_malloc(cx, (length + 1) * sizeof(jschar));
+        cp = chars = (jschar *) cx->malloc((length + 1) * sizeof(jschar));
         if (!chars)
             return JS_FALSE;
 
@@ -815,7 +818,7 @@ exn_toString(JSContext *cx, uintN argc, jsval *vp)
 
         result = js_NewString(cx, chars, length);
         if (!result) {
-            JS_free(cx, chars);
+            cx->free(chars);
             return JS_FALSE;
         }
     } else {
@@ -836,135 +839,118 @@ exn_toSource(JSContext *cx, uintN argc, jsval *vp)
     JSObject *obj;
     JSString *name, *message, *filename, *lineno_as_str, *result;
     jsval localroots[3] = {JSVAL_NULL, JSVAL_NULL, JSVAL_NULL};
-    JSTempValueRooter tvr;
-    JSBool ok;
-    uint32 lineno;
     size_t lineno_length, name_length, message_length, filename_length, length;
     jschar *chars, *cp;
 
     obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj ||
-        !OBJ_GET_PROPERTY(cx, obj,
-                          ATOM_TO_JSID(cx->runtime->atomState.nameAtom),
-                          vp)) {
-        return JS_FALSE;
-    }
-    name = js_ValueToString(cx, *vp);
+    if (!obj || !obj->getProperty(cx, ATOM_TO_JSID(cx->runtime->atomState.nameAtom), Valueify(vp)))
+        return false;
+    name = js_ValueToString(cx, Valueify(*vp));
     if (!name)
-        return JS_FALSE;
+        return false;
     *vp = STRING_TO_JSVAL(name);
 
-    MUST_FLOW_THROUGH("out");
-    JS_PUSH_TEMP_ROOT(cx, 3, localroots, &tvr);
+    {
+        AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(localroots), Valueify(localroots));
 
 #ifdef __GNUC__
-    message = filename = NULL;
+        message = filename = NULL;
 #endif
-    ok = JS_GetProperty(cx, obj, js_message_str, &localroots[0]) &&
-         (message = js_ValueToSource(cx, localroots[0]));
-    if (!ok)
-        goto out;
-    localroots[0] = STRING_TO_JSVAL(message);
-
-    ok = JS_GetProperty(cx, obj, js_fileName_str, &localroots[1]) &&
-         (filename = js_ValueToSource(cx, localroots[1]));
-    if (!ok)
-        goto out;
-    localroots[1] = STRING_TO_JSVAL(filename);
-
-    ok = JS_GetProperty(cx, obj, js_lineNumber_str, &localroots[2]);
-    if (!ok)
-        goto out;
-    lineno = js_ValueToECMAUint32 (cx, &localroots[2]);
-    ok = !JSVAL_IS_NULL(localroots[2]);
-    if (!ok)
-        goto out;
-
-    if (lineno != 0) {
-        lineno_as_str = js_ValueToString(cx, localroots[2]);
-        if (!lineno_as_str) {
-            ok = JS_FALSE;
-            goto out;
+        if (!JS_GetProperty(cx, obj, js_message_str, &localroots[0]) ||
+            !(message = js_ValueToSource(cx, Valueify(localroots[0])))) {
+            return false;
         }
-        lineno_length = lineno_as_str->length();
-    } else {
-        lineno_as_str = NULL;
-        lineno_length = 0;
-    }
+        localroots[0] = STRING_TO_JSVAL(message);
 
-    /* Magic 8, for the characters in ``(new ())''. */
-    name_length = name->length();
-    message_length = message->length();
-    length = 8 + name_length + message_length;
+        if (!JS_GetProperty(cx, obj, js_fileName_str, &localroots[1]) ||
+            !(filename = js_ValueToSource(cx, Valueify(localroots[1])))) {
+            return false;
+        }
+        localroots[1] = STRING_TO_JSVAL(filename);
 
-    filename_length = filename->length();
-    if (filename_length != 0) {
-        /* append filename as ``, {filename}'' */
-        length += 2 + filename_length;
+        if (!JS_GetProperty(cx, obj, js_lineNumber_str, &localroots[2]))
+            return false;
+        uint32_t lineno;
+        if (!ValueToECMAUint32(cx, Valueify(localroots[2]), &lineno))
+            return false;
+
+        if (lineno != 0) {
+            lineno_as_str = js_ValueToString(cx, Valueify(localroots[2]));
+            if (!lineno_as_str)
+                return false;
+            lineno_length = lineno_as_str->length();
+        } else {
+            lineno_as_str = NULL;
+            lineno_length = 0;
+        }
+
+        /* Magic 8, for the characters in ``(new ())''. */
+        name_length = name->length();
+        message_length = message->length();
+        length = 8 + name_length + message_length;
+
+        filename_length = filename->length();
+        if (filename_length != 0) {
+            /* append filename as ``, {filename}'' */
+            length += 2 + filename_length;
+            if (lineno_as_str) {
+                /* append lineno as ``, {lineno_as_str}'' */
+                length += 2 + lineno_length;
+            }
+        } else {
+            if (lineno_as_str) {
+                /*
+                 * no filename, but have line number,
+                 * need to append ``, "", {lineno_as_str}''
+                 */
+                length += 6 + lineno_length;
+            }
+        }
+
+        cp = chars = (jschar *) cx->malloc((length + 1) * sizeof(jschar));
+        if (!chars)
+            return false;
+
+        *cp++ = '('; *cp++ = 'n'; *cp++ = 'e'; *cp++ = 'w'; *cp++ = ' ';
+        js_strncpy(cp, name->chars(), name_length);
+        cp += name_length;
+        *cp++ = '(';
+        if (message_length != 0) {
+            js_strncpy(cp, message->chars(), message_length);
+            cp += message_length;
+        }
+
+        if (filename_length != 0) {
+            /* append filename as ``, {filename}'' */
+            *cp++ = ','; *cp++ = ' ';
+            js_strncpy(cp, filename->chars(), filename_length);
+            cp += filename_length;
+        } else {
+            if (lineno_as_str) {
+                /*
+                 * no filename, but have line number,
+                 * need to append ``, "", {lineno_as_str}''
+                 */
+                *cp++ = ','; *cp++ = ' '; *cp++ = '"'; *cp++ = '"';
+            }
+        }
         if (lineno_as_str) {
             /* append lineno as ``, {lineno_as_str}'' */
-            length += 2 + lineno_length;
+            *cp++ = ','; *cp++ = ' ';
+            js_strncpy(cp, lineno_as_str->chars(), lineno_length);
+            cp += lineno_length;
         }
-    } else {
-        if (lineno_as_str) {
-            /*
-             * no filename, but have line number,
-             * need to append ``, "", {lineno_as_str}''
-             */
-            length += 6 + lineno_length;
+
+        *cp++ = ')'; *cp++ = ')'; *cp = 0;
+
+        result = js_NewString(cx, chars, length);
+        if (!result) {
+            cx->free(chars);
+            return false;
         }
+        *vp = STRING_TO_JSVAL(result);
+        return true;
     }
-
-    cp = chars = (jschar *) JS_malloc(cx, (length + 1) * sizeof(jschar));
-    if (!chars) {
-        ok = JS_FALSE;
-        goto out;
-    }
-
-    *cp++ = '('; *cp++ = 'n'; *cp++ = 'e'; *cp++ = 'w'; *cp++ = ' ';
-    js_strncpy(cp, name->chars(), name_length);
-    cp += name_length;
-    *cp++ = '(';
-    if (message_length != 0) {
-        js_strncpy(cp, message->chars(), message_length);
-        cp += message_length;
-    }
-
-    if (filename_length != 0) {
-        /* append filename as ``, {filename}'' */
-        *cp++ = ','; *cp++ = ' ';
-        js_strncpy(cp, filename->chars(), filename_length);
-        cp += filename_length;
-    } else {
-        if (lineno_as_str) {
-            /*
-             * no filename, but have line number,
-             * need to append ``, "", {lineno_as_str}''
-             */
-            *cp++ = ','; *cp++ = ' '; *cp++ = '"'; *cp++ = '"';
-        }
-    }
-    if (lineno_as_str) {
-        /* append lineno as ``, {lineno_as_str}'' */
-        *cp++ = ','; *cp++ = ' ';
-        js_strncpy(cp, lineno_as_str->chars(), lineno_length);
-        cp += lineno_length;
-    }
-
-    *cp++ = ')'; *cp++ = ')'; *cp = 0;
-
-    result = js_NewString(cx, chars, length);
-    if (!result) {
-        JS_free(cx, chars);
-        ok = JS_FALSE;
-        goto out;
-    }
-    *vp = STRING_TO_JSVAL(result);
-    ok = JS_TRUE;
-
-out:
-    JS_POP_TEMP_ROOT(cx, &tvr);
-    return ok;
 }
 #endif
 
@@ -999,41 +985,34 @@ js_InitExceptionClasses(JSContext *cx, JSObject *obj)
 {
     jsval roots[3];
     JSObject *obj_proto, *error_proto;
-    jsval empty;
 
     /*
      * If lazy class initialization occurs for any Error subclass, then all
      * classes are initialized, starting with Error.  To avoid reentry and
      * redundant initialization, we must not pass a null proto parameter to
-     * js_NewObject below, when called for the Error superclass.  We need to
+     * NewNonFunction below, when called for the Error superclass.  We need to
      * ensure that Object.prototype is the proto of Error.prototype.
      *
      * See the equivalent code to ensure that parent_proto is non-null when
-     * JS_InitClass calls js_NewObject, in jsapi.c.
+     * js_InitClass calls NewObject, in jsobj.cpp.
      */
-    if (!js_GetClassPrototype(cx, obj, INT_TO_JSID(JSProto_Object),
-                              &obj_proto)) {
+    if (!js_GetClassPrototype(cx, obj, JSProto_Object, &obj_proto))
         return NULL;
-    }
 
-    memset(roots, 0, sizeof(roots));
-    JSAutoTempValueRooter tvr(cx, JS_ARRAY_LENGTH(roots), roots);
+    PodArrayZero(roots);
+    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), Valueify(roots));
 
 #ifdef __GNUC__
     error_proto = NULL;   /* quell GCC overwarning */
 #endif
 
+    jsval empty = STRING_TO_JSVAL(cx->runtime->emptyString);
+
     /* Initialize the prototypes first. */
     for (intN i = JSEXN_ERR; i != JSEXN_LIMIT; i++) {
-        JSObject *proto;
-        JSProtoKey protoKey;
-        JSAtom *atom;
-        JSFunction *fun;
-
         /* Make the prototype for the current constructor name. */
-        proto = js_NewObject(cx, &js_ErrorClass,
-                             (i != JSEXN_ERR) ? error_proto : obj_proto,
-                             obj);
+        JSObject *proto =
+            NewNonFunction<WithProto::Class>(cx, &js_ErrorClass, (i != JSEXN_ERR) ? error_proto : obj_proto, obj);
         if (!proto)
             return NULL;
         if (i == JSEXN_ERR) {
@@ -1046,12 +1025,12 @@ js_InitExceptionClasses(JSContext *cx, JSObject *obj)
         }
 
         /* So exn_finalize knows whether to destroy private data. */
-        STOBJ_SET_SLOT(proto, JSSLOT_PRIVATE, JSVAL_VOID);
+        proto->setPrivate(NULL);
 
         /* Make a constructor function for the current name. */
-        protoKey = GetExceptionProtoKey(i);
-        atom = cx->runtime->atomState.classAtoms[protoKey];
-        fun = js_DefineFunction(cx, obj, atom, Exception, 3, 0);
+        JSProtoKey protoKey = GetExceptionProtoKey(i);
+        JSAtom *atom = cx->runtime->atomState.classAtoms[protoKey];
+        JSFunction *fun = js_DefineFunction(cx, obj, atom, Exception, 3, 0);
         if (!fun)
             return NULL;
         roots[2] = OBJECT_TO_JSVAL(FUN_OBJECT(fun));
@@ -1066,30 +1045,27 @@ js_InitExceptionClasses(JSContext *cx, JSObject *obj)
         }
 
         /* Add the name property to the prototype. */
-        if (!JS_DefineProperty(cx, proto, js_name_str, ATOM_KEY(atom),
+        if (!JS_DefineProperty(cx, proto, js_name_str,
+                               STRING_TO_JSVAL(ATOM_TO_STRING(atom)),
                                NULL, NULL, JSPROP_ENUMERATE)) {
             return NULL;
         }
 
         /* Finally, stash the constructor for later uses. */
-        if (!js_SetClassObject(cx, obj, protoKey, FUN_OBJECT(fun)))
+        if (!js_SetClassObject(cx, obj, protoKey, FUN_OBJECT(fun), proto))
             return NULL;
+
+        /* Set default values. */
+        if (!JS_DefineProperty(cx, proto, js_message_str, empty, NULL, NULL, JSPROP_ENUMERATE) ||
+            !JS_DefineProperty(cx, proto, js_fileName_str, empty, NULL, NULL, JSPROP_ENUMERATE) ||
+            !JS_DefineProperty(cx, proto, js_lineNumber_str, JSVAL_ZERO, NULL, NULL,
+                               JSPROP_ENUMERATE)) {
+            return NULL;
+        }
     }
 
-    /*
-     * Set default values and add methods. We do it only for Error.prototype
-     * as the rest of exceptions delegate to it.
-     */
-    empty = STRING_TO_JSVAL(cx->runtime->emptyString);
-    if (!JS_DefineProperty(cx, error_proto, js_message_str, empty,
-                           NULL, NULL, JSPROP_ENUMERATE) ||
-        !JS_DefineProperty(cx, error_proto, js_fileName_str, empty,
-                           NULL, NULL, JSPROP_ENUMERATE) ||
-        !JS_DefineProperty(cx, error_proto, js_lineNumber_str, JSVAL_ZERO,
-                           NULL, NULL, JSPROP_ENUMERATE) ||
-        !JS_DefineFunctions(cx, error_proto, exception_methods)) {
+    if (!JS_DefineFunctions(cx, error_proto, exception_methods))
         return NULL;
-    }
 
     return error_proto;
 }
@@ -1120,13 +1096,13 @@ static struct exnname { char *name; char *exception; } errortoexnname[] = {
 #endif /* DEBUG */
 
 JSBool
-js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp)
+js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp,
+                    JSErrorCallback callback, void *userRef)
 {
     JSErrNum errorNumber;
     const JSErrorFormatString *errorString;
     JSExnType exn;
     jsval tv[4];
-    JSTempValueRooter tvr;
     JSBool ok;
     JSObject *errProto, *errObject;
     JSString *messageStr, *filenameStr;
@@ -1140,7 +1116,10 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp)
 
     /* Find the exception index associated with this error. */
     errorNumber = (JSErrNum) reportp->errorNumber;
-    errorString = js_GetLocalizedErrorMessage(cx, NULL, NULL, errorNumber);
+    if (!callback || callback == js_GetErrorMessage)
+        errorString = js_GetLocalizedErrorMessage(cx, NULL, NULL, errorNumber);
+    else
+        errorString = callback(userRef, NULL, errorNumber);
     exn = errorString ? (JSExnType) errorString->exnType : JSEXN_NONE;
     JS_ASSERT(exn < JSEXN_LIMIT);
 
@@ -1171,21 +1150,20 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp)
     cx->generatingError = JS_TRUE;
 
     /* Protect the newly-created strings below from nesting GCs. */
-    memset(tv, 0, sizeof tv);
-    JS_PUSH_TEMP_ROOT(cx, JS_ARRAY_LENGTH(tv), tv, &tvr);
+    PodArrayZero(tv);
+    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(tv), Valueify(tv));
 
     /*
      * Try to get an appropriate prototype by looking up the corresponding
      * exception constructor name in the scope chain of the current context's
      * top stack frame, or in the global object if no frame is active.
      */
-    ok = js_GetClassPrototype(cx, NULL, INT_TO_JSID(GetExceptionProtoKey(exn)),
-                              &errProto);
+    ok = js_GetClassPrototype(cx, NULL, GetExceptionProtoKey(exn), &errProto);
     if (!ok)
         goto out;
     tv[0] = OBJECT_TO_JSVAL(errProto);
 
-    errObject = js_NewObject(cx, &js_ErrorClass, errProto, NULL);
+    errObject = NewNativeClassInstance(cx, &js_ErrorClass, errProto, errProto->getParent());
     if (!errObject) {
         ok = JS_FALSE;
         goto out;
@@ -1217,7 +1195,6 @@ js_ErrorToException(JSContext *cx, const char *message, JSErrorReport *reportp)
     reportp->flags |= JSREPORT_EXCEPTION;
 
 out:
-    JS_POP_TEMP_ROOT(cx, &tvr);
     cx->generatingError = JS_FALSE;
     return ok;
 }
@@ -1228,20 +1205,18 @@ js_ReportUncaughtException(JSContext *cx)
     jsval exn;
     JSObject *exnObject;
     jsval roots[5];
-    JSTempValueRooter tvr;
     JSErrorReport *reportp, report;
     JSString *str;
     const char *bytes;
-    JSBool ok;
 
     if (!JS_IsExceptionPending(cx))
-        return JS_TRUE;
+        return true;
 
     if (!JS_GetPendingException(cx, &exn))
-        return JS_FALSE;
+        return false;
 
-    memset(roots, 0, sizeof roots);
-    JS_PUSH_TEMP_ROOT(cx, JS_ARRAY_LENGTH(roots), roots, &tvr);
+    PodArrayZero(roots);
+    AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), Valueify(roots));
 
     /*
      * Because js_ValueToString below could error and an exception object
@@ -1260,62 +1235,51 @@ js_ReportUncaughtException(JSContext *cx)
     reportp = js_ErrorFromException(cx, exn);
 
     /* XXX L10N angels cry once again (see also jsemit.c, /L10N gaffes/) */
-    str = js_ValueToString(cx, exn);
+    str = js_ValueToString(cx, Valueify(exn));
     if (!str) {
         bytes = "unknown (can't convert to string)";
     } else {
         roots[1] = STRING_TO_JSVAL(str);
         bytes = js_GetStringBytes(cx, str);
-        if (!bytes) {
-            ok = JS_FALSE;
-            goto out;
-        }
+        if (!bytes)
+            return false;
     }
-    ok = JS_TRUE;
 
-    if (!reportp &&
-        exnObject &&
-        OBJ_GET_CLASS(cx, exnObject) == &js_ErrorClass) {
+    if (!reportp && exnObject && exnObject->getClass() == &js_ErrorClass) {
         const char *filename;
-        uint32 lineno;
 
-        ok = JS_GetProperty(cx, exnObject, js_message_str, &roots[2]);
-        if (!ok)
-            goto out;
+        if (!JS_GetProperty(cx, exnObject, js_message_str, &roots[2]))
+            return false;
         if (JSVAL_IS_STRING(roots[2])) {
             bytes = js_GetStringBytes(cx, JSVAL_TO_STRING(roots[2]));
-            if (!bytes) {
-                ok = JS_FALSE;
-                goto out;
-            }
+            if (!bytes)
+                return false;
         }
 
-        ok = JS_GetProperty(cx, exnObject, js_fileName_str, &roots[3]);
-        if (!ok)
-            goto out;
-        str = js_ValueToString(cx, roots[3]);
-        if (!str) {
-            ok = JS_FALSE;
-            goto out;
-        }
+        if (!JS_GetProperty(cx, exnObject, js_fileName_str, &roots[3]))
+            return false;
+        str = js_ValueToString(cx, Valueify(roots[3]));
+        if (!str)
+            return false;
         filename = StringToFilename(cx, str);
-        if (!filename) {
-            ok = JS_FALSE;
-            goto out;
-        }
+        if (!filename)
+            return false;
 
-        ok = JS_GetProperty(cx, exnObject, js_lineNumber_str, &roots[4]);
-        if (!ok)
-            goto out;
-        lineno = js_ValueToECMAUint32 (cx, &roots[4]);
-        ok = !JSVAL_IS_NULL(roots[4]);
-        if (!ok)
-            goto out;
+        if (!JS_GetProperty(cx, exnObject, js_lineNumber_str, &roots[4]))
+            return false;
+        uint32_t lineno;
+        if (!ValueToECMAUint32 (cx, Valueify(roots[4]), &lineno))
+            return false;
 
         reportp = &report;
-        memset(&report, 0, sizeof report);
+        PodZero(&report);
         report.filename = filename;
         report.lineno = (uintN) lineno;
+        if (JSVAL_IS_STRING(roots[2])) {
+            report.ucmessage = js_GetStringChars(cx, JSVAL_TO_STRING(roots[2]));
+            if (!report.ucmessage)
+                return false;
+        }
     }
 
     if (!reportp) {
@@ -1331,7 +1295,5 @@ js_ReportUncaughtException(JSContext *cx)
         JS_ClearPendingException(cx);
     }
 
-out:
-    JS_POP_TEMP_ROOT(cx, &tvr);
-    return ok;
+    return true;
 }
