@@ -52,11 +52,12 @@
 #include "jscntxt.h"
 #include "jsdtoa.h"
 #include "jsgc.h"
-#include "jsfun.h"      /* for VALUE_IS_FUNCTION from LOCKED_OBJ_WRITE_SLOT */
 #include "jslock.h"
 #include "jsscope.h"
 #include "jsstr.h"
 
+using namespace js;
+
 #define ReadWord(W) (W)
 
 #if !defined(__GNUC__)
@@ -66,7 +67,7 @@
 
 /* Implement NativeCompareAndSwap. */
 
-#if defined(_WIN32) && defined(_M_IX86)
+#if defined(_MSC_VER) && defined(_M_IX86)
 #pragma warning( disable : 4035 )
 JS_BEGIN_EXTERN_C
 extern long __cdecl
@@ -112,7 +113,9 @@ static JS_ALWAYS_INLINE int
 NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
 {
     /* Details on these functions available in the manpage for atomic */
-    return OSAtomicCompareAndSwapPtrBarrier(ov, nv, w);
+    return OSAtomicCompareAndSwapPtrBarrier(reinterpret_cast<void *>(ov),
+                                            reinterpret_cast<void *>(nv),
+                                            reinterpret_cast<void **>(w));
 }
 
 #elif defined(__i386) && (defined(__GNUC__) || defined(__SUNPRO_CC))
@@ -129,11 +132,15 @@ NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
                           "sete %%al\n"
                           "andl $1, %%eax\n"
                           : "=a" (res)
+#ifdef __SUNPRO_CC
+/* Different code for Sun Studio because of a bug of SS12U1 */
+                          : "c" (w), "d" (nv), "a" (ov)
+#else
                           : "r" (w), "r" (nv), "a" (ov)
+#endif
                           : "cc", "memory");
     return (int)res;
 }
-
 #elif defined(__x86_64) && (defined(__GNUC__) || defined(__SUNPRO_CC))
 
 static JS_ALWAYS_INLINE int
@@ -152,7 +159,8 @@ NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
     return (int)res;
 }
 
-#elif defined(__sparc) && (defined(__GNUC__) || defined(__SUNPRO_CC))
+#elif defined(__sparc)
+#if defined(__GNUC__)
 
 static JS_ALWAYS_INLINE int
 NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
@@ -160,8 +168,13 @@ NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
     unsigned int res;
 
     __asm__ __volatile__ (
-                  "stbar\n"
+                  "membar #StoreLoad | #LoadLoad\n"
+#if JS_BITS_PER_WORD == 32
                   "cas [%1],%2,%3\n"
+#else
+                  "casx [%1],%2,%3\n"
+#endif
+                  "membar #StoreLoad | #LoadLoad\n"
                   "cmp %2,%3\n"
                   "be,a 1f\n"
                   "mov 1,%0\n"
@@ -172,6 +185,14 @@ NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
     return (int)res;
 }
 
+#elif defined(__SUNPRO_CC)
+
+/* Implementation in lock_sparc*.il */
+extern "C" int
+NativeCompareAndSwap(jsword *w, jsword ov, jsword nv);
+
+#endif
+
 #elif defined(AIX)
 
 #include <sys/atomic_op.h>
@@ -179,7 +200,13 @@ NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
 static JS_ALWAYS_INLINE int
 NativeCompareAndSwap(jsword *w, jsword ov, jsword nv)
 {
-    return !_check_lock((atomic_p)w, ov, nv);
+    int res;
+    JS_STATIC_ASSERT(sizeof(jsword) == sizeof(long));
+
+    res = compare_and_swaplp((atomic_l)w, &ov, nv);
+    if (res)
+        __asm__("isync");
+    return res;
 }
 
 #elif defined(USE_ARM_KUSER)
@@ -305,7 +332,7 @@ js_InitLock(JSThinLock *tl)
     tl->owner = 0;
     tl->fat = (JSFatLock*)JS_NEW_LOCK();
 #else
-    memset(tl, 0, sizeof(JSThinLock));
+    PodZero(tl);
 #endif
 }
 
@@ -394,7 +421,7 @@ WillDeadlock(JSContext *ownercx, JSThread *thread)
 
      for (;;) {
         JS_ASSERT(ownercx->thread);
-        JS_ASSERT(ownercx->requestDepth > 0);
+        JS_ASSERT(ownercx->thread->requestContext);
         JSTitle *title = ownercx->thread->titleToShare;
         if (!title || !title->ownercx) {
             /*
@@ -455,7 +482,7 @@ ShareTitle(JSContext *cx, JSTitle *title)
  * making mutable strings in the title's object's slots be immutable. We have
  * to do this because such strings will soon be available to multiple threads,
  * so their buffers can't be realloc'd any longer in js_ConcatStrings, and
- * their members can't be modified by js_ConcatStrings, js_UndependString or
+ * their members can't be modified by js_ConcatStrings, JSString::undepend, or
  * MinimizeDependentStrings.
  *
  * The last bit of work done by this function nulls title->ownercx and updates
@@ -464,28 +491,25 @@ ShareTitle(JSContext *cx, JSTitle *title)
 static void
 FinishSharingTitle(JSContext *cx, JSTitle *title)
 {
-    JSScope *scope;
-    JSObject *obj;
-    uint32 nslots, i;
-    jsval v;
-
     js_InitLock(&title->lock);
     title->u.count = 0;     /* NULL may not pun as 0 */
-    scope = TITLE_TO_SCOPE(title);
-    obj = scope->object;
+
+    JSScope *scope = TITLE_TO_SCOPE(title);
+    JSObject *obj = scope->object;
     if (obj) {
-        nslots = scope->freeslot;
-        for (i = 0; i != nslots; ++i) {
-            v = STOBJ_GET_SLOT(obj, i);
-            if (JSVAL_IS_STRING(v) &&
-                !js_MakeStringImmutable(cx, JSVAL_TO_STRING(v))) {
+        uint32 nslots = scope->freeslot;
+        JS_ASSERT(nslots >= JSSLOT_START(obj->getClass()));
+        for (uint32 i = JSSLOT_START(obj->getClass()); i != nslots; ++i) {
+            Value v = obj->getSlot(i);
+            if (v.isString() &&
+                !js_MakeStringImmutable(cx, v.toString())) {
                 /*
                  * FIXME bug 363059: The following error recovery changes
                  * runtime execution semantics, arbitrarily and silently
                  * ignoring errors except out-of-memory, which should have been
                  * reported through JS_ReportOutOfMemory at this point.
                  */
-                STOBJ_SET_SLOT(obj, i, JSVAL_VOID);
+                obj->setSlot(i, UndefinedValue());
             }
         }
     }
@@ -494,41 +518,6 @@ FinishSharingTitle(JSContext *cx, JSTitle *title)
     JS_RUNTIME_METER(cx->runtime, sharedTitles);
 }
 
-/*
- * Notify all contexts that are currently in a request, which will give them a
- * chance to yield their current request.
- */
-void
-js_NudgeOtherContexts(JSContext *cx)
-{
-    JSRuntime *rt = cx->runtime;
-    JSContext *acx = NULL;
-
-    while ((acx = js_NextActiveContext(rt, acx)) != NULL) {
-        if (cx != acx)
-            JS_TriggerOperationCallback(acx);
-    }
-}
-
-/*
- * Notify all contexts that are currently in a request and execute on this
- * specific thread.
- */
-static void
-NudgeThread(JSThread *thread)
-{
-    JSCList *link;
-    JSContext *acx;
-
-    link = &thread->contextList;
-    while ((link = link->next) != &thread->contextList) {
-        acx = CX_FROM_THREAD_LINKS(link);
-        JS_ASSERT(acx->thread == thread);
-        if (acx->requestDepth)
-            JS_TriggerOperationCallback(acx);
-    }
-}
-
 /*
  * Given a title with apparently non-null ownercx different from cx, try to
  * set ownercx to cx, claiming exclusive (single-threaded) ownership of title.
@@ -540,52 +529,49 @@ NudgeThread(JSThread *thread)
 static JSBool
 ClaimTitle(JSTitle *title, JSContext *cx)
 {
-    JSRuntime *rt;
-    JSContext *ownercx;
-    uint32 requestDebit;
+    JSRuntime *rt = cx->runtime;
+    JS_ASSERT_IF(!cx->thread->requestContext,
+                 cx->thread == rt->gcThread && rt->gcRunning);
 
-    rt = cx->runtime;
     JS_RUNTIME_METER(rt, claimAttempts);
-    JS_LOCK_GC(rt);
+    AutoLockGC lock(rt);
 
     /* Reload in case ownercx went away while we blocked on the lock. */
-    while ((ownercx = title->ownercx) != NULL) {
+    while (JSContext *ownercx = title->ownercx) {
         /*
          * Avoid selflock if ownercx is dead, or is not running a request, or
-         * has the same thread as cx.  Set title->ownercx to cx so that the
-         * matching JS_UNLOCK_SCOPE or JS_UNLOCK_OBJ macro call will take the
-         * fast path around the corresponding js_UnlockTitle or js_UnlockObj
-         * function call.
+         * has the same thread as cx, or cx->thread runs the GC (in which case
+         * all other requests must be suspended), or ownercx->thread runs a GC
+         * and the GC waits for all requests to finish. Set title->ownercx to
+         * cx so that the matching JS_UNLOCK_SCOPE or JS_UNLOCK_OBJ macro call
+         * will take the fast path around the corresponding js_UnlockTitle or
+         * js_UnlockObj function call.
          *
          * If title->u.link is non-null, title has already been inserted on
          * the rt->titleSharingTodo list, because another thread's context
          * already wanted to lock title while ownercx was running a request.
-         * That context must still be in request and cannot be dead. We can
-         * claim it if its thread matches ours but only if cx itself is in a
-         * request.
-         *
-         * The latter check covers the case when the embedding triggers a call
-         * to js_GC on a cx outside a request while having ownercx running a
-         * request on the same thread, and then js_GC calls a mark hook or a
-         * finalizer accessing the title. In this case we cannot claim the
-         * title but must share it now as no title-sharing JS_EndRequest will
-         * follow.
+         * That context must still be in request and cannot be dead. Moreover,
+         * the GC can not run at this moment as it must wait until all the
+         * titles are shared and the threads that want to lock them finish
+         * their requests. Thus we can claim the title if its thread matches
+         * ours.
          */
         bool canClaim;
         if (title->u.link) {
             JS_ASSERT(js_ValidContextPointer(rt, ownercx));
-            JS_ASSERT(ownercx->requestDepth > 0);
-            JS_ASSERT_IF(cx->requestDepth == 0, cx->thread == rt->gcThread);
-            canClaim = (ownercx->thread == cx->thread &&
-                        cx->requestDepth > 0);
+            JS_ASSERT(ownercx->thread->requestContext);
+            JS_ASSERT(!rt->gcRunning);
+            canClaim = (ownercx->thread == cx->thread);
         } else {
             canClaim = (!js_ValidContextPointer(rt, ownercx) ||
-                        !ownercx->requestDepth ||
-                        ownercx->thread == cx->thread);
+                        !ownercx->thread ||
+                        !ownercx->thread->requestContext ||
+                        cx->thread == ownercx->thread  ||
+                        cx->thread == rt->gcThread ||
+                        ownercx->thread->gcWaiting);
         }
         if (canClaim) {
             title->ownercx = cx;
-            JS_UNLOCK_GC(rt);
             JS_RUNTIME_METER(rt, claimedTitles);
             return JS_TRUE;
         }
@@ -602,14 +588,8 @@ ClaimTitle(JSTitle *title, JSContext *cx)
          * so that control would unwind properly once these locks became
          * "thin" or "fat". The engine promotes a title from exclusive to
          * shared access only when locking, never when holding or unlocking.
-         *
-         * Avoid deadlock before any of this title/context cycle detection if
-         * cx is on the active GC's thread, because in that case, no requests
-         * will run until the GC completes.  Any title wanted by the GC (from
-         * a finalizer or a mark hook) that can't be claimed must become
-         * shared.
          */
-        if (rt->gcThread == cx->thread || WillDeadlock(ownercx, cx->thread)) {
+        if (WillDeadlock(ownercx, cx->thread)) {
             ShareTitle(cx, title);
             break;
         }
@@ -620,26 +600,10 @@ ClaimTitle(JSTitle *title, JSContext *cx)
          * non-null test, and avoid double-insertion bugs.
          */
         if (!title->u.link) {
-            TITLE_TO_SCOPE(title)->hold();
             title->u.link = rt->titleSharingTodo;
             rt->titleSharingTodo = title;
         }
 
-        /*
-         * Discount all the requests running on the current thread so a
-         * possible GC can proceed on another thread while we wait on
-         * rt->titleSharingDone.
-         */
-        requestDebit = js_DiscountRequestsForGC(cx);
-        if (title->ownercx != ownercx) {
-            /*
-             * js_DiscountRequestsForGC released and reacquired the GC lock,
-             * and the title was taken or shared. Start over.
-             */
-            js_RecountRequestsAfterGC(rt, requestDebit);
-            continue;
-        }
-
         /*
          * We know that some other thread's context owns title, which is now
          * linked onto rt->titleSharingTodo, awaiting the end of that other
@@ -647,7 +611,7 @@ ClaimTitle(JSTitle *title, JSContext *cx)
          * But before waiting, we force the operation callback for that other
          * thread so it can quickly suspend.
          */
-        NudgeThread(ownercx->thread);
+        JS_THREAD_DATA(ownercx)->triggerOperationCallback();
 
         JS_ASSERT(!cx->thread->titleToShare);
         cx->thread->titleToShare = title;
@@ -656,25 +620,8 @@ ClaimTitle(JSTitle *title, JSContext *cx)
 #endif
             PR_WaitCondVar(rt->titleSharingDone, PR_INTERVAL_NO_TIMEOUT);
         JS_ASSERT(stat != PR_FAILURE);
-
-        js_RecountRequestsAfterGC(rt, requestDebit);
-
-        /*
-         * Don't clear titleToShare until after we're through waiting on
-         * all condition variables protected by rt->gcLock -- that includes
-         * rt->titleSharingDone *and* rt->gcDone (hidden in the call to
-         * js_RecountRequestsAfterGC immediately above).
-         *
-         * Otherwise, the GC could easily deadlock with another thread that
-         * owns a title wanted by a finalizer.  By keeping cx->titleToShare
-         * set till here, we ensure that such deadlocks are detected, which
-         * results in the finalized object's title being shared (it must, of
-         * course, have other, live objects sharing it).
-         */
         cx->thread->titleToShare = NULL;
     }
-
-    JS_UNLOCK_GC(rt);
     return JS_FALSE;
 }
 
@@ -688,24 +635,15 @@ js_ShareWaitingTitles(JSContext *cx)
     todop = &cx->runtime->titleSharingTodo;
     shared = false;
     while ((title = *todop) != NO_TITLE_SHARING_TODO) {
-        if (title->ownercx != cx) {
+        if (title->ownercx->thread != cx->thread) {
             todop = &title->u.link;
             continue;
         }
         *todop = title->u.link;
-        title->u.link = NULL;       /* null u.link for sanity ASAP */
+        title->u.link = NULL;           /* null u.link for sanity ASAP */
 
-        /*
-         * If JSScope::drop returns false, we held the last ref to scope. The
-         * waiting thread(s) must have been killed, after which the GC
-         * collected the object that held this scope.  Unlikely, because it
-         * requires that the GC ran (e.g., from an operation callback)
-         * during this request, but possible.
-         */
-        if (TITLE_TO_SCOPE(title)->drop(cx, NULL)) {
-            FinishSharingTitle(cx, title); /* set ownercx = NULL */
-            shared = true;
-        }
+        FinishSharingTitle(cx, title);  /* set ownercx = NULL */
+        shared = true;
     }
     if (shared)
         JS_NOTIFY_ALL_CONDVAR(cx->runtime->titleSharingDone);
@@ -723,24 +661,13 @@ js_GetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot)
     jsword me;
 #endif
 
-    /*
-     * We handle non-native objects via JSObjectOps.getRequiredSlot, treating
-     * all slots starting from 0 as required slots.  A property definition or
-     * some prior arrangement must have allocated slot.
-     *
-     * Note once again (see jspubtd.h, before JSGetRequiredSlotOp's typedef)
-     * the crucial distinction between a |required slot number| that's passed
-     * to the get/setRequiredSlot JSObjectOps, and a |reserved slot index|
-     * passed to the JS_Get/SetReservedSlot APIs.
-     */
-    if (!OBJ_IS_NATIVE(obj))
-        return OBJ_GET_REQUIRED_SLOT(cx, obj, slot);
+    OBJ_CHECK_SLOT(obj, slot);
 
     /*
      * Native object locking is inlined here to optimize the single-threaded
      * and contention-free multi-threaded cases.
      */
-    scope = OBJ_SCOPE(obj);
+    scope = obj->scope();
     title = &scope->title;
     JS_ASSERT(title->ownercx != cx);
     JS_ASSERT(slot < scope->freeslot);
@@ -754,7 +681,7 @@ js_GetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot)
     if (CX_THREAD_IS_RUNNING_GC(cx) ||
         scope->sealed() ||
         (title->ownercx && ClaimTitle(title, cx))) {
-        return STOBJ_GET_SLOT(obj, slot);
+        return Jsvalify(obj->getSlot(slot));
     }
 
 #ifndef NSPR_LOCK
@@ -768,8 +695,8 @@ js_GetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot)
          * require either a restart from the top of this routine, or a thin
          * lock release followed by fat lock acquisition.
          */
-        if (scope == OBJ_SCOPE(obj)) {
-            v = STOBJ_GET_SLOT(obj, slot);
+        if (scope == obj->scope()) {
+            v = Jsvalify(obj->getSlot(slot));
             if (!NativeCompareAndSwap(&tl->owner, me, 0)) {
                 /* Assert that scope locks never revert to flyweight. */
                 JS_ASSERT(title->ownercx != cx);
@@ -783,12 +710,12 @@ js_GetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot)
             js_Dequeue(tl);
     }
     else if (Thin_RemoveWait(ReadWord(tl->owner)) == me) {
-        return STOBJ_GET_SLOT(obj, slot);
+        return Jsvalify(obj->getSlot(slot));
     }
 #endif
 
     js_LockObj(cx, obj);
-    v = STOBJ_GET_SLOT(obj, slot);
+    v = Jsvalify(obj->getSlot(slot));
 
     /*
      * Test whether cx took ownership of obj's scope during js_LockObj.
@@ -799,7 +726,7 @@ js_GetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot)
      * object's scope (whose lock was not flyweight, else we wouldn't be here
      * in the first place!).
      */
-    title = &OBJ_SCOPE(obj)->title;
+    title = &obj->scope()->title;
     if (title->ownercx != cx)
         js_UnlockTitle(cx, title);
     return v;
@@ -815,6 +742,8 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
     jsword me;
 #endif
 
+    OBJ_CHECK_SLOT(obj, slot);
+
     /* Any string stored in a thread-safe object must be immutable. */
     if (JSVAL_IS_STRING(v) &&
         !js_MakeStringImmutable(cx, JSVAL_TO_STRING(v))) {
@@ -822,20 +751,11 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
         v = JSVAL_NULL;
     }
 
-    /*
-     * We handle non-native objects via JSObjectOps.setRequiredSlot, as above
-     * for the Get case.
-     */
-    if (!OBJ_IS_NATIVE(obj)) {
-        OBJ_SET_REQUIRED_SLOT(cx, obj, slot, v);
-        return;
-    }
-
     /*
      * Native object locking is inlined here to optimize the single-threaded
      * and contention-free multi-threaded cases.
      */
-    scope = OBJ_SCOPE(obj);
+    scope = obj->scope();
     title = &scope->title;
     JS_ASSERT(title->ownercx != cx);
     JS_ASSERT(slot < scope->freeslot);
@@ -849,7 +769,7 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
     if (CX_THREAD_IS_RUNNING_GC(cx) ||
         scope->sealed() ||
         (title->ownercx && ClaimTitle(title, cx))) {
-        LOCKED_OBJ_WRITE_SLOT(cx, obj, slot, v);
+        obj->lockedSetSlot(slot, Valueify(v));
         return;
     }
 
@@ -858,8 +778,8 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
     me = CX_THINLOCK_ID(cx);
     JS_ASSERT(CURRENT_THREAD_IS_ME(me));
     if (NativeCompareAndSwap(&tl->owner, 0, me)) {
-        if (scope == OBJ_SCOPE(obj)) {
-            LOCKED_OBJ_WRITE_SLOT(cx, obj, slot, v);
+        if (scope == obj->scope()) {
+            obj->lockedSetSlot(slot, Valueify(v));
             if (!NativeCompareAndSwap(&tl->owner, me, 0)) {
                 /* Assert that scope locks never revert to flyweight. */
                 JS_ASSERT(title->ownercx != cx);
@@ -871,20 +791,19 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
         }
         if (!NativeCompareAndSwap(&tl->owner, me, 0))
             js_Dequeue(tl);
-    }
-    else if (Thin_RemoveWait(ReadWord(tl->owner)) == me) {
-        LOCKED_OBJ_WRITE_SLOT(cx, obj, slot, v);
+    } else if (Thin_RemoveWait(ReadWord(tl->owner)) == me) {
+        obj->lockedSetSlot(slot, Valueify(v));
         return;
     }
 #endif
 
     js_LockObj(cx, obj);
-    LOCKED_OBJ_WRITE_SLOT(cx, obj, slot, v);
+    obj->lockedSetSlot(slot, Valueify(v));
 
     /*
      * Same drill as above, in js_GetSlotThreadSafe.
      */
-    title = &OBJ_SCOPE(obj)->title;
+    title = &obj->scope()->title;
     if (title->ownercx != cx)
         js_UnlockTitle(cx, title);
 }
@@ -894,7 +813,7 @@ js_SetSlotThreadSafe(JSContext *cx, JSObject *obj, uint32 slot, jsval v)
 static JSFatLock *
 NewFatlock()
 {
-    JSFatLock *fl = (JSFatLock *)malloc(sizeof(JSFatLock)); /* for now */
+    JSFatLock *fl = (JSFatLock *)js_malloc(sizeof(JSFatLock)); /* for now */
     if (!fl) return NULL;
     fl->susp = 0;
     fl->next = NULL;
@@ -909,7 +828,7 @@ DestroyFatlock(JSFatLock *fl)
 {
     PR_DestroyLock(fl->slock);
     PR_DestroyCondVar(fl->svar);
-    free(fl);
+    js_free(fl);
 }
 
 static JSFatLock *
@@ -1003,7 +922,7 @@ js_SetupLocks(int listc, int globc)
     global_locks_log2 = JS_CeilingLog2(globc);
     global_locks_mask = JS_BITMASK(global_locks_log2);
     global_lock_count = JS_BIT(global_locks_log2);
-    global_locks = (PRLock **) malloc(global_lock_count * sizeof(PRLock*));
+    global_locks = (PRLock **) js_malloc(global_lock_count * sizeof(PRLock*));
     if (!global_locks)
         return JS_FALSE;
     for (i = 0; i < global_lock_count; i++) {
@@ -1014,7 +933,7 @@ js_SetupLocks(int listc, int globc)
             return JS_FALSE;
         }
     }
-    fl_list_table = (JSFatLockTable *) malloc(i * sizeof(JSFatLockTable));
+    fl_list_table = (JSFatLockTable *) js_malloc(i * sizeof(JSFatLockTable));
     if (!fl_list_table) {
         js_CleanupLocks();
         return JS_FALSE;
@@ -1036,7 +955,7 @@ js_CleanupLocks()
     if (global_locks) {
         for (i = 0; i < global_lock_count; i++)
             PR_DestroyLock(global_locks[i]);
-        free(global_locks);
+        js_free(global_locks);
         global_locks = NULL;
         global_lock_count = 1;
         global_locks_log2 = 0;
@@ -1049,7 +968,7 @@ js_CleanupLocks()
             DeleteListOfFatlocks(fl_list_table[i].taken);
             fl_list_table[i].taken = NULL;
         }
-        free(fl_list_table);
+        js_free(fl_list_table);
         fl_list_table = NULL;
         fl_list_table_len = 0;
     }
@@ -1251,7 +1170,7 @@ void
 js_UnlockRuntime(JSRuntime *rt)
 {
 #ifdef DEBUG
-    rt->rtLockOwner = 0;
+    rt->rtLockOwner = NULL;
 #endif
     PR_Unlock(rt->rtLock);
 }
@@ -1329,17 +1248,18 @@ js_UnlockTitle(JSContext *cx, JSTitle *title)
  * dropped the last reference to oldtitle.
  */
 void
-js_TransferTitle(JSContext *cx, JSTitle *oldtitle, JSTitle *newtitle)
+js_DropAllEmptyScopeLocks(JSContext *cx, JSScope *scope)
 {
-    JS_ASSERT(JS_IS_TITLE_LOCKED(cx, newtitle));
+    JS_ASSERT(!CX_OWNS_SCOPE_TITLE(cx,scope));
+    JS_ASSERT(scope->isSharedEmpty());
+    JS_ASSERT(JS_IS_TITLE_LOCKED(cx, &scope->title));
 
     /*
-     * If the last reference to oldtitle went away, newtitle needs no lock
-     * state update.
+     * Shared empty scope cannot be sealed so we do not need to deal with
+     * cx->lockedSealedTitle.
      */
-    if (!oldtitle)
-        return;
-    JS_ASSERT(JS_IS_TITLE_LOCKED(cx, oldtitle));
+    JS_ASSERT(!scope->sealed());
+    JS_ASSERT(cx->lockedSealedTitle != &scope->title);
 
     /*
      * Special case in js_LockTitle and js_UnlockTitle for the GC calling
@@ -1351,45 +1271,14 @@ js_TransferTitle(JSContext *cx, JSTitle *oldtitle, JSTitle *newtitle)
         return;
 
     /*
-     * Special case in js_LockObj and js_UnlockTitle for locking the sealed
-     * scope of an object that owns that scope (the prototype or mutated obj
-     * for which OBJ_SCOPE(obj)->object == obj), and unlocking it.
+     * The title cannot be owned at this point by another cx on this or
+     * another thread as that would imply a missing JS_LOCK_OBJ call.
      */
-    JS_ASSERT(cx->lockedSealedTitle != newtitle);
-    if (cx->lockedSealedTitle == oldtitle) {
-        JS_ASSERT(newtitle->ownercx == cx ||
-                  (!newtitle->ownercx && newtitle->u.count == 1));
-        cx->lockedSealedTitle = NULL;
-        return;
-    }
+    JS_ASSERT(!scope->title.ownercx);
 
-    /*
-     * If oldtitle is single-threaded, there's nothing to do.
-     */
-    if (oldtitle->ownercx) {
-        JS_ASSERT(oldtitle->ownercx == cx);
-        JS_ASSERT(newtitle->ownercx == cx ||
-                  (!newtitle->ownercx && newtitle->u.count == 1));
-        return;
-    }
-
-    /*
-     * We transfer oldtitle->u.count only if newtitle is not single-threaded.
-     * Flow unwinds from here through some number of JS_UNLOCK_TITLE and/or
-     * JS_UNLOCK_OBJ macro calls, which will decrement newtitle->u.count only
-     * if they find newtitle->ownercx != cx.
-     */
-    if (newtitle->ownercx != cx) {
-        JS_ASSERT(!newtitle->ownercx);
-        newtitle->u.count = oldtitle->u.count;
-    }
-
-    /*
-     * Reset oldtitle's lock state so that it is completely unlocked.
-     */
-    LOGIT(oldtitle, '0');
-    oldtitle->u.count = 0;
-    ThinUnlock(&oldtitle->lock, CX_THINLOCK_ID(cx));
+    LOGIT(&scope->title, '0');
+    scope->title.u.count = 0;
+    ThinUnlock(&scope->title.lock, CX_THINLOCK_ID(cx));
 }
 
 void
@@ -1398,7 +1287,7 @@ js_LockObj(JSContext *cx, JSObject *obj)
     JSScope *scope;
     JSTitle *title;
 
-    JS_ASSERT(OBJ_IS_NATIVE(obj));
+    JS_ASSERT(obj->isNative());
 
     /*
      * We must test whether the GC is calling and return without mutating any
@@ -1409,7 +1298,7 @@ js_LockObj(JSContext *cx, JSObject *obj)
         return;
 
     for (;;) {
-        scope = OBJ_SCOPE(obj);
+        scope = obj->scope();
         title = &scope->title;
         if (scope->sealed() && !cx->lockedSealedTitle) {
             cx->lockedSealedTitle = title;
@@ -1419,7 +1308,7 @@ js_LockObj(JSContext *cx, JSObject *obj)
         js_LockTitle(cx, title);
 
         /* If obj still has this scope, we're done. */
-        if (scope == OBJ_SCOPE(obj))
+        if (scope == obj->scope())
             return;
 
         /* Lost a race with a mutator; retry with obj's new scope. */
@@ -1430,19 +1319,8 @@ js_LockObj(JSContext *cx, JSObject *obj)
 void
 js_UnlockObj(JSContext *cx, JSObject *obj)
 {
-    JS_ASSERT(OBJ_IS_NATIVE(obj));
-    js_UnlockTitle(cx, &OBJ_SCOPE(obj)->title);
-}
-
-bool
-js_LockObjIfShape(JSContext *cx, JSObject *obj, uint32 shape)
-{
-    JS_ASSERT(OBJ_SCOPE(obj)->title.ownercx != cx);
-    js_LockObj(cx, obj);
-    if (OBJ_SHAPE(obj) == shape)
-        return true;
-    js_UnlockObj(cx, obj);
-    return false;
+    JS_ASSERT(obj->isNative());
+    js_UnlockTitle(cx, &obj->scope()->title);
 }
 
 void
@@ -1450,7 +1328,7 @@ js_InitTitle(JSContext *cx, JSTitle *title)
 {
 #ifdef JS_THREADSAFE
     title->ownercx = cx;
-    memset(&title->lock, 0, sizeof title->lock);
+    js_InitLock(&title->lock);
 
     /*
      * Set u.link = NULL, not u.count = 0, in case the target architecture's
@@ -1491,7 +1369,7 @@ js_IsRuntimeLocked(JSRuntime *rt)
 JSBool
 js_IsObjLocked(JSContext *cx, JSObject *obj)
 {
-    return js_IsTitleLocked(cx, &OBJ_SCOPE(obj)->title);
+    return js_IsTitleLocked(cx, &obj->scope()->title);
 }
 
 JSBool
@@ -1506,13 +1384,14 @@ js_IsTitleLocked(JSContext *cx, JSTitle *title)
         return JS_TRUE;
 
     /*
-     * General case: the title is either exclusively owned (by cx), or it has
-     * a thin or fat lock to cope with shared (concurrent) ownership.
+     * General case: the title is either exclusively owned by some context, or
+     * it has a thin or fat lock to cope with shared (concurrent) ownership.
+     *
+     * js_LockTitle(cx, title) must set ownercx to cx when claiming the title
+     * from another context on the same thread.
      */
-    if (title->ownercx) {
-        JS_ASSERT(title->ownercx == cx || title->ownercx->thread == cx->thread);
-        return JS_TRUE;
-    }
+    if (title->ownercx)
+        return title->ownercx == cx;
     return js_CurrentThreadId() ==
            ((JSThread *)Thin_RemoveWait(ReadWord(title->lock.owner)))->id;
 }
