@@ -54,9 +54,8 @@
 #include "jsmath.h"
 #include "jsnum.h"
 #include "jslibmath.h"
-#include "jsobj.h"
 
-extern jsdouble js_NaN;
+using namespace js;
 
 #ifndef M_E
 #define M_E             2.7182818284590452354
@@ -95,91 +94,95 @@ static JSConstDoubleSpec math_constants[] = {
     {0,0,0,{0,0,0}}
 };
 
-JSClass js_MathClass = {
+Class js_MathClass = {
     js_Math_str,
     JSCLASS_HAS_CACHED_PROTO(JSProto_Math),
-    JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,
-    JS_EnumerateStub, JS_ResolveStub,   JS_ConvertStub,   NULL,
-    JSCLASS_NO_OPTIONAL_MEMBERS
+    PropertyStub,   /* addProperty */
+    PropertyStub,   /* delProperty */
+    PropertyStub,   /* getProperty */
+    PropertyStub,   /* setProperty */
+    EnumerateStub,
+    ResolveStub,
+    ConvertStub
 };
 
 static JSBool
-math_abs(JSContext *cx, uintN argc, jsval *vp)
+math_abs(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = fabs(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_acos(JSContext *cx, uintN argc, jsval *vp)
+math_acos(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
 #if defined(SOLARIS) && defined(__GNUC__)
     if (x < -1 || 1 < x) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
 #endif
     z = acos(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_asin(JSContext *cx, uintN argc, jsval *vp)
+math_asin(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
 #if defined(SOLARIS) && defined(__GNUC__)
     if (x < -1 || 1 < x) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
 #endif
     z = asin(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_atan(JSContext *cx, uintN argc, jsval *vp)
+math_atan(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = atan(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
-static jsdouble JS_FASTCALL
+static inline jsdouble JS_FASTCALL
 math_atan2_kernel(jsdouble x, jsdouble y)
 {
 #if defined(_MSC_VER)
@@ -210,149 +213,148 @@ math_atan2_kernel(jsdouble x, jsdouble y)
 }
 
 static JSBool
-math_atan2(JSContext *cx, uintN argc, jsval *vp)
+math_atan2(JSContext *cx, uintN argc, Value *vp)
 {
-    jsdouble x, y;
+    jsdouble x, y, z;
 
     if (argc <= 1) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
-    y = js_ValueToNumber(cx, &vp[3]);
-    if (JSVAL_IS_NULL(vp[3]))
+    if (!ValueToNumber(cx, vp[3], &y))
         return JS_FALSE;
-    return js_NewNumberInRootedValue(cx, math_atan2_kernel (x, y), vp);
+    z = math_atan2_kernel(x, y);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
-static inline jsdouble
+static inline jsdouble JS_FASTCALL
 math_ceil_kernel(jsdouble x)
 {
 #ifdef __APPLE__
-    if (x < 0 && x > -1.0) 
+    if (x < 0 && x > -1.0)
         return js_copysign(0, -1);
 #endif
     return ceil(x);
 }
 
-static JSBool
-math_ceil(JSContext *cx, uintN argc, jsval *vp)
+JSBool
+js_math_ceil(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = math_ceil_kernel(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_cos(JSContext *cx, uintN argc, jsval *vp)
+math_cos(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = cos(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_exp(JSContext *cx, uintN argc, jsval *vp)
+math_exp(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
 #ifdef _WIN32
     if (!JSDOUBLE_IS_NaN(x)) {
-        if (x == *cx->runtime->jsPositiveInfinity) {
-            *vp = DOUBLE_TO_JSVAL(cx->runtime->jsPositiveInfinity);
+        if (x == js_PositiveInfinity) {
+            vp->setDouble(js_PositiveInfinity);
             return JS_TRUE;
         }
-        if (x == *cx->runtime->jsNegativeInfinity) {
-            *vp = JSVAL_ZERO;
+        if (x == js_NegativeInfinity) {
+            vp->setInt32(0);
             return JS_TRUE;
         }
     }
 #endif
     z = exp(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
-static JSBool
-math_floor(JSContext *cx, uintN argc, jsval *vp)
+JSBool
+js_math_floor(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = floor(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_log(JSContext *cx, uintN argc, jsval *vp)
+math_log(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
 #if defined(SOLARIS) && defined(__GNUC__)
     if (x < 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
 #endif
     z = log(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
-static JSBool
-math_max(JSContext *cx, uintN argc, jsval *vp)
+JSBool
+js_math_max(JSContext *cx, uintN argc, Value *vp)
 {
-    jsdouble x, z = *cx->runtime->jsNegativeInfinity;
-    jsval *argv;
+    jsdouble x, z = js_NegativeInfinity;
+    Value *argv;
     uintN i;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNegativeInfinity);
+        vp->setDouble(js_NegativeInfinity);
         return JS_TRUE;
     }
     argv = vp + 2;
     for (i = 0; i < argc; i++) {
-        x = js_ValueToNumber(cx, &argv[i]);
-        if (JSVAL_IS_NULL(argv[i]))
+        if (!ValueToNumber(cx, argv[i], &x))
             return JS_FALSE;
         if (JSDOUBLE_IS_NaN(x)) {
-            *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+            vp->setDouble(js_NaN);
             return JS_TRUE;
         }
         if (x == 0 && x == z) {
@@ -362,27 +364,27 @@ math_max(JSContext *cx, uintN argc, jsval *vp)
             z = (x > z) ? x : z;
         }
     }
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
-static JSBool
-math_min(JSContext *cx, uintN argc, jsval *vp)
+JSBool
+js_math_min(JSContext *cx, uintN argc, Value *vp)
 {
-    jsdouble x, z = *cx->runtime->jsPositiveInfinity;
-    jsval *argv;
+    jsdouble x, z = js_PositiveInfinity;
+    Value *argv;
     uintN i;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsPositiveInfinity);
+        vp->setDouble(js_PositiveInfinity);
         return JS_TRUE;
     }
     argv = vp + 2;
     for (i = 0; i < argc; i++) {
-        x = js_ValueToNumber(cx, &argv[i]);
-        if (JSVAL_IS_NULL(argv[i]))
+        if (!ValueToNumber(cx, argv[i], &x))
             return JS_FALSE;
         if (JSDOUBLE_IS_NaN(x)) {
-            *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+            vp->setDouble(js_NaN);
             return JS_TRUE;
         }
         if (x == 0 && x == z) {
@@ -392,130 +394,96 @@ math_min(JSContext *cx, uintN argc, jsval *vp)
             z = (x < z) ? x : z;
         }
     }
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_pow(JSContext *cx, uintN argc, jsval *vp)
+math_pow(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, y, z;
 
     if (argc <= 1) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
-    y = js_ValueToNumber(cx, &vp[3]);
-    if (JSVAL_IS_NULL(vp[3]))
+    if (!ValueToNumber(cx, vp[3], &y))
         return JS_FALSE;
     /*
      * Because C99 and ECMA specify different behavior for pow(),
      * we need to wrap the libm call to make it ECMA compliant.
      */
     if (!JSDOUBLE_IS_FINITE(y) && (x == 1.0 || x == -1.0)) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
     /* pow(x, +-0) is always 1, even for x = NaN. */
     if (y == 0) {
-        *vp = JSVAL_ONE;
+        vp->setInt32(1);
         return JS_TRUE;
     }
     z = pow(x, y);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
+static const int64 RNG_MULTIPLIER = 0x5DEECE66DLL;
+static const int64 RNG_ADDEND = 0xBLL;
+static const int64 RNG_MASK = (1LL << 48) - 1;
+static const jsdouble RNG_DSCALE = jsdouble(1LL << 53);
+
 /*
  * Math.random() support, lifted from java.util.Random.java.
  */
-static void
-random_setSeed(JSRuntime *rt, int64 seed)
+static inline void
+random_setSeed(JSContext *cx, int64 seed)
 {
-    int64 tmp;
-
-    JSLL_I2L(tmp, 1000);
-    JSLL_DIV(seed, seed, tmp);
-    JSLL_XOR(tmp, seed, rt->rngMultiplier);
-    JSLL_AND(rt->rngSeed, tmp, rt->rngMask);
+    cx->rngSeed = (seed ^ RNG_MULTIPLIER) & RNG_MASK;
 }
 
 void
-js_random_init(JSRuntime *rt)
+js_InitRandom(JSContext *cx)
 {
-    int64 tmp, tmp2;
-
-    /* Do at most once. */
-    if (rt->rngInitialized)
-        return;
-    rt->rngInitialized = JS_TRUE;
-
-    /* rt->rngMultiplier = 0x5DEECE66DL */
-    JSLL_ISHL(tmp, 0x5, 32);
-    JSLL_UI2L(tmp2, 0xDEECE66DL);
-    JSLL_OR(rt->rngMultiplier, tmp, tmp2);
-
-    /* rt->rngAddend = 0xBL */
-    JSLL_I2L(rt->rngAddend, 0xBL);
-
-    /* rt->rngMask = (1L << 48) - 1 */
-    JSLL_I2L(tmp, 1);
-    JSLL_SHL(tmp2, tmp, 48);
-    JSLL_SUB(rt->rngMask, tmp2, tmp);
-
-    /* rt->rngDscale = (jsdouble)(1L << 53) */
-    JSLL_SHL(tmp2, tmp, 53);
-    JSLL_L2D(rt->rngDscale, tmp2);
-
-    /* Finally, set the seed from current time. */
-    random_setSeed(rt, PRMJ_Now());
+    /*
+     * Set the seed from current time. Since we have a RNG per context and we often bring
+     * up several contexts at the same time, we xor in some additional values, namely
+     * the context and its successor. We don't just use the context because it might be
+     * possible to reverse engineer the context pointer if one guesses the time right.
+     */
+    random_setSeed(cx,
+                   (PRMJ_Now() / 1000) ^
+                   int64(cx) ^
+                   int64(cx->link.next));
 }
 
-static uint32
-random_next(JSRuntime *rt, int bits)
+static inline uint64
+random_next(JSContext *cx, int bits)
 {
-    int64 nextseed, tmp;
-    uint32 retval;
-
-    JSLL_MUL(nextseed, rt->rngSeed, rt->rngMultiplier);
-    JSLL_ADD(nextseed, nextseed, rt->rngAddend);
-    JSLL_AND(nextseed, nextseed, rt->rngMask);
-    rt->rngSeed = nextseed;
-    JSLL_USHR(tmp, nextseed, 48 - bits);
-    JSLL_L2I(retval, tmp);
-    return retval;
+    uint64 nextseed = cx->rngSeed * RNG_MULTIPLIER;
+    nextseed += RNG_ADDEND;
+    nextseed &= RNG_MASK;
+    cx->rngSeed = nextseed;
+    return nextseed >> (48 - bits);
 }
 
-jsdouble
-js_random_nextDouble(JSRuntime *rt)
+static inline jsdouble
+random_nextDouble(JSContext *cx)
 {
-    int64 tmp, tmp2;
-    jsdouble d;
-
-    JSLL_ISHL(tmp, random_next(rt, 26), 27);
-    JSLL_UI2L(tmp2, random_next(rt, 27));
-    JSLL_ADD(tmp, tmp, tmp2);
-    JSLL_L2D(d, tmp);
-    return d / rt->rngDscale;
+    return jsdouble((random_next(cx, 26) << 27) + random_next(cx, 27)) / RNG_DSCALE;
 }
 
 static JSBool
-math_random(JSContext *cx, uintN argc, jsval *vp)
+math_random(JSContext *cx, uintN argc, Value *vp)
 {
-    JSRuntime *rt;
-    jsdouble z;
-
-    rt = cx->runtime;
-    JS_LOCK_RUNTIME(rt);
-    js_random_init(rt);
-    z = js_random_nextDouble(rt);
-    JS_UNLOCK_RUNTIME(rt);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    jsdouble z = random_nextDouble(cx);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 #if defined _WIN32 && !defined WINCE && _MSC_VER < 1400
-/* Try to work around apparent _copysign bustage in VC6 and VC7. */
+/* Try to work around apparent _copysign bustage in VC7.x. */
 double
 js_copysign(double x, double y)
 {
@@ -529,75 +497,75 @@ js_copysign(double x, double y)
 }
 #endif
 
-static JSBool
-math_round(JSContext *cx, uintN argc, jsval *vp)
+JSBool
+js_math_round(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = js_copysign(floor(x + 0.5), x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setNumber(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_sin(JSContext *cx, uintN argc, jsval *vp)
+math_sin(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = sin(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_sqrt(JSContext *cx, uintN argc, jsval *vp)
+math_sqrt(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = sqrt(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 static JSBool
-math_tan(JSContext *cx, uintN argc, jsval *vp)
+math_tan(JSContext *cx, uintN argc, Value *vp)
 {
     jsdouble x, z;
 
     if (argc == 0) {
-        *vp = DOUBLE_TO_JSVAL(cx->runtime->jsNaN);
+        vp->setDouble(js_NaN);
         return JS_TRUE;
     }
-    x = js_ValueToNumber(cx, &vp[2]);
-    if (JSVAL_IS_NULL(vp[2]))
+    if (!ValueToNumber(cx, vp[2], &x))
         return JS_FALSE;
     z = tan(x);
-    return js_NewNumberInRootedValue(cx, z, vp);
+    vp->setDouble(z);
+    return JS_TRUE;
 }
 
 #if JS_HAS_TOSOURCE
 static JSBool
-math_toSource(JSContext *cx, uintN argc, jsval *vp)
+math_toSource(JSContext *cx, uintN argc, Value *vp)
 {
-    *vp = ATOM_KEY(CLASS_ATOM(cx, Math));
+    vp->setString(ATOM_TO_STRING(CLASS_ATOM(cx, Math)));
     return JS_TRUE;
 }
 #endif
@@ -608,14 +576,13 @@ math_toSource(JSContext *cx, uintN argc, jsval *vp)
 #define MATH_BUILTIN_CFUN_1(name, cfun)                                       \
     static jsdouble FASTCALL math_##name##_tn(jsdouble d) { return cfun(d); } \
     JS_DEFINE_TRCINFO_1(math_##name,                                          \
-        (1, (static, DOUBLE, math_##name##_tn, DOUBLE, 1, 1)))
+        (1, (static, DOUBLE, math_##name##_tn, DOUBLE, 1, nanojit::ACCSET_NONE)))
 
 MATH_BUILTIN_CFUN_1(abs, fabs)
 MATH_BUILTIN_1(atan)
 MATH_BUILTIN_1(sin)
 MATH_BUILTIN_1(cos)
 MATH_BUILTIN_1(sqrt)
-MATH_BUILTIN_1(floor)
 MATH_BUILTIN_1(tan)
 
 static jsdouble FASTCALL
@@ -646,18 +613,16 @@ static jsdouble FASTCALL
 math_exp_tn(JSContext *cx, jsdouble d)
 {
     if (!JSDOUBLE_IS_NaN(d)) {
-        if (d == *cx->runtime->jsPositiveInfinity) {
-            return *cx->runtime->jsPositiveInfinity;
-        }
-        if (d == *cx->runtime->jsNegativeInfinity) {
+        if (d == js_PositiveInfinity)
+            return js_PositiveInfinity;
+        if (d == js_NegativeInfinity)
             return 0.0;
-        }
     }
     return exp(d);
 }
 
 JS_DEFINE_TRCINFO_1(math_exp,
-    (2, (static, DOUBLE, math_exp_tn,  CONTEXT, DOUBLE,  1, 1)))
+    (2, (static, DOUBLE, math_exp_tn,  CONTEXT, DOUBLE,  1, nanojit::ACCSET_NONE)))
 
 #else
 
@@ -716,13 +681,9 @@ math_pow_tn(jsdouble d, jsdouble p)
 }
 
 static jsdouble FASTCALL
-math_random_tn(JSRuntime* rt)
+math_random_tn(JSContext *cx)
 {
-    JS_LOCK_RUNTIME(rt);
-    js_random_init(rt);
-    jsdouble z = js_random_nextDouble(rt);
-    JS_UNLOCK_RUNTIME(rt);
-    return z;
+    return random_nextDouble(cx);
 }
 
 static jsdouble FASTCALL
@@ -737,26 +698,34 @@ math_ceil_tn(jsdouble x)
     return math_ceil_kernel(x);
 }
 
+static jsdouble FASTCALL
+math_floor_tn(jsdouble x)
+{
+    return floor(x);
+}
+
 JS_DEFINE_TRCINFO_1(math_acos,
-    (1, (static, DOUBLE, math_acos_tn, DOUBLE,          1, 1)))
+    (1, (static, DOUBLE, math_acos_tn, DOUBLE,          1, nanojit::ACCSET_NONE)))
 JS_DEFINE_TRCINFO_1(math_asin,
-    (1, (static, DOUBLE, math_asin_tn, DOUBLE,          1, 1)))
+    (1, (static, DOUBLE, math_asin_tn, DOUBLE,          1, nanojit::ACCSET_NONE)))
 JS_DEFINE_TRCINFO_1(math_atan2,
-    (2, (static, DOUBLE, math_atan2_kernel, DOUBLE, DOUBLE, 1, 1)))
+    (2, (static, DOUBLE, math_atan2_kernel, DOUBLE, DOUBLE, 1, nanojit::ACCSET_NONE)))
+JS_DEFINE_TRCINFO_1(js_math_floor,
+    (1, (static, DOUBLE, math_floor_tn, DOUBLE,         1, nanojit::ACCSET_NONE)))
 JS_DEFINE_TRCINFO_1(math_log,
-    (1, (static, DOUBLE, math_log_tn, DOUBLE,           1, 1)))
-JS_DEFINE_TRCINFO_1(math_max,
-    (2, (static, DOUBLE, math_max_tn, DOUBLE, DOUBLE,   1, 1)))
-JS_DEFINE_TRCINFO_1(math_min,
-    (2, (static, DOUBLE, math_min_tn, DOUBLE, DOUBLE,   1, 1)))
+    (1, (static, DOUBLE, math_log_tn, DOUBLE,           1, nanojit::ACCSET_NONE)))
+JS_DEFINE_TRCINFO_1(js_math_max,
+    (2, (static, DOUBLE, math_max_tn, DOUBLE, DOUBLE,   1, nanojit::ACCSET_NONE)))
+JS_DEFINE_TRCINFO_1(js_math_min,
+    (2, (static, DOUBLE, math_min_tn, DOUBLE, DOUBLE,   1, nanojit::ACCSET_NONE)))
 JS_DEFINE_TRCINFO_1(math_pow,
-    (2, (static, DOUBLE, math_pow_tn, DOUBLE, DOUBLE,   1, 1)))
+    (2, (static, DOUBLE, math_pow_tn, DOUBLE, DOUBLE,   1, nanojit::ACCSET_NONE)))
 JS_DEFINE_TRCINFO_1(math_random,
-    (1, (static, DOUBLE, math_random_tn, RUNTIME,       0, 0)))
-JS_DEFINE_TRCINFO_1(math_round,
-    (1, (static, DOUBLE, math_round_tn, DOUBLE,         1, 1)))
-JS_DEFINE_TRCINFO_1(math_ceil,
-    (1, (static, DOUBLE, math_ceil_tn, DOUBLE,          1, 1)))
+    (1, (static, DOUBLE, math_random_tn, CONTEXT,       0, nanojit::ACCSET_STORE_ANY)))
+JS_DEFINE_TRCINFO_1(js_math_round,
+    (1, (static, DOUBLE, math_round_tn, DOUBLE,         1, nanojit::ACCSET_NONE)))
+JS_DEFINE_TRCINFO_1(js_math_ceil,
+    (1, (static, DOUBLE, math_ceil_tn, DOUBLE,          1, nanojit::ACCSET_NONE)))
 
 #endif /* JS_TRACER */
 
@@ -764,24 +733,24 @@ static JSFunctionSpec math_static_methods[] = {
 #if JS_HAS_TOSOURCE
     JS_FN(js_toSource_str,  math_toSource,        0, 0),
 #endif
-    JS_TN("abs",            math_abs,             1, 0, math_abs_trcinfo),
-    JS_TN("acos",           math_acos,            1, 0, math_acos_trcinfo),
-    JS_TN("asin",           math_asin,            1, 0, math_asin_trcinfo),
-    JS_TN("atan",           math_atan,            1, 0, math_atan_trcinfo),
-    JS_TN("atan2",          math_atan2,           2, 0, math_atan2_trcinfo),
-    JS_TN("ceil",           math_ceil,            1, 0, math_ceil_trcinfo),
-    JS_TN("cos",            math_cos,             1, 0, math_cos_trcinfo),
-    JS_TN("exp",            math_exp,             1, 0, math_exp_trcinfo),
-    JS_TN("floor",          math_floor,           1, 0, math_floor_trcinfo),
-    JS_TN("log",            math_log,             1, 0, math_log_trcinfo),
-    JS_TN("max",            math_max,             2, 0, math_max_trcinfo),
-    JS_TN("min",            math_min,             2, 0, math_min_trcinfo),
-    JS_TN("pow",            math_pow,             2, 0, math_pow_trcinfo),
-    JS_TN("random",         math_random,          0, 0, math_random_trcinfo),
-    JS_TN("round",          math_round,           1, 0, math_round_trcinfo),
-    JS_TN("sin",            math_sin,             1, 0, math_sin_trcinfo),
-    JS_TN("sqrt",           math_sqrt,            1, 0, math_sqrt_trcinfo),
-    JS_TN("tan",            math_tan,             1, 0, math_tan_trcinfo),
+    JS_TN("abs",            math_abs,             1, 0, &math_abs_trcinfo),
+    JS_TN("acos",           math_acos,            1, 0, &math_acos_trcinfo),
+    JS_TN("asin",           math_asin,            1, 0, &math_asin_trcinfo),
+    JS_TN("atan",           math_atan,            1, 0, &math_atan_trcinfo),
+    JS_TN("atan2",          math_atan2,           2, 0, &math_atan2_trcinfo),
+    JS_TN("ceil",           js_math_ceil,         1, 0, &js_math_ceil_trcinfo),
+    JS_TN("cos",            math_cos,             1, 0, &math_cos_trcinfo),
+    JS_TN("exp",            math_exp,             1, 0, &math_exp_trcinfo),
+    JS_TN("floor",          js_math_floor,        1, 0, &js_math_floor_trcinfo),
+    JS_TN("log",            math_log,             1, 0, &math_log_trcinfo),
+    JS_TN("max",            js_math_max,          2, 0, &js_math_max_trcinfo),
+    JS_TN("min",            js_math_min,          2, 0, &js_math_min_trcinfo),
+    JS_TN("pow",            math_pow,             2, 0, &math_pow_trcinfo),
+    JS_TN("random",         math_random,          0, 0, &math_random_trcinfo),
+    JS_TN("round",          js_math_round,        1, 0, &js_math_round_trcinfo),
+    JS_TN("sin",            math_sin,             1, 0, &math_sin_trcinfo),
+    JS_TN("sqrt",           math_sqrt,            1, 0, &math_sqrt_trcinfo),
+    JS_TN("tan",            math_tan,             1, 0, &math_tan_trcinfo),
     JS_FS_END
 };
 
@@ -790,7 +759,7 @@ js_InitMathClass(JSContext *cx, JSObject *obj)
 {
     JSObject *Math;
 
-    Math = JS_NewObject(cx, &js_MathClass, NULL, obj);
+    Math = JS_NewObject(cx, Jsvalify(&js_MathClass), NULL, obj);
     if (!Math)
         return NULL;
     if (!JS_DefineProperty(cx, obj, js_Math_str, OBJECT_TO_JSVAL(Math),
