@@ -80,7 +80,7 @@ JS_BEGIN_EXTERN_C
  *                            defined in enclosing scopes, or ultimately not
  *                            defined (free variables, either global property
  *                            references or reference errors).
- *                          pn_argsbody: TOK_ARGSBODY or TOK_LC node
+ *                          pn_tree: TOK_ARGSBODY or TOK_LC node
  *
  * <Statements>
  * TOK_LC       list        pn_head: list of pn_count statements
@@ -194,7 +194,6 @@ JS_BEGIN_EXTERN_C
  *                          pn_kid: primary function, paren, name, object or
  *                                  array literal expressions
  * TOK_USESHARP nullary     pn_num: jsint value of n in #n#
- * TOK_RP       unary       pn_kid: parenthesized expression
  * TOK_NAME,    name        pn_atom: name, string, or object atom
  * TOK_STRING,              pn_op: JSOP_NAME, JSOP_STRING, or JSOP_OBJECT, or
  *                                 JSOP_REGEXP
@@ -291,17 +290,20 @@ struct JSDefinition;
 struct JSParseNode {
     uint32              pn_type:16,     /* TOK_* type, see jsscan.h */
                         pn_op:8,        /* see JSOp enum and jsopcode.tbl */
-                        pn_arity:6,     /* see JSParseNodeArity enum */
+                        pn_arity:5,     /* see JSParseNodeArity enum */
+                        pn_parens:1,    /* this expr was enclosed in parens */
                         pn_used:1,      /* name node is on a use-chain */
                         pn_defn:1;      /* this node is a JSDefinition */
 
 #define PN_OP(pn)    ((JSOp)(pn)->pn_op)
-#define PN_TYPE(pn)  ((JSTokenType)(pn)->pn_type)
+#define PN_TYPE(pn)  ((js::TokenKind)(pn)->pn_type)
 
-    JSTokenPos          pn_pos;         /* two 16-bit pairs here, for 64 bits */
+    js::TokenPos        pn_pos;         /* two 16-bit pairs here, for 64 bits */
     int32               pn_offset;      /* first generated bytecode offset */
     JSParseNode         *pn_next;       /* intrinsic link in parent PN_LIST */
-    JSParseNode         *pn_link;       /* def/use link (alignment freebie) */
+    JSParseNode         *pn_link;       /* def/use link (alignment freebie);
+                                           also links JSFunctionBox::methods
+                                           lists of would-be |this| methods */
     union {
         struct {                        /* list of next-linked nodes */
             JSParseNode *head;          /* first node in list */
@@ -318,7 +320,7 @@ struct JSParseNode {
         struct {                        /* two kids if binary */
             JSParseNode *left;
             JSParseNode *right;
-            jsval       val;            /* switch case value */
+            js::Value   *pval;          /* switch case value */
             uintN       iflags;         /* JSITER_* flags for TOK_FOR node */
         } binary;
         struct {                        /* one kid if unary */
@@ -337,7 +339,7 @@ struct JSParseNode {
                                            base object of TOK_DOT */
                 JSDefinition *lexdef;   /* lexical definition for this use */
             };
-            uint32      cookie;         /* upvar cookie with absolute frame
+            js::UpvarCookie cookie;     /* upvar cookie with absolute frame
                                            level (not relative skip), possibly
                                            in current frame */
             uint32      dflags:12,      /* definition/use flags, see below */
@@ -370,7 +372,7 @@ struct JSParseNode {
 #define pn_kid3         pn_u.ternary.kid3
 #define pn_left         pn_u.binary.left
 #define pn_right        pn_u.binary.right
-#define pn_val          pn_u.binary.val
+#define pn_pval         pn_u.binary.pval
 #define pn_iflags       pn_u.binary.iflags
 #define pn_kid          pn_u.unary.kid
 #define pn_num          pn_u.unary.num
@@ -384,6 +386,23 @@ struct JSParseNode {
 #define pn_dval         pn_u.dval
 #define pn_atom2        pn_u.apair.atom2
 
+protected:
+    void inline init(js::TokenKind type, JSOp op, JSParseNodeArity arity) {
+        pn_type = type;
+        pn_op = op;
+        pn_arity = arity;
+        pn_parens = false;
+        JS_ASSERT(!pn_used);
+        JS_ASSERT(!pn_defn);
+        pn_next = pn_link = NULL;
+    }
+
+    static JSParseNode *create(JSParseNodeArity arity, JSTreeContext *tc);
+
+public:
+    static JSParseNode *newBinaryOrAppend(js::TokenKind tt, JSOp op, JSParseNode *left,
+                                          JSParseNode *right, JSTreeContext *tc);
+
     /*
      * The pn_expr and lexdef members are arms of an unsafe union. Unless you
      * know exactly what you're doing, use only the following methods to access
@@ -397,7 +416,7 @@ struct JSParseNode {
     }
 
     JSDefinition *lexdef() const {
-        JS_ASSERT(pn_used);
+        JS_ASSERT(pn_used || isDeoptimized());
         JS_ASSERT(pn_arity == PN_NAME);
         return pn_lexdef;
     }
@@ -417,6 +436,9 @@ struct JSParseNode {
 #define PND_PLACEHOLDER 0x80            /* placeholder definition for lexdep */
 #define PND_FUNARG     0x100            /* downward or upward funarg usage */
 #define PND_BOUND      0x200            /* bound to a stack or global slot */
+#define PND_DEOPTIMIZED 0x400           /* former pn_used name node, pn_lexdef
+                                           still valid, but this use no longer
+                                           optimizable via an upvar opcode */
 
 /* Flags to propagate from uses to definition. */
 #define PND_USE2DEF_FLAGS (PND_ASSIGNED | PND_FUNARG)
@@ -443,18 +465,15 @@ struct JSParseNode {
 
     uintN frameLevel() const {
         JS_ASSERT(pn_arity == PN_FUNC || pn_arity == PN_NAME);
-        return UPVAR_FRAME_SKIP(pn_cookie);
+        return pn_cookie.level();
     }
 
     uintN frameSlot() const {
         JS_ASSERT(pn_arity == PN_FUNC || pn_arity == PN_NAME);
-        return UPVAR_FRAME_SLOT(pn_cookie);
+        return pn_cookie.slot();
     }
 
-    bool test(uintN flag) const {
-        JS_ASSERT(pn_arity == PN_FUNC || pn_arity == PN_NAME);
-        return !!(pn_dflags & flag);
-    }
+    inline bool test(uintN flag) const;
 
     bool isLet() const          { return test(PND_LET); }
     bool isConst() const        { return test(PND_CONST); }
@@ -462,10 +481,11 @@ struct JSParseNode {
     bool isTopLevel() const     { return test(PND_TOPLEVEL); }
     bool isBlockChild() const   { return test(PND_BLOCKCHILD); }
     bool isPlaceholder() const  { return test(PND_PLACEHOLDER); }
+    bool isDeoptimized() const  { return test(PND_DEOPTIMIZED); }
+    bool isAssigned() const     { return test(PND_ASSIGNED); }
+    bool isFunArg() const       { return test(PND_FUNARG); }
 
     /* Defined below, see after struct JSDefinition. */
-    bool isAssigned() const;
-    bool isFunArg() const;
     void setFunArg();
 
     void become(JSParseNode *pn2);
@@ -473,9 +493,43 @@ struct JSParseNode {
 
     /* True if pn is a parsenode representing a literal constant. */
     bool isLiteral() const {
-        return PN_TYPE(this) == TOK_NUMBER ||
-               PN_TYPE(this) == TOK_STRING ||
-               (PN_TYPE(this) == TOK_PRIMARY && PN_OP(this) != JSOP_THIS);
+        return PN_TYPE(this) == js::TOK_NUMBER ||
+               PN_TYPE(this) == js::TOK_STRING ||
+               (PN_TYPE(this) == js::TOK_PRIMARY && PN_OP(this) != JSOP_THIS);
+    }
+
+    /*
+     * True if this statement node could be a member of a Directive
+     * Prologue.  Note that the prologue may contain strings that
+     * cannot themselves be directives; that's a stricter test.
+     * If Statement begins to simplify trees into this form, then
+     * we'll need additional flags that we can test here.
+     */
+    bool isDirectivePrologueMember() const {
+        if (PN_TYPE(this) == js::TOK_SEMI) {
+            JS_ASSERT(pn_arity == PN_UNARY);
+            JSParseNode *kid = pn_kid;
+            return kid && PN_TYPE(kid) == js::TOK_STRING && !kid->pn_parens;
+        }
+        return false;
+    }
+
+    /*
+     * True if this node, known to be a Directive Prologue member,
+     * could be a directive itself.
+     */
+    bool isDirective() const {
+        JS_ASSERT(isDirectivePrologueMember());
+        JSParseNode *kid = pn_kid;
+        JSString *str = ATOM_TO_STRING(kid->pn_atom);
+
+        /*
+         * Directives must contain no EscapeSequences or LineContinuations.
+         * If the string's length in the source code is its length as a value,
+         * accounting for the quotes, then it qualifies.
+         */
+        return (pn_pos.begin.lineno == pn_pos.end.lineno &&
+                pn_pos.begin.index + str->length() + 2 == pn_pos.end.index);
     }
 
     /*
@@ -514,6 +568,64 @@ struct JSParseNode {
     }
 };
 
+namespace js {
+
+struct NullaryNode : public JSParseNode {
+    static inline NullaryNode *create(JSTreeContext *tc) {
+        return (NullaryNode *)JSParseNode::create(PN_NULLARY, tc);
+    }
+};
+
+struct UnaryNode : public JSParseNode {
+    static inline UnaryNode *create(JSTreeContext *tc) {
+        return (UnaryNode *)JSParseNode::create(PN_UNARY, tc);
+    }
+};
+
+struct BinaryNode : public JSParseNode {
+    static inline BinaryNode *create(JSTreeContext *tc) {
+        return (BinaryNode *)JSParseNode::create(PN_BINARY, tc);
+    }
+};
+
+struct TernaryNode : public JSParseNode {
+    static inline TernaryNode *create(JSTreeContext *tc) {
+        return (TernaryNode *)JSParseNode::create(PN_TERNARY, tc);
+    }
+};
+
+struct ListNode : public JSParseNode {
+    static inline ListNode *create(JSTreeContext *tc) {
+        return (ListNode *)JSParseNode::create(PN_LIST, tc);
+    }
+};
+
+struct FunctionNode : public JSParseNode {
+    static inline FunctionNode *create(JSTreeContext *tc) {
+        return (FunctionNode *)JSParseNode::create(PN_FUNC, tc);
+    }
+};
+
+struct NameNode : public JSParseNode {
+    static NameNode *create(JSAtom *atom, JSTreeContext *tc);
+
+    void inline initCommon(JSTreeContext *tc);
+};
+
+struct NameSetNode : public JSParseNode {
+    static inline NameSetNode *create(JSTreeContext *tc) {
+        return (NameSetNode *)JSParseNode::create(PN_NAMESET, tc);
+    }
+};
+
+struct LexicalScopeNode : public JSParseNode {
+    static inline LexicalScopeNode *create(JSTreeContext *tc) {
+        return (LexicalScopeNode *)JSParseNode::create(PN_NAME, tc);
+    }
+};
+
+} /* namespace js */
+
 /*
  * JSDefinition is a degenerate subtype of the PN_FUNC and PN_NAME variants of
  * JSParseNode, allocated only for function, var, const, and let declarations
@@ -551,12 +663,11 @@ struct JSParseNode {
  *               pn = allocate a PN_NAME JSParseNode;
  *           } else {                                       // defining
  *               dn = lookup x in tc->lexdeps;
- *               if (dn) {                                  // use before def
+ *               if (dn)                                    // use before def
  *                   remove x from tc->lexdeps;
- *               } else {                                   // def before use
+ *               else                                       // def before use
  *                   dn = allocate a PN_NAME JSDefinition;
- *                   map x to dn via tc->decls;
- *               }
+ *               map x to dn via tc->decls;
  *               pn = dn;
  *           }
  *           insert pn into its parent TOK_VAR list;
@@ -638,14 +749,14 @@ struct JSDefinition : public JSParseNode
      * We store definition pointers in PN_NAMESET JSAtomLists in the AST, but
      * due to redefinition these nodes may become uses of other definitions.
      * This is unusual, so we simply chase the pn_lexdef link to find the final
-     * definition node. See methods called from JSCompiler::analyzeFunctions.
+     * definition node. See methods called from Parser::analyzeFunctions.
      *
      * FIXME: MakeAssignment mutates for want of a parent link...
      */
     JSDefinition *resolve() {
         JSParseNode *pn = this;
         while (!pn->pn_defn) {
-            if (pn->pn_type == TOK_ASSIGN) {
+            if (pn->pn_type == js::TOK_ASSIGN) {
                 pn = pn->pn_left;
                 continue;
             }
@@ -654,30 +765,9 @@ struct JSDefinition : public JSParseNode
         return (JSDefinition *) pn;
     }
 
-    bool test(uintN flag) const {
-        JS_ASSERT(pn_defn);
-        if (pn_dflags & flag)
-            return true;
-#ifdef DEBUG
-        for (JSParseNode *pn = dn_uses; pn; pn = pn->pn_link) {
-            JS_ASSERT(!pn->pn_defn);
-            JS_ASSERT(!(pn->pn_dflags & flag));
-        }
-#endif
-        return false;
-    }
-
-    bool isAssigned() const {
-        return test(PND_ASSIGNED);
-    }
-
-    bool isFunArg() const {
-        return test(PND_FUNARG);
-    }
-
     bool isFreeVar() const {
         JS_ASSERT(pn_defn);
-        return pn_cookie == FREE_UPVAR_COOKIE || test(PND_GVAR);
+        return pn_cookie.isFree() || test(PND_GVAR);
     }
 
     // Grr, windows.h or something under it #defines CONST...
@@ -691,9 +781,9 @@ struct JSDefinition : public JSParseNode
     static const char *kindString(Kind kind);
 
     Kind kind() {
-        if (PN_TYPE(this) == TOK_FUNCTION)
+        if (PN_TYPE(this) == js::TOK_FUNCTION)
             return FUNCTION;
-        JS_ASSERT(PN_TYPE(this) == TOK_NAME);
+        JS_ASSERT(PN_TYPE(this) == js::TOK_NAME);
         if (PN_OP(this) == JSOP_NOP)
             return UNKNOWN;
         if (PN_OP(this) == JSOP_GETARG)
@@ -706,28 +796,19 @@ struct JSDefinition : public JSParseNode
     }
 };
 
-/*
- * These two are overridden by JSDefinition and we cannot afford virtual
- * methods -- so we use the mighty 'if' statement!
- */
 inline bool
-JSParseNode::isAssigned() const
+JSParseNode::test(uintN flag) const
 {
+    JS_ASSERT(pn_defn || pn_arity == PN_FUNC || pn_arity == PN_NAME);
 #ifdef DEBUG
-    if (pn_defn)
-        return ((JSDefinition *)this)->isAssigned();
+    if ((flag & (PND_ASSIGNED | PND_FUNARG)) && pn_defn && !(pn_dflags & flag)) {
+        for (JSParseNode *pn = ((JSDefinition *) this)->dn_uses; pn; pn = pn->pn_link) {
+            JS_ASSERT(!pn->pn_defn);
+            JS_ASSERT(!(pn->pn_dflags & flag));
+        }
+    }
 #endif
-    return test(PND_ASSIGNED);
-}
-
-inline bool
-JSParseNode::isFunArg() const
-{
-#ifdef DEBUG
-    if (pn_defn)
-        return ((JSDefinition *)this)->isFunArg();
-#endif
-    return test(PND_FUNARG);
+    return !!(pn_dflags & flag);
 }
 
 inline void
@@ -763,10 +844,29 @@ struct JSFunctionBox : public JSObjectBox
     JSFunctionBox       *siblings;
     JSFunctionBox       *kids;
     JSFunctionBox       *parent;
+    JSParseNode         *methods;               /* would-be methods set on this;
+                                                   these nodes are linked via
+                                                   pn_link, since lambdas are
+                                                   neither definitions nor uses
+                                                   of a binding */
     uint32              queued:1,
                         inLoop:1,               /* in a loop in parent function */
-                        level:JSFB_LEVEL_BITS,
-                        tcflags:16;
+                        level:JSFB_LEVEL_BITS;
+    uint32              tcflags;
+
+    bool joinable() const;
+
+    /*
+     * Unbrand an object being initialized or constructed if any method cannot
+     * be joined to one compiler-created null closure shared among N different
+     * closure environments.
+     *
+     * We despecialize from caching function objects, caching slots or sprops
+     * instead, because an unbranded object may still have joined methods (for
+     * which sprop->isMethod), since PropertyCache::fill gives precedence to
+     * joined methods over branded methods.
+     */
+    bool shouldUnbrand(uintN methods, uintN slowMethods) const;
 };
 
 struct JSFunctionBoxQueue {
@@ -808,35 +908,51 @@ struct JSFunctionBoxQueue {
 
 #define NUM_TEMP_FREELISTS      6U      /* 32 to 2048 byte size classes (32 bit) */
 
-struct JSCompiler {
-    JSContext           *context;
+typedef struct BindData BindData;
+
+namespace js {
+
+struct Parser : private js::AutoGCRooter
+{
+    JSContext           * const context; /* FIXME Bug 551291: use AutoGCRooter::context? */
     JSAtomListElement   *aleFreeList;
     void                *tempFreeList[NUM_TEMP_FREELISTS];
-    JSTokenStream       tokenStream;
+    js::TokenStream     tokenStream;
     void                *tempPoolMark;  /* initial JSContext.tempPool mark */
     JSPrincipals        *principals;    /* principals associated with source */
-    JSStackFrame        *callerFrame;   /* scripted caller frame for eval and dbgapi */
+    JSStackFrame *const callerFrame;    /* scripted caller frame for eval and dbgapi */
+    JSObject     *const callerVarObj;   /* callerFrame's varObj */
     JSParseNode         *nodeList;      /* list of recyclable parse-node structs */
     uint32              functionCount;  /* number of functions in current unit */
     JSObjectBox         *traceListHead; /* list of parsed object for GC tracing */
-    JSTempValueRooter   tempRoot;       /* root to trace traceListHead */
+    JSTreeContext       *tc;            /* innermost tree context (stack-allocated) */
 
-    JSCompiler(JSContext *cx, JSPrincipals *prin = NULL, JSStackFrame *cfp = NULL)
-      : context(cx), aleFreeList(NULL), principals(NULL), callerFrame(cfp),
-        nodeList(NULL), functionCount(0), traceListHead(NULL)
+    /* Root atoms and objects allocated for the parsed tree. */
+    js::AutoKeepAtoms   keepAtoms;
+
+    Parser(JSContext *cx, JSPrincipals *prin = NULL, JSStackFrame *cfp = NULL)
+      : js::AutoGCRooter(cx, PARSER), context(cx),
+        aleFreeList(NULL), tokenStream(cx), principals(NULL), callerFrame(cfp),
+        callerVarObj(cfp ? cfp->varobj(cx->containingSegment(cfp)) : NULL),
+        nodeList(NULL), functionCount(0), traceListHead(NULL), tc(NULL),
+        keepAtoms(cx->runtime)
     {
-        memset(tempFreeList, 0, sizeof tempFreeList);
+        js::PodArrayZero(tempFreeList);
         setPrincipals(prin);
         JS_ASSERT_IF(cfp, cfp->script);
     }
 
-    ~JSCompiler();
+    ~Parser();
+
+    friend void js::AutoGCRooter::trace(JSTracer *trc);
+    friend struct ::JSTreeContext;
+    friend struct Compiler;
 
     /*
-     * Initialize a compiler. Parameters are passed on to js_InitTokenStream.
+     * Initialize a parser. Parameters are passed on to init tokenStream.
      * The compiler owns the arena pool "tops-of-stack" space above the current
      * JSContext.tempPool mark. This means you cannot allocate from tempPool
-     * and save the pointer beyond the next JSCompiler destructor invocation.
+     * and save the pointer beyond the next Parser destructor invocation.
      */
     bool init(const jschar *base, size_t length,
               FILE *fp, const char *filename, uintN lineno);
@@ -870,12 +986,111 @@ struct JSCompiler {
      * starting at funbox, recursively walking its kids, then following its
      * siblings, their kids, etc.
      */
-    bool analyzeFunctions(JSFunctionBox *funbox, uint16& tcflags);
+    bool analyzeFunctions(JSFunctionBox *funbox, uint32& tcflags);
     bool markFunArgs(JSFunctionBox *funbox, uintN tcflags);
-    void setFunctionKinds(JSFunctionBox *funbox, uint16& tcflags);
+    void setFunctionKinds(JSFunctionBox *funbox, uint32& tcflags);
 
     void trace(JSTracer *trc);
 
+    /*
+     * Report a parse (compile) error.
+     */
+    inline bool reportErrorNumber(JSParseNode *pn, uintN flags, uintN errorNumber, ...);
+
+private:
+    /*
+     * JS parsers, from lowest to highest precedence.
+     *
+     * Each parser must be called during the dynamic scope of a JSTreeContext
+     * object, pointed to by this->tc.
+     *
+     * Each returns a parse node tree or null on error.
+     */
+    JSParseNode *functionStmt();
+    JSParseNode *functionExpr();
+    JSParseNode *statements();
+    JSParseNode *statement();
+    JSParseNode *variables(bool inLetHead);
+    JSParseNode *expr();
+    JSParseNode *assignExpr();
+    JSParseNode *condExpr();
+    JSParseNode *orExpr();
+    JSParseNode *andExpr();
+    JSParseNode *bitOrExpr();
+    JSParseNode *bitXorExpr();
+    JSParseNode *bitAndExpr();
+    JSParseNode *eqExpr();
+    JSParseNode *relExpr();
+    JSParseNode *shiftExpr();
+    JSParseNode *addExpr();
+    JSParseNode *mulExpr();
+    JSParseNode *unaryExpr();
+    JSParseNode *memberExpr(JSBool allowCallSyntax);
+    JSParseNode *primaryExpr(js::TokenKind tt, JSBool afterDot);
+    JSParseNode *parenExpr(JSParseNode *pn1, JSBool *genexp);
+
+    /*
+     * Additional JS parsers.
+     */
+    bool recognizeDirectivePrologue(JSParseNode *pn);
+    JSParseNode *functionBody();
+    JSParseNode *functionDef(uintN lambda, bool namePermitted);
+    JSParseNode *condition();
+    JSParseNode *comprehensionTail(JSParseNode *kid, uintN blockid,
+                                   js::TokenKind type = js::TOK_SEMI, JSOp op = JSOP_NOP);
+    JSParseNode *generatorExpr(JSParseNode *pn, JSParseNode *kid);
+    JSBool argumentList(JSParseNode *listNode);
+    JSParseNode *bracketedExpr();
+    JSParseNode *letBlock(JSBool statement);
+    JSParseNode *returnOrYield(bool useAssignExpr);
+    JSParseNode *destructuringExpr(BindData *data, js::TokenKind tt);
+
+#if JS_HAS_XML_SUPPORT
+    JSParseNode *endBracketedExpr();
+
+    JSParseNode *propertySelector();
+    JSParseNode *qualifiedSuffix(JSParseNode *pn);
+    JSParseNode *qualifiedIdentifier();
+    JSParseNode *attributeIdentifier();
+    JSParseNode *xmlExpr(JSBool inTag);
+    JSParseNode *xmlAtomNode();
+    JSParseNode *xmlNameExpr();
+    JSParseNode *xmlTagContent(js::TokenKind tagtype, JSAtom **namep);
+    JSBool xmlElementContent(JSParseNode *pn);
+    JSParseNode *xmlElementOrList(JSBool allowList);
+    JSParseNode *xmlElementOrListRoot(JSBool allowList);
+#endif /* JS_HAS_XML_SUPPORT */
+};
+
+inline bool
+Parser::reportErrorNumber(JSParseNode *pn, uintN flags, uintN errorNumber, ...)
+{
+    va_list args;
+    va_start(args, errorNumber);
+    bool result = tokenStream.reportCompileErrorNumberVA(pn, flags, errorNumber, args);
+    va_end(args);
+    return result;
+}
+
+struct Compiler
+{
+    Parser parser;
+
+    Compiler(JSContext *cx, JSPrincipals *prin = NULL, JSStackFrame *cfp = NULL)
+      : parser(cx, prin, cfp)
+    {
+    }
+
+    /*
+     * Initialize a compiler. Parameters are passed on to init parser.
+     */
+    inline bool
+    init(const jschar *base, size_t length,
+         FILE *fp, const char *filename, uintN lineno)
+    {
+        return parser.init(base, length, fp, filename, lineno);
+    }
+
     static bool
     compileFunctionBody(JSContext *cx, JSFunction *fun, JSPrincipals *principals,
                         const jschar *chars, size_t length,
@@ -886,13 +1101,16 @@ struct JSCompiler {
                   JSPrincipals *principals, uint32 tcflags,
                   const jschar *chars, size_t length,
                   FILE *file, const char *filename, uintN lineno,
-                  JSString *source = NULL);
+                  JSString *source = NULL,
+                  unsigned staticLevel = 0);
 };
 
+} /* namespace js */
+
 /*
- * Convenience macro to access JSCompiler.tokenStream as a pointer.
+ * Convenience macro to access Parser.tokenStream as a pointer.
  */
-#define TS(jsc) (&(jsc)->tokenStream)
+#define TS(p) (&(p)->tokenStream)
 
 extern JSBool
 js_FoldConstants(JSContext *cx, JSParseNode *pn, JSTreeContext *tc,
