@@ -40,7 +40,7 @@
 #include "xpcprivate.h"
 #include "nsDOMError.h"
 #include "jsdbgapi.h"
-#include "jscntxt.h"  // For JSAutoTempValueRooter.
+#include "jscntxt.h"  // For js::AutoValueRooter.
 #include "XPCNativeWrapper.h"
 #include "XPCWrapper.h"
 
@@ -48,36 +48,36 @@
 // be safely injected into untrusted code.
 
 static JSBool
-XPC_SOW_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+XPC_SOW_AddProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
-XPC_SOW_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+XPC_SOW_DelProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
-XPC_SOW_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+XPC_SOW_GetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
-XPC_SOW_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp);
+XPC_SOW_SetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp);
 
 static JSBool
 XPC_SOW_Enumerate(JSContext *cx, JSObject *obj);
 
 static JSBool
-XPC_SOW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+XPC_SOW_NewResolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
                    JSObject **objp);
 
 static JSBool
 XPC_SOW_Convert(JSContext *cx, JSObject *obj, JSType type, jsval *vp);
 
 static JSBool
-XPC_SOW_CheckAccess(JSContext *cx, JSObject *obj, jsval id, JSAccessMode mode,
+XPC_SOW_CheckAccess(JSContext *cx, JSObject *obj, jsid id, JSAccessMode mode,
                     jsval *vp);
 
 static JSBool
-XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, jsval v, JSBool *bp);
+XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, const jsval *valp, JSBool *bp);
 
 static JSBool
-XPC_SOW_Equality(JSContext *cx, JSObject *obj, jsval v, JSBool *bp);
+XPC_SOW_Equality(JSContext *cx, JSObject *obj, const jsval *valp, JSBool *bp);
 
 static JSObject *
 XPC_SOW_Iterator(JSContext *cx, JSObject *obj, JSBool keysonly);
@@ -85,86 +85,101 @@ XPC_SOW_Iterator(JSContext *cx, JSObject *obj, JSBool keysonly);
 static JSObject *
 XPC_SOW_WrappedObject(JSContext *cx, JSObject *obj);
 
-JSExtendedClass sXPC_SOW_JSClass = {
-  // JSClass (JSExtendedClass.base) initialization
-  { "SystemOnlyWrapper",
-    JSCLASS_NEW_RESOLVE | JSCLASS_IS_EXTENDED |
-    JSCLASS_HAS_RESERVED_SLOTS(XPCWrapper::sNumSlots),
-    XPC_SOW_AddProperty, XPC_SOW_DelProperty,
-    XPC_SOW_GetProperty, XPC_SOW_SetProperty,
-    XPC_SOW_Enumerate,   (JSResolveOp)XPC_SOW_NewResolve,
-    XPC_SOW_Convert,     nsnull,
-    nsnull,              XPC_SOW_CheckAccess,
-    nsnull,              nsnull,
-    nsnull,              XPC_SOW_HasInstance,
-    nsnull,              nsnull
-  },
-
-  // JSExtendedClass initialization
-  XPC_SOW_Equality,
-  nsnull,             // outerObject
-  nsnull,             // innerObject
-  XPC_SOW_Iterator,
-  XPC_SOW_WrappedObject,
-  JSCLASS_NO_RESERVED_MEMBERS
-};
-
-static JSBool
-XPC_SOW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                 jsval *rval);
+using namespace XPCWrapper;
 
 // Throws an exception on context |cx|.
 static inline JSBool
 ThrowException(nsresult rv, JSContext *cx)
 {
-  return XPCWrapper::ThrowException(rv, cx);
+  return DoThrowException(rv, cx);
 }
 
-// Like GetWrappedObject, but works on other types of wrappers, too.
-// TODO Move to XPCWrapper?
-static inline JSObject *
-GetWrappedJSObject(JSContext *cx, JSObject *obj)
-{
-  JSClass *clasp = STOBJ_GET_CLASS(obj);
-  if (!(clasp->flags & JSCLASS_IS_EXTENDED)) {
-    return obj;
-  }
+static const char prefix[] = "chrome://global/";
 
-  JSExtendedClass *xclasp = (JSExtendedClass *)clasp;
-  if (!xclasp->wrappedObject) {
-    return obj;
-  }
+namespace SystemOnlyWrapper {
 
-  return xclasp->wrappedObject(cx, obj);
-}
+js::Class SOWClass = {
+    "SystemOnlyWrapper",
+    JSCLASS_NEW_RESOLVE |
+    JSCLASS_HAS_RESERVED_SLOTS(XPCWrapper::sNumSlots),
+    js::Valueify(XPC_SOW_AddProperty),
+    js::Valueify(XPC_SOW_DelProperty),
+    js::Valueify(XPC_SOW_GetProperty),
+    js::Valueify(XPC_SOW_SetProperty),
+    XPC_SOW_Enumerate,
+    (JSResolveOp)XPC_SOW_NewResolve,
+    js::Valueify(XPC_SOW_Convert),
+    nsnull,   // finalize
+    nsnull,   // reserved0
+    js::Valueify(XPC_SOW_CheckAccess),
+    nsnull,   // call
+    nsnull,   // construct
+    nsnull,   // xdrObject
+    js::Valueify(XPC_SOW_HasInstance),
+    nsnull,   // mark
 
-// Get the (possibly non-existant) SOW off of an object
-static inline
-JSObject *
-GetWrapper(JSObject *obj)
-{
-  while (STOBJ_GET_CLASS(obj) != &sXPC_SOW_JSClass.base) {
-    obj = STOBJ_GET_PROTO(obj);
-    if (!obj) {
-      break;
+    // ClassExtension
+    {
+      js::Valueify(XPC_SOW_Equality),
+      nsnull, // outerObject
+      nsnull, // innerObject
+      XPC_SOW_Iterator,
+      XPC_SOW_WrappedObject
     }
+};
+
+JSBool
+WrapObject(JSContext *cx, JSObject *parent, jsval v, jsval *vp)
+{
+  // Slim wrappers don't expect to be wrapped, so morph them to fat wrappers
+  // if we're about to wrap one.
+  JSObject *innerObj = JSVAL_TO_OBJECT(v);
+  if (IS_SLIM_WRAPPER(innerObj) && !MorphSlimWrapper(cx, innerObj)) {
+    return ThrowException(NS_ERROR_FAILURE, cx);
   }
 
-  return obj;
+  JSObject *wrapperObj =
+    JS_NewObjectWithGivenProto(cx, js::Jsvalify(&SOWClass), NULL, parent);
+  if (!wrapperObj) {
+    return JS_FALSE;
+  }
+
+  *vp = OBJECT_TO_JSVAL(wrapperObj);
+  js::AutoObjectRooter tvr(cx, wrapperObj);
+
+  if (!JS_SetReservedSlot(cx, wrapperObj, sWrappedObjSlot, v) ||
+      !JS_SetReservedSlot(cx, wrapperObj, sFlagsSlot, JSVAL_ZERO)) {
+    return JS_FALSE;
+  }
+
+  return JS_TRUE;
 }
 
-static inline
-JSObject *
-GetWrappedObject(JSContext *cx, JSObject *wrapper)
+JSBool
+MakeSOW(JSContext *cx, JSObject *obj)
 {
-  return XPCWrapper::UnwrapGeneric(cx, &sXPC_SOW_JSClass, wrapper);
+#ifdef DEBUG
+  {
+    js::Class *clasp = obj->getClass();
+    NS_ASSERTION(clasp != &SystemOnlyWrapper::SOWClass &&
+                 clasp != &XPCCrossOriginWrapper::XOWClass,
+                 "bad call");
+  }
+#endif
+
+  jsval flags;
+  return JS_GetReservedSlot(cx, obj, sFlagsSlot, &flags) &&
+         JS_SetReservedSlot(cx, obj, sFlagsSlot,
+                            INT_TO_JSVAL(JSVAL_TO_INT(flags) | FLAG_SOW));
 }
 
+
 // If you change this code, change also nsContentUtils::CanAccessNativeAnon()!
 JSBool
-AllowedToAct(JSContext *cx, jsval idval)
+AllowedToAct(JSContext *cx, jsid id)
 {
-  nsIScriptSecurityManager *ssm = XPCWrapper::GetSecurityManager();
+  // TODO bug 508928: Refactor this with the XOW security checking code.
+  nsIScriptSecurityManager *ssm = GetSecurityManager();
   if (!ssm) {
     return JS_TRUE;
   }
@@ -185,21 +200,19 @@ AllowedToAct(JSContext *cx, jsval idval)
     // Some code is running, we can't make the assumption, as above, but we
     // can't use a native frame, so clear fp.
     fp = nsnull;
+  } else if (!fp->script) {
+    fp = nsnull;
   }
 
-  void *annotation = fp ? JS_GetFrameAnnotation(cx, fp) : nsnull;
   PRBool privileged;
-  if (NS_SUCCEEDED(principal->IsCapabilityEnabled("UniversalXPConnect",
-                                                  annotation,
-                                                  &privileged)) &&
+  if (NS_SUCCEEDED(ssm->IsSystemPrincipal(principal, &privileged)) &&
       privileged) {
-    // UniversalXPConnect things are allowed to touch us.
+    // Chrome things are allowed to touch us.
     return JS_TRUE;
   }
 
   // XXX HACK EWW! Allow chrome://global/ access to these things, even
   // if they've been cloned into less privileged contexts.
-  static const char prefix[] = "chrome://global/";
   const char *filename;
   if (fp &&
       (filename = fp->script->filename) &&
@@ -207,12 +220,19 @@ AllowedToAct(JSContext *cx, jsval idval)
     return JS_TRUE;
   }
 
-  if (JSVAL_IS_VOID(idval)) {
+  // Before we throw, check for UniversalXPConnect.
+  nsresult rv = ssm->IsCapabilityEnabled("UniversalXPConnect", &privileged);
+  if (NS_SUCCEEDED(rv) && privileged) {
+    return JS_TRUE;
+  }
+
+  if (JSID_IS_VOID(id)) {
     ThrowException(NS_ERROR_XPC_SECURITY_MANAGER_VETO, cx);
   } else {
     // TODO Localize me?
-    JSString *str = JS_ValueToString(cx, idval);
-    if (str) {
+    jsval idval;
+    JSString *str;
+    if (JS_IdToValue(cx, id, &idval) && (str = JS_ValueToString(cx, idval))) {
       JS_ReportError(cx, "Permission denied to access property '%hs' from a non-chrome context",
                      JS_GetStringChars(str));
     }
@@ -221,11 +241,71 @@ AllowedToAct(JSContext *cx, jsval idval)
   return JS_FALSE;
 }
 
+JSBool
+CheckFilename(JSContext *cx, jsid id, JSStackFrame *fp)
+{
+  const char *filename;
+  if (fp &&
+      (filename = fp->script->filename) &&
+      !strncmp(filename, prefix, NS_ARRAY_LENGTH(prefix) - 1)) {
+    return JS_TRUE;
+  }
+
+  if (JSID_IS_VOID(id)) {
+    ThrowException(NS_ERROR_XPC_SECURITY_MANAGER_VETO, cx);
+  } else {
+    jsval idval;
+    JSString *str;
+    if (JS_IdToValue(cx, id, &idval) && (str = JS_ValueToString(cx, idval))) {
+      JS_ReportError(cx, "Permission denied to access property '%hs' from a non-chrome context",
+                     JS_GetStringChars(str));
+    }
+  }
+
+  return JS_FALSE;
+}
+
+} // namespace SystemOnlyWrapper
+
+using namespace SystemOnlyWrapper;
+
+// Like GetWrappedObject, but works on other types of wrappers, too.
+// TODO Move to XPCWrapper?
+static inline JSObject *
+GetWrappedJSObject(JSContext *cx, JSObject *obj)
+{
+  if (JSObjectOp op = obj->getClass()->ext.wrappedObject)
+    return op(cx, obj);
+  return obj;
+}
+
+// Get the (possibly nonexistent) SOW off of an object
+static inline
+JSObject *
+GetWrapper(JSObject *obj)
+{
+  while (obj->getClass() != &SOWClass) {
+    obj = obj->getProto();
+    if (!obj) {
+      break;
+    }
+  }
+
+  return obj;
+}
+
+static inline
+JSObject *
+GetWrappedObject(JSContext *cx, JSObject *wrapper)
+{
+  return UnwrapGeneric(cx, &SOWClass, wrapper);
+}
+
 static JSBool
 XPC_SOW_FunctionWrapper(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
                         jsval *rval)
 {
-  if (!AllowedToAct(cx, JSVAL_VOID)) {
+  if (!AllowedToAct(cx, JSID_VOID)) {
     return JS_FALSE;
   }
 
@@ -248,8 +328,7 @@ XPC_SOW_FunctionWrapper(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
 
   JSObject *funObj = JSVAL_TO_OBJECT(argv[-2]);
   jsval funToCall;
-  if (!JS_GetReservedSlot(cx, funObj, XPCWrapper::eWrappedFunctionSlot,
-                          &funToCall)) {
+  if (!JS_GetReservedSlot(cx, funObj, eWrappedFunctionSlot, &funToCall)) {
     return JS_FALSE;
   }
 
@@ -282,7 +361,7 @@ XPC_SOW_WrapFunction(JSContext *cx, JSObject *outerObj, JSObject *funobj,
   *rval = OBJECT_TO_JSVAL(funWrapperObj);
 
   return JS_SetReservedSlot(cx, funWrapperObj,
-                            XPCWrapper::eWrappedFunctionSlot,
+                            eWrappedFunctionSlot,
                             funobjVal);
 }
 
@@ -310,12 +389,12 @@ XPC_SOW_RewrapValue(JSContext *cx, JSObject *wrapperObj, jsval *vp)
     if (native == XPC_SOW_FunctionWrapper) {
       // If this is a system function wrapper, make sure its ours, otherwise,
       // its prototype could come from the wrong scope.
-      if (STOBJ_GET_PROTO(wrapperObj) == STOBJ_GET_PARENT(obj)) {
+      if (wrapperObj->getProto() == obj->getParent()) {
         return JS_TRUE;
       }
 
       // It isn't ours, rewrap the wrapped function.
-      if (!JS_GetReservedSlot(cx, obj, XPCWrapper::eWrappedFunctionSlot, &v)) {
+      if (!JS_GetReservedSlot(cx, obj, eWrappedFunctionSlot, &v)) {
         return JS_FALSE;
       }
       obj = JSVAL_TO_OBJECT(v);
@@ -324,12 +403,12 @@ XPC_SOW_RewrapValue(JSContext *cx, JSObject *wrapperObj, jsval *vp)
     return XPC_SOW_WrapFunction(cx, wrapperObj, obj, vp);
   }
 
-  if (STOBJ_GET_CLASS(obj) == &sXPC_SOW_JSClass.base) {
+  if (obj->getClass() == &SOWClass) {
     // We are extra careful about content-polluted wrappers here. I don't know
     // if it's possible to reach them through objects that we wrap, but figuring
     // that out is more expensive (and harder) than simply checking and
     // rewrapping here.
-    if (STOBJ_GET_PARENT(wrapperObj) == STOBJ_GET_PARENT(obj)) {
+    if (wrapperObj->getParent() == obj->getParent()) {
       // Already wrapped.
       return JS_TRUE;
     }
@@ -343,16 +422,16 @@ XPC_SOW_RewrapValue(JSContext *cx, JSObject *wrapperObj, jsval *vp)
     v = *vp = OBJECT_TO_JSVAL(obj);
   }
 
-  return XPC_SOW_WrapObject(cx, STOBJ_GET_PARENT(wrapperObj), v, vp);
+  return WrapObject(cx, wrapperObj->getParent(), v, vp);
 }
 
 static JSBool
-XPC_SOW_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+XPC_SOW_AddProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
-  NS_ASSERTION(STOBJ_GET_CLASS(obj) == &sXPC_SOW_JSClass.base, "Wrong object");
+  NS_ASSERTION(obj->getClass() == &SOWClass, "Wrong object");
 
   jsval resolving;
-  if (!JS_GetReservedSlot(cx, obj, XPCWrapper::sFlagsSlot, &resolving)) {
+  if (!JS_GetReservedSlot(cx, obj, sFlagsSlot, &resolving)) {
     return JS_FALSE;
   }
 
@@ -370,11 +449,11 @@ XPC_SOW_AddProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     return JS_TRUE;
   }
 
-  return XPCWrapper::AddProperty(cx, obj, JS_TRUE, wrappedObj, id, vp);
+  return AddProperty(cx, obj, JS_TRUE, wrappedObj, id, vp);
 }
 
 static JSBool
-XPC_SOW_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+XPC_SOW_DelProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   JSObject *wrappedObj = GetWrappedObject(cx, obj);
   if (!wrappedObj) {
@@ -385,17 +464,13 @@ XPC_SOW_DelProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
     return JS_FALSE;
   }
 
-  return XPCWrapper::DelProperty(cx, wrappedObj, id, vp);
+  return DelProperty(cx, wrappedObj, id, vp);
 }
 
 static JSBool
-XPC_SOW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp,
+XPC_SOW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp,
                          JSBool isSet)
 {
-  if (id == GetRTStringByIndex(cx, XPCJSRuntime::IDX_TO_STRING)) {
-    return JS_TRUE;
-  }
-
   obj = GetWrapper(obj);
   if (!obj) {
     return ThrowException(NS_ERROR_ILLEGAL_VALUE, cx);
@@ -405,26 +480,21 @@ XPC_SOW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp,
     return JS_FALSE;
   }
 
-  JSAutoTempValueRooter tvr(cx, 1, vp);
+  js::AutoArrayRooter tvr(cx, 1, vp);
 
   JSObject *wrappedObj = GetWrappedObject(cx, obj);
   if (!wrappedObj) {
     return ThrowException(NS_ERROR_ILLEGAL_VALUE, cx);
   }
 
-  if (isSet && id == GetRTStringByIndex(cx, XPCJSRuntime::IDX_PROTO)) {
+  if (isSet && id == GetRTIdByIndex(cx, XPCJSRuntime::IDX_PROTO)) {
     // No setting __proto__ on my object.
     return ThrowException(NS_ERROR_INVALID_ARG, cx); // XXX better error message
   }
 
-  jsid interned_id;
-  if (!JS_ValueToId(cx, id, &interned_id)) {
-    return JS_FALSE;
-  }
-
   JSBool ok = isSet
-              ? JS_SetPropertyById(cx, wrappedObj, interned_id, vp)
-              : JS_GetPropertyById(cx, wrappedObj, interned_id, vp);
+              ? JS_SetPropertyById(cx, wrappedObj, id, vp)
+              : JS_GetPropertyById(cx, wrappedObj, id, vp);
   if (!ok) {
     return JS_FALSE;
   }
@@ -433,13 +503,13 @@ XPC_SOW_GetOrSetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp,
 }
 
 static JSBool
-XPC_SOW_GetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+XPC_SOW_GetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   return XPC_SOW_GetOrSetProperty(cx, obj, id, vp, JS_FALSE);
 }
 
 static JSBool
-XPC_SOW_SetProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
+XPC_SOW_SetProperty(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
 {
   return XPC_SOW_GetOrSetProperty(cx, obj, id, vp, JS_TRUE);
 }
@@ -454,15 +524,15 @@ XPC_SOW_Enumerate(JSContext *cx, JSObject *obj)
     return JS_TRUE;
   }
 
-  if (!AllowedToAct(cx, JSVAL_VOID)) {
+  if (!AllowedToAct(cx, JSID_VOID)) {
     return JS_FALSE;
   }
 
-  return XPCWrapper::Enumerate(cx, obj, wrappedObj);
+  return Enumerate(cx, obj, wrappedObj);
 }
 
 static JSBool
-XPC_SOW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
+XPC_SOW_NewResolve(JSContext *cx, JSObject *obj, jsid id, uintN flags,
                    JSObject **objp)
 {
   obj = GetWrapper(obj);
@@ -478,34 +548,13 @@ XPC_SOW_NewResolve(JSContext *cx, JSObject *obj, jsval id, uintN flags,
     return JS_FALSE;
   }
 
-  if (id == GetRTStringByIndex(cx, XPCJSRuntime::IDX_TO_STRING)) {
-    jsval oldSlotVal;
-    if (!JS_GetReservedSlot(cx, obj, XPCWrapper::sFlagsSlot, &oldSlotVal) ||
-        !JS_SetReservedSlot(cx, obj, XPCWrapper::sFlagsSlot,
-                            INT_TO_JSVAL(JSVAL_TO_INT(oldSlotVal) |
-                                         FLAG_RESOLVING))) {
-      return JS_FALSE;
-    }
-
-    JSBool ok = JS_DefineFunction(cx, obj, "toString",
-                                  XPC_SOW_toString, 0, 0) != nsnull;
-
-    JS_SetReservedSlot(cx, obj, XPCWrapper::sFlagsSlot, oldSlotVal);
-
-    if (ok) {
-      *objp = obj;
-    }
-
-    return ok;
-  }
-
-  return XPCWrapper::NewResolve(cx, obj, JS_TRUE, wrappedObj, id, flags, objp);
+  return NewResolve(cx, obj, JS_FALSE, wrappedObj, id, flags, objp);
 }
 
 static JSBool
 XPC_SOW_Convert(JSContext *cx, JSObject *obj, JSType type, jsval *vp)
 {
-  if (!AllowedToAct(cx, JSVAL_VOID)) {
+  if (!AllowedToAct(cx, JSID_VOID)) {
     return JS_FALSE;
   }
 
@@ -518,20 +567,15 @@ XPC_SOW_Convert(JSContext *cx, JSObject *obj, JSType type, jsval *vp)
   JSObject *wrappedObj = GetWrappedObject(cx, obj);
   if (!wrappedObj) {
     // Converting the prototype to something.
-
-    if (type == JSTYPE_STRING || type == JSTYPE_VOID) {
-      return XPC_SOW_toString(cx, obj, 0, nsnull, vp);
-    }
-
-    *vp = OBJECT_TO_JSVAL(obj);
+    // XXX Can this happen?
     return JS_TRUE;
   }
 
-  return STOBJ_GET_CLASS(wrappedObj)->convert(cx, wrappedObj, type, vp);
+  return wrappedObj->getJSClass()->convert(cx, wrappedObj, type, vp);
 }
 
 static JSBool
-XPC_SOW_CheckAccess(JSContext *cx, JSObject *obj, jsval prop, JSAccessMode mode,
+XPC_SOW_CheckAccess(JSContext *cx, JSObject *obj, jsid id, JSAccessMode mode,
                     jsval *vp)
 {
   // Simply forward checkAccess to our wrapped object. It's already expecting
@@ -544,15 +588,13 @@ XPC_SOW_CheckAccess(JSContext *cx, JSObject *obj, jsval prop, JSAccessMode mode,
   }
 
   uintN junk;
-  jsid id;
-  return JS_ValueToId(cx, prop, &id) &&
-         JS_CheckAccess(cx, wrappedObj, id, mode, vp, &junk);
+  return JS_CheckAccess(cx, wrappedObj, id, mode, vp, &junk);
 }
 
 static JSBool
-XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
+XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, const jsval *valp, JSBool *bp)
 {
-  if (!AllowedToAct(cx, JSVAL_VOID)) {
+  if (!AllowedToAct(cx, JSID_VOID)) {
     return JS_FALSE;
   }
 
@@ -562,7 +604,7 @@ XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
     return JS_TRUE;
   }
 
-  JSClass *clasp = STOBJ_GET_CLASS(iface);
+  JSClass *clasp = iface->getJSClass();
 
   *bp = JS_FALSE;
   if (!clasp->hasInstance) {
@@ -572,6 +614,7 @@ XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
   // Prematurely unwrap the left hand side. This isn't necessary, but could be
   // faster than waiting until XPCWrappedNative::GetWrappedNativeOfJSObject to
   // do it.
+  jsval v = *valp;
   if (!JSVAL_IS_PRIMITIVE(v)) {
     JSObject *test = JSVAL_TO_OBJECT(v);
 
@@ -582,13 +625,14 @@ XPC_SOW_HasInstance(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
     }
   }
 
-  return clasp->hasInstance(cx, iface, v, bp);
+  return clasp->hasInstance(cx, iface, &v, bp);
 }
 
 static JSBool
-XPC_SOW_Equality(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
+XPC_SOW_Equality(JSContext *cx, JSObject *obj, const jsval *valp, JSBool *bp)
 {
   // Delegate to our wrapped object.
+  jsval v = *valp;
   if (JSVAL_IS_PRIMITIVE(v)) {
     *bp = JS_FALSE;
     return JS_TRUE;
@@ -607,21 +651,16 @@ XPC_SOW_Equality(JSContext *cx, JSObject *obj, jsval v, JSBool *bp)
   }
 
   if (lhs) {
-    // Delegate to our wrapped object if we can.
-    JSClass *clasp = STOBJ_GET_CLASS(lhs);
-    if (clasp->flags & JSCLASS_IS_EXTENDED) {
-      JSExtendedClass *xclasp = (JSExtendedClass *) clasp;
-      // NB: JSExtendedClass.equality is a required field.
-      return xclasp->equality(cx, lhs, OBJECT_TO_JSVAL(rhs), bp);
+    if (JSEqualityOp op = js::Jsvalify(lhs->getClass()->ext.equality)) {
+      jsval rhsVal = OBJECT_TO_JSVAL(rhs);
+      return op(cx, lhs, &rhsVal, bp);
     }
   }
 
   // We know rhs is non-null.
-  JSClass *clasp = STOBJ_GET_CLASS(rhs);
-  if (clasp->flags & JSCLASS_IS_EXTENDED) {
-    JSExtendedClass *xclasp = (JSExtendedClass *) clasp;
-    // NB: JSExtendedClass.equality is a required field.
-    return xclasp->equality(cx, rhs, OBJECT_TO_JSVAL(lhs), bp);
+  if (JSEqualityOp op = js::Jsvalify(rhs->getClass()->ext.equality)) {
+    jsval lhsVal = OBJECT_TO_JSVAL(lhs);
+    return op(cx, rhs, &lhsVal, bp);
   }
 
   *bp = JS_FALSE;
@@ -637,24 +676,22 @@ XPC_SOW_Iterator(JSContext *cx, JSObject *obj, JSBool keysonly)
     return nsnull;
   }
 
-  JSObject *wrapperIter = JS_NewObject(cx, &sXPC_SOW_JSClass.base, nsnull,
+  JSObject *wrapperIter = JS_NewObject(cx, js::Jsvalify(&SOWClass), nsnull,
                                        JS_GetGlobalForObject(cx, obj));
   if (!wrapperIter) {
     return nsnull;
   }
 
-  JSAutoTempValueRooter tvr(cx, OBJECT_TO_JSVAL(wrapperIter));
+  js::AutoObjectRooter tvr(cx, wrapperIter);
 
   // Initialize our SOW.
   jsval v = OBJECT_TO_JSVAL(wrappedObj);
-  if (!JS_SetReservedSlot(cx, wrapperIter, XPCWrapper::sWrappedObjSlot, v) ||
-      !JS_SetReservedSlot(cx, wrapperIter, XPCWrapper::sFlagsSlot,
-                          JSVAL_ZERO)) {
+  if (!JS_SetReservedSlot(cx, wrapperIter, sWrappedObjSlot, v) ||
+      !JS_SetReservedSlot(cx, wrapperIter, sFlagsSlot, JSVAL_ZERO)) {
     return nsnull;
   }
 
-  return XPCWrapper::CreateIteratorObj(cx, wrapperIter, obj, wrappedObj,
-                                       keysonly);
+  return CreateIteratorObj(cx, wrapperIter, obj, wrappedObj, keysonly);
 }
 
 static JSObject *
@@ -662,56 +699,3 @@ XPC_SOW_WrappedObject(JSContext *cx, JSObject *obj)
 {
   return GetWrappedObject(cx, obj);
 }
-
-static JSBool
-XPC_SOW_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv,
-                 jsval *rval)
-{
-  if (!AllowedToAct(cx, JSVAL_VOID)) {
-    return JS_FALSE;
-  }
-
-  obj = GetWrapper(obj);
-  if (!obj) {
-    return ThrowException(NS_ERROR_UNEXPECTED, cx);
-  }
-
-  JSObject *wrappedObj = GetWrappedObject(cx, obj);
-  if (!wrappedObj) {
-    // Someone's calling toString on our prototype.
-    NS_NAMED_LITERAL_CSTRING(protoString, "[object XPCCrossOriginWrapper]");
-    JSString *str =
-      JS_NewStringCopyN(cx, protoString.get(), protoString.Length());
-    if (!str) {
-      return JS_FALSE;
-    }
-    *rval = STRING_TO_JSVAL(str);
-    return JS_TRUE;
-  }
-
-  XPCWrappedNative *wn =
-    XPCWrappedNative::GetWrappedNativeOfJSObject(cx, wrappedObj);
-  return XPCWrapper::NativeToString(cx, wn, argc, argv, rval, JS_FALSE);
-}
-
-JSBool
-XPC_SOW_WrapObject(JSContext *cx, JSObject *parent, jsval v,
-                   jsval *vp)
-{
-  JSObject *wrapperObj =
-    JS_NewObjectWithGivenProto(cx, &sXPC_SOW_JSClass.base, NULL, parent);
-  if (!wrapperObj) {
-    return JS_FALSE;
-  }
-
-  *vp = OBJECT_TO_JSVAL(wrapperObj);
-  JSAutoTempValueRooter tvr(cx, *vp);
-
-  if (!JS_SetReservedSlot(cx, wrapperObj, XPCWrapper::sWrappedObjSlot, v) ||
-      !JS_SetReservedSlot(cx, wrapperObj, XPCWrapper::sFlagsSlot,
-                          JSVAL_ZERO)) {
-    return JS_FALSE;
-  }
-
-  return JS_TRUE;
-}
