@@ -113,7 +113,7 @@ static uint8 xpc_reflectable_flags[XPC_FLAG_COUNT] = {
     XPC_MK_FLAG(  0  ,  1  ,   0 ,  0 ), /* T_UTF8STRING        */
     XPC_MK_FLAG(  0  ,  1  ,   0 ,  0 ), /* T_CSTRING           */
     XPC_MK_FLAG(  0  ,  1  ,   0 ,  0 ), /* T_ASTRING           */
-    XPC_MK_FLAG(  0  ,  0  ,   0 ,  0 ), /* 26 - reserved       */
+    XPC_MK_FLAG(  1  ,  0  ,   1 ,  0 ), /* T_JSVAL             */
     XPC_MK_FLAG(  0  ,  0  ,   0 ,  0 ), /* 27 - reserved       */
     XPC_MK_FLAG(  0  ,  0  ,   0 ,  0 ), /* 28 - reserved       */
     XPC_MK_FLAG(  0  ,  0  ,   0 ,  0 ), /* 29 - reserved       */
@@ -153,7 +153,7 @@ XPCConvert::IsMethodReflectable(const XPTMethodDescriptor& info)
 JSBool
 XPCConvert::GetISupportsFromJSObject(JSObject* obj, nsISupports** iface)
 {
-    JSClass* jsclass = STOBJ_GET_CLASS(obj);
+    JSClass* jsclass = obj->getJSClass();
     NS_ASSERTION(jsclass, "obj has no class");
     if(jsclass &&
        (jsclass->flags & JSCLASS_HAS_PRIVATE) &&
@@ -202,13 +202,9 @@ XPCConvert::RemoveXPCOMUCStringFinalizer()
 }
 
 
-#define FIT_32(cx,i,d)      (INT_FITS_IN_JSVAL(i) \
-                             ? *d = INT_TO_JSVAL(i), JS_TRUE    \
-                             : JS_NewDoubleValue(cx, i, d))
-
-#define FIT_U32(cx,i,d)     ((i) <= JSVAL_INT_MAX \
-                             ? *d = INT_TO_JSVAL(i), JS_TRUE    \
-                             : JS_NewDoubleValue(cx, i, d))
+#define FIT_U32(i)     ((i) <= JSVAL_INT_MAX      \
+                        ? INT_TO_JSVAL(i)         \
+                        : DOUBLE_TO_JSVAL(i))
 
 /*
  * Support for 64 bit conversions where 'long long' not supported.
@@ -238,33 +234,31 @@ INT64_TO_DOUBLE(const int64 &v)
 
 // static
 JSBool
-XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
+XPCConvert::NativeData2JS(XPCLazyCallContext& lccx, jsval* d, const void* s,
                           const nsXPTType& type, const nsID* iid,
                           JSObject* scope, nsresult* pErr)
 {
     NS_PRECONDITION(s, "bad param");
     NS_PRECONDITION(d, "bad param");
 
-    JSContext* cx = ccx.GetJSContext();
+    JSContext* cx = lccx.GetJSContext();
 
     if(pErr)
         *pErr = NS_ERROR_XPC_BAD_CONVERT_NATIVE;
 
     switch(type.TagPart())
     {
-    case nsXPTType::T_I8    : *d = INT_TO_JSVAL((int32)*((int8*)s));     break;
-    case nsXPTType::T_I16   : *d = INT_TO_JSVAL((int32)*((int16*)s));    break;
-    case nsXPTType::T_I32   : return FIT_32(cx,*((int32*)s),d);
-    case nsXPTType::T_I64   :
-        return JS_NewNumberValue(cx, INT64_TO_DOUBLE(*((int64*)s)), d);
-    case nsXPTType::T_U8    : *d = INT_TO_JSVAL((int32)*((uint8*)s));    break;
-    case nsXPTType::T_U16   : *d = INT_TO_JSVAL((int32)*((uint16*)s));   break;
-    case nsXPTType::T_U32   : return FIT_U32(cx,*((uint32*)s),d);
-    case nsXPTType::T_U64   :
-        return JS_NewNumberValue(cx, UINT64_TO_DOUBLE(*((uint64*)s)), d);
-    case nsXPTType::T_FLOAT : return JS_NewNumberValue(cx, *((float*)s), d);
-    case nsXPTType::T_DOUBLE: return JS_NewNumberValue(cx, *((double*)s), d);
-    case nsXPTType::T_BOOL  : *d = *((PRBool*)s)?JSVAL_TRUE:JSVAL_FALSE; break;
+    case nsXPTType::T_I8    : *d = INT_TO_JSVAL((int32)*((int8*)s));                 break;
+    case nsXPTType::T_I16   : *d = INT_TO_JSVAL((int32)*((int16*)s));                break;
+    case nsXPTType::T_I32   : *d = INT_TO_JSVAL(*((int32*)s));                       break;
+    case nsXPTType::T_I64   : *d = DOUBLE_TO_JSVAL(INT64_TO_DOUBLE(*((int64*)s)));   break;
+    case nsXPTType::T_U8    : *d = INT_TO_JSVAL((int32)*((uint8*)s));                break;
+    case nsXPTType::T_U16   : *d = INT_TO_JSVAL((int32)*((uint16*)s));               break;
+    case nsXPTType::T_U32   : *d = FIT_U32(*((uint32*)s));                           break;
+    case nsXPTType::T_U64   : *d = DOUBLE_TO_JSVAL(UINT64_TO_DOUBLE(*((uint64*)s))); break;
+    case nsXPTType::T_FLOAT : *d = DOUBLE_TO_JSVAL(*((float*)s));                    break;
+    case nsXPTType::T_DOUBLE: *d = DOUBLE_TO_JSVAL(*((double*)s));                   break;
+    case nsXPTType::T_BOOL  : *d = BOOLEAN_TO_JSVAL(*((PRBool*)s));                  break;
     case nsXPTType::T_CHAR  :
         {
             char* p = (char*)s;
@@ -292,6 +286,12 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
             *d = STRING_TO_JSVAL(str);
             break;
         }
+
+    case nsXPTType::T_JSVAL :
+        JS_STATIC_ASSERT(sizeof(jsval) <= sizeof(uint64));
+        *d = **((jsval**)s);
+        break;
+
     default:
         if(!type.IsPointer())
         {
@@ -330,9 +330,12 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                     break;
 
                 if(!p->IsVoid()) {
-                    jsval str = XPCStringConvert::ReadableToJSVal(cx, *p);
-                    if(!str)
+                    nsStringBuffer* buf;
+                    jsval str = XPCStringConvert::ReadableToJSVal(cx, *p, &buf);
+                    if(JSVAL_IS_NULL(str))
                         return JS_FALSE;
+                    if(buf)
+                        buf->AddRef();
 
                     *d = str;
                 }
@@ -456,7 +459,7 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                         if(!variant)
                             return JS_FALSE;
 
-                        return XPCVariant::VariantDataToJS(ccx, variant, 
+                        return XPCVariant::VariantDataToJS(lccx, variant, 
                                                            scope, pErr, d);
                     }
                     // else...
@@ -466,22 +469,23 @@ XPCConvert::NativeData2JS(XPCCallContext& ccx, jsval* d, const void* s,
                     // global object will not have been collected, and
                     // therefore this NativeInterface2JSObject will not end up
                     // creating a new XPCNativeScriptableShared.
-                    if(!NativeInterface2JSObject(ccx, d, nsnull, iface, iid,
+                    if(!NativeInterface2JSObject(lccx, d, nsnull, iface, iid,
                                                  nsnull, nsnull, scope, PR_TRUE,
                                                  OBJ_IS_NOT_GLOBAL, pErr))
                         return JS_FALSE;
 
 #ifdef DEBUG
                     JSObject* jsobj = JSVAL_TO_OBJECT(*d);
-                    if(jsobj && !STOBJ_GET_PARENT(jsobj))
-                        NS_ASSERTION(STOBJ_GET_CLASS(jsobj)->flags & JSCLASS_IS_GLOBAL,
+                    if(jsobj && !jsobj->getParent())
+                        NS_ASSERTION(jsobj->getClass()->flags & JSCLASS_IS_GLOBAL,
                                      "Why did we recreate this wrapper?");
 #endif
                 }
                 break;
             }
+
         default:
-            NS_ASSERTION(0, "bad type");
+            NS_ERROR("bad type");
             return JS_FALSE;
         }
     }
@@ -596,8 +600,8 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 return JS_FALSE;
             }
 #ifdef DEBUG
-            jschar* chars=nsnull;
-            if(nsnull!=(chars = JS_GetStringChars(str)))
+            const jschar* chars=nsnull;
+            if(nsnull!=(chars = JS_GetStringCharsZ(cx, str)))
             {
                 NS_ASSERTION((! ILLEGAL_RANGE(chars[0])),"U+0080/U+0100 - U+FFFF data lost");
             }
@@ -607,20 +611,36 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
         }
     case nsXPTType::T_WCHAR  :
         {
-            jschar* chars=nsnull;
+            const jschar* chars=nsnull;
             JSString* str;
-            if(!(str = JS_ValueToString(cx, s))||
-               !(chars = JS_GetStringChars(str)))
+            if(!(str = JS_ValueToString(cx, s)))
             {
                 return JS_FALSE;
             }
-            *((uint16*)d)  = (uint16) chars[0];
+            if(JS_GetStringLength(str) == 0)
+            {
+                *((uint16*)d) = 0;
+                break;
+            }
+            chars = JS_GetStringChars(str);
+            *((uint16*)d) = (uint16) chars[0];
+            break;
+        }
+    case nsXPTType::T_JSVAL :
+        {
+            NS_ASSERTION(useAllocator, "trying to convert a jsval to const jsval & without allocator : this would leak");
+
+            // The C++ type is (const jsval &), which here means (jsval *).
+            jsval *buf = new jsval(s);
+            if(!buf)
+                return JS_FALSE;
+            *((jsval**)d) = buf;
             break;
         }
     default:
         if(!type.IsPointer())
         {
-            NS_ASSERTION(0,"unsupported type");
+            NS_ERROR("unsupported type");
             return JS_FALSE;
         }
 
@@ -628,7 +648,7 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
         {
         case nsXPTType::T_VOID:
             XPC_LOG_ERROR(("XPCConvert::JSData2Native : void* params not supported"));
-            NS_ASSERTION(0,"void* params not supported");
+            NS_ERROR("void* params not supported");
             return JS_FALSE;
         case nsXPTType::T_IID:
         {
@@ -811,7 +831,7 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
 
         case nsXPTType::T_WCHAR_STR:
         {
-            jschar* chars=nsnull;
+            const jschar* chars=nsnull;
             JSString* str;
 
             if(JSVAL_IS_VOID(s) || JSVAL_IS_NULL(s))
@@ -827,23 +847,35 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                 return JS_TRUE;
             }
 
-            if(!(str = JS_ValueToString(cx, s))||
-               !(chars = JS_GetStringChars(str)))
+            if(!(str = JS_ValueToString(cx, s)))
             {
                 return JS_FALSE;
             }
             if(useAllocator)
             {
-                int byte_len = (JS_GetStringLength(str)+1)*sizeof(jschar);
+                if(!(chars = JS_GetStringChars(str)))
+                {
+                    return JS_FALSE;
+                }
+                int len = JS_GetStringLength(str);
+                int byte_len = (len+1)*sizeof(jschar);
                 if(!(*((void**)d) = nsMemory::Alloc(byte_len)))
                 {
                     // XXX should report error
                     return JS_FALSE;
                 }
-                memcpy(*((void**)d), chars, byte_len);
+                jschar* destchars = *((jschar**)d);
+                memcpy(destchars, chars, byte_len);
+                destchars[len] = 0;
             }
             else
-                *((jschar**)d) = chars;
+            {
+                if(!(chars = JS_GetStringCharsZ(cx, str)))
+                {
+                    return JS_FALSE;
+                }
+                *((const jschar**)d) = chars;
+            }
 
             return JS_TRUE;
         }
@@ -898,8 +930,9 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
             {
                 rs = *((nsCString**)d);
             }
-            CopyUTF16toUTF8(nsDependentString((const PRUnichar*)chars, length),
-                            *rs);
+            const PRUnichar* start = (const PRUnichar*)chars;
+            const PRUnichar* end = start + length;
+            CopyUTF16toUTF8(nsDependentSubstring(start, end), *rs);
             return JS_TRUE;
         }
 
@@ -984,7 +1017,8 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                     return JS_FALSE;
                 }
                 PRUint32 length = JS_GetStringLength(str);
-                nsIAtom* atom = NS_NewAtom(nsDependentString(chars, length));
+                nsIAtom* atom = NS_NewAtom(nsDependentSubstring(chars,
+                                             chars + length));
                 if (!atom && pErr)
                     *pErr = NS_ERROR_OUT_OF_MEMORY;
                 *((nsISupports**)d) = atom;
@@ -1017,7 +1051,7 @@ XPCConvert::JSData2Native(XPCCallContext& ccx, void* d, jsval s,
                                             nsnull, pErr);
         }
         default:
-            NS_ASSERTION(0, "bad type");
+            NS_ERROR("bad type");
             return JS_FALSE;
         }
     }
@@ -1045,19 +1079,22 @@ CreateHolderIfNeeded(XPCCallContext& ccx, JSObject* obj, jsval* d,
 /***************************************************************************/
 // static
 JSBool
-XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
+XPCConvert::NativeInterface2JSObject(XPCLazyCallContext& lccx,
                                      jsval* d,
                                      nsIXPConnectJSObjectHolder** dest,
                                      nsISupports* src,
                                      const nsID* iid,
-                                     XPCNativeInterface* Interface,
+                                     XPCNativeInterface** Interface,
                                      nsWrapperCache *cache,
                                      JSObject* scope,
                                      PRBool allowNativeWrapper,
                                      PRBool isGlobal,
-                                     nsresult* pErr)
+                                     nsresult* pErr,
+                                     qsObjectHelper* aHelper)
 {
     NS_ASSERTION(scope, "bad param");
+    NS_ASSERTION(!Interface || iid,
+                 "Need the iid if you pass in an XPCNativeInterface cache.");
 
     *d = JSVAL_NULL;
     if(dest)
@@ -1085,10 +1122,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
         // verify that this wrapper is for the right interface
         nsCOMPtr<nsISupports> wrapper;
-        if(Interface)
-            src->QueryInterface(*Interface->GetIID(),
-                                (void**)getter_AddRefs(wrapper));
-        else if(iid)
+        if(iid)
             src->QueryInterface(*iid, (void**)getter_AddRefs(wrapper));
         else
             wrapper = do_QueryInterface(src);
@@ -1106,13 +1140,17 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
     else
 #endif /* XPC_DO_DOUBLE_WRAP */
     {
+        JSContext* cx = lccx.GetJSContext();
+
         XPCWrappedNativeScope* xpcscope =
-            XPCWrappedNativeScope::FindInJSObjectScope(ccx, scope);
+            XPCWrappedNativeScope::FindInJSObjectScope(cx, scope);
         if(!xpcscope)
             return JS_FALSE;
 
         if(!cache)
             CallQueryInterface(src, &cache);
+
+        PRBool tryConstructSlimWrapper = PR_FALSE;
         JSObject *flat;
         if(cache)
         {
@@ -1121,24 +1159,11 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
             {
                 if(!flat)
                 {
-                    jsval slim;
-                    if(ConstructSlimWrapper(ccx, src, cache, Interface,
-                                            xpcscope, &slim))
-                    {
-                        *d = slim;
-                        return JS_TRUE;
-                    }
-
-                    // Even if ConstructSlimWrapper returns JS_FALSE it might
-                    // have created a wrapper (while calling the PreCreate
-                    // hook). In that case we need to fall through because we
-                    // either have a slim wrapper that needs to be morphed or
-                    // we have an XPCWrappedNative.
-                    flat = cache->GetWrapper();
+                    tryConstructSlimWrapper = PR_TRUE;
                 }
-                else if(!IS_WRAPPER_CLASS(STOBJ_GET_CLASS(flat)))
+                else if(IS_SLIM_WRAPPER_OBJECT(flat))
                 {
-                    JSObject* global = JS_GetGlobalForObject(ccx, flat);
+                    JSObject* global = JS_GetGlobalForObject(cx, flat);
                     if(global == xpcscope->GetGlobalJSObject())
                     {
                         *d = OBJECT_TO_JSVAL(flat);
@@ -1152,26 +1177,68 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
             flat = nsnull;
         }
 
-        AutoMarkingNativeInterfacePtr iface(ccx, Interface);
-        if(!iface && iid)
+        if(tryConstructSlimWrapper)
         {
-            iface = XPCNativeInterface::GetNewOrUsed(ccx, iid);
-            if(!iface)
+            XPCCallContext &ccx = lccx.GetXPCCallContext();
+            if(!ccx.IsValid())
                 return JS_FALSE;
+
+            jsval slim;
+            if(ConstructSlimWrapper(ccx, src, aHelper, cache, xpcscope, &slim))
+            {
+                *d = slim;
+                return JS_TRUE;
+            }
+
+            // Even if ConstructSlimWrapper returns JS_FALSE it might have created a
+            // wrapper (while calling the PreCreate hook). In that case we need to
+            // fall through because we either have a slim wrapper that needs to be
+            // morphed or we have an XPCWrappedNative.
+            flat = cache->GetWrapper();
         }
 
+        AutoMarkingNativeInterfacePtr iface;
+        if(iid)
+        {
+            XPCCallContext &ccx = lccx.GetXPCCallContext();
+            if(!ccx.IsValid())
+                return JS_FALSE;
+
+            iface.Init(ccx);
+
+            if(Interface)
+                iface = *Interface;
+
+            if(!iface)
+            {
+                iface = XPCNativeInterface::GetNewOrUsed(ccx, iid);
+                if(!iface)
+                    return JS_FALSE;
+
+                if(Interface)
+                    *Interface = iface;
+            }
+        }
+
+        NS_ASSERTION(!flat || IS_WRAPPER_CLASS(flat->getClass()),
+                     "What kind of wrapper is this?");
+
         nsresult rv;
         XPCWrappedNative* wrapper;
         nsRefPtr<XPCWrappedNative> strongWrapper;
         if(!flat)
         {
+            XPCCallContext &ccx = lccx.GetXPCCallContext();
+            if(!ccx.IsValid())
+                return JS_FALSE;
+
             rv = XPCWrappedNative::GetNewOrUsed(ccx, src, xpcscope, iface,
                                                 cache, isGlobal,
                                                 getter_AddRefs(strongWrapper));
 
             wrapper = strongWrapper;
         }
-        else if(IS_WRAPPER_CLASS(STOBJ_GET_CLASS(flat)))
+        else if(IS_WN_WRAPPER_OBJECT(flat))
         {
             wrapper = static_cast<XPCWrappedNative*>(xpc_GetJSPrivate(flat));
 
@@ -1180,8 +1247,12 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
             // rooted in that case).
             if(dest)
                 strongWrapper = wrapper;
+            // If iface is not null we know lccx.GetXPCCallContext() returns
+            // a valid XPCCallContext because we checked when calling Init on
+            // iface.
             if(iface)
-                wrapper->FindTearOff(ccx, iface, JS_FALSE, &rv);
+                wrapper->FindTearOff(lccx.GetXPCCallContext(), iface, JS_FALSE,
+                                     &rv);
             else
                 rv = NS_OK;
         }
@@ -1189,6 +1260,11 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
         {
             NS_ASSERTION(IS_SLIM_WRAPPER(flat),
                          "What kind of wrapper is this?");
+
+            XPCCallContext &ccx = lccx.GetXPCCallContext();
+            if(!ccx.IsValid())
+                return JS_FALSE;
+
             SLIM_LOG(("***** morphing from XPCConvert::NativeInterface2JSObject"
                       "(%p)\n",
                       static_cast<nsISupports*>(xpc_GetJSPrivate(flat))));
@@ -1202,6 +1278,10 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
             *pErr = rv;
         if(NS_SUCCEEDED(rv) && wrapper)
         {
+            XPCCallContext &ccx = lccx.GetXPCCallContext();
+            if(!ccx.IsValid())
+                return JS_FALSE;
+
             uint32 flags = 0;
             flat = wrapper->GetFlatJSObject();
             jsval v = OBJECT_TO_JSVAL(flat);
@@ -1230,7 +1310,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                     if(fp)
                     {
                         script = fp->script;
-                        callee = fp->callee;
+                        callee = fp->callee();
                     }
                 }
                 else if(ccx.GetXPCContext()->CallerTypeIsNative())
@@ -1307,7 +1387,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
                         destObj =
                             XPCNativeWrapper::GetNewOrUsed(ccx, wrapper,
-                                                           objPrincipal);
+                                                           scope, objPrincipal);
                         triedWrapping = JS_TRUE;
                     }
                     else if (flags & JSFILENAME_SYSTEM)
@@ -1317,7 +1397,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                                "XPCSafeJSObjectWrapper\n");
 #endif
 
-                        if(XPC_SJOW_Construct(ccx, nsnull, 1, &v, &v))
+                        if(XPCSafeJSObjectWrapper::WrapObject(ccx, scope, v, &v))
                             destObj = JSVAL_TO_OBJECT(v);
                         triedWrapping = JS_TRUE;
                     }
@@ -1325,7 +1405,7 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                     {
                         // Reaching across scopes from content code. Wrap
                         // the new object in a XOW.
-                        if (XPC_XOW_WrapObject(ccx, scope, &v))
+                        if (XPCCrossOriginWrapper::WrapObject(ccx, scope, &v))
                             destObj = JSVAL_TO_OBJECT(v);
                         triedWrapping = JS_TRUE;
                     }
@@ -1337,11 +1417,12 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
 
                         jsval wrappedObjVal = OBJECT_TO_JSVAL(destObj);
                         AUTO_MARK_JSVAL(ccx, &wrappedObjVal);
-                        if(wrapper->NeedsChromeWrapper())
+                        if(wrapper->NeedsSOW())
                         {
-                            if(!XPC_SOW_WrapObject(ccx, xpcscope->GetGlobalJSObject(),
-                                                   OBJECT_TO_JSVAL(destObj),
-                                                   &wrappedObjVal))
+                            using SystemOnlyWrapper::WrapObject;
+                            if(!WrapObject(ccx, xpcscope->GetGlobalJSObject(),
+                                           OBJECT_TO_JSVAL(destObj),
+                                           &wrappedObjVal))
                                 return JS_FALSE;
                         }
 
@@ -1351,11 +1432,11 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                 }
             }
 
-            const char *name = STOBJ_GET_CLASS(flat)->name;
+            const char *name = flat->getClass()->name;
             if(allowNativeWrapper &&
                !(flags & JSFILENAME_SYSTEM) &&
                !JS_IsSystemObject(ccx, flat) &&
-               XPC_XOW_ClassNeedsXOW(name))
+               XPCCrossOriginWrapper::ClassNeedsXOW(name))
             {
                 // From here on we might create new JSObjects, so we need to
                 // make sure that wrapper stays alive.
@@ -1363,21 +1444,23 @@ XPCConvert::NativeInterface2JSObject(XPCCallContext& ccx,
                     strongWrapper = wrapper;
 
                 AUTO_MARK_JSVAL(ccx, &v);
-                return XPC_XOW_WrapObject(ccx, scope, &v) &&
-                       (!wrapper->NeedsChromeWrapper() ||
-                        XPC_SOW_WrapObject(ccx, xpcscope->GetGlobalJSObject(),
-                                           v, &v)) &&
+                return XPCCrossOriginWrapper::WrapObject(ccx, scope, &v) &&
+                       (!wrapper->NeedsSOW() ||
+                        SystemOnlyWrapper::WrapObject(ccx, xpcscope->GetGlobalJSObject(),
+                                                      v, &v)) &&
                        CreateHolderIfNeeded(ccx, JSVAL_TO_OBJECT(v), d, dest);
             }
 
             *d = v;
             if(allowNativeWrapper)
             {
-                if(wrapper->NeedsChromeWrapper())
-                    if(!XPC_SOW_WrapObject(ccx, xpcscope->GetGlobalJSObject(), v, d))
+                if(wrapper->NeedsSOW())
+                    if(!SystemOnlyWrapper::WrapObject(ccx,
+                                                      xpcscope->GetGlobalJSObject(),
+                                                      v, d))
                         return JS_FALSE;
-                if(wrapper->IsDoubleWrapper())
-                    if(!XPC_COW_WrapObject(ccx, xpcscope->GetGlobalJSObject(), v, d))
+                if(wrapper->NeedsCOW())
+                    if(!ChromeObjectWrapper::WrapObject(ccx, xpcscope->GetGlobalJSObject(), v, d))
                         return JS_FALSE;
             }
             if(dest)
@@ -1417,25 +1500,41 @@ XPCConvert::JSObject2NativeInterface(XPCCallContext& ccx,
         // wrappedNative or other wise has 'nsISupportness'. 
         // This allows wrapJSAggregatedToNative to work.
 
+        // If we're looking at a security wrapper, see now if we're allowed to
+        // pass it to C++. If we are, then fall through to the code below. If
+        // we aren't, throw an exception eagerly.
+        JSObject* inner = nsnull;
+        if(XPCWrapper::IsSecurityWrapper(src))
+        {
+            inner = XPCWrapper::Unwrap(cx, src);
+            if(!inner)
+            {
+                if(pErr)
+                    *pErr = NS_ERROR_XPC_SECURITY_MANAGER_VETO;
+                return JS_FALSE;
+            }
+        }
+
         // Is this really a native xpcom object with a wrapper?
         XPCWrappedNative* wrappedNative =
-                    XPCWrappedNative::GetWrappedNativeOfJSObject(cx, src);
+                    XPCWrappedNative::GetWrappedNativeOfJSObject(cx,
+                                                                 inner
+                                                                 ? inner
+                                                                 : src);
         if(wrappedNative)
         {
             iface = wrappedNative->GetIdentityObject();
             return NS_SUCCEEDED(iface->QueryInterface(*iid, dest));
         }
         // else...
-        
+
         // XXX E4X breaks the world. Don't try wrapping E4X objects!
         // This hack can be removed (or changed accordingly) when the
         // DOM <-> E4X bindings are complete, see bug 270553
         if(JS_TypeOfValue(cx, OBJECT_TO_JSVAL(src)) == JSTYPE_XML)
             return JS_FALSE;
 
-        // Does the JSObject have 'nsISupportness'?
-        // XXX hmm, I wonder if this matters anymore with no 
-        // oldstyle DOM objects around.
+        // Deal with slim wrappers here.
         if(GetISupportsFromJSObject(src, &iface))
         {
             if(iface)
@@ -1518,23 +1617,23 @@ XPCConvert::ConstructException(nsresult rv, const char* message,
 
 /********************************/
 
-class AutoExceptionRestorer : public JSAutoTempValueRooter
+class AutoExceptionRestorer
 {
 public:
     AutoExceptionRestorer(JSContext *cx, jsval v)
-        : JSAutoTempValueRooter(cx, v),
-          mVal(v)
+        : mContext(cx), tvr(cx, v)
     {
         JS_ClearPendingException(mContext);
     }
 
     ~AutoExceptionRestorer()
     {
-        JS_SetPendingException(mContext, mVal);
+        JS_SetPendingException(mContext, tvr.jsval_value());
     }
 
 private:
-    jsval mVal;
+    JSContext * const mContext;
+    js::AutoValueRooter tvr;
 };
 
 // static
@@ -1555,7 +1654,7 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
 
         if(!obj)
         {
-            NS_ASSERTION(0, "when is an object not an object?");
+            NS_ERROR("when is an object not an object?");
             return NS_ERROR_FAILURE;
         }
 
@@ -1665,7 +1764,7 @@ XPCConvert::JSValToXPCException(XPCCallContext& ccx,
         }
         else
         {
-            number = *(JSVAL_TO_DOUBLE(s));
+            number = JSVAL_TO_DOUBLE(s);
             if(number > 0.0 &&
                number < (double)0xffffffff &&
                0.0 == fmod(number,1))
@@ -1886,7 +1985,7 @@ XPC_JSArgumentFormatter(JSContext *cx, const char *format,
 
 // static
 JSBool
-XPCConvert::NativeArray2JS(XPCCallContext& ccx,
+XPCConvert::NativeArray2JS(XPCLazyCallContext& lccx,
                            jsval* d, const void** s,
                            const nsXPTType& type, const nsID* iid,
                            JSUint32 count, JSObject* scope,
@@ -1895,6 +1994,10 @@ XPCConvert::NativeArray2JS(XPCCallContext& ccx,
     NS_PRECONDITION(s, "bad param");
     NS_PRECONDITION(d, "bad param");
 
+    XPCCallContext& ccx = lccx.GetXPCCallContext();
+    if(!ccx.IsValid())
+        return JS_FALSE;
+
     JSContext* cx = ccx.GetJSContext();
 
     // XXX add support for putting chars in a string rather than an array
@@ -1945,17 +2048,17 @@ XPCConvert::NativeArray2JS(XPCCallContext& ccx,
     case nsXPTType::T_BOOL          : POPULATE(PRBool);         break;
     case nsXPTType::T_CHAR          : POPULATE(char);           break;
     case nsXPTType::T_WCHAR         : POPULATE(jschar);         break;
-    case nsXPTType::T_VOID          : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_VOID          : NS_ERROR("bad type"); goto failure;
     case nsXPTType::T_IID           : POPULATE(nsID*);          break;
-    case nsXPTType::T_DOMSTRING     : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_DOMSTRING     : NS_ERROR("bad type"); goto failure;
     case nsXPTType::T_CHAR_STR      : POPULATE(char*);          break;
     case nsXPTType::T_WCHAR_STR     : POPULATE(jschar*);        break;
     case nsXPTType::T_INTERFACE     : POPULATE(nsISupports*);   break;
     case nsXPTType::T_INTERFACE_IS  : POPULATE(nsISupports*);   break;
-    case nsXPTType::T_UTF8STRING    : NS_ASSERTION(0,"bad type"); goto failure;
-    case nsXPTType::T_CSTRING       : NS_ASSERTION(0,"bad type"); goto failure;
-    case nsXPTType::T_ASTRING       : NS_ASSERTION(0,"bad type"); goto failure;
-    default                         : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_UTF8STRING    : NS_ERROR("bad type"); goto failure;
+    case nsXPTType::T_CSTRING       : NS_ERROR("bad type"); goto failure;
+    case nsXPTType::T_ASTRING       : NS_ERROR("bad type"); goto failure;
+    default                         : NS_ERROR("bad type"); goto failure;
     }
 
     if(pErr)
@@ -2041,7 +2144,7 @@ XPCConvert::JSArray2Native(XPCCallContext& ccx, void** d, jsval s,
 #define POPULATE(_mode, _t)                                                  \
     PR_BEGIN_MACRO                                                           \
         cleanupMode = _mode;                                                 \
-        if (capacity > ~(size_t)0 / sizeof(_t) ||                            \
+        if (capacity > PR_UINT32_MAX / sizeof(_t) ||                         \
             nsnull == (array = nsMemory::Alloc(capacity * sizeof(_t))))      \
         {                                                                    \
             if(pErr)                                                         \
@@ -2078,17 +2181,17 @@ fill_array:
     case nsXPTType::T_BOOL          : POPULATE(na, PRBool);         break;
     case nsXPTType::T_CHAR          : POPULATE(na, char);           break;
     case nsXPTType::T_WCHAR         : POPULATE(na, jschar);         break;
-    case nsXPTType::T_VOID          : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_VOID          : NS_ERROR("bad type"); goto failure;
     case nsXPTType::T_IID           : POPULATE(fr, nsID*);          break;
-    case nsXPTType::T_DOMSTRING     : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_DOMSTRING     : NS_ERROR("bad type"); goto failure;
     case nsXPTType::T_CHAR_STR      : POPULATE(fr, char*);          break;
     case nsXPTType::T_WCHAR_STR     : POPULATE(fr, jschar*);        break;
     case nsXPTType::T_INTERFACE     : POPULATE(re, nsISupports*);   break;
     case nsXPTType::T_INTERFACE_IS  : POPULATE(re, nsISupports*);   break;
-    case nsXPTType::T_UTF8STRING    : NS_ASSERTION(0,"bad type"); goto failure;
-    case nsXPTType::T_CSTRING       : NS_ASSERTION(0,"bad type"); goto failure;
-    case nsXPTType::T_ASTRING       : NS_ASSERTION(0,"bad type"); goto failure;
-    default                         : NS_ASSERTION(0,"bad type"); goto failure;
+    case nsXPTType::T_UTF8STRING    : NS_ERROR("bad type"); goto failure;
+    case nsXPTType::T_CSTRING       : NS_ERROR("bad type"); goto failure;
+    case nsXPTType::T_ASTRING       : NS_ERROR("bad type"); goto failure;
+    default                         : NS_ERROR("bad type"); goto failure;
     }
 
     *d = array;
@@ -2128,7 +2231,7 @@ failure:
 
 // static
 JSBool
-XPCConvert::NativeStringWithSize2JS(XPCCallContext& ccx,
+XPCConvert::NativeStringWithSize2JS(JSContext* cx,
                                     jsval* d, const void* s,
                                     const nsXPTType& type,
                                     JSUint32 count,
@@ -2137,8 +2240,6 @@ XPCConvert::NativeStringWithSize2JS(XPCCallContext& ccx,
     NS_PRECONDITION(s, "bad param");
     NS_PRECONDITION(d, "bad param");
 
-    JSContext* cx = ccx.GetJSContext();
-
     if(pErr)
         *pErr = NS_ERROR_XPC_BAD_CONVERT_NATIVE;
 
@@ -2186,7 +2287,7 @@ XPCConvert::JSStringWithSize2Native(XPCCallContext& ccx, void* d, jsval s,
                                     JSBool useAllocator,
                                     uintN* pErr)
 {
-    NS_PRECONDITION(s, "bad param");
+    NS_PRECONDITION(!JSVAL_IS_NULL(s), "bad param");
     NS_PRECONDITION(d, "bad param");
 
     JSContext* cx = ccx.GetJSContext();
@@ -2278,7 +2379,7 @@ XPCConvert::JSStringWithSize2Native(XPCCallContext& ccx, void* d, jsval s,
 
         case nsXPTType::T_PWSTRING_SIZE_IS:
         {
-            jschar* chars=nsnull;
+            const jschar* chars=nsnull;
             JSString* str;
 
             if(JSVAL_IS_VOID(s) || JSVAL_IS_NULL(s))
@@ -2305,12 +2406,11 @@ XPCConvert::JSStringWithSize2Native(XPCCallContext& ccx, void* d, jsval s,
                 }
 
                 // else ...
-                *((jschar**)d) = nsnull;
+                *((const jschar**)d) = nsnull;
                 return JS_TRUE;
             }
 
-            if(!(str = JS_ValueToString(cx, s))||
-               !(chars = JS_GetStringChars(str)))
+            if(!(str = JS_ValueToString(cx, s)))
             {
                 return JS_FALSE;
             }
@@ -2327,6 +2427,10 @@ XPCConvert::JSStringWithSize2Native(XPCCallContext& ccx, void* d, jsval s,
 
             if(useAllocator)
             {
+                if(!(chars = JS_GetStringChars(str)))
+                {
+                    return JS_FALSE;
+                }
                 JSUint32 alloc_len = (len + 1) * sizeof(jschar);
                 if(!(*((void**)d) = nsMemory::Alloc(alloc_len)))
                 {
@@ -2337,7 +2441,13 @@ XPCConvert::JSStringWithSize2Native(XPCCallContext& ccx, void* d, jsval s,
                 (*((jschar**)d))[count] = 0;
             }
             else
-                *((jschar**)d) = chars;
+            {
+                if(!(chars = JS_GetStringCharsZ(cx, str)))
+                {
+                    return JS_FALSE;
+                }
+                *((const jschar**)d) = chars;
+            }
 
             return JS_TRUE;
         }
