@@ -140,8 +140,11 @@ PropertyOpForwarder(JSContext *cx, uintN argc, jsval *vp)
         return JS_FALSE;
 
     jsval argval = (argc > 0) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
+    jsid id;
+    if (!JS_ValueToId(cx, argval, &id))
+        return JS_FALSE;
     JS_SET_RVAL(cx, vp, argval);
-    return (*popp)(cx, obj, v, vp);
+    return (*popp)(cx, obj, id, vp);
 }
 
 static void
@@ -173,7 +176,7 @@ GeneratePropertyOp(JSContext *cx, JSObject *obj, jsval idval, uintN argc,
 
     JSObject *funobj = JS_GetFunctionObject(fun);
 
-    JSAutoTempValueRooter tvr(cx, OBJECT_TO_JSVAL(funobj));
+    js::AutoObjectRooter tvr(cx, funobj);
 
     // Unfortunately, we cannot guarantee that JSPropertyOp is aligned. Use a
     // second object to work around this.
@@ -198,7 +201,7 @@ ReifyPropertyOps(JSContext *cx, JSObject *obj, jsval idval, jsid interned_id,
 {
     // Generate both getter and setter and stash them in the prototype.
     jsval roots[2] = { JSVAL_NULL, JSVAL_NULL };
-    JSAutoTempValueRooter tvr(cx, 2, roots);
+    js::AutoArrayRooter tvr(cx, JS_ARRAY_LENGTH(roots), roots);
 
     uintN attrs = JSPROP_SHARED;
     JSObject *getterobj;
@@ -236,34 +239,29 @@ ReifyPropertyOps(JSContext *cx, JSObject *obj, jsval idval, jsid interned_id,
 }
 
 static JSBool
-LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, jsval *vp)
+LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, uintN argc, jsval *vp)
 {
-    uintN attrs;
-    JSBool found;
-    JSPropertyOp getter, setter;
-    JSObject *obj2;
-    jsid interned_id;
-    jsval v;
-
     XPC_QS_ASSERT_CONTEXT_OK(cx);
-    JSObject *obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj)
-        return JS_FALSE;
-    jsval idval = JS_ARGV(cx, vp)[0];
 
-    const char *name = JSVAL_IS_STRING(idval)
-                       ? JS_GetStringBytes(JSVAL_TO_STRING(idval))
-                       : nsnull;
+    if(argc == 0)
+    {
+        JS_SET_RVAL(cx, vp, JSVAL_VOID);
+        return JS_TRUE;
+    }
+
+    JSObject *obj = JS_THIS_OBJECT(cx, vp);
+    if(!obj)
+        return JS_FALSE;
+
+    jsval idval = JS_ARGV(cx, vp)[0];
+    jsid interned_id;
+    JSPropertyDescriptor desc;
     if(!JS_ValueToId(cx, idval, &interned_id) ||
-       !JS_LookupPropertyWithFlagsById(cx, obj, interned_id,
-                                       JSRESOLVE_QUALIFIED, &obj2, &v) ||
-       (obj2 &&
-        !JS_GetPropertyAttrsGetterAndSetterById(cx, obj2, interned_id, &attrs,
-                                                &found, &getter, &setter)))
+       !JS_GetPropertyDescriptorById(cx, obj, interned_id, JSRESOLVE_QUALIFIED, &desc))
         return JS_FALSE;
 
     // No property at all means no getters or setters possible.
-    if(!obj2 || !found)
+    if(!desc.obj)
     {
         JS_SET_RVAL(cx, vp, JSVAL_VOID);
         return JS_TRUE;
@@ -272,19 +270,19 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, jsval *vp)
     // Inline obj_lookup[GS]etter here.
     if(wantGetter)
     {
-        if(attrs & JSPROP_GETTER)
+        if(desc.attrs & JSPROP_GETTER)
         {
             JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, getter)));
+                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.getter)));
             return JS_TRUE;
         }
     }
     else
     {
-        if(attrs & JSPROP_SETTER)
+        if(desc.attrs & JSPROP_SETTER)
         {
             JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, setter)));
+                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.setter)));
             return JS_TRUE;
         }
     }
@@ -293,22 +291,27 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, jsval *vp)
     // ensuring that we have an XPConnect prototype object ensures that
     // we are only going to expose quickstubbed properties to script.
     // Also be careful not to overwrite existing properties!
+
+    const char *name = JSVAL_IS_STRING(idval)
+                       ? JS_GetStringBytes(JSVAL_TO_STRING(idval))
+                       : nsnull;
     if(!name ||
-       !IS_PROTO_CLASS(STOBJ_GET_CLASS(obj2)) ||
-       (attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
-       !(getter || setter))
+       !IS_PROTO_CLASS(desc.obj->getClass()) ||
+       (desc.attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
+       !(desc.getter || desc.setter) ||
+       desc.setter == desc.obj->getJSClass()->setProperty)
     {
         JS_SET_RVAL(cx, vp, JSVAL_VOID);
         return JS_TRUE;
     }
 
     JSObject *getterobj, *setterobj;
-    if(!ReifyPropertyOps(cx, obj, idval, interned_id, name, getter, setter,
-                         &getterobj, &setterobj))
+    if(!ReifyPropertyOps(cx, obj, idval, interned_id, name,
+                         desc.getter, desc.setter, &getterobj, &setterobj))
         return JS_FALSE;
 
     JSObject *wantedobj = wantGetter ? getterobj : setterobj;
-    v = wantedobj ? OBJECT_TO_JSVAL(wantedobj) : JSVAL_VOID;
+    jsval v = wantedobj ? OBJECT_TO_JSVAL(wantedobj) : JSVAL_VOID;
     JS_SET_RVAL(cx, vp, v);
     return JS_TRUE;
 }
@@ -316,19 +319,15 @@ LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, jsval *vp)
 static JSBool
 SharedLookupGetter(JSContext *cx, uintN argc, jsval *vp)
 {
-    return LookupGetterOrSetter(cx, PR_TRUE, vp);
+    return LookupGetterOrSetter(cx, PR_TRUE, argc, vp);
 }
 
 static JSBool
 SharedLookupSetter(JSContext *cx, uintN argc, jsval *vp)
 {
-    return LookupGetterOrSetter(cx, PR_FALSE, vp);
+    return LookupGetterOrSetter(cx, PR_FALSE, argc, vp);
 }
 
-// XXX Hack! :-/
-JS_FRIEND_API(JSBool) js_obj_defineGetter(JSContext *cx, uintN argc, jsval *vp);
-JS_FRIEND_API(JSBool) js_obj_defineSetter(JSContext *cx, uintN argc, jsval *vp);
-
 static JSBool
 DefineGetterOrSetter(JSContext *cx, uintN argc, JSBool wantGetter, jsval *vp)
 {
@@ -343,7 +342,8 @@ DefineGetterOrSetter(JSContext *cx, uintN argc, JSBool wantGetter, jsval *vp)
     JSObject *obj = JS_THIS_OBJECT(cx, vp);
     if (!obj)
         return JS_FALSE;
-    JSFastNative forward = wantGetter ? js_obj_defineGetter : js_obj_defineSetter;
+    JSFastNative forward = wantGetter ? Jsvalify(js_obj_defineGetter)
+                                      : Jsvalify(js_obj_defineSetter);
     jsval id = (argc >= 1) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
     if(!JSVAL_IS_STRING(id))
         return forward(cx, argc, vp);
@@ -363,7 +363,7 @@ DefineGetterOrSetter(JSContext *cx, uintN argc, JSBool wantGetter, jsval *vp)
     if(!obj2 ||
        (attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
        !(getter || setter) ||
-       !IS_PROTO_CLASS(STOBJ_GET_CLASS(obj2)))
+       !IS_PROTO_CLASS(obj2->getClass()))
         return forward(cx, argc, vp);
 
     // Reify the getter and setter...
@@ -447,7 +447,7 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, uintN flags,
                         if(!JS_DefineFunction(
                                cx, proto, ts->name, ts->native, ts->arity,
                                flags | JSFUN_FAST_NATIVE | JSFUN_STUB_GSOPS |
-                                       JSFUN_TRACEABLE))
+                                       JSFUN_TRCINFO))
                             return JS_FALSE;
                     }
                 }
@@ -493,7 +493,7 @@ xpc_qsThrow(JSContext *cx, nsresult rv)
  */
 static void
 GetMemberInfo(JSObject *obj,
-              jsval memberId,
+              jsid memberId,
               const char **ifaceName,
               const char **memberName)
 {
@@ -504,9 +504,8 @@ GetMemberInfo(JSObject *obj,
     // but this code often produces a more specific error message, e.g.
     *ifaceName = "Unknown";
 
-    NS_ASSERTION(IS_WRAPPER_CLASS(STOBJ_GET_CLASS(obj)) ||
-                 STOBJ_GET_CLASS(obj) == &XPC_WN_Tearoff_JSClass ||
-                 IS_SLIM_WRAPPER(obj),
+    NS_ASSERTION(IS_WRAPPER_CLASS(obj->getClass()) ||
+                 obj->getClass() == &XPC_WN_Tearoff_JSClass,
                  "obj must be a wrapper");
     XPCWrappedNativeProto *proto;
     if(IS_SLIM_WRAPPER(obj))
@@ -515,7 +514,7 @@ GetMemberInfo(JSObject *obj,
     }
     else
     {
-        XPCWrappedNative *wrapper = (XPCWrappedNative *) STOBJ_GET_PRIVATE(obj);
+        XPCWrappedNative *wrapper = (XPCWrappedNative *) obj->getPrivate();
         proto = wrapper->GetProto();
     }
     if(proto)
@@ -531,8 +530,8 @@ GetMemberInfo(JSObject *obj,
         }
     }
 
-    *memberName = (JSVAL_IS_STRING(memberId)
-                   ? JS_GetStringBytes(JSVAL_TO_STRING(memberId))
+    *memberName = (JSID_IS_STRING(memberId)
+                   ? JS_GetStringBytes(JSID_TO_STRING(memberId))
                    : "unknown");
 }
 
@@ -546,7 +545,7 @@ GetMethodInfo(JSContext *cx,
     NS_ASSERTION(JS_ObjectIsFunction(cx, funobj),
                  "JSFastNative callee should be Function object");
     JSString *str = JS_GetFunctionId((JSFunction *) JS_GetPrivate(cx, funobj));
-    jsval methodId = str ? STRING_TO_JSVAL(str) : JSVAL_NULL;
+    jsid methodId = str ? INTERNED_STRING_TO_JSID(str) : JSID_VOID;
 
     GetMemberInfo(JSVAL_TO_OBJECT(vp[1]), methodId, ifaceName, memberName);
 }
@@ -600,7 +599,7 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
 
 JSBool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv, JSObject *obj,
-                              jsval memberId)
+                              jsid memberId)
 {
     const char *ifaceName, *memberName;
     GetMemberInfo(obj, memberId, &ifaceName, &memberName);
@@ -673,33 +672,58 @@ xpc_qsThrowBadArgWithDetails(JSContext *cx, nsresult rv, uintN paramnum,
 
 void
 xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv,
-                          JSObject *obj, jsval propId)
+                          JSObject *obj, jsid propId)
 {
     const char *ifaceName, *memberName;
     GetMemberInfo(obj, propId, &ifaceName, &memberName);
     ThrowBadArg(cx, rv, ifaceName, memberName, 0);
 }
 
-xpc_qsDOMString::xpc_qsDOMString(JSContext *cx, jsval *pval)
+JSBool
+xpc_qsGetterOnlyPropertyStub(JSContext *cx, JSObject *obj, jsid id, jsval *vp)
+{
+    return JS_ReportErrorFlagsAndNumber(cx,
+                                        JSREPORT_WARNING | JSREPORT_STRICT |
+                                        JSREPORT_STRICT_MODE_ERROR,
+                                        js_GetErrorMessage, NULL,
+                                        JSMSG_GETTER_ONLY);
+}
+
+xpc_qsDOMString::xpc_qsDOMString(JSContext *cx, jsval v, jsval *pval,
+                                 StringificationBehavior nullBehavior,
+                                 StringificationBehavior undefinedBehavior)
 {
     // From the T_DOMSTRING case in XPCConvert::JSData2Native.
     typedef implementation_type::char_traits traits;
-    jsval v;
     JSString *s;
     const PRUnichar *chars;
     size_t len;
 
-    v = *pval;
     if(JSVAL_IS_STRING(v))
     {
         s = JSVAL_TO_STRING(v);
     }
     else
     {
+        StringificationBehavior behavior = eStringify;
         if(JSVAL_IS_NULL(v))
         {
+            behavior = nullBehavior;
+        }
+        else if(JSVAL_IS_VOID(v))
+        {
+            behavior = undefinedBehavior;
+        }
+
+        // If pval is null, that means the argument was optional and
+        // not passed; turn those into void strings if they're
+        // supposed to be stringified.
+        if (behavior != eStringify || !pval)
+        {
+            // Here behavior == eStringify implies !pval, so both eNull and
+            // eStringify should end up with void strings.
             (new(mBuf) implementation_type(
-                traits::sEmptyBuffer, PRUint32(0)))->SetIsVoid(PR_TRUE);
+                traits::sEmptyBuffer, PRUint32(0)))->SetIsVoid(behavior != eEmpty);
             mValid = JS_TRUE;
             return;
         }
@@ -713,58 +737,18 @@ xpc_qsDOMString::xpc_qsDOMString(JSContext *cx, jsval *pval)
         *pval = STRING_TO_JSVAL(s);  // Root the new string.
     }
 
-    len = JS_GetStringLength(s);
-    chars = (len == 0 ? traits::sEmptyBuffer : (const PRUnichar*)JS_GetStringChars(s));
+    len = s->length();
+    chars = (len == 0 ? traits::sEmptyBuffer :
+                        reinterpret_cast<const PRUnichar*>(JS_GetStringChars(s)));
     new(mBuf) implementation_type(chars, len);
     mValid = JS_TRUE;
 }
 
-xpc_qsAString::xpc_qsAString(JSContext *cx, jsval *pval)
-{
-    // From the T_ASTRING case in XPCConvert::JSData2Native.
-    typedef implementation_type::char_traits traits;
-    jsval v;
-    JSString *s;
-    const PRUnichar *chars;
-    size_t len;
-
-    v = *pval;
-    if(JSVAL_IS_STRING(v))
-    {
-        s = JSVAL_TO_STRING(v);
-    }
-    else
-    {
-        if(JSVAL_IS_NULL(v) || JSVAL_IS_VOID(v))
-        {
-            (new(mBuf) implementation_type(
-                traits::sEmptyBuffer, PRUint32(0)))->SetIsVoid(PR_TRUE);
-            mValid = JS_TRUE;
-            return;
-        }
-
-        s = JS_ValueToString(cx, v);
-        if(!s)
-        {
-            mValid = JS_FALSE;
-            return;
-        }
-        *pval = STRING_TO_JSVAL(s);  // Root the new string.
-    }
-
-    len = JS_GetStringLength(s);
-    chars = (len == 0 ? traits::sEmptyBuffer : (const PRUnichar*)JS_GetStringChars(s));
-    new(mBuf) implementation_type(chars, len);
-    mValid = JS_TRUE;
-}
-
-xpc_qsACString::xpc_qsACString(JSContext *cx, jsval *pval)
+xpc_qsACString::xpc_qsACString(JSContext *cx, jsval v, jsval *pval)
 {
     // From the T_CSTRING case in XPCConvert::JSData2Native.
-    jsval v;
     JSString *s;
 
-    v = *pval;
     if(JSVAL_IS_STRING(v))
     {
         s = JSVAL_TO_STRING(v);
@@ -788,7 +772,7 @@ xpc_qsACString::xpc_qsACString(JSContext *cx, jsval *pval)
     }
 
     const char *bytes = JS_GetStringBytes(s);
-    size_t len = JS_GetStringLength(s);
+    size_t len = s->length();
     new(mBuf) implementation_type(bytes, len);
     mValid = JS_TRUE;
 }
@@ -824,7 +808,7 @@ getNative(nsISupports *idobj,
     return rv;
 }
 
-static nsresult
+inline nsresult
 getNativeFromWrapper(XPCWrappedNative *wrapper,
                      const nsIID &iid,
                      void **ppThis,
@@ -836,100 +820,66 @@ getNativeFromWrapper(XPCWrappedNative *wrapper,
 }
 
 
-JSBool
-xpc_qsUnwrapThisImpl(JSContext *cx,
-                     JSObject *obj,
-                     const nsIID &iid,
-                     void **ppThis,
-                     nsISupports **pThisRef,
-                     jsval *vp)
+nsresult
+getWrapper(JSContext *cx,
+           JSObject *obj,
+           JSObject *callee,
+           XPCWrappedNative **wrapper,
+           JSObject **cur,
+           XPCWrappedNativeTearOff **tearoff)
 {
-    // From XPCWrappedNative::GetWrappedNativeOfJSObject.
-    //
-    // Usually IS_WRAPPER_CLASS is true the first time through the while loop,
-    // and the QueryInterface then succeeds.
-
-    NS_ASSERTION(obj, "this == null");
-
-    JSObject *cur = obj;
-    while(cur)
+    if(XPCWrapper::IsSecurityWrapper(obj) &&
+       !(obj = XPCWrapper::Unwrap(cx, obj)))
     {
-        JSClass *clazz;
-
-        clazz = STOBJ_GET_CLASS(cur);
-        if(IS_SLIM_WRAPPER_CLASS(clazz))
-        {
-            nsISupports *native =
-                static_cast<nsISupports*>(xpc_GetJSPrivate(cur));
-            if(NS_SUCCEEDED(getNative(native, GetOffsetsFromSlimWrapper(cur),
-                                      cur, iid, ppThis, pThisRef, vp)))
-                return JS_TRUE;
-            *pThisRef = nsnull;
-            return xpc_qsThrow(cx, NS_ERROR_XPC_BAD_OP_ON_WN_PROTO);
-        }
-
-        XPCWrappedNative *wrapper;
-        nsresult rv;
-
-        if(IS_WRAPPER_CLASS(clazz))
-        {
-            wrapper = (XPCWrappedNative*) xpc_GetJSPrivate(cur);
-            NS_ASSERTION(wrapper, "XPCWN wrapping nothing");
-        }
-        else if(clazz == &XPC_WN_Tearoff_JSClass)
-        {
-            wrapper = (XPCWrappedNative*) xpc_GetJSPrivate(STOBJ_GET_PARENT(cur));
-            NS_ASSERTION(wrapper, "XPCWN wrapping nothing");
-        }
-        else
-        {
-            JSObject *unsafeObj = XPCWrapper::Unwrap(cx, cur);
-            if(unsafeObj)
-            {
-                cur = unsafeObj;
-                continue;
-            }
-
-            // This goto is a bug, dutifully copied from
-            // XPCWrappedNative::GetWrappedNativeOfJSObject.
-            goto next;
-        }
-
-        rv = getNativeFromWrapper(wrapper, iid, ppThis, pThisRef, vp);
-        if(NS_SUCCEEDED(rv))
-            return JS_TRUE;
-        if(rv != NS_ERROR_NO_INTERFACE)
-            return xpc_qsThrow(cx, rv);
-
-    next:
-        cur = STOBJ_GET_PROTO(cur);
+        return NS_ERROR_XPC_SECURITY_MANAGER_VETO;
     }
 
-    // If we didn't find a wrapper using the given obj, try again with obj's
-    // outer object, if it's got one.
+    *cur = obj;
+    *tearoff = nsnull;
 
-    JSClass *clazz = STOBJ_GET_CLASS(obj);
+    *wrapper =
+        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, obj, callee, cur,
+                                                     tearoff);
 
-    if((clazz->flags & JSCLASS_IS_EXTENDED) &&
-        ((JSExtendedClass*)clazz)->outerObject)
+    return NS_OK;
+}
+
+nsresult
+castNative(JSContext *cx,
+           XPCWrappedNative *wrapper,
+           JSObject *cur,
+           XPCWrappedNativeTearOff *tearoff,
+           const nsIID &iid,
+           void **ppThis,
+           nsISupports **pThisRef,
+           jsval *vp,
+           XPCLazyCallContext *lccx)
+{
+    if(wrapper)
     {
-        JSObject *outer = ((JSExtendedClass*)clazz)->outerObject(cx, obj);
+        nsresult rv = getNativeFromWrapper(wrapper, iid, ppThis, pThisRef, vp);
 
-        // Protect against infinite recursion through XOWs.
-        JSObject *unsafeObj;
-        clazz = STOBJ_GET_CLASS(outer);
-        if(clazz == &sXPC_XOW_JSClass.base &&
-           (unsafeObj = XPCWrapper::Unwrap(cx, outer)))
+        if(lccx && NS_SUCCEEDED(rv))
+            lccx->SetWrapper(wrapper, tearoff);
+
+        if(rv != NS_ERROR_NO_INTERFACE)
+            return rv;
+    }
+    else if(cur)
+    {
+        nsISupports *native = static_cast<nsISupports*>(xpc_GetJSPrivate(cur));
+        if(NS_SUCCEEDED(getNative(native, GetOffsetsFromSlimWrapper(cur),
+                                  cur, iid, ppThis, pThisRef, vp)))
         {
-            outer = unsafeObj;
-        }
+            if(lccx)
+                lccx->SetWrapper(cur);
 
-        if(outer && outer != obj)
-            return xpc_qsUnwrapThisImpl(cx, outer, iid, ppThis, pThisRef, vp);
+            return NS_OK;
+        }
     }
 
     *pThisRef = nsnull;
-    return xpc_qsThrow(cx, NS_ERROR_XPC_BAD_OP_ON_WN_PROTO);
+    return NS_ERROR_XPC_BAD_OP_ON_WN_PROTO;
 }
 
 JSBool
@@ -951,6 +901,29 @@ xpc_qsUnwrapThisFromCcxImpl(XPCCallContext &ccx,
     return JS_TRUE;
 }
 
+JSObject*
+xpc_qsUnwrapObj(jsval v, nsISupports **ppArgRef, nsresult *rv)
+{
+    if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
+    {
+        *ppArgRef = nsnull;
+        *rv = NS_OK;
+        return nsnull;
+    }
+
+    if(!JSVAL_IS_OBJECT(v))
+    {
+        *ppArgRef = nsnull;
+        *rv = ((JSVAL_IS_INT(v) && JSVAL_TO_INT(v) == 0)
+              ? NS_ERROR_XPC_BAD_CONVERT_JS_ZERO_ISNOT_NULL
+              : NS_ERROR_XPC_BAD_CONVERT_JS);
+        return nsnull;
+    }
+
+    *rv = NS_OK;
+    return JSVAL_TO_OBJECT(v);
+}
+
 nsresult
 xpc_qsUnwrapArgImpl(JSContext *cx,
                     jsval v,
@@ -959,41 +932,25 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
                     nsISupports **ppArgRef,
                     jsval *vp)
 {
-    // From XPCConvert::JSData2Native
-    if(JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v))
+    nsresult rv;
+    JSObject *src = xpc_qsUnwrapObj(v, ppArgRef, &rv);
+    if(!src)
     {
         *ppArg = nsnull;
-        *ppArgRef = nsnull;
-        return NS_OK;
+
+        return rv;
     }
 
-    if(!JSVAL_IS_OBJECT(v))
-    {
-        *ppArgRef = nsnull;
-        return ((JSVAL_IS_INT(v) && JSVAL_TO_INT(v) == 0)
-                ? NS_ERROR_XPC_BAD_CONVERT_JS_ZERO_ISNOT_NULL
-                : NS_ERROR_XPC_BAD_CONVERT_JS);
-    }
-    JSObject *src = JSVAL_TO_OBJECT(v);
+    XPCWrappedNative *wrapper;
+    XPCWrappedNativeTearOff *tearoff;
+    JSObject *obj2;
+    rv = getWrapper(cx, src, nsnull, &wrapper, &obj2, &tearoff);
+    NS_ENSURE_SUCCESS(rv, rv);
 
-    if(IS_SLIM_WRAPPER(src))
+    if(wrapper || obj2)
     {
-        nsISupports *iface = static_cast<nsISupports*>(xpc_GetJSPrivate(src));
-        if(NS_FAILED(getNative(iface, GetOffsetsFromSlimWrapper(src),
-                               src, iid, ppArg, ppArgRef, vp)))
-            return NS_ERROR_XPC_BAD_CONVERT_JS;
-        return NS_OK;
-    }
-
-    // From XPCConvert::JSObject2NativeInterface
-    XPCWrappedNative* wrappedNative =
-        XPCWrappedNative::GetWrappedNativeOfJSObject(cx, src);
-    nsISupports *iface;
-    if(wrappedNative)
-    {
-        iface = wrappedNative->GetIdentityObject();
-        if(NS_FAILED(getNativeFromWrapper(wrappedNative, iid, ppArg, ppArgRef,
-                                          vp)))
+        if(NS_FAILED(castNative(cx, wrapper, obj2, tearoff, iid, ppArg,
+                                ppArgRef, vp, nsnull)))
             return NS_ERROR_XPC_BAD_CONVERT_JS;
         return NS_OK;
     }
@@ -1009,9 +966,8 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
         return NS_ERROR_XPC_BAD_CONVERT_JS;
     }
 
-    // Does the JSObject have 'nsISupportness'?
-    // XXX hmm, I wonder if this matters anymore with no
-    // oldstyle DOM objects around.
+    // Try to unwrap a slim wrapper.
+    nsISupports *iface;
     if(XPCConvert::GetISupportsFromJSObject(src, &iface))
     {
         if(!iface || NS_FAILED(iface->QueryInterface(iid, ppArg)))
@@ -1032,10 +988,10 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
         return NS_ERROR_XPC_BAD_CONVERT_JS;
     }
 
-    nsXPCWrappedJS *wrapper;
-    nsresult rv =
-        nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nsnull, &wrapper);
-    if(NS_FAILED(rv) || !wrapper)
+    nsRefPtr<nsXPCWrappedJS> wrappedJS;
+    rv = nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nsnull,
+                                      getter_AddRefs(wrappedJS));
+    if(NS_FAILED(rv) || !wrappedJS)
     {
         *ppArgRef = nsnull;
         return rv;
@@ -1045,20 +1001,18 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
     // the right thing for the various 'special' interfaces; e.g.
     // nsIPropertyBag. We must use AggregatedQueryInterface in cases where
     // there is an outer to avoid nasty recursion.
-    rv = wrapper->QueryInterface(iid, ppArg);
+    rv = wrappedJS->QueryInterface(iid, ppArg);
     if(NS_SUCCEEDED(rv))
     {
         *ppArgRef = static_cast<nsISupports*>(*ppArg);
-        *vp = OBJECT_TO_JSVAL(wrapper->GetJSObject());
+        *vp = OBJECT_TO_JSVAL(wrappedJS->GetJSObject());
     }
-    NS_RELEASE(wrapper);
     return rv;
 }
 
 JSBool
-xpc_qsJsvalToCharStr(JSContext *cx, jsval *pval, char **pstr)
+xpc_qsJsvalToCharStr(JSContext *cx, jsval v, jsval *pval, char **pstr)
 {
-    jsval v = *pval;
     JSString *str;
 
     if(JSVAL_IS_STRING(v))
@@ -1082,9 +1036,8 @@ xpc_qsJsvalToCharStr(JSContext *cx, jsval *pval, char **pstr)
 }
 
 JSBool
-xpc_qsJsvalToWcharStr(JSContext *cx, jsval *pval, PRUnichar **pstr)
+xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, jsval *pval, PRUnichar **pstr)
 {
-    jsval v = *pval;
     JSString *str;
 
     if(JSVAL_IS_STRING(v))
@@ -1103,12 +1056,13 @@ xpc_qsJsvalToWcharStr(JSContext *cx, jsval *pval, PRUnichar **pstr)
         *pval = STRING_TO_JSVAL(str);  // Root the new string.
     }
 
+    // XXXbz this is casting away constness too...  That seems like a bad idea.
     *pstr = (PRUnichar*)JS_GetStringChars(str);
     return JS_TRUE;
 }
 
 JSBool
-xpc_qsStringToJsval(JSContext *cx, const nsAString &str, jsval *rval)
+xpc_qsStringToJsval(JSContext *cx, nsString &str, jsval *rval)
 {
     // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
     if(str.IsVoid())
@@ -1117,49 +1071,84 @@ xpc_qsStringToJsval(JSContext *cx, const nsAString &str, jsval *rval)
         return JS_TRUE;
     }
 
-    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str);
-    if(!jsstr)
+    nsStringBuffer* sharedBuffer;
+    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
+    if (JSVAL_IS_NULL(jsstr))
         return JS_FALSE;
     *rval = jsstr;
+    if (sharedBuffer)
+    {
+        // The string was shared but ReadableToJSVal didn't addref it.
+        // Move the ownership from str to jsstr.
+        str.ForgetSharedBuffer();
+    }
     return JS_TRUE;
 }
 
 JSBool
-xpc_qsXPCOMObjectToJsval(XPCCallContext &ccx, nsISupports *p,
-                         nsWrapperCache *cache, XPCNativeInterface *iface,
+xpc_qsStringToJsstring(JSContext *cx, nsString &str, JSString **rval)
+{
+    // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
+    if(str.IsVoid())
+    {
+        *rval = nsnull;
+        return JS_TRUE;
+    }
+
+    nsStringBuffer* sharedBuffer;
+    jsval jsstr = XPCStringConvert::ReadableToJSVal(cx, str, &sharedBuffer);
+    if(JSVAL_IS_NULL(jsstr))
+        return JS_FALSE;
+    *rval = JSVAL_TO_STRING(jsstr);
+    if (sharedBuffer)
+    {
+        // The string was shared but ReadableToJSVal didn't addref it.
+        // Move the ownership from str to jsstr.
+        str.ForgetSharedBuffer();
+    }
+    return JS_TRUE;
+}
+
+JSBool
+xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, qsObjectHelper* aHelper,
+                         nsWrapperCache *cache,
+                         const nsIID *iid, XPCNativeInterface **iface,
                          jsval *rval)
 {
+    NS_PRECONDITION(iface, "Who did that and why?");
+
     // From the T_INTERFACE case in XPCConvert::NativeData2JS.
     // This is one of the slowest things quick stubs do.
 
-    JSObject *scope = ccx.GetCurrentJSObject();
-    NS_ASSERTION(scope, "bad ccx");
-
-    if(!iface)
-        return xpc_qsThrow(ccx, NS_ERROR_XPC_BAD_CONVERT_NATIVE);
+    JSContext *cx = lccx.GetJSContext();
 
     // XXX The OBJ_IS_NOT_GLOBAL here is not really right. In
     // fact, this code is depending on the fact that the
     // global object will not have been collected, and
     // therefore this NativeInterface2JSObject will not end up
     // creating a new XPCNativeScriptableShared.
+
     nsresult rv;
-    if(!XPCConvert::NativeInterface2JSObject(ccx, rval, nsnull, p, nsnull,
-                                             iface, cache, scope, PR_TRUE,
-                                             OBJ_IS_NOT_GLOBAL, &rv))
+    if(!XPCConvert::NativeInterface2JSObject(lccx, rval, nsnull,
+                                             aHelper->Object(),
+                                             iid, iface,
+                                             cache,
+                                             lccx.GetCurrentJSObject(), PR_TRUE,
+                                             OBJ_IS_NOT_GLOBAL, &rv,
+                                             aHelper))
     {
         // I can't tell if NativeInterface2JSObject throws JS exceptions
         // or not.  This is a sloppy stab at the right semantics; the
         // method really ought to be fixed to behave consistently.
-        if(!JS_IsExceptionPending(ccx))
-            xpc_qsThrow(ccx, NS_FAILED(rv) ? rv : NS_ERROR_UNEXPECTED);
+        if(!JS_IsExceptionPending(cx))
+            xpc_qsThrow(cx, NS_FAILED(rv) ? rv : NS_ERROR_UNEXPECTED);
         return JS_FALSE;
     }
 
 #ifdef DEBUG
     JSObject* jsobj = JSVAL_TO_OBJECT(*rval);
-    if(jsobj && !STOBJ_GET_PARENT(jsobj))
-        NS_ASSERTION(STOBJ_GET_CLASS(jsobj)->flags & JSCLASS_IS_GLOBAL,
+    if(jsobj && !jsobj->getParent())
+        NS_ASSERTION(jsobj->getClass()->flags & JSCLASS_IS_GLOBAL,
                      "Why did we recreate this wrapper?");
 #endif
 
@@ -1167,9 +1156,8 @@ xpc_qsXPCOMObjectToJsval(XPCCallContext &ccx, nsISupports *p,
 }
 
 JSBool
-xpc_qsVariantToJsval(XPCCallContext &ccx,
+xpc_qsVariantToJsval(XPCLazyCallContext &lccx,
                      nsIVariant *p,
-                     uintN paramNum,
                      jsval *rval)
 {
     // From the T_INTERFACE case in XPCConvert::NativeData2JS.
@@ -1177,11 +1165,11 @@ xpc_qsVariantToJsval(XPCCallContext &ccx,
     if(p)
     {
         nsresult rv;
-        JSBool ok = XPCVariant::VariantDataToJS(ccx, p,
-                                                ccx.GetCurrentJSObject(),
+        JSBool ok = XPCVariant::VariantDataToJS(lccx, p,
+                                                lccx.GetCurrentJSObject(),
                                                 &rv, rval);
         if (!ok)
-            XPCThrower::ThrowBadParam(rv, 0, ccx);
+            xpc_qsThrow(lccx.GetJSContext(), rv);
         return ok;
     }
     *rval = JSVAL_NULL;
