@@ -53,6 +53,10 @@
 #include "nsHTMLContainerFrame.h"
 #include "nsInlineFrame.h"
 #include "nsPlaceholderFrame.h"
+#include "nsContainerFrame.h"
+#include "nsFirstLetterFrame.h"
+
+using namespace mozilla;
 
 static const PRUnichar kSpace            = 0x0020;
 static const PRUnichar kLineSeparator    = 0x2028;
@@ -117,7 +121,6 @@ SplitInlineAncestors(nsIFrame*     aFrame)
   nsIPresShell *presShell = presContext->PresShell();
   nsIFrame* frame = aFrame;
   nsIFrame* parent = aFrame->GetParent();
-  nsIFrame* newFrame = aFrame->GetNextSibling();
   nsIFrame* newParent;
 
   while (IsBidiSplittable(parent)) {
@@ -130,27 +133,29 @@ SplitInlineAncestors(nsIFrame*     aFrame)
       return rv;
     }
     
-    // The new parent adopts the new frame
-    frame->SetNextSibling(nsnull);
-    rv = newParent->InsertFrames(nsGkAtoms::nextBidi, nsnull, newFrame);
-    if (NS_FAILED(rv)) {
-      return rv;
-    }
+    // Split the child list after |frame|.
+    nsContainerFrame* container = do_QueryFrame(parent);
+    nsFrameList tail = container->StealFramesAfter(frame);
 
     // Reparent views as necessary
-    rv = nsHTMLContainerFrame::ReparentFrameViewList(presContext, newFrame, parent, newParent);
+    rv = nsHTMLContainerFrame::ReparentFrameViewList(presContext, tail, parent, newParent);
     if (NS_FAILED(rv)) {
       return rv;
     }
     
+    // The parent's continuation adopts the siblings after the split.
+    rv = newParent->InsertFrames(nsGkAtoms::nextBidi, nsnull, tail);
+    if (NS_FAILED(rv)) {
+      return rv;
+    }
     // The list name nsGkAtoms::nextBidi would indicate we don't want reflow
-    rv = grandparent->InsertFrames(nsGkAtoms::nextBidi, parent, newParent);
+    nsFrameList temp(newParent, newParent);
+    rv = grandparent->InsertFrames(nsGkAtoms::nextBidi, parent, temp);
     if (NS_FAILED(rv)) {
       return rv;
     }
     
     frame = parent;
-    newFrame = newParent;
     parent = grandparent;
   }
   
@@ -196,15 +201,30 @@ CreateBidiContinuation(nsIFrame*       aFrame,
 
   nsIFrame* parent = aFrame->GetParent();
   NS_ASSERTION(parent, "Couldn't get frame parent in nsBidiPresUtils::CreateBidiContinuation");
+
+  nsresult rv = NS_OK;
   
-  nsresult rv = presShell->FrameConstructor()->
+  // Have to special case floating first letter frames because the continuation
+  // doesn't go in the first letter frame. The continuation goes with the rest
+  // of the text that the first letter frame was made out of.
+  if (parent->GetType() == nsGkAtoms::letterFrame &&
+      parent->GetStyleDisplay()->IsFloating()) {
+    nsFirstLetterFrame* letterFrame = do_QueryFrame(parent);
+    rv = letterFrame->CreateContinuationForFloatingParent(presContext, aFrame,
+                                                          aNewFrame, PR_FALSE);
+    return rv;
+  }
+
+  rv = presShell->FrameConstructor()->
     CreateContinuingFrame(presContext, aFrame, parent, aNewFrame, PR_FALSE);
   if (NS_FAILED(rv)) {
     return rv;
   }
-  
+
   // The list name nsGkAtoms::nextBidi would indicate we don't want reflow
-  rv = parent->InsertFrames(nsGkAtoms::nextBidi, aFrame, *aNewFrame);
+  // XXXbz this needs higher-level framelist love
+  nsFrameList temp(*aNewFrame, *aNewFrame);
+  rv = parent->InsertFrames(nsGkAtoms::nextBidi, aFrame, temp);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -225,7 +245,12 @@ IsFrameInCurrentLine(nsBlockInFlowLineIterator* aLineIter,
   nsIFrame* endFrame = aLineIter->IsLastLineInList() ? nsnull :
     aLineIter->GetLine().next()->mFirstChild;
   nsIFrame* startFrame = aPrevFrame ? aPrevFrame : aLineIter->GetLine()->mFirstChild;
-  return nsFrameList(startFrame).ContainsFrameBefore(aFrame, endFrame);
+  for (nsIFrame* frame = startFrame; frame && frame != endFrame;
+       frame = frame->GetNextSibling()) {
+    if (frame == aFrame)
+      return PR_TRUE;
+  }
+  return PR_FALSE;
 }
 
 static void
@@ -275,18 +300,16 @@ AdvanceLineIteratorToFrame(nsIFrame* aFrame,
  *
  *  Finally, walk these runs in logical order using nsBidi::GetLogicalRun() and
  *  correlate them with the frames indexed in mLogicalFrames, setting the
- *  baseLevel, embeddingLevel, and charType properties according to the results
- *  returned by the Bidi engine and CalculateCharType().
+ *  baseLevel and embeddingLevel properties according to the results returned
+ *  by the Bidi engine.
  *
  *  The rendering layer requires each text frame to contain text in only one
- *  direction and of only one character type, so we may need to call
- *  EnsureBidiContinuation() to split frames. We may also need to call
- *  RemoveBidiContinuation() to convert frames created by
- *  EnsureBidiContinuation() in previous reflows into fluid continuations.
+ *  direction, so we may need to call EnsureBidiContinuation() to split frames.
+ *  We may also need to call RemoveBidiContinuation() to convert frames created
+ *  by EnsureBidiContinuation() in previous reflows into fluid continuations.
  */
 nsresult
-nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
-                         PRBool          aIsVisualFormControl)
+nsBidiPresUtils::Resolve(nsBlockFrame* aBlockFrame)
 {
   mLogicalFrames.Clear();
   mContentToFrameIndex.Clear();
@@ -347,12 +370,6 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
       return mSuccess;
   }
 
-  PRBool isVisual;
-  if (aIsVisualFormControl) {
-    isVisual = PR_FALSE;
-  } else {
-    isVisual = presContext->IsVisualMode();
-  }
   mSuccess = mBidiEngine->CountRuns(&runCount);
   if (NS_FAILED(mSuccess) ) {
     return mSuccess;
@@ -365,15 +382,13 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
   PRInt32     frameIndex     = -1;  // index to the frames in mLogicalFrames
   PRInt32     frameCount     = mLogicalFrames.Length();
   PRInt32     contentOffset  = 0;   // offset of current frame in its content node
-  PRUint8     charType;
-  PRUint8     prevType       = eCharType_LeftToRight;
   PRBool      isTextFrame    = PR_FALSE;
   nsIFrame*   frame = nsnull;
   nsIContent* content = nsnull;
   PRInt32     contentTextLength;
   nsIAtom*    frameType = nsnull;
 
-  nsPropertyTable *propTable = presContext->PropertyTable();
+  FramePropertyTable *propTable = presContext->PropertyTable();
 
   nsBlockInFlowLineIterator lineIter(aBlockFrame, aBlockFrame->begin_lines(), PR_FALSE);
   if (lineIter.GetLine() == aBlockFrame->end_lines()) {
@@ -382,6 +397,30 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
   }
   nsIFrame* prevFrame = nsnull;
   PRBool lineNeedsUpdate = PR_FALSE;
+
+  PRBool isVisual = presContext->IsVisualMode();
+  if (isVisual) {
+    /**
+     * Drill up in content to detect whether this is an element that needs to be
+     * rendered with logical order even on visual pages.
+     *
+     * We always use logical order on form controls, firstly so that text entry
+     * will be in logical order, but also because visual pages were written with
+     * the assumption that even if the browser had no support for right-to-left
+     * text rendering, it would use native widgets with bidi support to display
+     * form controls.
+     *
+     * We also use logical order in XUL elements, since we expect that if a XUL
+     * element appears in a visual page, it will be generated by an XBL binding
+     * and contain localized text which will be in logical order.
+     */
+    for (content = aBlockFrame->GetContent() ; content; content = content->GetParent()) {
+      if (content->IsNodeOfType(nsINode::eHTML_FORM_CONTROL) || content->IsXUL()) {
+        isVisual = PR_FALSE;
+        break;
+      }
+    }
+  }
   
   for (; ;) {
     if (fragmentLength <= 0) {
@@ -403,16 +442,17 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
           frame->AdjustOffsetsForBidi(0, 0);
           // Set the base level and embedding level of the current run even
           // on an empty frame. Otherwise frame reordering will not be correct.
-          propTable->SetProperty(frame, nsGkAtoms::embeddingLevel,
-                                 NS_INT32_TO_PTR(embeddingLevel),
-                                 nsnull, nsnull);
-          propTable->SetProperty(frame, nsGkAtoms::baseLevel,
-                                 NS_INT32_TO_PTR(paraLevel), nsnull, nsnull);
+          propTable->Set(frame, nsIFrame::EmbeddingLevelProperty(),
+                         NS_INT32_TO_PTR(embeddingLevel));
+          propTable->Set(frame, nsIFrame::BaseLevelProperty(),
+                         NS_INT32_TO_PTR(paraLevel));
           continue;
         }
         PRInt32 start, end;
         frame->GetOffsets(start, end);
-        fragmentLength = end - start;
+        NS_ASSERTION(!(contentTextLength < end - start),
+                     "Frame offsets don't fit in content");
+        fragmentLength = NS_MIN(contentTextLength, end - start);
         contentOffset = start;
         isTextFrame = PR_TRUE;
       }
@@ -448,19 +488,11 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
       ++lineOffset;
     }
     else {
-      propTable->SetProperty(frame, nsGkAtoms::embeddingLevel,
-                             NS_INT32_TO_PTR(embeddingLevel), nsnull, nsnull);
-      propTable->SetProperty(frame, nsGkAtoms::baseLevel,
-                             NS_INT32_TO_PTR(paraLevel), nsnull, nsnull);
+      propTable->Set(frame, nsIFrame::EmbeddingLevelProperty(),
+                     NS_INT32_TO_PTR(embeddingLevel));
+      propTable->Set(frame, nsIFrame::BaseLevelProperty(),
+                     NS_INT32_TO_PTR(paraLevel));
       if (isTextFrame) {
-        PRInt32 typeLimit = PR_MIN(logicalLimit, lineOffset + fragmentLength);
-        CalculateCharType(lineOffset, typeLimit, logicalLimit, runLength,
-                           runCount, charType, prevType);
-        // IBMBIDI - Egypt - Start
-        propTable->SetProperty(frame, nsGkAtoms::charType,
-                               NS_INT32_TO_PTR(charType), nsnull, nsnull);
-        // IBMBIDI - Egypt - End
-
         if ( (runLength > 0) && (runLength < fragmentLength) ) {
           /*
            * The text in this frame continues beyond the end of this directional run.
@@ -472,14 +504,17 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
           }
           lineIter.GetLine()->MarkDirty();
           nsIFrame* nextBidi;
+          PRInt32 runEnd = contentOffset + runLength;
           EnsureBidiContinuation(frame, &nextBidi, frameIndex,
                                  contentOffset,
-                                 contentOffset + runLength);
+                                 runEnd);
           if (NS_FAILED(mSuccess)) {
             break;
           }
+          nextBidi->AdjustOffsetsForBidi(runEnd,
+                                         contentOffset + fragmentLength);
           frame = nextBidi;
-          contentOffset += runLength;
+          contentOffset = runEnd;
         } // if (runLength < fragmentLength)
         else {
           if (contentOffset + fragmentLength == contentTextLength) {
@@ -505,6 +540,16 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
             do {
             } while (mLogicalFrames[++newIndex]->GetType() == nsGkAtoms::directionalFrame);
             RemoveBidiContinuation(frame, frameIndex, newIndex, lineOffset);
+          } else if (runLength == fragmentLength) {
+            /*
+             * The directional run ends at the end of the frame. Make sure that
+             * the next frame is a non-fluid continuation
+             */
+            nsIFrame* next = frame->GetNextInFlow();
+            if (next) {
+              frame->SetNextContinuation(next);
+              next->SetPrevContinuation(frame);
+            }
           }
           frame->AdjustOffsetsForBidi(contentOffset, contentOffset + fragmentLength);
           if (lineNeedsUpdate) {
@@ -523,14 +568,28 @@ nsBidiPresUtils::Resolve(nsBlockFrame*   aBlockFrame,
     fragmentLength -= temp;
 
     if (frame && fragmentLength <= 0) {
-      if (runLength <= 0) {
-        // If the frame is at the end of a run, split all ancestor inlines that need splitting.
+      // If the frame is at the end of a run, split all ancestor inlines that
+      // need splitting.
+      // To determine whether we're at the end of the run, we check that we've
+      // finished processing the current run, and that the current frame
+      // doesn't have a fluid continuation (it could have a fluid continuation
+      // of zero length, so testing runLength alone is not sufficient).
+      if (runLength <= 0 && !frame->GetNextInFlow()) {
         nsIFrame* child = frame;
         nsIFrame* parent = frame->GetParent();
         // As long as we're on the last sibling, the parent doesn't have to be split.
+        // However, if the parent has a fluid continuation, we do have to make
+        // it non-fluid. This can happen e.g. when we have a first-letter frame
+        // and the end of the first-letter coincides with the end of a
+        // directional run.
         while (parent &&
                IsBidiSplittable(parent) &&
                !child->GetNextSibling()) {
+          nsIFrame* next = parent->GetNextInFlow();
+          if (next) {
+            parent->SetNextContinuation(next);
+            next->SetPrevContinuation(parent);
+          }
           child = parent;
           parent = child->GetParent();
         }
@@ -784,7 +843,8 @@ nsBidiPresUtils::GetFrameEmbeddingLevel(nsIFrame* aFrame)
 {
   nsIFrame* firstLeaf = aFrame;
   while (!IsBidiLeaf(firstLeaf)) {
-    firstLeaf = firstLeaf->GetFirstChild(nsnull);
+    firstLeaf = 
+      nsPlaceholderFrame::GetRealFrameFor(firstLeaf->GetFirstChild(nsnull));
   }
   return NS_GET_EMBEDDING_LEVEL(firstLeaf);
 }
@@ -867,18 +927,19 @@ nsBidiPresUtils::IsLeftOrRightMost(nsIFrame*              aFrame,
 
   if ((aIsLeftMost || aIsRightMost) &&
       (aFrame->GetStateBits() & NS_FRAME_IS_SPECIAL)) {
-    // For ib splits, don't treat the first part as endmost or the
-    // last part as startmost.
-    if (nsLayoutUtils::FrameIsInFirstPartOfIBSplit(aFrame)) {
+    // For ib splits, don't treat anything except the last part as
+    // endmost or anything except the first part as startmost.
+    // As an optimization, only get the first continuation once.
+    nsIFrame* firstContinuation = aFrame->GetFirstContinuation();
+    if (nsLayoutUtils::FrameIsNonLastInIBSplit(firstContinuation)) {
       // We are not endmost
       if (isLTR) {
         aIsRightMost = PR_FALSE;
       } else {
         aIsLeftMost = PR_FALSE;
       }
-    } else {
-      NS_ASSERTION(nsLayoutUtils::FrameIsInLastPartOfIBSplit(aFrame),
-                   "How did that happen?");
+    }
+    if (nsLayoutUtils::FrameIsNonFirstInIBSplit(firstContinuation)) {
       // We are not startmost
       if (isLTR) {
         aIsLeftMost = PR_FALSE;
@@ -1011,7 +1072,7 @@ nsBidiPresUtils::RepositionInlineFrames(nsIFrame* aFirstChild) const
   // have been reflowed, which is required for GetUsedMargin/Border/Padding
   nsMargin margin = aFirstChild->GetUsedMargin();
   if (!aFirstChild->GetPrevContinuation() &&
-      !nsLayoutUtils::FrameIsInLastPartOfIBSplit(aFirstChild))
+      !nsLayoutUtils::FrameIsNonFirstInIBSplit(aFirstChild))
     leftSpace = isLTR ? margin.left : margin.right;
 
   nscoord left = aFirstChild->GetPosition().x - leftSpace;
@@ -1141,14 +1202,12 @@ nsBidiPresUtils::RemoveBidiContinuation(nsIFrame*       aFrame,
                                         PRInt32         aLastIndex,
                                         PRInt32&        aOffset) const
 {
-  nsresult rv;
-  nsBidiLevel embeddingLevel = (nsCharType)NS_PTR_TO_INT32(aFrame->GetProperty(nsGkAtoms::embeddingLevel, &rv));
-  NS_ASSERTION(NS_SUCCEEDED(rv), "embeddingLevel attribute missing from aFrame");
-  nsBidiLevel baseLevel = (nsCharType)NS_PTR_TO_INT32(aFrame->GetProperty(nsGkAtoms::baseLevel, &rv));
-  NS_ASSERTION(NS_SUCCEEDED(rv), "baseLevel attribute missing from aFrame");
-  nsCharType charType = (nsCharType)NS_PTR_TO_INT32(aFrame->GetProperty(nsGkAtoms::charType, &rv));
-  NS_ASSERTION(NS_SUCCEEDED(rv), "charType attribute missing from aFrame");
-  
+  FrameProperties props = aFrame->Properties();
+  nsBidiLevel embeddingLevel =
+    (nsBidiLevel)NS_PTR_TO_INT32(props.Get(nsIFrame::EmbeddingLevelProperty()));
+  nsBidiLevel baseLevel =
+    (nsBidiLevel)NS_PTR_TO_INT32(props.Get(nsIFrame::BaseLevelProperty()));
+
   for (PRInt32 index = aFirstIndex + 1; index <= aLastIndex; index++) {
     nsIFrame* frame = mLogicalFrames[index];
     if (nsGkAtoms::directionalFrame == frame->GetType()) {
@@ -1158,9 +1217,11 @@ nsBidiPresUtils::RemoveBidiContinuation(nsIFrame*       aFrame,
     else {
       // Make the frame and its continuation ancestors fluid,
       // so they can be reused or deleted by normal reflow code
-      frame->SetProperty(nsGkAtoms::embeddingLevel, NS_INT32_TO_PTR(embeddingLevel));
-      frame->SetProperty(nsGkAtoms::baseLevel, NS_INT32_TO_PTR(baseLevel));
-      frame->SetProperty(nsGkAtoms::charType, NS_INT32_TO_PTR(charType));
+      FrameProperties frameProps = frame->Properties();
+      frameProps.Set(nsIFrame::EmbeddingLevelProperty(),
+                     NS_INT32_TO_PTR(embeddingLevel));
+      frameProps.Set(nsIFrame::BaseLevelProperty(),
+                     NS_INT32_TO_PTR(baseLevel));
       frame->AddStateBits(NS_FRAME_IS_BIDI);
       while (frame) {
         nsIFrame* prev = frame->GetPrevContinuation();
@@ -1354,16 +1415,6 @@ nsBidiPresUtils::CalculateCharType(PRInt32& aOffset,
   aOffset = offset;
 }
 
-nsresult nsBidiPresUtils::GetBidiEngine(nsBidi** aBidiEngine)
-{
-  nsresult rv = NS_ERROR_FAILURE;
-  if (mBidiEngine) {
-    *aBidiEngine = mBidiEngine;
-    rv = NS_OK;
-  }
-  return rv; 
-}
-
 nsresult nsBidiPresUtils::ProcessText(const PRUnichar*       aText,
                                       PRInt32                aLength,
                                       nsBidiDirection        aBaseDirection,
@@ -1415,7 +1466,7 @@ nsresult nsBidiPresUtils::ProcessText(const PRUnichar*       aText,
 
     PRInt32 subRunLength = limit - start;
     PRInt32 lineOffset = start;
-    PRInt32 typeLimit = PR_MIN(limit, aLength);
+    PRInt32 typeLimit = NS_MIN(limit, aLength);
     PRInt32 subRunCount = 1;
     PRInt32 subRunLimit = typeLimit;
 
@@ -1629,4 +1680,19 @@ nsresult nsBidiPresUtils::ProcessTextForRenderingContext(const PRUnichar*
   return ProcessText(aText, aLength, aBaseDirection, aPresContext, processor,
                      aMode, aPosResolve, aPosResolveCount, aWidth);
 }
+
+PRUint32 nsBidiPresUtils::EstimateMemoryUsed()
+{
+  PRUint32 size = 0;
+
+  size += sizeof(nsBidiPresUtils);
+  size += mBuffer.Length() * sizeof(PRUnichar);
+  size += moz_malloc_usable_size(mBidiEngine->mDirPropsMemory);
+  size += moz_malloc_usable_size(mBidiEngine->mLevelsMemory);
+  size += moz_malloc_usable_size(mBidiEngine->mRunsMemory);
+
+  return size;
+}
+
+
 #endif // IBMBIDI
