@@ -21,6 +21,7 @@
  *
  * Contributor(s):
  *   Mats Palmgren <mats.palmgren@bredband.net>
+ *   Jonathon Jongsma <jonathon.jongsma@collabora.co.uk>, Collabora Ltd.
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -41,6 +42,7 @@
 #ifndef nsStyleConsts_h___
 #define nsStyleConsts_h___
 
+#include "gfxRect.h"
 #include "nsFont.h"
 
 // cairo doesn't support invert
@@ -49,18 +51,18 @@
 // XXX fold this into nsStyleContext and group by nsStyleXXX struct
 
 // Indices into border/padding/margin arrays
-#define NS_SIDE_TOP     0
-#define NS_SIDE_RIGHT   1
-#define NS_SIDE_BOTTOM  2
-#define NS_SIDE_LEFT    3
+#define NS_SIDE_TOP     mozilla::css::eSideTop
+#define NS_SIDE_RIGHT   mozilla::css::eSideRight
+#define NS_SIDE_BOTTOM  mozilla::css::eSideBottom
+#define NS_SIDE_LEFT    mozilla::css::eSideLeft
 
-#define NS_FOR_CSS_SIDES(var_) for (PRInt32 var_ = 0; var_ < 4; ++var_)
-
-// Indices into "full corner" arrays (nsCSSCornerSizes e.g.)
-#define NS_CORNER_TOP_LEFT     0
-#define NS_CORNER_TOP_RIGHT    1
-#define NS_CORNER_BOTTOM_RIGHT 2
-#define NS_CORNER_BOTTOM_LEFT  3
+#define NS_FOR_CSS_SIDES(var_) for (mozilla::css::Side var_ = NS_SIDE_TOP; var_ <= NS_SIDE_LEFT; var_++)
+static inline mozilla::css::Side operator++(mozilla::css::Side& side, int) {
+    NS_PRECONDITION(side >= NS_SIDE_TOP &&
+                    side <= NS_SIDE_LEFT, "Out of range side");
+    side = mozilla::css::Side(side + 1);
+    return side;
+}
 
 #define NS_FOR_CSS_FULL_CORNERS(var_) for (PRInt32 var_ = 0; var_ < 4; ++var_)
 
@@ -197,19 +199,19 @@
 #define NS_STYLE_SPEAK_NORMAL             1
 #define NS_STYLE_SPEAK_SPELL_OUT          2
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_HEADER_ONCE        0
 #define NS_STYLE_SPEAK_HEADER_ALWAYS      1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_NUMERAL_DIGITS     0
 #define NS_STYLE_SPEAK_NUMERAL_CONTINUOUS 1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEAK_PUNCTUATION_NONE   0
 #define NS_STYLE_SPEAK_PUNCTUATION_CODE   1
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_SPEECH_RATE_X_SLOW       0
 #define NS_STYLE_SPEECH_RATE_SLOW         1
 #define NS_STYLE_SPEECH_RATE_MEDIUM       2
@@ -218,7 +220,7 @@
 #define NS_STYLE_SPEECH_RATE_FASTER       10
 #define NS_STYLE_SPEECH_RATE_SLOWER       11
 
-// See nsStyleAural 
+// See nsStyleAural
 #define NS_STYLE_VOLUME_SILENT            0
 #define NS_STYLE_VOLUME_X_SOFT            1
 #define NS_STYLE_VOLUME_SOFT              2
@@ -246,8 +248,7 @@
 // Code depends on these constants having the same values as BG_ORIGIN_*
 #define NS_STYLE_BG_CLIP_BORDER           0
 #define NS_STYLE_BG_CLIP_PADDING          1
-// When we add NS_STYLE_BG_CLIP_CONTENT, we should add the PR_STATIC_ASSERTs
-// to the places that assert equality for BORDER and PADDING.
+#define NS_STYLE_BG_CLIP_CONTENT          2
 
 // See nsStyleBackground
 #define NS_STYLE_BG_INLINE_POLICY_EACH_BOX      0
@@ -269,11 +270,16 @@
 #define NS_STYLE_BG_POSITION_RIGHT   (1<<4)
 
 // See nsStyleBackground
+// Code depends on (BG_REPEAT_X | BG_REPEAT_Y) == BG_REPEAT_XY
 #define NS_STYLE_BG_REPEAT_OFF                  0x00
 #define NS_STYLE_BG_REPEAT_X                    0x01
 #define NS_STYLE_BG_REPEAT_Y                    0x02
 #define NS_STYLE_BG_REPEAT_XY                   0x03
 
+// See nsStyleBackground
+#define NS_STYLE_BG_SIZE_CONTAIN  0
+#define NS_STYLE_BG_SIZE_COVER    1
+
 // See nsStyleTable
 #define NS_STYLE_BORDER_COLLAPSE                0
 #define NS_STYLE_BORDER_SEPARATE                1
@@ -296,9 +302,6 @@
 #define NS_STYLE_BORDER_STYLE_OUTSET            8
 #define NS_STYLE_BORDER_STYLE_HIDDEN            9
 #define NS_STYLE_BORDER_STYLE_AUTO              10 // for outline-style only
-// a bit ORed onto the style for table border collapsing indicating that the style was 
-// derived from a table with its rules attribute set
-#define NS_STYLE_BORDER_STYLE_RULES_MARKER      0x10  
 
 // See nsStyleBorder mBorderImage
 #define NS_STYLE_BORDER_IMAGE_STRETCH           0
@@ -328,15 +331,15 @@
 #define NS_STYLE_CURSOR_CROSSHAIR               2
 #define NS_STYLE_CURSOR_DEFAULT                 3    // ie: an arrow
 #define NS_STYLE_CURSOR_POINTER                 4    // for links
-#define NS_STYLE_CURSOR_MOVE                    5 
-#define NS_STYLE_CURSOR_E_RESIZE                6     
-#define NS_STYLE_CURSOR_NE_RESIZE               7      
-#define NS_STYLE_CURSOR_NW_RESIZE               8      
-#define NS_STYLE_CURSOR_N_RESIZE                9     
-#define NS_STYLE_CURSOR_SE_RESIZE               10      
-#define NS_STYLE_CURSOR_SW_RESIZE               11     
-#define NS_STYLE_CURSOR_S_RESIZE                12    
-#define NS_STYLE_CURSOR_W_RESIZE                13    
+#define NS_STYLE_CURSOR_MOVE                    5
+#define NS_STYLE_CURSOR_E_RESIZE                6
+#define NS_STYLE_CURSOR_NE_RESIZE               7
+#define NS_STYLE_CURSOR_NW_RESIZE               8
+#define NS_STYLE_CURSOR_N_RESIZE                9
+#define NS_STYLE_CURSOR_SE_RESIZE               10
+#define NS_STYLE_CURSOR_SW_RESIZE               11
+#define NS_STYLE_CURSOR_S_RESIZE                12
+#define NS_STYLE_CURSOR_W_RESIZE                13
 #define NS_STYLE_CURSOR_TEXT                    14   // ie: i-beam
 #define NS_STYLE_CURSOR_WAIT                    15
 #define NS_STYLE_CURSOR_HELP                    16
@@ -492,7 +495,7 @@
 #define NS_STYLE_CLIP_BOTTOM_AUTO               0x80
 
 // FRAME/FRAMESET/IFRAME specific values including backward compatibility. Boolean values with
-// the same meaning (e.g. 1 & yes) may need to be distinguished for correct mode processing 
+// the same meaning (e.g. 1 & yes) may need to be distinguished for correct mode processing
 #define NS_STYLE_FRAME_YES                      0
 #define NS_STYLE_FRAME_NO                       1
 #define NS_STYLE_FRAME_0                        2
@@ -549,7 +552,7 @@
 #define NS_STYLE_LIST_STYLE_MOZ_JAPANESE_FORMAL       31
 #define NS_STYLE_LIST_STYLE_MOZ_ARABIC_INDIC          32
 #define NS_STYLE_LIST_STYLE_MOZ_PERSIAN               33
-#define NS_STYLE_LIST_STYLE_MOZ_URDU                  34 
+#define NS_STYLE_LIST_STYLE_MOZ_URDU                  34
 #define NS_STYLE_LIST_STYLE_MOZ_DEVANAGARI            35
 #define NS_STYLE_LIST_STYLE_MOZ_GURMUKHI              36
 #define NS_STYLE_LIST_STYLE_MOZ_GUJARATI              37
@@ -578,6 +581,24 @@
 // See nsStyleMargin
 #define NS_STYLE_MARGIN_SIZE_AUTO               0
 
+// See nsStyleDisplay
+#define NS_STYLE_POINTER_EVENTS_NONE            0
+#define NS_STYLE_POINTER_EVENTS_VISIBLEPAINTED  1
+#define NS_STYLE_POINTER_EVENTS_VISIBLEFILL     2
+#define NS_STYLE_POINTER_EVENTS_VISIBLESTROKE   3
+#define NS_STYLE_POINTER_EVENTS_VISIBLE         4
+#define NS_STYLE_POINTER_EVENTS_PAINTED         5
+#define NS_STYLE_POINTER_EVENTS_FILL            6
+#define NS_STYLE_POINTER_EVENTS_STROKE          7
+#define NS_STYLE_POINTER_EVENTS_ALL             8
+#define NS_STYLE_POINTER_EVENTS_AUTO            9
+
+// See nsStyleDisplay
+#define NS_STYLE_RESIZE_NONE                    0
+#define NS_STYLE_RESIZE_BOTH                    1
+#define NS_STYLE_RESIZE_HORIZONTAL              2
+#define NS_STYLE_RESIZE_VERTICAL                3
+
 // See nsStyleText
 #define NS_STYLE_TEXT_ALIGN_DEFAULT               0
 #define NS_STYLE_TEXT_ALIGN_LEFT                  1
@@ -613,6 +634,13 @@
 #define NS_STYLE_TEXT_TRANSFORM_LOWERCASE       2
 #define NS_STYLE_TEXT_TRANSFORM_UPPERCASE       3
 
+// See nsStyleDisplay
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE         0
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_LINEAR       1
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_IN      2
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_OUT     3
+#define NS_STYLE_TRANSITION_TIMING_FUNCTION_EASE_IN_OUT  4
+
 // See nsStyleText
 // Note: these values pickup after the text-align values because there
 // are a few html cases where an object can have both types of
@@ -632,6 +660,9 @@
 #define NS_STYLE_VISIBILITY_VISIBLE             1
 #define NS_STYLE_VISIBILITY_COLLAPSE            2
 
+// See nsStyleText
+#define NS_STYLE_TABSIZE_INITIAL                8
+
 // See nsStyleText
 #define NS_STYLE_WHITESPACE_NORMAL              0
 #define NS_STYLE_WHITESPACE_PRE                 1
@@ -720,7 +751,16 @@
 #define NS_STYLE_IME_MODE_DISABLED              3
 #define NS_STYLE_IME_MODE_INACTIVE              4
 
-#ifdef MOZ_SVG
+// See nsStyleGradient
+#define NS_STYLE_GRADIENT_SHAPE_LINEAR          0
+#define NS_STYLE_GRADIENT_SHAPE_ELLIPTICAL      1
+#define NS_STYLE_GRADIENT_SHAPE_CIRCULAR        2
+
+#define NS_STYLE_GRADIENT_SIZE_CLOSEST_SIDE     0
+#define NS_STYLE_GRADIENT_SIZE_CLOSEST_CORNER   1
+#define NS_STYLE_GRADIENT_SIZE_FARTHEST_SIDE    2
+#define NS_STYLE_GRADIENT_SIZE_FARTHEST_CORNER  3
+
 // See nsStyleSVG
 
 // dominant-baseline
@@ -747,17 +787,6 @@
 #define NS_STYLE_IMAGE_RENDERING_OPTIMIZEQUALITY  2
 #define NS_STYLE_IMAGE_RENDERING_CRISPEDGES       3
 
-// pointer-events
-#define NS_STYLE_POINTER_EVENTS_NONE            0
-#define NS_STYLE_POINTER_EVENTS_VISIBLEPAINTED  1
-#define NS_STYLE_POINTER_EVENTS_VISIBLEFILL     2
-#define NS_STYLE_POINTER_EVENTS_VISIBLESTROKE   3
-#define NS_STYLE_POINTER_EVENTS_VISIBLE         4
-#define NS_STYLE_POINTER_EVENTS_PAINTED         5
-#define NS_STYLE_POINTER_EVENTS_FILL            6
-#define NS_STYLE_POINTER_EVENTS_STROKE          7
-#define NS_STYLE_POINTER_EVENTS_ALL             8
-
 // shape-rendering
 #define NS_STYLE_SHAPE_RENDERING_AUTO               0
 #define NS_STYLE_SHAPE_RENDERING_OPTIMIZESPEED      1
@@ -775,9 +804,9 @@
 #define NS_STYLE_STROKE_LINEJOIN_BEVEL          2
 
 // text-anchor
-#define NS_STYLE_TEXT_ANCHOR_START              0 
-#define NS_STYLE_TEXT_ANCHOR_MIDDLE             1 
-#define NS_STYLE_TEXT_ANCHOR_END                2 
+#define NS_STYLE_TEXT_ANCHOR_START              0
+#define NS_STYLE_TEXT_ANCHOR_MIDDLE             1
+#define NS_STYLE_TEXT_ANCHOR_END                2
 
 // text-rendering
 #define NS_STYLE_TEXT_RENDERING_AUTO               0
@@ -790,8 +819,6 @@
 #define NS_STYLE_COLOR_INTERPOLATION_SRGB           1
 #define NS_STYLE_COLOR_INTERPOLATION_LINEARRGB      2
 
-#endif // MOZ_SVG
-
 /*****************************************************************************
  * Constants for media features.                                             *
  *****************************************************************************/
