@@ -101,14 +101,6 @@
 #include "nsBidiUtils.h"
 #include "nsBidiPresUtils.h"
 
-#ifdef DEBUG
-#undef NOISY_IMAGE_LOADING
-#undef NOISY_ICON_LOADING
-#else
-#undef NOISY_IMAGE_LOADING
-#undef NOISY_ICON_LOADING
-#endif
-
 // sizes (pixels) for image icon, padding and border frame
 #define ICON_SIZE        (16)
 #define ICON_PADDING     (3)
@@ -133,13 +125,8 @@ static PRBool HaveFixedSize(const nsStylePosition* aStylePosition)
   // check the width and height values in the reflow state's style struct
   // - if width and height are specified as either coord or percentage, then
   //   the size of the image frame is constrained
-  nsStyleUnit widthUnit = aStylePosition->mWidth.GetUnit();
-  nsStyleUnit heightUnit = aStylePosition->mHeight.GetUnit();
-
-  return ((widthUnit  == eStyleUnit_Coord ||
-           widthUnit  == eStyleUnit_Percent) &&
-          (heightUnit == eStyleUnit_Coord ||
-           heightUnit == eStyleUnit_Percent));
+  return aStylePosition->mWidth.IsCoordPercentCalcUnit() &&
+         aStylePosition->mHeight.IsCoordPercentCalcUnit();
 }
 // use the data in the reflow state to decide if the image has a constrained size
 // (i.e. width and height that are based on the containing block size and not the image size) 
@@ -154,11 +141,15 @@ inline PRBool HaveFixedSize(const nsHTMLReflowState& aReflowState)
   // during pass 1 reflow, ComputedWidth() is NS_UNCONSTRAINEDSIZE
   // in pass 2 reflow, ComputedWidth() is 0, it also needs to return PR_FALSE
   // see bug 156731
-  nsStyleUnit heightUnit = (*(aReflowState.mStylePosition)).mHeight.GetUnit();
-  nsStyleUnit widthUnit = (*(aReflowState.mStylePosition)).mWidth.GetUnit();
-  return ((eStyleUnit_Percent == heightUnit && NS_UNCONSTRAINEDSIZE == aReflowState.ComputedHeight()) ||
-          (eStyleUnit_Percent == widthUnit && (NS_UNCONSTRAINEDSIZE == aReflowState.ComputedWidth() ||
-           0 == aReflowState.ComputedWidth())))
+  const nsStyleCoord &height = aReflowState.mStylePosition->mHeight;
+  const nsStyleCoord &width = aReflowState.mStylePosition->mWidth;
+  return (((eStyleUnit_Percent == height.GetUnit() ||
+            (height.IsCalcUnit() && height.CalcHasPercent())) &&
+           NS_UNCONSTRAINEDSIZE == aReflowState.ComputedHeight()) ||
+          ((eStyleUnit_Percent == width.GetUnit() ||
+            (width.IsCalcUnit() && width.CalcHasPercent())) &&
+           (NS_UNCONSTRAINEDSIZE == aReflowState.ComputedWidth() ||
+            0 == aReflowState.ComputedWidth())))
           ? PR_FALSE
           : HaveFixedSize(aReflowState.mStylePosition); 
 }
@@ -169,11 +160,14 @@ NS_NewImageFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
   return new (aPresShell) nsImageFrame(aContext);
 }
 
+NS_IMPL_FRAMEARENA_HELPERS(nsImageFrame)
+
 
 nsImageFrame::nsImageFrame(nsStyleContext* aContext) :
   ImageFrameSuper(aContext),
   mComputedSize(0, 0),
-  mIntrinsicSize(0, 0)
+  mIntrinsicSize(0, 0),
+  mDisplayingIcon(PR_FALSE)
 {
   // We assume our size is not constrained and we haven't gotten an
   // initial reflow yet, so don't touch those flags.
@@ -188,20 +182,22 @@ NS_QUERYFRAME_HEAD(nsImageFrame)
 NS_QUERYFRAME_TAIL_INHERITING(ImageFrameSuper)
 
 #ifdef ACCESSIBILITY
-NS_IMETHODIMP nsImageFrame::GetAccessible(nsIAccessible** aAccessible)
+already_AddRefed<nsAccessible>
+nsImageFrame::CreateAccessible()
 {
   nsCOMPtr<nsIAccessibilityService> accService = do_GetService("@mozilla.org/accessibilityService;1");
 
   if (accService) {
-    return accService->CreateHTMLImageAccessible(static_cast<nsIFrame*>(this), aAccessible);
+    return accService->CreateHTMLImageAccessible(mContent,
+                                                 PresContext()->PresShell());
   }
 
-  return NS_ERROR_FAILURE;
+  return nsnull;
 }
 #endif
 
 void
-nsImageFrame::Destroy()
+nsImageFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
   // Tell our image map, if there is one, to clean up
   // This causes the nsImageMap to unregister itself as
@@ -223,7 +219,11 @@ nsImageFrame::Destroy()
   
   mListener = nsnull;
 
-  nsSplittableFrame::Destroy();
+  // If we were displaying an icon, take ourselves off the list
+  if (mDisplayingIcon)
+    gIconLoad->RemoveIconObserver(this);
+
+  nsSplittableFrame::DestroyFrom(aDestructRoot);
 }
 
 
@@ -280,8 +280,10 @@ nsImageFrame::UpdateIntrinsicSize(imgIContainer* aImage)
   
   if (aImage) {
     nsIntSize imageSizeInPx;
-    aImage->GetWidth(&imageSizeInPx.width);
-    aImage->GetHeight(&imageSizeInPx.height);
+    if (NS_FAILED(aImage->GetWidth(&imageSizeInPx.width)) ||
+        NS_FAILED(aImage->GetHeight(&imageSizeInPx.height))) {
+      imageSizeInPx.SizeTo(0, 0);
+    }
     nsSize newSize(nsPresContext::CSSPixelsToAppUnits(imageSizeInPx.width),
                    nsPresContext::CSSPixelsToAppUnits(imageSizeInPx.height));
     if (mIntrinsicSize != newSize) {
@@ -473,11 +475,6 @@ nsImageFrame::OnStartContainer(imgIRequest *aRequest, imgIContainer *aImage)
 {
   if (!aImage) return NS_ERROR_INVALID_ARG;
 
-  // handle iconLoads first...
-  if (HandleIconLoads(aRequest, PR_FALSE)) {
-    return NS_OK;
-  }
-
   /* Get requested animation policy from the pres context:
    *   normal = 0
    *   one frame = 1
@@ -539,13 +536,6 @@ nsImageFrame::OnDataAvailable(imgIRequest *aRequest,
   // image scaling!
   nsRect r = SourceRectToDest(*aRect);
 
-  // handle iconLoads first...
-  if (HandleIconLoads(aRequest, PR_FALSE)) {
-    // Image changed, invalidate
-    Invalidate(r);
-    return NS_OK;
-  }
-
   if (IsPendingLoad(aRequest)) {
     // We don't care
     return NS_OK;
@@ -576,11 +566,6 @@ nsImageFrame::OnStopDecode(imgIRequest *aRequest,
   nsIPresShell *presShell = presContext->GetPresShell();
   NS_ASSERTION(presShell, "No PresShell.");
 
-  // handle iconLoads first...
-  if (HandleIconLoads(aRequest, NS_SUCCEEDED(aStatus))) {
-    return NS_OK;
-  }
-
   // Check what request type we're dealing with
   nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mContent);
   NS_ASSERTION(imageLoader, "Who's notifying us??");
@@ -624,7 +609,8 @@ nsImageFrame::OnStopDecode(imgIRequest *aRequest,
 }
 
 nsresult
-nsImageFrame::FrameChanged(imgIContainer *aContainer, nsIntRect *aDirtyRect)
+nsImageFrame::FrameChanged(imgIContainer *aContainer,
+                           const nsIntRect *aDirtyRect)
 {
   if (!GetStyleVisibility()->IsVisible()) {
     return NS_OK;
@@ -648,19 +634,23 @@ nsImageFrame::EnsureIntrinsicSize(nsPresContext* aPresContext)
   // if mIntrinsicSize.width and height are 0, then we should
   // check to see if the size is already known by the image container.
   if (mIntrinsicSize.width == 0 && mIntrinsicSize.height == 0) {
+
+    // Jump through all the hoops to get the status of the request
     nsCOMPtr<imgIRequest> currentRequest;
     nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mContent);
-    if (imageLoader) {
+    if (imageLoader)
       imageLoader->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
                               getter_AddRefs(currentRequest));
-    }
-    nsCOMPtr<imgIContainer> currentContainer;
-    if (currentRequest) {
-      currentRequest->GetImage(getter_AddRefs(currentContainer));
-    }
+    PRUint32 status = 0;
+    if (currentRequest)
+      currentRequest->GetImageStatus(&status);
 
-    if (currentContainer) {
-      UpdateIntrinsicSize(currentContainer);
+    // If we know the size, we can grab it and use it for an update
+    if (status & imgIRequest::STATUS_SIZE_AVAILABLE) {
+      nsCOMPtr<imgIContainer> imgCon;
+      currentRequest->GetImage(getter_AddRefs(imgCon));
+      NS_ABORT_IF_FALSE(imgCon, "SIZE_AVAILABLE, but no imgContainer?");
+      UpdateIntrinsicSize(imgCon);
     } else {
       // image request is null or image size not known, probably an
       // invalid image specified
@@ -793,7 +783,7 @@ nsImageFrame::Reflow(nsPresContext*          aPresContext,
     aMetrics.width = GetPrevInFlow()->GetSize().width;
     nscoord y = GetContinuationOffset();
     aMetrics.height -= y + aReflowState.mComputedBorderPadding.top;
-    aMetrics.height = PR_MAX(0, aMetrics.height);
+    aMetrics.height = NS_MAX(0, aMetrics.height);
   }
 
 
@@ -816,7 +806,7 @@ nsImageFrame::Reflow(nsPresContext*          aPresContext,
       aMetrics.height > aReflowState.availableHeight) { 
     // our desired height was greater than 0, so to avoid infinite
     // splitting, use 1 pixel as the min
-    aMetrics.height = PR_MAX(nsPresContext::CSSPixelsToAppUnits(1), aReflowState.availableHeight);
+    aMetrics.height = NS_MAX(nsPresContext::CSSPixelsToAppUnits(1), aReflowState.availableHeight);
     aStatus = NS_FRAME_NOT_COMPLETE;
   }
 
@@ -998,6 +988,9 @@ nsImageFrame::DisplayAltFeedback(nsIRenderingContext& aRenderingContext,
                                  imgIRequest*         aRequest,
                                  nsPoint              aPt)
 {
+  // We should definitely have a gIconLoad here.
+  NS_ABORT_IF_FALSE(gIconLoad, "How did we succeed in Init then?");
+
   // Calculate the inner area
   nsRect  inner = GetInnerArea() + aPt;
 
@@ -1018,8 +1011,9 @@ nsImageFrame::DisplayAltFeedback(nsIRenderingContext& aRenderingContext,
 
   // Paint the border
   nsRecessedBorder recessedBorder(borderEdgeWidth, PresContext());
-  nsCSSRendering::PaintBorder(PresContext(), aRenderingContext, this, inner,
-                              inner, recessedBorder, mStyleContext);
+  nsCSSRendering::PaintBorderWithStyleBorder(PresContext(), aRenderingContext,
+                                             this, inner, inner,
+                                             recessedBorder, mStyleContext);
 
   // Adjust the inner rect to account for the one pixel recessed border,
   // and a six pixel padding on each edge
@@ -1033,34 +1027,41 @@ nsImageFrame::DisplayAltFeedback(nsIRenderingContext& aRenderingContext,
   aRenderingContext.PushState();
   aRenderingContext.SetClipRect(inner, nsClipCombine_kIntersect);
 
-  PRBool dispIcon = gIconLoad ? gIconLoad->mPrefShowPlaceholders : PR_TRUE;
-
   // Check if we should display image placeholders
-  if (dispIcon) {
+  if (gIconLoad->mPrefShowPlaceholders) {
     const nsStyleVisibility* vis = GetStyleVisibility();
     nscoord size = nsPresContext::CSSPixelsToAppUnits(ICON_SIZE);
 
     PRBool iconUsed = PR_FALSE;
 
-    // see if the icon images are present...
-    if (gIconLoad && gIconLoad->mIconsLoaded) {
-      // pick the correct image
-      nsCOMPtr<imgIContainer> imgCon;
-      if (aRequest) {
-        aRequest->GetImage(getter_AddRefs(imgCon));
-      }
-      if (imgCon) {
-        // draw it
-        nsRect dest((vis->mDirection == NS_STYLE_DIRECTION_RTL) ?
-                    inner.XMost() - size : inner.x,
-                    inner.y, size, size);
-        nsLayoutUtils::DrawSingleImage(&aRenderingContext, imgCon,
-          nsLayoutUtils::GetGraphicsFilterForFrame(this), dest, aDirtyRect);
-        iconUsed = PR_TRUE;
-      }
+    // If we weren't previously displaying an icon, register ourselves
+    // as an observer for load and animation updates and flag that we're
+    // doing so now.
+    if (aRequest && !mDisplayingIcon) {
+      gIconLoad->AddIconObserver(this);
+      mDisplayingIcon = PR_TRUE;
     }
 
-    // if we could not draw the image, then just draw some graffiti
+
+    // If the image in question is loaded and decoded, draw it
+    PRUint32 imageStatus = 0;
+    if (aRequest)
+      aRequest->GetImageStatus(&imageStatus);
+    if (imageStatus & imgIRequest::STATUS_FRAME_COMPLETE) {
+      nsCOMPtr<imgIContainer> imgCon;
+      aRequest->GetImage(getter_AddRefs(imgCon));
+      NS_ABORT_IF_FALSE(imgCon, "Frame Complete, but no image container?");
+      nsRect dest((vis->mDirection == NS_STYLE_DIRECTION_RTL) ?
+                  inner.XMost() - size : inner.x,
+                  inner.y, size, size);
+      nsLayoutUtils::DrawSingleImage(&aRenderingContext, imgCon,
+        nsLayoutUtils::GetGraphicsFilterForFrame(this), dest, aDirtyRect,
+        imgIContainer::FLAG_NONE);
+      iconUsed = PR_TRUE;
+    }
+
+    // if we could not draw the icon, flag that we're waiting for it and
+    // just draw some graffiti in the mean time
     if (!iconUsed) {
       nscolor oldColor;
       nscoord iconXPos = (vis->mDirection ==   NS_STYLE_DIRECTION_RTL) ?
@@ -1138,23 +1139,27 @@ public:
   virtual ~nsDisplayImage() {
     MOZ_COUNT_DTOR(nsDisplayImage);
   }
-  virtual void Paint(nsDisplayListBuilder* aBuilder, nsIRenderingContext* aCtx,
-     const nsRect& aDirtyRect);
-  NS_DISPLAY_DECL_NAME("Image")
+  virtual void Paint(nsDisplayListBuilder* aBuilder,
+                     nsIRenderingContext* aCtx);
+  NS_DISPLAY_DECL_NAME("Image", TYPE_IMAGE)
 private:
   nsCOMPtr<imgIContainer> mImage;
 };
 
 void
 nsDisplayImage::Paint(nsDisplayListBuilder* aBuilder,
-     nsIRenderingContext* aCtx, const nsRect& aDirtyRect) {
+                      nsIRenderingContext* aCtx) {
   static_cast<nsImageFrame*>(mFrame)->
-    PaintImage(*aCtx, aBuilder->ToReferenceFrame(mFrame), aDirtyRect, mImage);
+    PaintImage(*aCtx, aBuilder->ToReferenceFrame(mFrame), mVisibleRect, mImage,
+               aBuilder->ShouldSyncDecodeImages()
+                 ? (PRUint32) imgIContainer::FLAG_SYNC_DECODE
+                 : (PRUint32) imgIContainer::FLAG_NONE);
 }
 
 void
 nsImageFrame::PaintImage(nsIRenderingContext& aRenderingContext, nsPoint aPt,
-                         const nsRect& aDirtyRect, imgIContainer* aImage)
+                         const nsRect& aDirtyRect, imgIContainer* aImage,
+                         PRUint32 aFlags)
 {
   // Render the image into our content area (the area inside
   // the borders and padding)
@@ -1164,7 +1169,8 @@ nsImageFrame::PaintImage(nsIRenderingContext& aRenderingContext, nsPoint aPt,
   dest.y -= GetContinuationOffset();
 
   nsLayoutUtils::DrawSingleImage(&aRenderingContext, aImage,
-    nsLayoutUtils::GetGraphicsFilterForFrame(this), dest, aDirtyRect);
+    nsLayoutUtils::GetGraphicsFilterForFrame(this), dest, aDirtyRect,
+    aFlags);
 
   nsPresContext* presContext = PresContext();
   nsImageMap* map = GetImageMap(presContext);
@@ -1214,22 +1220,42 @@ nsImageFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
       currentRequest->GetImage(getter_AddRefs(imgCon));
     }
 
-    if (!imageOK || !imgCon) {
+    // Determine if the size is available
+    PRBool haveSize = PR_FALSE;
+    PRUint32 imageStatus = 0;
+    if (currentRequest)
+      currentRequest->GetImageStatus(&imageStatus);
+    if (imageStatus & imgIRequest::STATUS_SIZE_AVAILABLE)
+      haveSize = PR_TRUE;
+
+    // We should never have the size and not have an image container
+    NS_ABORT_IF_FALSE(!haveSize || imgCon, "Have size but not container?");
+
+    if (!imageOK || !haveSize) {
       // No image yet, or image load failed. Draw the alt-text and an icon
       // indicating the status
       rv = aLists.Content()->AppendNewToTop(new (aBuilder)
-          nsDisplayGeneric(this, PaintAltFeedback, "AltFeedback"));
+          nsDisplayGeneric(this, PaintAltFeedback, "AltFeedback",
+                           nsDisplayItem::TYPE_ALT_FEEDBACK));
       NS_ENSURE_SUCCESS(rv, rv);
     }
     else {
       rv = aLists.Content()->AppendNewToTop(new (aBuilder)
           nsDisplayImage(this, imgCon));
       NS_ENSURE_SUCCESS(rv, rv);
+
+      // If we were previously displaying an icon, we're not anymore
+      if (mDisplayingIcon) {
+        gIconLoad->RemoveIconObserver(this);
+        mDisplayingIcon = PR_FALSE;
+      }
+
         
 #ifdef DEBUG
       if (GetShowFrameBorders() && GetImageMap(PresContext())) {
         rv = aLists.Outlines()->AppendNewToTop(new (aBuilder)
-            nsDisplayGeneric(this, PaintDebugImageMap, "DebugImageMap"));
+            nsDisplayGeneric(this, PaintDebugImageMap, "DebugImageMap",
+                             nsDisplayItem::TYPE_DEBUG_IMAGE_MAP));
         NS_ENSURE_SUCCESS(rv, rv);
       }
 #endif
@@ -1237,12 +1263,9 @@ nsImageFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   }
 
   // XXX what on EARTH is this code for?
-  PRInt16 displaySelection = 0;
   nsresult result;
   nsPresContext* presContext = PresContext();
-  result = presContext->PresShell()->GetSelectionFlags(&displaySelection);
-  if (NS_FAILED(result))
-    return result;
+  PRInt16 displaySelection = presContext->PresShell()->GetSelectionFlags();
   if (!(displaySelection & nsISelectionDisplay::DISPLAY_IMAGES))
     return NS_OK;//no need to check the blue border, we cannot be drawn selected
 //insert hook here for image selection drawing
@@ -1368,7 +1391,10 @@ nsImageFrame::GetAnchorHREFTargetAndNode(nsIURI** aHref, nsString& aTarget,
        content; content = content->GetParent()) {
     nsCOMPtr<nsILink> link(do_QueryInterface(content));
     if (link) {
-      *aHref = content->GetHrefURI().get();
+      nsCOMPtr<nsIURI> href = content->GetHrefURI();
+      if (href) {
+        href->Clone(aHref);
+      }
       status = (*aHref != nsnull);
 
       nsCOMPtr<nsIDOMHTMLAnchorElement> anchor(do_QueryInterface(content));
@@ -1388,6 +1414,22 @@ nsImageFrame::GetContentForEvent(nsPresContext* aPresContext,
                                  nsIContent** aContent)
 {
   NS_ENSURE_ARG_POINTER(aContent);
+
+  nsIFrame* f = nsLayoutUtils::GetNonGeneratedAncestor(this);
+  if (f != this) {
+    return f->GetContentForEvent(aPresContext, aEvent, aContent);
+  }
+
+  // XXX We need to make this special check for area element's capturing the
+  // mouse due to bug 135040. Remove it once that's fixed.
+  nsIContent* capturingContent =
+    NS_IS_MOUSE_EVENT(aEvent) ? nsIPresShell::GetCapturingContent() : nsnull;
+  if (capturingContent && capturingContent->GetPrimaryFrame() == this) {
+    *aContent = capturingContent;
+    NS_IF_ADDREF(*aContent);
+    return NS_OK;
+  }
+
   nsImageMap* map;
   map = GetImageMap(aPresContext);
 
@@ -1493,7 +1535,7 @@ nsImageFrame::GetCursor(const nsPoint& aPoint,
       // specified will inherit the style from the image.
       nsRefPtr<nsStyleContext> areaStyle = 
         PresContext()->PresShell()->StyleSet()->
-          ResolveStyleFor(area, GetStyleContext());
+          ResolveStyleFor(area->AsElement(), GetStyleContext());
       if (areaStyle) {
         FillCursorInformationFromStyle(areaStyle->GetStyleUserInterface(),
                                        aCursor);
@@ -1551,12 +1593,12 @@ nsImageFrame::List(FILE* out, PRInt32 aIndent) const
   if (HasView()) {
     fprintf(out, " [view=%p]", (void*)GetView());
   }
-  fprintf(out, " {%d,%d,%d,%d}", mRect.x, mRect.y, mRect.width, 
-mRect.height);
+  fprintf(out, " {%d,%d,%d,%d}", mRect.x, mRect.y, mRect.width, mRect.height);
   if (0 != mState) {
-    fprintf(out, " [state=%08x]", mState);
+    fprintf(out, " [state=%016llx]", mState);
   }
   fprintf(out, " [content=%p]", (void*)mContent);
+  fprintf(out, " [sc=%p]", static_cast<void*>(mStyleContext));
 
   // output the img src url
   nsCOMPtr<nsIImageLoadingContent> imageLoader = do_QueryInterface(mContent);
@@ -1628,11 +1670,12 @@ nsImageFrame::LoadIcon(const nsAString& aSpec,
                                        apply to icons. */
                        nsnull,      /* referrer (not relevant for icons) */
                        loadGroup,
-                       mListener,
+                       gIconLoad,
                        nsnull,      /* Not associated with any particular document */
                        loadFlags,
                        nsnull,
                        nsnull,
+                       nsnull,      /* channel policy not needed */
                        aRequest);
 }
 
@@ -1685,10 +1728,10 @@ nsresult nsImageFrame::LoadIcons(nsPresContext *aPresContext)
 {
   NS_ASSERTION(!gIconLoad, "called LoadIcons twice");
 
-  NS_NAMED_LITERAL_STRING(loadingSrc,"resource://gre/res/loading-image.png");
-  NS_NAMED_LITERAL_STRING(brokenSrc,"resource://gre/res/broken-image.png");
+  NS_NAMED_LITERAL_STRING(loadingSrc,"resource://gre-resources/loading-image.png");
+  NS_NAMED_LITERAL_STRING(brokenSrc,"resource://gre-resources/broken-image.png");
 
-  gIconLoad = new IconLoad(mListener);
+  gIconLoad = new IconLoad();
   if (!gIconLoad) 
     return NS_ERROR_OUT_OF_MEMORY;
   NS_ADDREF(gIconLoad);
@@ -1698,11 +1741,6 @@ nsresult nsImageFrame::LoadIcons(nsPresContext *aPresContext)
   rv = LoadIcon(loadingSrc,
                 aPresContext,
                 getter_AddRefs(gIconLoad->mLoadingImage));
-#ifdef NOISY_ICON_LOADING
-  printf("Loading request %p, rv=%u\n",
-         gIconLoad->mLoadingImage.get(), rv);
-#endif
-
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -1710,59 +1748,20 @@ nsresult nsImageFrame::LoadIcons(nsPresContext *aPresContext)
   rv = LoadIcon(brokenSrc,
                 aPresContext,
                 getter_AddRefs(gIconLoad->mBrokenImage));
-#ifdef NOISY_ICON_LOADING
-  printf("Loading request %p, rv=%u\n",
-         gIconLoad->mBrokenImage.get(), rv);
-#endif
-
-  // ImageLoader will callback into OnStartContainer, which will
-  // handle the mIconsLoaded flag
-
   return rv;
 }
 
-PRBool nsImageFrame::HandleIconLoads(imgIRequest* aRequest, PRBool aLoaded)
-{
-  PRBool result = PR_FALSE;
-
-  if (gIconLoad) {
-    // check which image it is
-    if (aRequest == gIconLoad->mLoadingImage ||
-        aRequest == gIconLoad->mBrokenImage) {
-      result = PR_TRUE;
-      if (aLoaded && (++gIconLoad->mIconsLoaded == 2))
-        gIconLoad->mLoadObserver = nsnull;
-    }
-
-#ifdef NOISY_ICON_LOADING
-    if (gIconLoad->mIconsLoaded && result) {
-      printf( "Icons Loaded: request for %s\n",
-              aRequest == gIconLoad->mLoadingImage
-                ? "mLoadingImage" : "mBrokenImage" );
-    }
-#endif
-  }
-  
-#ifdef NOISY_ICON_LOADING
-  printf( "HandleIconLoads returned %s (%p)\n", result ? "TRUE" : "FALSE", this);
-#endif
-
-  return result;
-}
-
-NS_IMPL_ISUPPORTS1(nsImageFrame::IconLoad, nsIObserver)
+NS_IMPL_ISUPPORTS2(nsImageFrame::IconLoad, nsIObserver,
+                   imgIDecoderObserver)
 
 static const char kIconLoadPrefs[][40] = {
   "browser.display.force_inline_alttext",
   "browser.display.show_image_placeholders"
 };
 
-nsImageFrame::IconLoad::IconLoad(imgIDecoderObserver *aObserver)
-  : mLoadObserver(aObserver),
-    mIconsLoaded(0)
+nsImageFrame::IconLoad::IconLoad()
 {
-  nsCOMPtr<nsIPrefBranch2> prefBranch =
-    do_QueryInterface(nsContentUtils::GetPrefBranch());
+  nsIPrefBranch2* prefBranch = nsContentUtils::GetPrefBranch();
 
   // register observers
   for (PRUint32 i = 0; i < NS_ARRAY_LENGTH(kIconLoadPrefs); ++i)
@@ -1800,6 +1799,100 @@ void nsImageFrame::IconLoad::GetPrefs()
                                 PR_TRUE);
 }
 
+
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStartRequest(imgIRequest *aRequest)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStartDecode(imgIRequest *aRequest)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStartContainer(imgIRequest *aRequest,
+                                         imgIContainer *aContainer)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStartFrame(imgIRequest *aRequest,
+                                     PRUint32 aFrame)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnDataAvailable(imgIRequest *aRequest,
+                                        PRBool aCurrentFrame,
+                                        const nsIntRect * aRect)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStopFrame(imgIRequest *aRequest,
+                                    PRUint32 aFrame)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStopContainer(imgIRequest *aRequest,
+                                        imgIContainer *aContainer)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStopDecode(imgIRequest *aRequest,
+                                     nsresult status,
+                                     const PRUnichar *statusArg)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnStopRequest(imgIRequest *aRequest,
+                                      PRBool aIsLastPart)
+{
+  nsTObserverArray<nsImageFrame*>::ForwardIterator iter(mIconObservers);
+  nsImageFrame *frame;
+  while (iter.HasMore()) {
+    frame = iter.GetNext();
+    frame->Invalidate(frame->GetRect());
+  }
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::OnDiscard(imgIRequest *aRequest)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsImageFrame::IconLoad::FrameChanged(imgIContainer *aContainer,
+                                     const nsIntRect *aDirtyRect)
+{
+  nsTObserverArray<nsImageFrame*>::ForwardIterator iter(mIconObservers);
+  nsImageFrame *frame;
+  while (iter.HasMore()) {
+    frame = iter.GetNext();
+    frame->Invalidate(frame->GetRect());
+  }
+
+  return NS_OK;
+}
+
+
+
 NS_IMPL_ISUPPORTS2(nsImageListener, imgIDecoderObserver, imgIContainerObserver)
 
 nsImageListener::nsImageListener(nsImageFrame *aFrame) :
@@ -1841,12 +1934,12 @@ NS_IMETHODIMP nsImageListener::OnStopDecode(imgIRequest *aRequest,
 }
 
 NS_IMETHODIMP nsImageListener::FrameChanged(imgIContainer *aContainer,
-                                            nsIntRect * dirtyRect)
+                                            const nsIntRect *aDirtyRect)
 {
   if (!mFrame)
     return NS_ERROR_FAILURE;
 
-  return mFrame->FrameChanged(aContainer, dirtyRect);
+  return mFrame->FrameChanged(aContainer, aDirtyRect);
 }
 
 static PRBool
@@ -1856,7 +1949,7 @@ IsInAutoWidthTableCellForQuirk(nsIFrame *aFrame)
     return PR_FALSE;
   // Check if the parent of the closest nsBlockFrame has auto width.
   nsBlockFrame *ancestor = nsLayoutUtils::FindNearestBlockAncestor(aFrame);
-  if (ancestor->GetStyleContext()->GetPseudoType() == nsCSSAnonBoxes::cellContent) {
+  if (ancestor->GetStyleContext()->GetPseudo() == nsCSSAnonBoxes::cellContent) {
     // Assume direct parent is a table cell frame.
     nsFrame *grandAncestor = static_cast<nsFrame*>(ancestor->GetParent());
     return grandAncestor &&
