@@ -96,15 +96,15 @@ NS_NewLineBox(nsIPresShell* aPresShell, nsIFrame* aFrame,
 
 // Overloaded new operator. Uses an arena (which comes from the presShell)
 // to perform the allocation.
-void* 
+void*
 nsLineBox::operator new(size_t sz, nsIPresShell* aPresShell) CPP_THROW_NEW
 {
-  return aPresShell->AllocateFrame(sz);
+  return aPresShell->AllocateMisc(sz);
 }
 
 // Overloaded delete operator. Doesn't actually free the memory, because we
 // use an arena
-void 
+void
 nsLineBox::operator delete(void* aPtr, size_t sz)
 {
 }
@@ -116,7 +116,7 @@ nsLineBox::Destroy(nsIPresShell* aPresShell)
   delete this;
 
   // Have the pres shell recycle the memory
-  aPresShell->FreeFrame(sizeof(*this), (void*)this);
+  aPresShell->FreeMisc(sizeof(*this), (void*)this);
 }
 
 void
@@ -137,27 +137,20 @@ nsLineBox::Cleanup()
 static void
 ListFloats(FILE* out, PRInt32 aIndent, const nsFloatCacheList& aFloats)
 {
-  nsAutoString frameName;
   nsFloatCache* fc = aFloats.Head();
   while (fc) {
     nsFrame::IndentBy(out, aIndent);
-    nsPlaceholderFrame* ph = fc->mPlaceholder;
-    if (ph) {
-      fprintf(out, "placeholder@%p ", static_cast<void*>(ph));
-      nsIFrame* frame = ph->GetOutOfFlowFrame();
-      if (frame) {
-        nsIFrameDebug* frameDebug = do_QueryFrame(frame);
-        if (frameDebug) {
-          frameDebug->GetFrameName(frameName);
-          fputs(NS_LossyConvertUTF16toASCII(frameName).get(), out);
-        }
-      }
-
-      if (!frame) {
-        fputs("\n###!!! NULL out-of-flow frame", out);
-      }
-      fprintf(out, "\n");
+    nsIFrame* frame = fc->mFloat;
+    fprintf(out, "floatframe@%p ", static_cast<void*>(frame));
+    if (frame) {
+      nsAutoString frameName;
+      frame->GetFrameName(frameName);
+      fputs(NS_LossyConvertUTF16toASCII(frameName).get(), out);
     }
+    else {
+      fputs("\n###!!! NULL out-of-flow frame", out);
+    }
+    fprintf(out, "\n");
     fc = fc->Next();
   }
 }
@@ -222,10 +215,7 @@ nsLineBox::List(FILE* out, PRInt32 aIndent) const
   nsIFrame* frame = mFirstChild;
   PRInt32 n = GetChildCount();
   while (--n >= 0) {
-    nsIFrameDebug* frameDebug = do_QueryFrame(frame);
-    if (frameDebug) {
-      frameDebug->List(out, aIndent + 1);
-    }
+    frame->List(out, aIndent + 1);
     frame = frame->GetNextSibling();
   }
 
@@ -286,6 +276,9 @@ nsLineBox::IsEmpty() const
     if (!kid->IsEmpty())
       return PR_FALSE;
   }
+  if (HasBullet()) {
+    return PR_FALSE;
+  }
   return PR_TRUE;
 }
 
@@ -316,6 +309,9 @@ nsLineBox::CachedIsEmpty()
           break;
         }
       }
+    if (HasBullet()) {
+      result = PR_FALSE;
+    }
   }
 
   mFlags.mEmptyCacheValid = PR_TRUE;
@@ -324,61 +320,63 @@ nsLineBox::CachedIsEmpty()
 }
 
 void
-nsLineBox::DeleteLineList(nsPresContext* aPresContext, nsLineList& aLines)
+nsLineBox::DeleteLineList(nsPresContext* aPresContext, nsLineList& aLines,
+                          nsIFrame* aDestructRoot)
 {
   if (! aLines.empty()) {
     // Delete our child frames before doing anything else. In particular
     // we do all of this before our base class releases it's hold on the
     // view.
+#ifdef DEBUG
+    PRInt32 numFrames = 0;
+#endif
     for (nsIFrame* child = aLines.front()->mFirstChild; child; ) {
       nsIFrame* nextChild = child->GetNextSibling();
-      child->Destroy();
+      child->SetNextSibling(nsnull);
+      child->DestroyFrom((aDestructRoot) ? aDestructRoot : child);
       child = nextChild;
+#ifdef DEBUG
+      numFrames++;
+#endif
     }
 
     nsIPresShell *shell = aPresContext->PresShell();
 
     do {
       nsLineBox* line = aLines.front();
+#ifdef DEBUG
+      numFrames -= line->GetChildCount();
+#endif
       aLines.pop_front();
       line->Destroy(shell);
     } while (! aLines.empty());
+#ifdef DEBUG
+    NS_ASSERTION(numFrames == 0, "number of frames deleted does not match");
+#endif
   }
 }
 
-nsLineBox*
-nsLineBox::FindLineContaining(nsLineList& aLines, nsIFrame* aFrame,
-                              PRInt32* aFrameIndexInLine)
-{
-  NS_PRECONDITION(aFrameIndexInLine && !aLines.empty() && aFrame, "null ptr");
-  for (nsLineList::iterator line = aLines.begin(),
-                            line_end = aLines.end();
-       line != line_end;
-       ++line)
-  {
-    PRInt32 ix = line->IndexOf(aFrame);
-    if (ix >= 0) {
-      *aFrameIndexInLine = ix;
-      return line;
-    }
-  }
-  *aFrameIndexInLine = -1;
-  return nsnull;
-}
-
 PRBool
 nsLineBox::RFindLineContaining(nsIFrame* aFrame,
                                const nsLineList::iterator& aBegin,
                                nsLineList::iterator& aEnd,
+                               nsIFrame* aLastFrameBeforeEnd,
                                PRInt32* aFrameIndexInLine)
 {
   NS_PRECONDITION(aFrame, "null ptr");
+  nsIFrame* curFrame = aLastFrameBeforeEnd;
   while (aBegin != aEnd) {
     --aEnd;
-    PRInt32 ix = aEnd->IndexOf(aFrame);
-    if (ix >= 0) {
-      *aFrameIndexInLine = ix;
-      return PR_TRUE;
+    NS_ASSERTION(aEnd->IsLastChild(curFrame), "Unexpected curFrame");
+    // i is the index of curFrame in aEnd
+    PRInt32 i = aEnd->GetChildCount() - 1;
+    while (i >= 0) {
+      if (curFrame == aFrame) {
+        *aFrameIndexInLine = i;
+        return PR_TRUE;
+      }
+      --i;
+      curFrame = curFrame->GetPrevSibling();
     }
   }
   *aFrameIndexInLine = -1;
@@ -638,23 +636,6 @@ nsLineIterator::FindLineContaining(nsIFrame* aFrame)
   return -1;
 }
 
-/* virtual */ PRInt32
-nsLineIterator::FindLineAt(nscoord aY)
-{
-  nsLineBox* line = mLines[0];
-  if (!line || (aY < line->mBounds.y)) {
-    return -1;
-  }
-  PRInt32 lineNumber = 0;
-  while (lineNumber != mNumLines) {
-    if ((aY >= line->mBounds.y) && (aY < line->mBounds.YMost())) {
-      return lineNumber;
-    }
-    line = mLines[++lineNumber];
-  }
-  return mNumLines;
-}
-
 #ifdef IBMBIDI
 NS_IMETHODIMP
 nsLineIterator::CheckLineOrder(PRInt32                  aLine,
@@ -836,7 +817,7 @@ nsFloatCacheList::Find(nsIFrame* aOutOfFlowFrame)
 {
   nsFloatCache* fc = mHead;
   while (fc) {
-    if (fc->mPlaceholder->GetOutOfFlowFrame() == aOutOfFlowFrame) {
+    if (fc->mFloat == aOutOfFlowFrame) {
       break;
     }
     fc = fc->Next();
@@ -913,8 +894,10 @@ nsFloatCacheFreeList::DeleteAll()
 }
 
 nsFloatCache*
-nsFloatCacheFreeList::Alloc()
+nsFloatCacheFreeList::Alloc(nsIFrame* aFloat)
 {
+  NS_PRECONDITION(aFloat->GetStateBits() & NS_FRAME_OUT_OF_FLOW,
+                  "This is a float cache, why isn't the frame out-of-flow?");
   nsFloatCache* fc = mHead;
   if (mHead) {
     if (mHead == mTail) {
@@ -928,6 +911,7 @@ nsFloatCacheFreeList::Alloc()
   else {
     fc = new nsFloatCache();
   }
+  fc->mFloat = aFloat;
   return fc;
 }
 
@@ -950,7 +934,7 @@ nsFloatCacheFreeList::Append(nsFloatCache* aFloat)
 //----------------------------------------------------------------------
 
 nsFloatCache::nsFloatCache()
-  : mPlaceholder(nsnull),
+  : mFloat(nsnull),
     mNext(nsnull)
 {
   MOZ_COUNT_CTOR(nsFloatCache);
