@@ -57,19 +57,24 @@
 #include "nsBoxFrame.h"
 #include "nsImageFrame.h"
 #include "nsIImageLoadingContent.h"
+#include "nsDisplayList.h"
 
 #ifdef ACCESSIBILITY
 #include "nsIServiceManager.h"
-#include "nsIAccessible.h"
 #include "nsIAccessibilityService.h"
 #endif
 
+using namespace mozilla;
+using namespace mozilla::layers;
+
 nsIFrame*
 NS_NewHTMLVideoFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
   return new (aPresShell) nsVideoFrame(aContext);
 }
 
+NS_IMPL_FRAMEARENA_HELPERS(nsVideoFrame)
+
 nsVideoFrame::nsVideoFrame(nsStyleContext* aContext) :
   nsContainerFrame(aContext)
 {
@@ -81,9 +86,6 @@ nsVideoFrame::~nsVideoFrame()
 
 NS_QUERYFRAME_HEAD(nsVideoFrame)
   NS_QUERYFRAME_ENTRY(nsIAnonymousContentCreator)
-#ifdef DEBUG
-  NS_QUERYFRAME_ENTRY(nsIFrameDebug)
-#endif
 NS_QUERYFRAME_TAIL_INHERITING(nsContainerFrame)
 
 nsresult
@@ -97,11 +99,11 @@ nsVideoFrame::CreateAnonymousContent(nsTArray<nsIContent*>& aElements)
     // before we load, or on a subsequent load.
     nodeInfo = nodeInfoManager->GetNodeInfo(nsGkAtoms::img,
                                             nsnull,
-                                            kNameSpaceID_None);
+                                            kNameSpaceID_XHTML);
     NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
-    mPosterImage = NS_NewHTMLImageElement(nodeInfo);
+    mPosterImage = NS_NewHTMLImageElement(nodeInfo.forget());
     NS_ENSURE_TRUE(mPosterImage, NS_ERROR_OUT_OF_MEMORY);
-    
+
     // Set the nsImageLoadingContent::ImageState() to 0. This means that the
     // image will always report its state as 0, so it will never be reframed
     // to show frames for loading or the broken image icon. This is important,
@@ -109,14 +111,14 @@ nsVideoFrame::CreateAnonymousContent(nsTArray<nsIContent*>& aElements)
     nsCOMPtr<nsIImageLoadingContent> imgContent = do_QueryInterface(mPosterImage);
     NS_ENSURE_TRUE(imgContent, NS_ERROR_FAILURE);
 
-    imgContent->ForceImageState(PR_TRUE, 0);    
+    imgContent->ForceImageState(PR_TRUE, 0);
 
     nsresult res = UpdatePosterSource(PR_FALSE);
     NS_ENSURE_SUCCESS(res,res);
-    
+
     if (!aElements.AppendElement(mPosterImage))
       return NS_ERROR_OUT_OF_MEMORY;
-  } 
+  }
 
   // Set up "videocontrols" XUL element which will be XBL-bound to the
   // actual controls.
@@ -127,7 +129,7 @@ nsVideoFrame::CreateAnonymousContent(nsTArray<nsIContent*>& aElements)
 
   nsresult rv = NS_NewElement(getter_AddRefs(mVideoControls),
                               kNameSpaceID_XUL,
-                              nodeInfo,
+                              nodeInfo.forget(),
                               PR_FALSE);
   NS_ENSURE_SUCCESS(rv, rv);
   if (!aElements.AppendElement(mVideoControls))
@@ -137,11 +139,18 @@ nsVideoFrame::CreateAnonymousContent(nsTArray<nsIContent*>& aElements)
 }
 
 void
-nsVideoFrame::Destroy()
+nsVideoFrame::AppendAnonymousContentTo(nsBaseContentList& aElements)
+{
+  aElements.MaybeAppendElement(mPosterImage);
+  aElements.MaybeAppendElement(mVideoControls);
+}
+
+void
+nsVideoFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
   nsContentUtils::DestroyAnonymousContent(&mVideoControls);
   nsContentUtils::DestroyAnonymousContent(&mPosterImage);
-  nsContainerFrame::Destroy();
+  nsContainerFrame::DestroyFrom(aDestructRoot);
 }
 
 PRBool
@@ -159,32 +168,109 @@ CorrectForAspectRatio(const gfxRect& aRect, const nsIntSize& aRatio)
                "Nothing to draw");
   // Choose scale factor that scales aRatio to just fit into aRect
   gfxFloat scale =
-    PR_MIN(aRect.Width()/aRatio.width, aRect.Height()/aRatio.height);
+    NS_MIN(aRect.Width()/aRatio.width, aRect.Height()/aRatio.height);
   gfxSize scaledRatio(scale*aRatio.width, scale*aRatio.height);
   gfxPoint topLeft((aRect.Width() - scaledRatio.width)/2,
                    (aRect.Height() - scaledRatio.height)/2);
   return gfxRect(aRect.TopLeft() + topLeft, scaledRatio);
 }
 
-void
-nsVideoFrame::PaintVideo(nsIRenderingContext& aRenderingContext,
-                         const nsRect& aDirtyRect, nsPoint aPt) 
+already_AddRefed<Layer>
+nsVideoFrame::BuildLayer(nsDisplayListBuilder* aBuilder,
+                         LayerManager* aManager,
+                         nsDisplayItem* aItem)
 {
-  nsRect area = GetContentRect() - GetPosition() + aPt;
+  nsRect area = GetContentRect() + aBuilder->ToReferenceFrame(GetParent());
   nsHTMLVideoElement* element = static_cast<nsHTMLVideoElement*>(GetContent());
   nsIntSize videoSize = element->GetVideoSize(nsIntSize(0, 0));
   if (videoSize.width <= 0 || videoSize.height <= 0 || area.IsEmpty())
-    return;
+    return nsnull;
 
-  gfxContext* ctx = static_cast<gfxContext*>(aRenderingContext.GetNativeGraphicData(nsIRenderingContext::NATIVE_THEBES_CONTEXT));
+  nsRefPtr<ImageContainer> container = element->GetImageContainer();
+  // If we have a container with a different layer manager, try to hand
+  // off the container to the new one.
+  if (container && container->Manager() != aManager) {
+    // we don't care about the return type here -- if the set didn't take, it'll
+    // be handled when we next check the manager
+    container->SetLayerManager(aManager);
+  }
+
+  // If we have a container with the right layer manager already, we don't
+  // need to do anything here. Otherwise we need to set up a temporary
+  // ImageContainer, capture the video data and store it in the temp
+  // container.
+  if (!container || container->Manager() != aManager) {
+    nsRefPtr<ImageContainer> tmpContainer = aManager->CreateImageContainer();
+    if (!tmpContainer)
+      return nsnull;
+    
+    // We get a reference to the video data as a cairo surface.
+    CairoImage::Data cairoData;
+    nsRefPtr<gfxASurface> imageSurface;
+    if (container) {
+      // Get video from the existing container. It was created for a
+      // different layer manager, so we do fallback through cairo.
+      imageSurface = container->GetCurrentAsSurface(&cairoData.mSize);
+      if (!imageSurface) {
+        // we couldn't do fallback, so we've got nothing to do here
+        return nsnull;
+      }
+      cairoData.mSurface = imageSurface;
+    } else {
+      // We're probably printing.
+      cairoData.mSurface = element->GetPrintSurface();
+      if (!cairoData.mSurface)
+        return nsnull;
+      cairoData.mSize = gfxIntSize(videoSize.width, videoSize.height);
+    }
+
+    // Now create a CairoImage to display the surface.
+    Image::Format cairoFormat = Image::CAIRO_SURFACE;
+    nsRefPtr<Image> image = tmpContainer->CreateImage(&cairoFormat, 1);
+    if (!image)
+      return nsnull;
+
+    NS_ASSERTION(image->GetFormat() == cairoFormat, "Wrong format");
+    static_cast<CairoImage*>(image.get())->SetData(cairoData);
+    tmpContainer->SetCurrentImage(image);
+    container = tmpContainer.forget();
+  }
+
+  // Retrieve the size of the decoded video frame, before being scaled
+  // by pixel aspect ratio.
+  gfxIntSize frameSize = container->GetCurrentSize();
+  if (frameSize.width == 0 || frameSize.height == 0) {
+    // No image, or zero-sized image. No point creating a layer.
+    return nsnull;
+  }
+
+  // Compute the rectangle in which to paint the video. We need to use
+  // the largest rectangle that fills our content-box and has the
+  // correct aspect ratio.
   nsPresContext* presContext = PresContext();
-  gfxRect r = gfxRect(presContext->AppUnitsToGfxUnits(area.x), 
-                      presContext->AppUnitsToGfxUnits(area.y), 
-                      presContext->AppUnitsToGfxUnits(area.width), 
+  gfxRect r = gfxRect(presContext->AppUnitsToGfxUnits(area.x),
+                      presContext->AppUnitsToGfxUnits(area.y),
+                      presContext->AppUnitsToGfxUnits(area.width),
                       presContext->AppUnitsToGfxUnits(area.height));
-
   r = CorrectForAspectRatio(r, videoSize);
-  element->Paint(ctx, nsLayoutUtils::GetGraphicsFilterForFrame(this), r);
+
+  nsRefPtr<ImageLayer> layer = static_cast<ImageLayer*>
+    (aBuilder->LayerBuilder()->GetLeafLayerFor(aBuilder, aManager, aItem));
+  if (!layer) {
+    layer = aManager->CreateImageLayer();
+    if (!layer)
+      return nsnull;
+  }
+
+  layer->SetContainer(container);
+  layer->SetFilter(nsLayoutUtils::GetGraphicsFilterForFrame(this));
+  // Set a transform on the layer to draw the video in the right place
+  gfxMatrix transform;
+  transform.Translate(r.pos);
+  transform.Scale(r.Width()/frameSize.width, r.Height()/frameSize.height);
+  layer->SetTransform(gfx3DMatrix::From2D(transform));
+  nsRefPtr<Layer> result = layer.forget();
+  return result.forget();
 }
 
 NS_IMETHODIMP
@@ -234,7 +320,7 @@ nsVideoFrame::Reflow(nsPresContext*           aPresContext,
         kidReflowState.SetComputedHeight(aReflowState.ComputedHeight());
       } else {
         kidReflowState.SetComputedWidth(0);
-        kidReflowState.SetComputedHeight(0);      
+        kidReflowState.SetComputedHeight(0);
       }
       ReflowChild(imageFrame, aPresContext, kidDesiredSize, kidReflowState,
                   mBorderPadding.left, mBorderPadding.top, 0, aStatus);
@@ -268,20 +354,59 @@ nsVideoFrame::Reflow(nsPresContext*           aPresContext,
   return NS_OK;
 }
 
-static void PaintVideo(nsIFrame* aFrame, nsIRenderingContext* aCtx,
-                        const nsRect& aDirtyRect, nsPoint aPt)
-{
-#if 0
-  double start = double(PR_IntervalToMilliseconds(PR_IntervalNow()))/1000.0;
+class nsDisplayVideo : public nsDisplayItem {
+public:
+  nsDisplayVideo(nsVideoFrame* aFrame)
+    : nsDisplayItem(aFrame)
+  {
+    MOZ_COUNT_CTOR(nsDisplayVideo);
+  }
+#ifdef NS_BUILD_REFCNT_LOGGING
+  virtual ~nsDisplayVideo() {
+    MOZ_COUNT_DTOR(nsDisplayVideo);
+  }
 #endif
+  
+  NS_DISPLAY_DECL_NAME("Video", TYPE_VIDEO)
 
-  static_cast<nsVideoFrame*>(aFrame)->PaintVideo(*aCtx, aDirtyRect, aPt);
-#if 0
-  double end = double(PR_IntervalToMilliseconds(PR_IntervalNow()))/1000.0;
-  printf("PaintVideo: %f\n", (float)end - (float)start);
+  // It would be great if we could override IsOpaque to return false here,
+  // but it's probably not safe to do so in general. Video frames are
+  // updated asynchronously from decoder threads, and it's possible that
+  // we might have an opaque video frame when IsOpaque is called, but
+  // when we come to paint, the video frame is transparent or has gone
+  // away completely (e.g. because of a decoder error). The problem would
+  // be especially acute if we have off-main-thread rendering.
 
-#endif
-}
+  virtual nsRect GetBounds(nsDisplayListBuilder* aBuilder)
+  {
+    nsIFrame* f = GetUnderlyingFrame();
+    return f->GetContentRect() + aBuilder->ToReferenceFrame(f->GetParent());
+  }
+
+  virtual already_AddRefed<Layer> BuildLayer(nsDisplayListBuilder* aBuilder,
+                                             LayerManager* aManager)
+  {
+    return static_cast<nsVideoFrame*>(mFrame)->BuildLayer(aBuilder, aManager, this);
+  }
+
+  virtual LayerState GetLayerState(nsDisplayListBuilder* aBuilder,
+                                   LayerManager* aManager)
+  {
+    if (aManager->GetBackendType() != LayerManager::LAYERS_BASIC) {
+      // For non-basic layer managers we can assume that compositing
+      // layers is very cheap, and since ImageLayers don't require
+      // additional memory of the video frames we have to have anyway,
+      // we can't save much by making layers inactive. Also, for many
+      // accelerated layer managers calling
+      // imageContainer->GetCurrentAsSurface can be very expensive. So
+      // just always be active for these managers.
+      return LAYER_ACTIVE;
+    }
+    nsHTMLMediaElement* elem =
+      static_cast<nsHTMLMediaElement*>(mFrame->GetContent());
+    return elem->IsPotentiallyPlaying() ? LAYER_ACTIVE : LAYER_INACTIVE;
+  }
+};
 
 NS_IMETHODIMP
 nsVideoFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
@@ -296,8 +421,9 @@ nsVideoFrame::BuildDisplayList(nsDisplayListBuilder*   aBuilder,
   nsresult rv = DisplayBorderBackgroundOutline(aBuilder, aLists);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (!ShouldDisplayPoster() && HasVideoData()) {
-    rv = aLists.Content()->AppendNewToTop(new (aBuilder) nsDisplayGeneric(this, ::PaintVideo, "Video"));
+  if (HasVideoElement() && !ShouldDisplayPoster()) {
+    rv = aLists.Content()->AppendNewToTop(
+      new (aBuilder) nsDisplayVideo(this));
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -329,15 +455,14 @@ nsVideoFrame::GetType() const
 }
 
 #ifdef ACCESSIBILITY
-NS_IMETHODIMP
-nsVideoFrame::GetAccessible(nsIAccessible** aAccessible)
+already_AddRefed<nsAccessible>
+nsVideoFrame::CreateAccessible()
 {
   nsCOMPtr<nsIAccessibilityService> accService =
     do_GetService("@mozilla.org/accessibilityService;1");
-  NS_ENSURE_STATE(accService);
-
-  return accService->CreateHTMLMediaAccessible(static_cast<nsIFrame*>(this),
-                                               aAccessible);
+  return accService ?
+    accService->CreateHTMLMediaAccessible(mContent, PresContext()->PresShell()) :
+    nsnull;
 }
 #endif
 
@@ -350,14 +475,14 @@ nsVideoFrame::GetFrameName(nsAString& aResult) const
 #endif
 
 nsSize nsVideoFrame::ComputeSize(nsIRenderingContext *aRenderingContext,
-                                     nsSize aCBSize, 
+                                     nsSize aCBSize,
                                      nscoord aAvailableWidth,
-                                     nsSize aMargin, 
-                                     nsSize aBorder, 
+                                     nsSize aMargin,
+                                     nsSize aBorder,
                                      nsSize aPadding,
                                      PRBool aShrinkWrap)
 {
-  nsSize size = GetIntrinsicSize(aRenderingContext);
+  nsSize size = GetVideoIntrinsicSize(aRenderingContext);
 
   IntrinsicSize intrinsicSize;
   intrinsicSize.width.SetCoordValue(size.width);
@@ -365,33 +490,33 @@ nsSize nsVideoFrame::ComputeSize(nsIRenderingContext *aRenderingContext,
 
   nsSize& intrinsicRatio = size; // won't actually be used
 
-  return nsLayoutUtils::ComputeSizeWithIntrinsicDimensions(aRenderingContext, 
+  return nsLayoutUtils::ComputeSizeWithIntrinsicDimensions(aRenderingContext,
                                                            this,
                                                            intrinsicSize,
                                                            intrinsicRatio,
-                                                           aCBSize, 
-                                                           aMargin, 
-                                                           aBorder, 
+                                                           aCBSize,
+                                                           aMargin,
+                                                           aBorder,
                                                            aPadding);
 }
 
 nscoord nsVideoFrame::GetMinWidth(nsIRenderingContext *aRenderingContext)
 {
-  nscoord result = GetIntrinsicSize(aRenderingContext).width;
+  nscoord result = GetVideoIntrinsicSize(aRenderingContext).width;
   DISPLAY_MIN_WIDTH(this, result);
   return result;
 }
 
 nscoord nsVideoFrame::GetPrefWidth(nsIRenderingContext *aRenderingContext)
 {
-  nscoord result = GetIntrinsicSize(aRenderingContext).width;
+  nscoord result = GetVideoIntrinsicSize(aRenderingContext).width;
   DISPLAY_PREF_WIDTH(this, result);
   return result;
 }
 
 nsSize nsVideoFrame::GetIntrinsicRatio()
 {
-  return GetIntrinsicSize(nsnull);
+  return GetVideoIntrinsicSize(nsnull);
 }
 
 PRBool nsVideoFrame::ShouldDisplayPoster()
@@ -405,7 +530,7 @@ PRBool nsVideoFrame::ShouldDisplayPoster()
 
   nsCOMPtr<nsIImageLoadingContent> imgContent = do_QueryInterface(mPosterImage);
   NS_ENSURE_TRUE(imgContent, PR_FALSE);
-  
+
   nsCOMPtr<imgIRequest> request;
   nsresult res = imgContent->GetRequest(nsIImageLoadingContent::CURRENT_REQUEST,
                                         getter_AddRefs(request));
@@ -416,15 +541,16 @@ PRBool nsVideoFrame::ShouldDisplayPoster()
   PRUint32 status = 0;
   res = request->GetImageStatus(&status);
   if (NS_FAILED(res) || (status & imgIRequest::STATUS_ERROR))
-    return PR_FALSE;  
-  
+    return PR_FALSE;
+
   return PR_TRUE;
 }
 
-nsSize nsVideoFrame::GetIntrinsicSize(nsIRenderingContext *aRenderingContext)
+nsSize
+nsVideoFrame::GetVideoIntrinsicSize(nsIRenderingContext *aRenderingContext)
 {
   // Defaulting size to 300x150 if no size given.
-  nsIntSize size(300,150);
+  nsIntSize size(300, 150);
 
   if (ShouldDisplayPoster()) {
     // Use the poster image frame's size.
@@ -437,7 +563,7 @@ nsSize nsVideoFrame::GetIntrinsicSize(nsIRenderingContext *aRenderingContext)
     }
   }
 
-  if (!HasVideoData()) {
+  if (!HasVideoElement()) {
     if (!aRenderingContext || !mFrames.FirstChild()) {
       // We just want our intrinsic ratio, but audio elements need no
       // intrinsic ratio, so just return "no ratio". Also, if there's
@@ -454,7 +580,7 @@ nsSize nsVideoFrame::GetIntrinsicSize(nsIRenderingContext *aRenderingContext)
   nsHTMLVideoElement* element = static_cast<nsHTMLVideoElement*>(GetContent());
   size = element->GetVideoSize(size);
 
-  return nsSize(nsPresContext::CSSPixelsToAppUnits(size.width), 
+  return nsSize(nsPresContext::CSSPixelsToAppUnits(size.width),
                 nsPresContext::CSSPixelsToAppUnits(size.height));
 }
 
@@ -498,6 +624,6 @@ PRBool nsVideoFrame::HasVideoData()
   if (!HasVideoElement())
     return PR_FALSE;
   nsHTMLVideoElement* element = static_cast<nsHTMLVideoElement*>(GetContent());
-  nsIntSize size = element->GetVideoSize(nsIntSize(0,0));    
+  nsIntSize size = element->GetVideoSize(nsIntSize(0,0));
   return size != nsIntSize(0,0);
 }
