@@ -41,10 +41,14 @@
  */
 
 #include "nsCSSDataBlock.h"
+#include "mozilla/css/Declaration.h"
 #include "nsCSSProps.h"
 #include "nsRuleData.h"
 #include "nsRuleNode.h"
 #include "nsStyleSet.h"
+#include "nsStyleContext.h"
+
+namespace css = mozilla::css;
 
 /*
  * nsCSSCompressedDataBlock holds property-value pairs corresponding to
@@ -170,7 +174,54 @@ ShouldIgnoreColors(nsRuleData *aRuleData)
            !aRuleData->mPresContext->UseDocumentColors();
 }
 
-nsresult
+/**
+ * Tries to call |nsCSSValue::StartImageLoad()| on an image source.
+ * Image sources are specified by |url()| or |-moz-image-rect()| function.
+ */
+static void
+TryToStartImageLoadOnValue(const nsCSSValue& aValue, nsIDocument* aDocument)
+{
+  if (aValue.GetUnit() == eCSSUnit_URL) {
+    aValue.StartImageLoad(aDocument);
+  }
+  else if (aValue.EqualsFunction(eCSSKeyword__moz_image_rect)) {
+    nsCSSValue::Array* arguments = aValue.GetArrayValue();
+    NS_ABORT_IF_FALSE(arguments->Count() == 6, "unexpected num of arguments");
+
+    const nsCSSValue& image = arguments->Item(1);
+    if (image.GetUnit() == eCSSUnit_URL)
+      image.StartImageLoad(aDocument);
+  }
+}
+
+static void
+TryToStartImageLoad(const nsCSSValue& aValue, nsIDocument* aDocument,
+                    nsCSSProperty aProperty)
+{
+  if (nsCSSProps::PropHasFlags(aProperty, CSS_PROPERTY_IMAGE_IS_IN_ARRAY_0)) {
+    if (aValue.GetUnit() == eCSSUnit_Array) {
+      TryToStartImageLoadOnValue(aValue.GetArrayValue()->Item(0), aDocument);
+    }
+  } else {
+    TryToStartImageLoadOnValue(aValue, aDocument);
+  }
+}
+
+static inline PRBool
+ShouldStartImageLoads(nsRuleData *aRuleData, nsCSSProperty aProperty)
+{
+  // Don't initiate image loads for if-visited styles.  This is
+  // important because:
+  //  (1) it's a waste of CPU and bandwidth
+  //  (2) in some cases we'd start the image load on a style change
+  //      where we wouldn't have started the load initially, which makes
+  //      which links are visited detectable to Web pages (see bug
+  //      557287)
+  return !aRuleData->mStyleContext->IsStyleIfVisited() &&
+         nsCSSProps::PropHasFlags(aProperty, CSS_PROPERTY_START_IMAGE_LOADS);
+}
+
+void
 nsCSSCompressedDataBlock::MapRuleInfoInto(nsRuleData *aRuleData) const
 {
     // If we have no data for these structs, then return immediately.
@@ -178,7 +229,9 @@ nsCSSCompressedDataBlock::MapRuleInfoInto(nsRuleData *aRuleData) const
     // have to worry much less (although still some) about the speed of
     // the rest of the function.
     if (!(aRuleData->mSIDs & mStyleBits))
-        return NS_OK;
+        return;
+
+    nsIDocument* doc = aRuleData->mPresContext->Document();
 
     const char* cursor = Block();
     const char* cursor_end = BlockEnd();
@@ -188,27 +241,15 @@ nsCSSCompressedDataBlock::MapRuleInfoInto(nsRuleData *aRuleData) const
                      "out of range");
         if (nsCachedStyleData::GetBitForSID(nsCSSProps::kSIDTable[iProp]) &
             aRuleData->mSIDs) {
-            void *prop =
-                nsCSSExpandedDataBlock::RuleDataPropertyAt(aRuleData, iProp);
+            void *prop = aRuleData->StorageFor(iProp);
             switch (nsCSSProps::kTypeTable[iProp]) {
                 case eCSSType_Value: {
                     nsCSSValue* target = static_cast<nsCSSValue*>(prop);
                     if (target->GetUnit() == eCSSUnit_Null) {
                         const nsCSSValue *val = ValueAtCursor(cursor);
                         NS_ASSERTION(val->GetUnit() != eCSSUnit_Null, "oops");
-                        if (iProp == eCSSProperty_list_style_image) {
-                            if (val->GetUnit() == eCSSUnit_URL) {
-                                val->StartImageLoad(
-                                    aRuleData->mPresContext->Document());
-                            }
-                        } else if (iProp == eCSSProperty_border_image) {
-                            if (val->GetUnit() == eCSSUnit_Array) {
-                                nsCSSValue::Array *array = val->GetArrayValue();
-                                if (array->Item(0).GetUnit() == eCSSUnit_URL) {
-                                    array->Item(0).StartImageLoad(
-                                        aRuleData->mPresContext->Document());
-                                }
-                            }
+                        if (ShouldStartImageLoads(aRuleData, iProp)) {
+                            TryToStartImageLoad(*val, doc, iProp);
                         }
                         *target = *val;
                         if (iProp == eCSSProperty_font_family) {
@@ -273,30 +314,16 @@ nsCSSCompressedDataBlock::MapRuleInfoInto(nsRuleData *aRuleData) const
                 } break;
 
                 case eCSSType_ValueList:
-                    if (iProp == eCSSProperty_background_image ||
-                        iProp == eCSSProperty_content) {
-                        for (nsCSSValueList* l = ValueListAtCursor(cursor);
-                             l; l = l->mNext)
-                            if (l->mValue.GetUnit() == eCSSUnit_URL)
-                                l->mValue.StartImageLoad(
-                                    aRuleData->mPresContext->Document());
-                    } else if (iProp == eCSSProperty_cursor) {
-                        for (nsCSSValueList* l = ValueListAtCursor(cursor);
-                             l; l = l->mNext)
-                            if (l->mValue.GetUnit() == eCSSUnit_Array) {
-                                // Don't try to restart loads we've already
-                                // started
-                                nsCSSValue& val =
-                                    l->mValue.GetArrayValue()->Item(0);
-                                if (val.GetUnit() == eCSSUnit_URL)
-                                    val.StartImageLoad(
-                                      aRuleData->mPresContext->Document());
-                            }
-                    }
-                // fall through
                 case eCSSType_ValuePairList: {
                     void** target = static_cast<void**>(prop);
                     if (!*target) {
+                        if (ShouldStartImageLoads(aRuleData, iProp)) {
+                            for (nsCSSValueList* l = ValueListAtCursor(cursor);
+                                 l; l = l->mNext) {
+                                TryToStartImageLoad(l->mValue, doc, iProp);
+                            }
+                        }
+
                         void* val = PointerAtCursor(cursor);
                         NS_ASSERTION(val, "oops");
                         *target = val;
@@ -333,8 +360,6 @@ nsCSSCompressedDataBlock::MapRuleInfoInto(nsRuleData *aRuleData) const
         }
     }
     NS_ASSERTION(cursor == cursor_end, "inconsistent data");
-
-    return NS_OK;
 }
 
 const void*
@@ -404,8 +429,8 @@ nsCSSCompressedDataBlock::Clone() const
     const char *cursor = Block(), *cursor_end = BlockEnd();
     char *result_cursor;
 
-    nsCSSCompressedDataBlock *result =
-        new(cursor_end - cursor) nsCSSCompressedDataBlock();
+    nsAutoPtr<nsCSSCompressedDataBlock> result
+        (new(cursor_end - cursor) nsCSSCompressedDataBlock());
     if (!result)
         return nsnull;
     result_cursor = result->Block();
@@ -462,8 +487,8 @@ nsCSSCompressedDataBlock::Clone() const
                         break;
                 }
                 if (!copy) {
+                    // so the destructor knows where to stop clearing
                     result->mBlockEnd = result_cursor;
-                    result->Destroy();
                     return nsnull;
                 }
                 PointerAtCursor(result_cursor) = copy;
@@ -477,11 +502,11 @@ nsCSSCompressedDataBlock::Clone() const
     result->mBlockEnd = result_cursor;
     result->mStyleBits = mStyleBits;
     NS_ASSERTION(result->DataSize() == DataSize(), "wrong size");
-    return result;
+
+    return result.forget();
 }
 
-void
-nsCSSCompressedDataBlock::Destroy()
+nsCSSCompressedDataBlock::~nsCSSCompressedDataBlock()
 {
     const char* cursor = Block();
     const char* cursor_end = BlockEnd();
@@ -529,24 +554,80 @@ nsCSSCompressedDataBlock::Destroy()
         }
     }
     NS_ASSERTION(cursor == cursor_end, "inconsistent data");
-    delete this;
 }
 
 /* static */ nsCSSCompressedDataBlock*
 nsCSSCompressedDataBlock::CreateEmptyBlock()
 {
     nsCSSCompressedDataBlock *result = new(0) nsCSSCompressedDataBlock();
-    if (!result)
-        return nsnull;
     result->mBlockEnd = result->Block();
     return result;
 }
 
+/* static */ void
+nsCSSCompressedDataBlock::MoveValue(void *aSource, void *aDest,
+                                    nsCSSProperty aPropID,
+                                    PRBool* aChanged)
+{
+  switch (nsCSSProps::kTypeTable[aPropID]) {
+    case eCSSType_Value: {
+      nsCSSValue *source = static_cast<nsCSSValue*>(aSource);
+      nsCSSValue *dest = static_cast<nsCSSValue*>(aDest);
+      if (*source != *dest)
+        *aChanged = PR_TRUE;
+      dest->~nsCSSValue();
+      memcpy(dest, source, sizeof(nsCSSValue));
+      new (source) nsCSSValue();
+    } break;
+
+    case eCSSType_Rect: {
+      nsCSSRect *source = static_cast<nsCSSRect*>(aSource);
+      nsCSSRect *dest = static_cast<nsCSSRect*>(aDest);
+      if (*source != *dest)
+        *aChanged = PR_TRUE;
+      dest->~nsCSSRect();
+      memcpy(dest, source, sizeof(nsCSSRect));
+      new (source) nsCSSRect();
+    } break;
+
+    case eCSSType_ValuePair: {
+      nsCSSValuePair *source = static_cast<nsCSSValuePair*>(aSource);
+      nsCSSValuePair *dest = static_cast<nsCSSValuePair*>(aDest);
+      if (*source != *dest)
+        *aChanged = PR_TRUE;
+      dest->~nsCSSValuePair();
+      memcpy(dest, source, sizeof(nsCSSValuePair));
+      new (source) nsCSSValuePair();
+    } break;
+
+    case eCSSType_ValueList: {
+      nsCSSValueList **source = static_cast<nsCSSValueList**>(aSource);
+      nsCSSValueList **dest = static_cast<nsCSSValueList**>(aDest);
+      if (**source != **dest)
+        *aChanged = PR_TRUE;
+      delete *dest;
+      *dest = *source;
+      *source = nsnull;
+    } break;
+
+    case eCSSType_ValuePairList: {
+      nsCSSValuePairList **source =
+        static_cast<nsCSSValuePairList**>(aSource);
+      nsCSSValuePairList **dest =
+        static_cast<nsCSSValuePairList**>(aDest);
+      if (**source != **dest)
+        *aChanged = PR_TRUE;
+      delete *dest;
+      *dest = *source;
+      *source = nsnull;
+    } break;
+  }
+}
+
 /*****************************************************************************/
 
 nsCSSExpandedDataBlock::nsCSSExpandedDataBlock()
 {
-    ClearSets();
     AssertInitialState();
 }
 
@@ -555,32 +636,22 @@ nsCSSExpandedDataBlock::~nsCSSExpandedDataBlock()
     AssertInitialState();
 }
 
-const nsCSSExpandedDataBlock::PropertyOffsetInfo
-nsCSSExpandedDataBlock::kOffsetTable[eCSSProperty_COUNT_no_shorthands] = {
-    #define CSS_PROP_BACKENDONLY(name_, id_, method_, flags_, datastruct_, member_, type_, kwtable_) \
-        { offsetof(nsCSSExpandedDataBlock, m##datastruct_.member_),           \
-          size_t(-1),                                                         \
-          size_t(-1) },
-    #define CSS_PROP(name_, id_, method_, flags_, datastruct_, member_, type_, kwtable_) \
-        { offsetof(nsCSSExpandedDataBlock, m##datastruct_.member_),           \
-          offsetof(nsRuleData, m##datastruct_##Data),                         \
-          offsetof(nsRuleData##datastruct_, member_) },
+const size_t
+nsCSSExpandedDataBlock::kOffsetTable[] = {
+    #define CSS_PROP(name_, id_, method_, flags_, datastruct_, member_, type_, \
+                     kwtable_, stylestruct_, stylestructoffset_, animtype_)    \
+        offsetof(nsCSSExpandedDataBlock, m##datastruct_.member_),
     #include "nsCSSPropList.h"
     #undef CSS_PROP
-    #undef CSS_PROP_BACKENDONLY
 };
 
 void
 nsCSSExpandedDataBlock::DoExpand(nsCSSCompressedDataBlock *aBlock,
                                  PRBool aImportant)
 {
-    NS_PRECONDITION(aBlock, "unexpected null block");
-
     /*
      * Save needless copying and allocation by copying the memory
-     * corresponding to the stored data in the compressed block, and
-     * then, to avoid destructors, deleting the compressed block by
-     * calling |delete| instead of using its |Destroy| method.
+     * corresponding to the stored data in the compressed block.
      */
     const char* cursor = aBlock->Block();
     const char* cursor_end = aBlock->BlockEnd();
@@ -650,21 +721,21 @@ nsCSSExpandedDataBlock::DoExpand(nsCSSCompressedDataBlock *aBlock,
     }
     NS_ASSERTION(cursor == cursor_end, "inconsistent data");
 
+    // Don't destroy remnants of what we just copied
+    aBlock->mBlockEnd = aBlock->Block();
     delete aBlock;
 }
 
 void
-nsCSSExpandedDataBlock::Expand(nsCSSCompressedDataBlock **aNormalBlock,
-                               nsCSSCompressedDataBlock **aImportantBlock)
+nsCSSExpandedDataBlock::Expand(nsCSSCompressedDataBlock *aNormalBlock,
+                               nsCSSCompressedDataBlock *aImportantBlock)
 {
-    NS_PRECONDITION(*aNormalBlock, "unexpected null block");
+    NS_PRECONDITION(aNormalBlock, "unexpected null block");
     AssertInitialState();
 
-    DoExpand(*aNormalBlock, PR_FALSE);
-    *aNormalBlock = nsnull;
-    if (*aImportantBlock) {
-        DoExpand(*aImportantBlock, PR_TRUE);
-        *aImportantBlock = nsnull;
+    DoExpand(aNormalBlock, PR_FALSE);
+    if (aImportantBlock) {
+        DoExpand(aImportantBlock, PR_TRUE);
     }
 }
 
@@ -672,14 +743,13 @@ nsCSSExpandedDataBlock::ComputeSizeResult
 nsCSSExpandedDataBlock::ComputeSize()
 {
     ComputeSizeResult result = {0, 0};
-    for (PRUint32 iHigh = 0; iHigh < NS_ARRAY_LENGTH(mPropertiesSet); ++iHigh) {
-        if (mPropertiesSet[iHigh] == 0)
+    for (size_t iHigh = 0; iHigh < nsCSSPropertySet::kChunkCount; ++iHigh) {
+        if (!mPropertiesSet.HasPropertyInChunk(iHigh))
             continue;
-        for (PRInt32 iLow = 0; iLow < kPropertiesSetChunkSize; ++iLow) {
-            if ((mPropertiesSet[iHigh] & (1 << iLow)) == 0)
+        for (size_t iLow = 0; iLow < nsCSSPropertySet::kBitsInChunk; ++iLow) {
+            if (!mPropertiesSet.HasPropertyAt(iHigh, iLow))
                 continue;
-            nsCSSProperty iProp =
-                nsCSSProperty(iHigh * kPropertiesSetChunkSize + iLow);
+            nsCSSProperty iProp = nsCSSPropertySet::CSSPropertyAt(iHigh, iLow);
             NS_ASSERTION(0 <= iProp && iProp < eCSSProperty_COUNT_no_shorthands,
                          "out of range");
 #ifdef DEBUG
@@ -724,10 +794,10 @@ nsCSSExpandedDataBlock::ComputeSize()
                     increment = CDBPointerStorage_advance;
                 } break;
             }
-            if ((mPropertiesImportant[iHigh] & (1 << iLow)) == 0)
-                result.normal += increment;
-            else
+            if (mPropertiesImportant.HasPropertyAt(iHigh, iLow))
                 result.important += increment;
+            else
+                result.normal += increment;
         }
     }
     return result;
@@ -737,11 +807,11 @@ void
 nsCSSExpandedDataBlock::Compress(nsCSSCompressedDataBlock **aNormalBlock,
                                  nsCSSCompressedDataBlock **aImportantBlock)
 {
-    nsCSSCompressedDataBlock *result_normal, *result_important;
+    nsAutoPtr<nsCSSCompressedDataBlock> result_normal, result_important;
     char *cursor_normal, *cursor_important;
 
     ComputeSizeResult size = ComputeSize();
-    
+
     result_normal = new(size.normal) nsCSSCompressedDataBlock();
     if (!result_normal) {
         *aNormalBlock = nsnull;
@@ -753,7 +823,6 @@ nsCSSExpandedDataBlock::Compress(nsCSSCompressedDataBlock **aNormalBlock,
     if (size.important != 0) {
         result_important = new(size.important) nsCSSCompressedDataBlock();
         if (!result_important) {
-            delete result_normal;
             *aNormalBlock = nsnull;
             *aImportantBlock = nsnull;
             return;
@@ -768,19 +837,18 @@ nsCSSExpandedDataBlock::Compress(nsCSSCompressedDataBlock **aNormalBlock,
      * corresponding to the stored data in the expanded block, and then
      * clearing the data in the expanded block.
      */
-    for (PRUint32 iHigh = 0; iHigh < NS_ARRAY_LENGTH(mPropertiesSet); ++iHigh) {
-        if (mPropertiesSet[iHigh] == 0)
+    for (size_t iHigh = 0; iHigh < nsCSSPropertySet::kChunkCount; ++iHigh) {
+        if (!mPropertiesSet.HasPropertyInChunk(iHigh))
             continue;
-        for (PRInt32 iLow = 0; iLow < kPropertiesSetChunkSize; ++iLow) {
-            if ((mPropertiesSet[iHigh] & (1 << iLow)) == 0)
+        for (size_t iLow = 0; iLow < nsCSSPropertySet::kBitsInChunk; ++iLow) {
+            if (!mPropertiesSet.HasPropertyAt(iHigh, iLow))
                 continue;
-            nsCSSProperty iProp =
-                nsCSSProperty(iHigh * kPropertiesSetChunkSize + iLow);
+            nsCSSProperty iProp = nsCSSPropertySet::CSSPropertyAt(iHigh, iLow);
             NS_ASSERTION(0 <= iProp && iProp < eCSSProperty_COUNT_no_shorthands,
                          "out of range");
             void *prop = PropertyAt(iProp);
             PRBool important =
-                (mPropertiesImportant[iHigh] & (1 << iLow)) != 0;
+                mPropertiesImportant.HasPropertyAt(iHigh, iLow);
             char *&cursor = important ? cursor_important : cursor_normal;
             nsCSSCompressedDataBlock *result =
                 important ? result_important : result_normal;
@@ -850,22 +918,21 @@ nsCSSExpandedDataBlock::Compress(nsCSSCompressedDataBlock **aNormalBlock,
 
     ClearSets();
     AssertInitialState();
-    *aNormalBlock = result_normal;
-    *aImportantBlock = result_important;
+    *aNormalBlock = result_normal.forget();
+    *aImportantBlock = result_important.forget();
 }
 
 void
 nsCSSExpandedDataBlock::Clear()
 {
-    for (PRUint32 iHigh = 0; iHigh < NS_ARRAY_LENGTH(mPropertiesSet); ++iHigh) {
-        if (mPropertiesSet[iHigh] == 0)
+    for (size_t iHigh = 0; iHigh < nsCSSPropertySet::kChunkCount; ++iHigh) {
+        if (!mPropertiesSet.HasPropertyInChunk(iHigh))
             continue;
-        for (PRInt32 iLow = 0; iLow < kPropertiesSetChunkSize; ++iLow) {
-            if ((mPropertiesSet[iHigh] & (1 << iLow)) == 0)
+        for (size_t iLow = 0; iLow < nsCSSPropertySet::kBitsInChunk; ++iLow) {
+            if (!mPropertiesSet.HasPropertyAt(iHigh, iLow))
                 continue;
-            nsCSSProperty iProp =
-                nsCSSProperty(iHigh * kPropertiesSetChunkSize + iLow);
-            ClearProperty(iProp);
+            nsCSSProperty iProp = nsCSSPropertySet::CSSPropertyAt(iHigh, iLow);
+            ClearLonghandProperty(iProp);
         }
     }
 
@@ -875,8 +942,21 @@ nsCSSExpandedDataBlock::Clear()
 void
 nsCSSExpandedDataBlock::ClearProperty(nsCSSProperty aPropID)
 {
-    NS_ASSERTION(0 <= aPropID && aPropID < eCSSProperty_COUNT_no_shorthands,
-                 "out of range");
+  if (nsCSSProps::IsShorthand(aPropID)) {
+    CSSPROPS_FOR_SHORTHAND_SUBPROPERTIES(p, aPropID) {
+      ClearLonghandProperty(*p);
+    }
+  } else {
+    ClearLonghandProperty(aPropID);
+  }
+}
+
+void
+nsCSSExpandedDataBlock::ClearLonghandProperty(nsCSSProperty aPropID)
+{
+    NS_ABORT_IF_FALSE(0 <= aPropID &&
+                      aPropID < eCSSProperty_COUNT_no_shorthands,
+                      "out of range");
 
     ClearPropertyBit(aPropID);
     ClearImportantBit(aPropID);
@@ -918,19 +998,81 @@ nsCSSExpandedDataBlock::ClearProperty(nsCSSProperty aPropID)
     }
 }
 
+void
+nsCSSExpandedDataBlock::TransferFromBlock(nsCSSExpandedDataBlock& aFromBlock,
+                                          nsCSSProperty aPropID,
+                                          PRBool aIsImportant,
+                                          PRBool aOverrideImportant,
+                                          PRBool aMustCallValueAppended,
+                                          css::Declaration* aDeclaration,
+                                          PRBool* aChanged)
+{
+  if (nsCSSProps::IsShorthand(aPropID)) {
+    CSSPROPS_FOR_SHORTHAND_SUBPROPERTIES(p, aPropID) {
+      DoTransferFromBlock(aFromBlock, *p, aIsImportant, aOverrideImportant,
+                          aMustCallValueAppended, aDeclaration, aChanged);
+    }
+  } else {
+    DoTransferFromBlock(aFromBlock, aPropID, aIsImportant, aOverrideImportant,
+                        aMustCallValueAppended, aDeclaration, aChanged);
+  }
+}
+
+void
+nsCSSExpandedDataBlock::DoTransferFromBlock(nsCSSExpandedDataBlock& aFromBlock,
+                                            nsCSSProperty aPropID,
+                                            PRBool aIsImportant,
+                                            PRBool aOverrideImportant,
+                                            PRBool aMustCallValueAppended,
+                                            css::Declaration* aDeclaration,
+                                            PRBool* aChanged)
+{
+  NS_ASSERTION(aFromBlock.HasPropertyBit(aPropID), "oops");
+  if (aIsImportant) {
+    if (!HasImportantBit(aPropID))
+      *aChanged = PR_TRUE;
+    SetImportantBit(aPropID);
+  } else {
+    if (HasImportantBit(aPropID)) {
+      // When parsing a declaration block, an !important declaration
+      // is not overwritten by an ordinary declaration of the same
+      // property later in the block.  However, CSSOM manipulations
+      // come through here too, and in that case we do want to
+      // overwrite the property.
+      if (!aOverrideImportant) {
+        aFromBlock.ClearLonghandProperty(aPropID);
+        return;
+      }
+      *aChanged = PR_TRUE;
+      ClearImportantBit(aPropID);
+    }
+  }
+
+  if (aMustCallValueAppended || !HasPropertyBit(aPropID)) {
+    aDeclaration->ValueAppended(aPropID);
+  }
+
+  SetPropertyBit(aPropID);
+  aFromBlock.ClearPropertyBit(aPropID);
+
+  /*
+   * Save needless copying and allocation by calling the destructor in
+   * the destination, copying memory directly, and then using placement
+   * new.
+   */
+  void *v_source = aFromBlock.PropertyAt(aPropID);
+  void *v_dest = PropertyAt(aPropID);
+  nsCSSCompressedDataBlock::MoveValue(v_source, v_dest, aPropID, aChanged);
+}
+
 #ifdef DEBUG
 void
 nsCSSExpandedDataBlock::DoAssertInitialState()
 {
-    PRUint32 i;
-    for (i = 0; i < NS_ARRAY_LENGTH(mPropertiesSet); ++i) {
-        NS_ASSERTION(mPropertiesSet[i] == 0, "not initial state");
-    }
-    for (i = 0; i < NS_ARRAY_LENGTH(mPropertiesImportant); ++i) {
-        NS_ASSERTION(mPropertiesImportant[i] == 0, "not initial state");
-    }
+    mPropertiesSet.AssertIsEmpty("not initial state");
+    mPropertiesImportant.AssertIsEmpty("not initial state");
 
-    for (i = 0; i < eCSSProperty_COUNT_no_shorthands; ++i) {
+    for (PRUint32 i = 0; i < eCSSProperty_COUNT_no_shorthands; ++i) {
         void *prop = PropertyAt(nsCSSProperty(i));
         switch (nsCSSProps::kTypeTable[i]) {
             case eCSSType_Value: {
