@@ -44,11 +44,14 @@
 #include "nsString.h"
 #include "nsCoord.h"
 #include "nsCSSProperty.h"
+#include "nsCSSKeywords.h"
 #include "nsIURI.h"
 #include "nsCOMPtr.h"
 #include "nsAutoPtr.h"
 #include "nsCRTGlue.h"
 #include "nsStringBuffer.h"
+#include "nsTArray.h"
+#include "nsISupportsImpl.h"
 
 class imgIRequest;
 class nsIDocument;
@@ -93,25 +96,50 @@ enum nsCSSUnit {
   eCSSUnit_None         = 4,      // (n/a) value is none
   eCSSUnit_Normal       = 5,      // (n/a) value is normal (algorithmic, different than auto)
   eCSSUnit_System_Font  = 6,      // (n/a) value is -moz-use-system-font
-  eCSSUnit_Dummy        = 7,      // (n/a) a fake but specified value, used
+  eCSSUnit_All          = 7,      // (n/a) value is all
+  eCSSUnit_Dummy        = 8,      // (n/a) a fake but specified value, used
                                   //       only in temporary values
-  eCSSUnit_DummyInherit = 8,      // (n/a) a fake but specified value, used
+  eCSSUnit_DummyInherit = 9,      // (n/a) a fake but specified value, used
                                   //       only in temporary values
-  eCSSUnit_RectIsAuto   = 9,      // (n/a) 'auto' for an entire rect()
-  eCSSUnit_String       = 10,     // (PRUnichar*) a string value
-  eCSSUnit_Ident        = 11,     // (PRUnichar*) a string value
-  eCSSUnit_Families     = 12,     // (PRUnichar*) a string value
-  eCSSUnit_Attr         = 13,     // (PRUnichar*) a attr(string) value
-  eCSSUnit_Local_Font   = 14,     // (PRUnichar*) a local font name
-  eCSSUnit_Font_Format  = 15,     // (PRUnichar*) a font format name
+  eCSSUnit_RectIsAuto   = 10,     // (n/a) 'auto' for an entire rect()
+  eCSSUnit_String       = 11,     // (PRUnichar*) a string value
+  eCSSUnit_Ident        = 12,     // (PRUnichar*) a string value
+  eCSSUnit_Families     = 13,     // (PRUnichar*) a string value
+  eCSSUnit_Attr         = 14,     // (PRUnichar*) a attr(string) value
+  eCSSUnit_Local_Font   = 15,     // (PRUnichar*) a local font name
+  eCSSUnit_Font_Format  = 16,     // (PRUnichar*) a font format name
+  eCSSUnit_Element      = 17,     // (PRUnichar*) an element id
   eCSSUnit_Array        = 20,     // (nsCSSValue::Array*) a list of values
   eCSSUnit_Counter      = 21,     // (nsCSSValue::Array*) a counter(string,[string]) value
   eCSSUnit_Counters     = 22,     // (nsCSSValue::Array*) a counters(string,string[,string]) value
-  eCSSUnit_Function     = 23,     // (nsCSSValue::Array*) a function with parameters.  First elem of array is name,
+  eCSSUnit_Cubic_Bezier = 23,     // (nsCSSValue::Array*) a list of float values
+  eCSSUnit_Function     = 24,     // (nsCSSValue::Array*) a function with parameters.  First elem of array is name,
                                   //  the rest of the values are arguments.
 
-  eCSSUnit_URL          = 30,     // (nsCSSValue::URL*) value
-  eCSSUnit_Image        = 31,     // (nsCSSValue::Image*) value
+  // The top level of a calc() expression is either -moz-calc()
+  // (eCSSUnit_Calc), -moz-min() (eCSSUnit_Calc_Minimum), or -moz-max()
+  // (eCSSUnit_Calc_Maximum).  All remaining eCSSUnit_Calc_* units only
+  // occur inside these toplevel calc values.
+
+  // eCSSUnit_Calc has an array with exactly 1 element.  eCSSUnit_Calc
+  // exists so we can distinguish calc(2em) from 2em as specified values
+  // (but we drop this distinction for nsStyleCoord when we store
+  // computed values).
+  eCSSUnit_Calc         = 25,     // (nsCSSValue::Array*) calc() value
+  // Plus, Minus, Times_* and Divided have arrays with exactly 2
+  // elements.  a + b + c + d is grouped as ((a + b) + c) + d
+  eCSSUnit_Calc_Plus    = 26,     // (nsCSSValue::Array*) + node within calc()
+  eCSSUnit_Calc_Minus   = 27,     // (nsCSSValue::Array*) - within calc
+  eCSSUnit_Calc_Times_L = 28,     // (nsCSSValue::Array*) num * val within calc
+  eCSSUnit_Calc_Times_R = 29,     // (nsCSSValue::Array*) val * num within calc
+  eCSSUnit_Calc_Divided = 30,     // (nsCSSValue::Array*) / within calc
+  // Minimum and Maximum have arrays with 1 or more elements
+  eCSSUnit_Calc_Minimum = 31,     // (nsCSSValue::Array*) min() within calc
+  eCSSUnit_Calc_Maximum = 32,     // (nsCSSValue::Array*) max() within calc
+
+  eCSSUnit_URL          = 40,     // (nsCSSValue::URL*) value
+  eCSSUnit_Image        = 41,     // (nsCSSValue::Image*) value
+  eCSSUnit_Gradient     = 42,     // (nsCSSValueGradient*) value
   eCSSUnit_Integer      = 50,     // (int) simple value
   eCSSUnit_Enumerated   = 51,     // (int) value has enumerated meaning
   eCSSUnit_EnumColor    = 80,     // (int) enumerated color (kColorKTable)
@@ -155,6 +183,8 @@ enum nsCSSUnit {
   eCSSUnit_Milliseconds = 3001     // (float) 1/1000 second
 };
 
+struct nsCSSValueGradient;
+
 class nsCSSValue {
 public:
   struct Array;
@@ -166,31 +196,37 @@ public:
   struct Image;
   friend struct Image;
   
-  // for valueless units only (null, auto, inherit, none, normal)
+  // for valueless units only (null, auto, inherit, none, all, normal)
   explicit nsCSSValue(nsCSSUnit aUnit = eCSSUnit_Null)
     : mUnit(aUnit)
   {
     NS_ASSERTION(aUnit <= eCSSUnit_RectIsAuto, "not a valueless unit");
   }
 
-  nsCSSValue(PRInt32 aValue, nsCSSUnit aUnit) NS_HIDDEN;
-  nsCSSValue(float aValue, nsCSSUnit aUnit) NS_HIDDEN;
-  nsCSSValue(const nsString& aValue, nsCSSUnit aUnit) NS_HIDDEN;
-  explicit nsCSSValue(nscolor aValue) NS_HIDDEN;
-  nsCSSValue(Array* aArray, nsCSSUnit aUnit) NS_HIDDEN;
-  explicit nsCSSValue(URL* aValue) NS_HIDDEN;
-  explicit nsCSSValue(Image* aValue) NS_HIDDEN;
-  nsCSSValue(const nsCSSValue& aCopy) NS_HIDDEN;
+  nsCSSValue(PRInt32 aValue, nsCSSUnit aUnit);
+  nsCSSValue(float aValue, nsCSSUnit aUnit);
+  nsCSSValue(const nsString& aValue, nsCSSUnit aUnit);
+  nsCSSValue(Array* aArray, nsCSSUnit aUnit);
+  explicit nsCSSValue(URL* aValue);
+  explicit nsCSSValue(Image* aValue);
+  explicit nsCSSValue(nsCSSValueGradient* aValue);
+  nsCSSValue(const nsCSSValue& aCopy);
   ~nsCSSValue() { Reset(); }
 
-  NS_HIDDEN_(nsCSSValue&)  operator=(const nsCSSValue& aCopy);
-  NS_HIDDEN_(PRBool)      operator==(const nsCSSValue& aOther) const;
+  nsCSSValue&  operator=(const nsCSSValue& aCopy);
+  PRBool      operator==(const nsCSSValue& aOther) const;
 
   PRBool operator!=(const nsCSSValue& aOther) const
   {
     return !(*this == aOther);
   }
 
+  /**
+   * Serialize |this| as a specified value for |aProperty| and append
+   * it to |aResult|.
+   */
+  void AppendToString(nsCSSProperty aProperty, nsAString& aResult) const;
+
   nsCSSUnit GetUnit() const { return mUnit; }
   PRBool    IsLengthUnit() const
     { return eCSSUnit_Inch <= mUnit && mUnit <= eCSSUnit_Pixel; }
@@ -204,9 +240,13 @@ public:
     { return eCSSUnit_Hertz <= mUnit && mUnit <= eCSSUnit_Kilohertz; }
   PRBool    IsTimeUnit() const  
     { return eCSSUnit_Seconds <= mUnit && mUnit <= eCSSUnit_Milliseconds; }
+  PRBool    IsCalcUnit() const
+    { return eCSSUnit_Calc <= mUnit && mUnit <= eCSSUnit_Calc_Maximum; }
 
   PRBool    UnitHasStringValue() const
-    { return eCSSUnit_String <= mUnit && mUnit <= eCSSUnit_Font_Format; }
+    { return eCSSUnit_String <= mUnit && mUnit <= eCSSUnit_Element; }
+  PRBool    UnitHasArrayValue() const
+    { return eCSSUnit_Array <= mUnit && mUnit <= eCSSUnit_Calc_Maximum; }
 
   PRInt32 GetIntValue() const
   {
@@ -228,6 +268,16 @@ public:
     return mValue.mFloat;
   }
 
+  float GetAngleValue() const
+  {
+    NS_ASSERTION(eCSSUnit_Degree <= mUnit &&
+                 mUnit <= eCSSUnit_Radian, "not an angle value");
+    return mValue.mFloat;
+  }
+
+  // Converts any angle to radians.
+  double GetAngleValueInRadians() const;
+
   nsAString& GetStringValue(nsAString& aBuffer) const
   {
     NS_ASSERTION(UnitHasStringValue(), "not a string value");
@@ -253,8 +303,7 @@ public:
 
   Array* GetArrayValue() const
   {
-    NS_ASSERTION(eCSSUnit_Array <= mUnit && mUnit <= eCSSUnit_Function,
-                 "not an array value");
+    NS_ASSERTION(UnitHasArrayValue(), "not an array value");
     return mValue.mArray;
   }
 
@@ -266,6 +315,12 @@ public:
       mValue.mURL->mURI : mValue.mImage->mURI;
   }
 
+  nsCSSValueGradient* GetGradientValue() const
+  {
+    NS_ASSERTION(mUnit == eCSSUnit_Gradient, "not a gradient value");
+    return mValue.mGradient;
+  }
+
   URL* GetURLStructValue() const
   {
     // Not allowing this for Image values, because if the caller takes
@@ -286,38 +341,43 @@ public:
   // Not making this inline because that would force us to include
   // imgIRequest.h, which leads to REQUIRES hell, since this header is included
   // all over.
-  NS_HIDDEN_(imgIRequest*) GetImageValue() const;
+  imgIRequest* GetImageValue() const;
 
-  NS_HIDDEN_(nscoord)   GetLengthTwips() const;
+  nscoord GetLengthTwips() const;
 
-  NS_HIDDEN_(void)  Reset()  // sets to null
+  void Reset()  // sets to null
   {
     if (mUnit != eCSSUnit_Null)
       DoReset();
   }
 private:
-  NS_HIDDEN_(void)  DoReset();
+  void DoReset();
 
 public:
-  NS_HIDDEN_(void)  SetIntValue(PRInt32 aValue, nsCSSUnit aUnit);
-  NS_HIDDEN_(void)  SetPercentValue(float aValue);
-  NS_HIDDEN_(void)  SetFloatValue(float aValue, nsCSSUnit aUnit);
-  NS_HIDDEN_(void)  SetStringValue(const nsString& aValue, nsCSSUnit aUnit);
-  NS_HIDDEN_(void)  SetColorValue(nscolor aValue);
-  NS_HIDDEN_(void)  SetArrayValue(nsCSSValue::Array* aArray, nsCSSUnit aUnit);
-  NS_HIDDEN_(void)  SetURLValue(nsCSSValue::URL* aURI);
-  NS_HIDDEN_(void)  SetImageValue(nsCSSValue::Image* aImage);
-  NS_HIDDEN_(void)  SetAutoValue();
-  NS_HIDDEN_(void)  SetInheritValue();
-  NS_HIDDEN_(void)  SetInitialValue();
-  NS_HIDDEN_(void)  SetNoneValue();
-  NS_HIDDEN_(void)  SetNormalValue();
-  NS_HIDDEN_(void)  SetSystemFontValue();
-  NS_HIDDEN_(void)  SetDummyValue();
-  NS_HIDDEN_(void)  SetDummyInheritValue();
-  NS_HIDDEN_(void)  SetRectIsAutoValue();
-  NS_HIDDEN_(void)  StartImageLoad(nsIDocument* aDocument)
-                                   const;  // Not really const, but pretending
+  void SetIntValue(PRInt32 aValue, nsCSSUnit aUnit);
+  void SetPercentValue(float aValue);
+  void SetFloatValue(float aValue, nsCSSUnit aUnit);
+  void SetStringValue(const nsString& aValue, nsCSSUnit aUnit);
+  void SetColorValue(nscolor aValue);
+  void SetArrayValue(nsCSSValue::Array* aArray, nsCSSUnit aUnit);
+  void SetURLValue(nsCSSValue::URL* aURI);
+  void SetImageValue(nsCSSValue::Image* aImage);
+  void SetGradientValue(nsCSSValueGradient* aGradient);
+  void SetAutoValue();
+  void SetInheritValue();
+  void SetInitialValue();
+  void SetNoneValue();
+  void SetAllValue();
+  void SetNormalValue();
+  void SetSystemFontValue();
+  void SetDummyValue();
+  void SetDummyInheritValue();
+  void StartImageLoad(nsIDocument* aDocument) const;  // Only pretend const
+
+  // Initializes as a function value with the specified function id.
+  Array* InitFunction(nsCSSKeyword aFunctionId, PRUint32 aNumArgs);
+  // Checks if this is a function value with the specified function id.
+  PRBool EqualsFunction(nsCSSKeyword aFunctionId) const;
 
   // Returns an already addrefed buffer.  Can return null on allocation
   // failure.
@@ -331,17 +391,17 @@ public:
     // aString must not be null.
     // aOriginPrincipal must not be null.
     URL(nsIURI* aURI, nsStringBuffer* aString, nsIURI* aReferrer,
-        nsIPrincipal* aOriginPrincipal) NS_HIDDEN;
+        nsIPrincipal* aOriginPrincipal);
 
-    ~URL() NS_HIDDEN;
+    ~URL();
 
-    NS_HIDDEN_(PRBool) operator==(const URL& aOther) const;
+    PRBool operator==(const URL& aOther) const;
 
     // URIEquals only compares URIs and principals (unlike operator==, which
     // also compares the original strings).  URIEquals also assumes that the
     // mURI member of both URL objects is non-null.  Do NOT call this method
     // unless you're sure this is the case.
-    NS_HIDDEN_(PRBool) URIEquals(const URL& aOther) const;
+    PRBool URIEquals(const URL& aOther) const;
 
     nsCOMPtr<nsIURI> mURI; // null == invalid URL
     nsStringBuffer* mString; // Could use nsRefPtr, but it'd add useless
@@ -349,23 +409,13 @@ public:
     nsCOMPtr<nsIURI> mReferrer;
     nsCOMPtr<nsIPrincipal> mOriginPrincipal;
 
-    void AddRef() {
-      if (mRefCnt == PR_UINT32_MAX) {
-        NS_WARNING("refcount overflow, leaking nsCSSValue::URL");
-        return;
-      }
-      ++mRefCnt;
-    }
-    void Release() {
-      if (mRefCnt == PR_UINT32_MAX) {
-        NS_WARNING("refcount overflow, leaking nsCSSValue::URL");
-        return;
-      }
-      if (--mRefCnt == 0)
-        delete this;
-    }
+    NS_INLINE_DECL_REFCOUNTING(nsCSSValue::URL)
+
   protected:
-    nsrefcnt mRefCnt;
+
+    // not to be implemented
+    URL(const URL& aOther);
+    URL& operator=(const URL& aOther);
   };
 
   struct Image : public URL {
@@ -374,22 +424,16 @@ public:
     // this header is included all over.
     // aString must not be null.
     Image(nsIURI* aURI, nsStringBuffer* aString, nsIURI* aReferrer,
-          nsIPrincipal* aOriginPrincipal, nsIDocument* aDocument) NS_HIDDEN;
-    ~Image() NS_HIDDEN;
+          nsIPrincipal* aOriginPrincipal, nsIDocument* aDocument);
+    ~Image();
 
     // Inherit operator== from nsCSSValue::URL
 
     nsCOMPtr<imgIRequest> mRequest; // null == image load blocked or somehow failed
 
-    // Override Release so we delete correctly without a virtual destructor
-    void Release() {
-      if (mRefCnt == PR_UINT32_MAX) {
-        NS_WARNING("refcount overflow, leaking nsCSSValue::Image");
-        return;
-      }
-      if (--mRefCnt == 0)
-        delete this;
-    }
+    // Override AddRef and Release to not only log ourselves correctly, but
+    // also so that we delete correctly without a virtual destructor
+    NS_INLINE_DECL_REFCOUNTING(nsCSSValue::Image)
   };
 
 private:
@@ -409,43 +453,120 @@ protected:
     Array*     mArray;
     URL*       mURL;
     Image*     mImage;
+    nsCSSValueGradient* mGradient;
   }         mValue;
 };
 
+struct nsCSSValueGradientStop {
+public:
+  nsCSSValueGradientStop();
+  // needed to keep bloat logs happy when we use the nsTArray in nsCSSValueGradient
+  nsCSSValueGradientStop(const nsCSSValueGradientStop& aOther);
+  ~nsCSSValueGradientStop();
+
+  nsCSSValue mLocation;
+  nsCSSValue mColor;
+
+  PRBool operator==(const nsCSSValueGradientStop& aOther) const
+  {
+    return (mLocation == aOther.mLocation &&
+            mColor == aOther.mColor);
+  }
+
+  PRBool operator!=(const nsCSSValueGradientStop& aOther) const
+  {
+    return !(*this == aOther);
+  }
+};
+
+struct nsCSSValueGradient {
+  nsCSSValueGradient(PRBool aIsRadial, PRBool aIsRepeating);
+
+  // true if gradient is radial, false if it is linear
+  PRPackedBool mIsRadial;
+  PRPackedBool mIsRepeating;
+  // line position and angle
+  nsCSSValue mBgPosX;
+  nsCSSValue mBgPosY;
+  nsCSSValue mAngle;
+
+  // Only meaningful if mIsRadial is true
+  nsCSSValue mRadialShape;
+  nsCSSValue mRadialSize;
+
+  nsTArray<nsCSSValueGradientStop> mStops;
+
+  PRBool operator==(const nsCSSValueGradient& aOther) const
+  {
+    if (mIsRadial != aOther.mIsRadial ||
+        mIsRepeating != aOther.mIsRepeating ||
+        mBgPosX != aOther.mBgPosX ||
+        mBgPosY != aOther.mBgPosY ||
+        mAngle != aOther.mAngle ||
+        mRadialShape != aOther.mRadialShape ||
+        mRadialSize != aOther.mRadialSize)
+      return PR_FALSE;
+
+    if (mStops.Length() != aOther.mStops.Length())
+      return PR_FALSE;
+
+    for (PRUint32 i = 0; i < mStops.Length(); i++) {
+      if (mStops[i] != aOther.mStops[i])
+        return PR_FALSE;
+    }
+
+    return PR_TRUE;
+  }
+
+  PRBool operator!=(const nsCSSValueGradient& aOther) const
+  {
+    return !(*this == aOther);
+  }
+
+  NS_INLINE_DECL_REFCOUNTING(nsCSSValueGradient)
+
+private:
+  // not to be implemented
+  nsCSSValueGradient(const nsCSSValueGradient& aOther);
+  nsCSSValueGradient& operator=(const nsCSSValueGradient& aOther);
+};
+
 struct nsCSSValue::Array {
 
   // return |Array| with reference count of zero
-  static Array* Create(PRUint16 aItemCount) {
+  static Array* Create(size_t aItemCount) {
     return new (aItemCount) Array(aItemCount);
   }
 
-  nsCSSValue& operator[](PRUint16 aIndex) {
+  nsCSSValue& operator[](size_t aIndex) {
     NS_ASSERTION(aIndex < mCount, "out of range");
     return mArray[aIndex];
   }
 
-  const nsCSSValue& operator[](PRUint16 aIndex) const {
+  const nsCSSValue& operator[](size_t aIndex) const {
     NS_ASSERTION(aIndex < mCount, "out of range");
     return mArray[aIndex];
   }
 
-  nsCSSValue& Item(PRUint16 aIndex) { return (*this)[aIndex]; }
-  const nsCSSValue& Item(PRUint16 aIndex) const { return (*this)[aIndex]; }
+  nsCSSValue& Item(size_t aIndex) { return (*this)[aIndex]; }
+  const nsCSSValue& Item(size_t aIndex) const { return (*this)[aIndex]; }
 
-  PRUint16 Count() const { return mCount; }
+  size_t Count() const { return mCount; }
 
   PRBool operator==(const Array& aOther) const
   {
     if (mCount != aOther.mCount)
       return PR_FALSE;
-    for (PRUint16 i = 0; i < mCount; ++i)
+    for (size_t i = 0; i < mCount; ++i)
       if ((*this)[i] != aOther[i])
         return PR_FALSE;
     return PR_TRUE;
   }
 
+  // XXXdholbert This uses a size_t ref count. Should we use a variant
+  // of NS_INLINE_DECL_REFCOUNTING that takes a type as an argument?
   void AddRef() {
-    if (mRefCnt == PR_UINT16_MAX) {
+    if (mRefCnt == size_t(-1)) { // really want SIZE_MAX
       NS_WARNING("refcount overflow, leaking nsCSSValue::Array");
       return;
     }
@@ -453,7 +574,7 @@ struct nsCSSValue::Array {
     NS_LOG_ADDREF(this, mRefCnt, "nsCSSValue::Array", sizeof(*this));
   }
   void Release() {
-    if (mRefCnt == PR_UINT16_MAX) {
+    if (mRefCnt == size_t(-1)) { // really want SIZE_MAX
       NS_WARNING("refcount overflow, leaking nsCSSValue::Array");
       return;
     }
@@ -465,14 +586,14 @@ struct nsCSSValue::Array {
 
 private:
 
-  PRUint16 mRefCnt;
-  const PRUint16 mCount;
+  size_t mRefCnt;
+  const size_t mCount;
   // This must be the last sub-object, since we extend this array to
   // be of size mCount; it needs to be a sub-object so it gets proper
   // alignment.
   nsCSSValue mArray[1];
 
-  void* operator new(size_t aSelfSize, PRUint16 aItemCount) CPP_THROW_NEW {
+  void* operator new(size_t aSelfSize, size_t aItemCount) CPP_THROW_NEW {
     NS_ABORT_IF_FALSE(aItemCount > 0, "cannot have a 0 item count");
     return ::operator new(aSelfSize + sizeof(nsCSSValue) * (aItemCount - 1));
   }
@@ -487,7 +608,7 @@ private:
   for (nsCSSValue *var = First() + 1, *var##_end = First() + mCount;          \
        var != var##_end; ++var)
 
-  Array(PRUint16 aItemCount)
+  Array(size_t aItemCount)
     : mRefCnt(0)
     , mCount(aItemCount)
   {
@@ -508,7 +629,9 @@ private:
 #undef CSSVALUE_LIST_FOR_EXTRA_VALUES
 
 private:
-  Array(const Array& aOther); // not to be implemented
+  // not to be implemented
+  Array(const Array& aOther);
+  Array& operator=(const Array& aOther);
 };
 
 #endif /* nsCSSValue_h___ */
