@@ -45,71 +45,50 @@
 #include "nsPresContext.h"
 #include "nsRuleNode.h"
 #include "nsCSSKeywords.h"
-#include <math.h>
+#include "nsMathUtils.h"
+#include "CSSCalc.h"
+#include "nsCSSStruct.h"
 
-/* Arguably, this loses precision, but it doesn't hurt! */
-const float kPi      = 3.1415926535897932384626433832795f;
-const float kTwoPi   = 6.283185307179586476925286766559f;
-const float kEpsilon = 0.0001f;
+namespace css = mozilla::css;
 
-/* Computes tan(theta).  For values of theta such that
- * tan(theta) is undefined or arbitrarily large, SafeTangent
- * returns a managably large or small value of the correct sign.
+/* Note on floating point precision: The transform matrix is an array
+ * of single precision 'float's, and so are most of the input values
+ * we get from the style system, but intermediate calculations
+ * involving angles need to be done in 'double'.
  */
-static float SafeTangent(float aTheta)
+
+/* Force small values to zero.  We do this to avoid having sin(360deg)
+ * evaluate to a tiny but nonzero value.
+ */
+static double FlushToZero(double aVal)
 {
-  /* We'll do this by computing sin and cos theta.  If cos(theta) is
-   * is too close to zero, we'll set it to some arbitrary epsilon value
-   * that avoid float overflow or undefined result.
+  if (-FLT_EPSILON < aVal && aVal < FLT_EPSILON)
+    return 0.0f;
+  else
+    return aVal;
+}
+
+/* Computes tan(aTheta).  For values of aTheta such that tan(aTheta) is
+ * undefined or very large, SafeTangent returns a manageably large value
+ * of the correct sign.
+ */
+static double SafeTangent(double aTheta)
+{
+  const double kEpsilon = 0.0001;
+
+  /* tan(theta) = sin(theta)/cos(theta); problems arise when
+   * cos(theta) is too close to zero.  Limit cos(theta) to the
+   * range [-1, -epsilon] U [epsilon, 1].
    */
-  float sinTheta = sin(aTheta);
-  float cosTheta = cos(aTheta);
-  
-  /* Bound cos(theta) to be in the range [-1, -epsilon) U (epsilon, 1] */
+  double sinTheta = sin(aTheta);
+  double cosTheta = cos(aTheta);
+
   if (cosTheta >= 0 && cosTheta < kEpsilon)
     cosTheta = kEpsilon;
   else if (cosTheta < 0 && cosTheta >= -kEpsilon)
     cosTheta = -kEpsilon;
-  
-  return sinTheta / cosTheta;
-}
 
-/* Helper function to constrain an angle to a value in the range [-pi, pi),
- * which reduces accumulated floating point errors from trigonometric functions
- * by keeping the error terms small.
- */
-static inline float ConstrainFloatValue(float aValue)
-{
-  /* Get in range [0, 2pi) */
-  aValue = fmod(aValue, kTwoPi);
-  return aValue >= kPi ? aValue - kTwoPi : aValue;
-}
-
-/* Converts an nsCSSValue containing an angle into an equivalent measure
- * of radians.  The value is guaranteed to be in the range (-pi, pi) to
- * minimize error.
- */
-static float CSSToRadians(const nsCSSValue &aValue)
-{
-  NS_PRECONDITION(aValue.IsAngularUnit(),
-                  "Expected an angle, but didn't find one!");
-  
-  switch (aValue.GetUnit()) {
-  case eCSSUnit_Degree:
-    /* 360deg = 2pi rad, so deg = pi / 180 rad */
-    return
-      ConstrainFloatValue(aValue.GetFloatValue() * kPi / 180.0f);
-  case eCSSUnit_Grad:
-    /* 400grad = 2pi rad, so grad = pi / 200 rad */
-    return
-      ConstrainFloatValue(aValue.GetFloatValue() * kPi / 200.0f);
-  case eCSSUnit_Radian:
-    /* Yay identity transforms! */
-    return ConstrainFloatValue(aValue.GetFloatValue());
-  default:
-    NS_NOTREACHED("Unexpected angular unit!");
-    return 0.0f;
-  }
+  return FlushToZero(sinTheta / cosTheta);
 }
 
 /* Constructor sets the data to the identity matrix. */
@@ -233,15 +212,118 @@ nsStyleTransformMatrix::operator *(const nsStyleTransformMatrix &aOther) const
 }
 
 /* Helper function to fill in an nscoord with the specified nsCSSValue. */
-static void SetCoordToValue(const nsCSSValue &aValue,
-                            nsStyleContext* aContext,
-                            nsPresContext* aPresContext,
-                            PRBool &aCanStoreInRuleTree, nscoord &aOut)
+static nscoord CalcLength(const nsCSSValue &aValue,
+                          nsStyleContext* aContext,
+                          nsPresContext* aPresContext,
+                          PRBool &aCanStoreInRuleTree)
 {
-  aOut = nsRuleNode::CalcLength(aValue, aContext, aPresContext,
+  if (aValue.GetUnit() == eCSSUnit_Pixel) {
+    // Handle this here (even though nsRuleNode::CalcLength handles it
+    // fine) so that callers are allowed to pass a null style context
+    // and pres context to SetToTransformFunction if they know (as
+    // nsStyleAnimation does) that all lengths within the transform
+    // function have already been computed to pixels and percents.
+    return nsPresContext::CSSPixelsToAppUnits(aValue.GetFloatValue());
+  }
+  return nsRuleNode::CalcLength(aValue, aContext, aPresContext,
                                 aCanStoreInRuleTree);
 }
 
+struct LengthPercentPairCalcOps : public css::NumbersAlreadyNormalizedOps
+{
+  struct result_type {
+    nscoord mLength;
+    float mPercent;
+
+    result_type(nscoord aLength, float aPercent)
+      : mLength(aLength), mPercent(aPercent) {}
+  };
+
+  LengthPercentPairCalcOps(nsStyleContext* aContext,
+                           nsPresContext* aPresContext,
+                           PRBool& aCanStoreInRuleTree)
+    : mContext(aContext),
+      mPresContext(aPresContext),
+      mCanStoreInRuleTree(aCanStoreInRuleTree) {}
+
+  nsStyleContext* mContext;
+  nsPresContext* mPresContext;
+  PRBool& mCanStoreInRuleTree;
+
+  result_type ComputeLeafValue(const nsCSSValue& aValue)
+  {
+    if (aValue.GetUnit() == eCSSUnit_Percent) {
+      return result_type(0, aValue.GetPercentValue());
+    } else {
+      return result_type(CalcLength(aValue, mContext, mPresContext,
+                                    mCanStoreInRuleTree),
+                         0.0f);
+    }
+  }
+
+  result_type
+  MergeAdditive(nsCSSUnit aCalcFunction,
+                result_type aValue1, result_type aValue2)
+  {
+    if (aCalcFunction == eCSSUnit_Calc_Plus) {
+      return result_type(NSCoordSaturatingAdd(aValue1.mLength,
+                                              aValue2.mLength),
+                         aValue1.mPercent + aValue2.mPercent);
+    }
+    NS_ABORT_IF_FALSE(aCalcFunction == eCSSUnit_Calc_Minus,
+                      "min() and max() are not allowed in calc() on "
+                      "transform");
+    return result_type(NSCoordSaturatingSubtract(aValue1.mLength,
+                                                 aValue2.mLength, 0),
+                       aValue1.mPercent - aValue2.mPercent);
+  }
+
+  result_type
+  MergeMultiplicativeL(nsCSSUnit aCalcFunction,
+                       float aValue1, result_type aValue2)
+  {
+    NS_ABORT_IF_FALSE(aCalcFunction == eCSSUnit_Calc_Times_L,
+                      "unexpected unit");
+    return result_type(NSCoordSaturatingMultiply(aValue2.mLength, aValue1),
+                       aValue1 * aValue2.mPercent);
+  }
+
+  result_type
+  MergeMultiplicativeR(nsCSSUnit aCalcFunction,
+                       result_type aValue1, float aValue2)
+  {
+    NS_ABORT_IF_FALSE(aCalcFunction == eCSSUnit_Calc_Times_R ||
+                      aCalcFunction == eCSSUnit_Calc_Divided,
+                      "unexpected unit");
+    if (aCalcFunction == eCSSUnit_Calc_Divided) {
+      aValue2 = 1.0f / aValue2;
+    }
+    return result_type(NSCoordSaturatingMultiply(aValue1.mLength, aValue2),
+                       aValue1.mPercent * aValue2);
+  }
+
+};
+
+static void ProcessTranslatePart(nscoord& aOffset, float& aPercent,
+                                 const nsCSSValue& aValue,
+                                 nsStyleContext* aContext,
+                                 nsPresContext* aPresContext,
+                                 PRBool& aCanStoreInRuleTree)
+{
+  if (aValue.GetUnit() == eCSSUnit_Percent) {
+    aPercent = aValue.GetPercentValue();
+  } else if (aValue.IsCalcUnit()) {
+    LengthPercentPairCalcOps ops(aContext, aPresContext, aCanStoreInRuleTree);
+    LengthPercentPairCalcOps::result_type result =
+      css::ComputeCalc(aValue, ops);
+    aPercent = result.mPercent;
+    aOffset = result.mLength;
+  } else {
+    aOffset = CalcLength(aValue, aContext, aPresContext,
+                         aCanStoreInRuleTree);
+  }
+}
+
 /* Helper function to process a matrix entry. */
 static void ProcessMatrix(float aMain[4], nscoord aDelta[2],
                           float aX[2], float aY[2],
@@ -258,23 +340,13 @@ static void ProcessMatrix(float aMain[4], nscoord aDelta[2],
   for (PRUint16 index = 1; index <= 4; ++index)
     aMain[index - 1] = aData->Item(index).GetFloatValue();
 
-  /* For the fifth element, if it's a percentage, store it in aX[0].
-   * Otherwise, it's a length that needs to go in aDelta[0]
+  /* The last two elements have their length parts stored in aDelta
+   * and their percent parts stored in aX[0] and aY[1].
    */
-  if (aData->Item(5).GetUnit() == eCSSUnit_Percent)
-    aX[0] = aData->Item(5).GetPercentValue();
-  else
-    SetCoordToValue(aData->Item(5), aContext, aPresContext, aCanStoreInRuleTree,
-                    aDelta[0]);
-
-  /* For the final element, if it's a percentage, store it in aY[1].
-   * Otherwise, it's a length that needs to go in aDelta[1].
-   */
-  if (aData->Item(6).GetUnit() == eCSSUnit_Percent)
-    aY[1] = aData->Item(6).GetPercentValue();
-  else
-    SetCoordToValue(aData->Item(6), aContext, aPresContext, aCanStoreInRuleTree,
-                    aDelta[1]);
+  ProcessTranslatePart(aDelta[0], aX[0], aData->Item(5),
+                       aContext, aPresContext, aCanStoreInRuleTree);
+  ProcessTranslatePart(aDelta[1], aY[1], aData->Item(6),
+                       aContext, aPresContext, aCanStoreInRuleTree);
 }
 
 /* Helper function to process a translatex function. */
@@ -297,11 +369,8 @@ static void ProcessTranslateX(nscoord aDelta[2], float aX[2],
    * Otherwise, we might have a percentage, so we want to set the dX component
    * to the percent.
    */
-  if (aData->Item(1).GetUnit() != eCSSUnit_Percent)
-    SetCoordToValue(aData->Item(1), aContext, aPresContext, aCanStoreInRuleTree,
-                    aDelta[0]);
-  else
-    aX[0] = aData->Item(1).GetPercentValue();
+  ProcessTranslatePart(aDelta[0], aX[0], aData->Item(1),
+                       aContext, aPresContext, aCanStoreInRuleTree);
 }
 
 /* Helper function to process a translatey function. */
@@ -324,11 +393,8 @@ static void ProcessTranslateY(nscoord aDelta[2], float aY[2],
    * Otherwise, we might have a percentage, so we want to set the dY component
    * to the percent.
    */
-  if (aData->Item(1).GetUnit() != eCSSUnit_Percent)
-    SetCoordToValue(aData->Item(1), aContext, aPresContext, aCanStoreInRuleTree,
-                    aDelta[1]);
-  else
-    aY[1] = aData->Item(1).GetPercentValue();
+  ProcessTranslatePart(aDelta[1], aY[1], aData->Item(1),
+                       aContext, aPresContext, aCanStoreInRuleTree);
 }
 
 /* Helper function to process a translate function. */
@@ -348,20 +414,13 @@ static void ProcessTranslate(nscoord aDelta[2], float aX[2], float aY[2],
    * the main matrix.
    */
 
-  const nsCSSValue &dx = aData->Item(1);
-  if (dx.GetUnit() == eCSSUnit_Percent)
-    aX[0] = dx.GetPercentValue();
-  else
-    SetCoordToValue(dx, aContext, aPresContext, aCanStoreInRuleTree, aDelta[0]);
+  ProcessTranslatePart(aDelta[0], aX[0], aData->Item(1),
+                       aContext, aPresContext, aCanStoreInRuleTree);
 
   /* If we read in a Y component, set it appropriately */
   if (aData->Count() == 3) {
-    const nsCSSValue &dy = aData->Item(2);
-    if (dy.GetUnit() == eCSSUnit_Percent)
-      aY[1] = dy.GetPercentValue();
-    else
-      SetCoordToValue(dy, aContext, aPresContext, aCanStoreInRuleTree,
-                      aDelta[1]); 
+    ProcessTranslatePart(aDelta[1], aY[1], aData->Item(2),
+                         aContext, aPresContext, aCanStoreInRuleTree);
   }
 }
 
@@ -410,13 +469,13 @@ static void ProcessScale(float aMain[4], const nsCSSValue::Array* aData)
 /* Helper function that, given a set of angles, constructs the appropriate
  * skew matrix.
  */
-static void ProcessSkewHelper(float aXAngle, float aYAngle, float aMain[4])
+static void ProcessSkewHelper(double aXAngle, double aYAngle, float aMain[4])
 {
   /* We want our matrix to look like this:
    * |  1           tan(ThetaX)  0|
    * |  tan(ThetaY) 1            0|
    * |  0           0            1|
-   * However, to avoid infinte values, we'll use the SafeTangent function
+   * However, to avoid infinite values, we'll use the SafeTangent function
    * instead of the C standard tan function.
    */
   aMain[2] = SafeTangent(aXAngle);
@@ -427,23 +486,24 @@ static void ProcessSkewHelper(float aXAngle, float aYAngle, float aMain[4])
 static void ProcessSkewX(float aMain[4], const nsCSSValue::Array* aData)
 {
   NS_ASSERTION(aData->Count() == 2, "Bad array!");
-  ProcessSkewHelper(CSSToRadians(aData->Item(1)), 0.0f, aMain);
+  ProcessSkewHelper(aData->Item(1).GetAngleValueInRadians(), 0.0, aMain);
 }
 
 /* Function that converts a skewy transform into a matrix. */
 static void ProcessSkewY(float aMain[4], const nsCSSValue::Array* aData)
 {
   NS_ASSERTION(aData->Count() == 2, "Bad array!");
-  ProcessSkewHelper(0.0f, CSSToRadians(aData->Item(1)), aMain);
+  ProcessSkewHelper(0.0, aData->Item(1).GetAngleValueInRadians(), aMain);
 }
 
 /* Function that converts a skew transform into a matrix. */
 static void ProcessSkew(float aMain[4], const nsCSSValue::Array* aData)
 {
   NS_ASSERTION(aData->Count() == 2 || aData->Count() == 3, "Bad array!");
-  
-  float xSkew = CSSToRadians(aData->Item(1));
-  float ySkew = (aData->Count() == 2 ? 0.0f : CSSToRadians(aData->Item(2)));
+
+  double xSkew = aData->Item(1).GetAngleValueInRadians();
+  double ySkew = (aData->Count() == 2
+                  ? 0.0 : aData->Item(2).GetAngleValueInRadians());
 
   ProcessSkewHelper(xSkew, ySkew, aMain);
 }
@@ -459,9 +519,9 @@ static void ProcessRotate(float aMain[4], const nsCSSValue::Array* aData)
    * |           0            0  1|
    * (see http://www.w3.org/TR/SVG/coords.html#RotationDefined)
    */
-  float theta = CSSToRadians(aData->Item(1));
-  float cosTheta = cos(theta);
-  float sinTheta = sin(theta);
+  double theta = aData->Item(1).GetAngleValueInRadians();
+  float cosTheta = FlushToZero(cos(theta));
+  float sinTheta = FlushToZero(sin(theta));
 
   aMain[0] = cosTheta;
   aMain[1] = sinTheta;
@@ -469,6 +529,18 @@ static void ProcessRotate(float aMain[4], const nsCSSValue::Array* aData)
   aMain[3] = cosTheta;
 }
 
+/**
+ * Return the transform function, as an nsCSSKeyword, for the given
+ * nsCSSValue::Array from a transform list.
+ */
+/* static */ nsCSSKeyword
+nsStyleTransformMatrix::TransformFunctionOf(const nsCSSValue::Array* aData)
+{
+  nsAutoString keyword;
+  aData->Item(0).GetStringValue(keyword);
+  return nsCSSKeywords::LookupKeyword(keyword);
+}
+
 /**
  * SetToTransformFunction is essentially a giant switch statement that fans
  * out to many smaller helper functions.
@@ -480,18 +552,17 @@ nsStyleTransformMatrix::SetToTransformFunction(const nsCSSValue::Array * aData,
                                                PRBool& aCanStoreInRuleTree)
 {
   NS_PRECONDITION(aData, "Why did you want to get data from a null array?");
-  NS_PRECONDITION(aContext, "Need a context for unit conversion!");
-  NS_PRECONDITION(aPresContext, "Need a context for unit conversion!");
-  
+  // It's OK if aContext and aPresContext are null if the caller already
+  // knows that all length units have been converted to pixels (as
+  // nsStyleAnimation does).
+
   /* Reset the matrix to the identity so that each subfunction can just
    * worry about its own components.
    */
   SetToIdentity();
 
   /* Get the keyword for the transform. */
-  nsAutoString keyword;
-  aData->Item(0).GetStringValue(keyword);
-  switch (nsCSSKeywords::LookupKeyword(keyword)) {
+  switch (TransformFunctionOf(aData)) {
   case eCSSKeyword_translatex:
     ProcessTranslateX(mDelta, mX, aData, aContext, aPresContext,
                       aCanStoreInRuleTree);
@@ -534,6 +605,39 @@ nsStyleTransformMatrix::SetToTransformFunction(const nsCSSValue::Array * aData,
   }
 }
 
+/* Given a -moz-transform token stream, accumulates them into an
+ * nsStyleTransformMatrix
+ *
+ * @param aList The nsCSSValueList of arrays to read into transform functions.
+ * @param aContext The style context to use for unit conversion.
+ * @param aPresContext The presentation context to use for unit conversion
+ * @param aCanStoreInRuleTree This is set to PR_FALSE if the value cannot be stored in the rule tree.
+ * @return An nsStyleTransformMatrix corresponding to the net transform.
+ */
+/* static */ nsStyleTransformMatrix
+nsStyleTransformMatrix::ReadTransforms(const nsCSSValueList* aList,
+                                       nsStyleContext* aContext,
+                                       nsPresContext* aPresContext,
+                                       PRBool &aCanStoreInRuleTree)
+{
+  nsStyleTransformMatrix result;
+
+  for (const nsCSSValueList* curr = aList; curr != nsnull; curr = curr->mNext) {
+    const nsCSSValue &currElem = curr->mValue;
+    NS_ASSERTION(currElem.GetUnit() == eCSSUnit_Function,
+                 "Stream should consist solely of functions!");
+    NS_ASSERTION(currElem.GetArrayValue()->Count() >= 1,
+                 "Incoming function is too short!");
+
+    /* Read in a single transform matrix, then accumulate it with the total. */
+    nsStyleTransformMatrix currMatrix;
+    currMatrix.SetToTransformFunction(currElem.GetArrayValue(), aContext,
+                                      aPresContext, aCanStoreInRuleTree);
+    result *= currMatrix;
+  }
+  return result;
+}
+
 /* Does an element-by-element comparison and returns whether or not the
  * matrices are equal.
  */
