@@ -79,7 +79,8 @@ public:
   nsDisplayTableItem(nsIFrame* aFrame) : nsDisplayItem(aFrame),
       mPartHasFixedBackground(PR_FALSE) {}
 
-  virtual PRBool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder);
+  virtual PRBool IsVaryingRelativeToMovingFrame(nsDisplayListBuilder* aBuilder,
+                                                nsIFrame* aFrame);
   // With collapsed borders, parts of the collapsed border can extend outside
   // the table part frames, so allow this display element to blow out to our
   // overflow rect. This is also useful for row frames that have spanning
@@ -139,8 +140,8 @@ private:
 class nsTableFrame : public nsHTMLContainerFrame, public nsITableLayout
 {
 public:
-
   NS_DECL_QUERYFRAME
+  NS_DECL_FRAMEARENA_HELPERS
 
   /** nsTableOuterFrame has intimate knowledge of the inner table frame */
   friend class nsTableOuterFrame;
@@ -159,11 +160,6 @@ public:
                   nsIFrame*        aParent,
                   nsIFrame*        aPrevInFlow);
 
-
-  static void* GetProperty(nsIFrame*            aFrame,
-                           nsIAtom*             aPropertyName,
-                           PRBool               aCreateIfNecessary = PR_FALSE);
-
   static float GetTwipsToPixels(nsPresContext* aPresContext);
 
   // Return true if aParentReflowState.frame or any of its ancestors within
@@ -182,7 +178,7 @@ public:
 
   static void RePositionViews(nsIFrame* aFrame);
 
-  static PRBool PageBreakAfter(nsIFrame& aSourceFrame,
+  static PRBool PageBreakAfter(nsIFrame* aSourceFrame,
                                nsIFrame* aNextFrame);
 
   nsPoint GetFirstSectionOrigin(const nsHTMLReflowState& aReflowState) const;
@@ -193,17 +189,17 @@ public:
                            nsIContent*     aContent, 
                            nsIAtom*        aAttribute); 
 
-  /** @see nsIFrame::Destroy */
-  virtual void Destroy();
+  /** @see nsIFrame::DestroyFrom */
+  virtual void DestroyFrom(nsIFrame* aDestructRoot);
   
   /** @see nsIFrame::DidSetStyleContext */
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext);
 
   NS_IMETHOD AppendFrames(nsIAtom*        aListName,
-                          nsIFrame*       aFrameList);
+                          nsFrameList&    aFrameList);
   NS_IMETHOD InsertFrames(nsIAtom*        aListName,
                           nsIFrame*       aPrevFrame,
-                          nsIFrame*       aFrameList);
+                          nsFrameList&    aFrameList);
   NS_IMETHOD RemoveFrame(nsIAtom*        aListName,
                          nsIFrame*       aOldFrame);
 
@@ -245,9 +241,7 @@ public:
   static nsIFrame* GetFrameAtOrBefore(nsIFrame*       aParentFrame,
                                       nsIFrame*       aPriorChildFrame,
                                       nsIAtom*        aChildType);
-  PRBool IsAutoWidth(PRBool* aIsPctWidth = nsnull);
   PRBool IsAutoHeight();
-  static PRBool IsPctHeight(nsStyleContext* aStyleContext);
   
   /** @return PR_TRUE if aDisplayType represents a rowgroup of any sort
     * (header, footer, or body)
@@ -258,12 +252,9 @@ public:
     * @see nsIFrame::SetInitialChildList 
     */
   NS_IMETHOD SetInitialChildList(nsIAtom*        aListName,
-                                 nsIFrame*       aChildList);
+                                 nsFrameList&    aChildList);
 
-  /** return the first child belonging to the list aListName. 
-    * @see nsIFrame::GetFirstChild
-    */
-  virtual nsIFrame* GetFirstChild(nsIAtom* aListName) const;
+  virtual nsFrameList GetChildList(nsIAtom* aListName) const;
 
   /** @see nsIFrame::GetAdditionalChildListName */
   virtual nsIAtom* GetAdditionalChildListName(PRInt32 aIndex) const;
@@ -279,7 +270,7 @@ public:
    */
   void PaintTableBorderBackground(nsIRenderingContext& aRenderingContext,
                                   const nsRect& aDirtyRect,
-                                  nsPoint aPt);
+                                  nsPoint aPt, PRUint32 aBGPaintFlags);
 
   /** Get the outer half (i.e., the part outside the height and width of
    *  the table) of the largest segment (?) of border-collapsed border on
@@ -389,9 +380,6 @@ public:
   /** return the width of the column at aColIndex    */
   virtual PRInt32 GetColumnWidth(PRInt32 aColIndex);
 
-  /** set the width of the column at aColIndex to aWidth    */
-  virtual void SetColumnWidth(PRInt32 aColIndex, nscoord aWidth);
-
   /** helper to get the cell spacing X style value */
   virtual nscoord GetCellSpacingX();
 
@@ -431,11 +419,6 @@ public:
     */
   PRBool HasMoreThanOneCell(PRInt32 aRowIndex) const;
 
-  /** return the value of the COLS attribute, adjusted for the 
-    * actual number of columns in the table
-    */
-  PRInt32 GetEffectiveCOLSAttribute();
-
   /** return the column frame associated with aColIndex
     * returns nsnull if the col frame has not yet been allocated, or if
     * aColIndex is out of range
@@ -454,14 +437,18 @@ public:
 
   PRInt32 DestroyAnonymousColFrames(PRInt32 aNumFrames);
 
+  // Append aNumColsToAdd anonymous col frames of type eColAnonymousCell to our
+  // last eColGroupAnonymousCell colgroup.  If we have no such colgroup, then
+  // create one.
   void AppendAnonymousColFrames(PRInt32 aNumColsToAdd);
 
-  void CreateAnonymousColFrames(nsTableColGroupFrame* aColGroupFrame,
+  // Append aNumColsToAdd anonymous col frames of type aColType to
+  // aColGroupFrame.  If aAddToTable is true, also call AddColsToTable on the
+  // new cols.
+  void AppendAnonymousColFrames(nsTableColGroupFrame* aColGroupFrame,
                                 PRInt32               aNumColsToAdd,
                                 nsTableColType        aColType,
-                                PRBool                aAddToColGroupAndTable,
-                                nsIFrame*             aPrevCol,
-                                nsIFrame**            aFirstNewFrame);
+                                PRBool                aAddToTable);
 
   void MatchCellMapToColCache(nsTableCellMap* aCellMap);
   /** empty the column frame cache */
@@ -479,16 +466,11 @@ public:
   virtual void RemoveCell(nsTableCellFrame* aCellFrame,
                           PRInt32           aRowIndex);
 
-  void AppendRows(nsTableRowGroupFrame&       aRowGroupFrame,
+  void AppendRows(nsTableRowGroupFrame*       aRowGroupFrame,
                   PRInt32                     aRowIndex,
                   nsTArray<nsTableRowFrame*>& aRowFrames);
 
-  PRInt32 InsertRow(nsTableRowGroupFrame& aRowGroupFrame,
-                    nsIFrame&             aFrame,
-                    PRInt32               aRowIndex,
-                    PRBool                aConsiderSpans);
-
-  PRInt32 InsertRows(nsTableRowGroupFrame&       aRowGroupFrame,
+  PRInt32 InsertRows(nsTableRowGroupFrame*       aRowGroupFrame,
                      nsTArray<nsTableRowFrame*>& aFrames,
                      PRInt32                     aRowIndex,
                      PRBool                      aConsiderSpans);
@@ -498,22 +480,12 @@ public:
                           PRBool           aConsiderSpans);
 
   /** Insert multiple rowgroups into the table cellmap handling
-    * @param aFirstRowGroupFrame - first row group to be inserted all siblings
-    *                              will be appended too.
+    * @param aRowGroups - iterator that iterates over the rowgroups to insert
     */
-  void AppendRowGroups(nsIFrame* aFirstRowGroupFrame);
+  void InsertRowGroups(const nsFrameList::Slice& aRowGroups);
 
-  /** Insert multiple rowgroups into the table cellmap handling
-    * @param aFirstRowGroupFrame - first row group to be inserted
-    * @param aLastRowGroupFrame  - when inserting the siblings of 
-    *                              aFirstRowGroupFrame stop at this row group
-    */
-  void InsertRowGroups(nsIFrame*       aFirstRowGroupFrame,
-                       nsIFrame*       aLastRowGroupFrame);
-
-  void InsertColGroups(PRInt32         aColIndex,
-                       nsIFrame*       aFirstFrame,
-                       nsIFrame*       aLastFrame = nsnull);
+  void InsertColGroups(PRInt32                   aStartColIndex,
+                       const nsFrameList::Slice& aColgroups);
 
   virtual void RemoveCol(nsTableColGroupFrame* aColGroupFrame,
                          PRInt32               aColIndex,
@@ -636,16 +608,20 @@ protected:
                   nsHTMLReflowMetrics& aKidDesiredSize,
                   const nsRect&        aOriginalKidRect,
                   const nsRect&        aOriginalKidOverflowRect);
+   void PlaceRepeatedFooter(nsTableReflowState& aReflowState,
+                            nsTableRowGroupFrame *aTfoot,
+                            nscoord aFooterHeight);
 
   nsIFrame* GetFirstBodyRowGroupFrame();
-  PRBool MoveOverflowToChildList(nsPresContext* aPresContext);
+public:
+  typedef nsAutoTPtrArray<nsTableRowGroupFrame, 8> RowGroupArray;
   /**
-   * Push all our child frames from the aFrames array, in order, starting from the
-   * frame at aPushFrom to the end of the array. The frames are put on our overflow
-   * list or moved directly to our next-in-flow if one exists.
+   * Push all our child frames from the aRowGroups array, in order, starting
+   * from the frame at aPushFrom to the end of the array. The frames are put on
+   * our overflow list or moved directly to our next-in-flow if one exists.
    */
-  typedef nsAutoTPtrArray<nsIFrame, 8> FrameArray;
-  void PushChildren(const FrameArray& aFrames, PRInt32 aPushFrom);
+protected:
+  void PushChildren(const RowGroupArray& aRowGroups, PRInt32 aPushFrom);
 
 public:
   // put the children frames in the display order (e.g. thead before tbodies
@@ -653,8 +629,10 @@ public:
   // children, and not append nulls, so the array is guaranteed to contain
   // nsTableRowGroupFrames.  If there are multiple theads or tfoots, all but
   // the first one are treated as tbodies instead.
-  typedef nsAutoTPtrArray<nsTableRowGroupFrame, 8> RowGroupArray;
-  void OrderRowGroups(RowGroupArray& aChildren) const;
+
+  void OrderRowGroups(RowGroupArray& aChildren,
+                      nsTableRowGroupFrame** aHead = nsnull,
+                      nsTableRowGroupFrame** aFoot = nsnull) const;
 
   // Return the thead, if any
   nsTableRowGroupFrame* GetTHead() const;
@@ -662,25 +640,6 @@ public:
   // Return the tfoot, if any
   nsTableRowGroupFrame* GetTFoot() const;
 
-protected:
-  // As above, but does NOT actually call GetRowGroupFrame() on the kids, so
-  // returns an array of nsIFrames.  This is to be used when you really want
-  // the flowable kids of the table, not the rowgroups.  This outputs the thead
-  // and tfoot if they happen to be rowgroups.  All the child nsIFrames of the
-  // table that return null if you call GetRowGroupFrame() on them will appear
-  // at the end of the array, after the tfoot, if any.
-  //
-  // aHead and aFoot must not be null.
-  //
-  // @return the number of frames in aChildren which return non-null if you
-  // call GetRowGroupFrame() on them.
-  //
-  // XXXbz why do we really care about the non-rowgroup kids?
-  PRUint32 OrderRowGroups(FrameArray& aChildren,
-                          nsTableRowGroupFrame** aHead,
-                          nsTableRowGroupFrame** aFoot) const;
-
-public:
   // Returns PR_TRUE if there are any cells above the row at
   // aRowIndex and spanning into the row at aRowIndex, the number of
   // effective columns limits the search up to that column
@@ -691,14 +650,6 @@ public:
   // columns limits the search up to that column
   PRBool RowHasSpanningCells(PRInt32 aRowIndex, PRInt32 aNumEffCols);
 
-  // Returns PR_TRUE if there are any cells to the left of the column at
-  // aColIndex and spanning into the column at aColIndex     
-  PRBool ColIsSpannedInto(PRInt32 aColIndex);
-
-  // Returns PR_TRUE if there is a cell originating in aColIndex
-  // which spans into the next col
-  PRBool ColHasSpanningCells(PRInt32 aColIndex);
-
 protected:
 
   PRBool HaveReflowedColGroups() const;
@@ -745,21 +696,13 @@ public:
   /** Reset the rowindices of all rows as they might have changed due to 
     * rowgroup reordering, exclude new row group frames that show in the
     * reordering but are not yet inserted into the cellmap
-    * @param aFirstRowGroupFrame - first row group to be excluded
-    * @param aLastRowGroupFrame  - last sibling of aFirstRowGroupFrame that
-    *                              should be excluded when reseting the row
-    *                              indices.
+    * @param aRowGroupsToExclude - an iterator that will produce the row groups
+    *                              to exclude.
     */
-  void ResetRowIndices(nsIFrame* aFirstRowGroupFrame = nsnull,
-                       nsIFrame* aLastRowGroupFrame = nsnull);
+  void ResetRowIndices(const nsFrameList::Slice& aRowGroupsToExclude);
 
   nsTArray<nsTableColFrame*>& GetColCache();
 
-  /** Return aFrame's child if aFrame is an nsScrollFrame, otherwise return aFrame
-    */
-  static nsTableRowGroupFrame* GetRowGroupFrame(nsIFrame* aFrame,
-                                                nsIAtom*  aFrameTypeIn = nsnull);
-
 protected:
 
   void SetBorderCollapse(PRBool aValue);
@@ -776,7 +719,7 @@ protected:
 
 public: /* ----- Cell Map public methods ----- */
 
-  PRInt32 GetStartRowIndex(nsTableRowGroupFrame& aRowGroupFrame);
+  PRInt32 GetStartRowIndex(nsTableRowGroupFrame* aRowGroupFrame);
 
   /** returns the number of rows in this table.
     */
