@@ -55,9 +55,6 @@
 #include "nsIDOMMouseListener.h"
 #include "nsIDOMMouseMotionListener.h"
 #include "nsIDOMEventTarget.h"
-#include "nsIView.h"
-#include "nsIViewManager.h"
-#include "nsIScrollableView.h"
 #include "nsIDOMMouseEvent.h"
 #include "nsIPresShell.h"
 #include "nsFrameList.h"
@@ -147,8 +144,6 @@ public:
   ResizeType GetResizeAfter();
   State GetState();
 
-  //nsresult CaptureMouse(nsPresContext* aPresContext, PRBool aGrabMouseEvents);
-  //PRBool IsMouseCaptured(nsPresContext* aPresContext);
   void Reverse(nsSplitterInfo*& aIndexes, PRInt32 aCount);
   PRBool SupportsCollapseDirection(CollapseDirection aDirection);
 
@@ -167,7 +162,6 @@ public:
   PRInt32 mChildInfosAfterCount;
   State mState;
   nscoord mSplitterPos;
-  nscoord mSplitterViewPos;
   PRBool mDragging;
 
 };
@@ -252,7 +246,9 @@ nsIFrame*
 NS_NewSplitterFrame (nsIPresShell* aPresShell, nsStyleContext* aContext)
 {
   return new (aPresShell) nsSplitterFrame(aPresShell, aContext);
-} // NS_NewSplitterFrame
+}
+
+NS_IMPL_FRAMEARENA_HELPERS(nsSplitterFrame)
 
 nsSplitterFrame::nsSplitterFrame(nsIPresShell* aPresShell, nsStyleContext* aContext)
 : nsBoxFrame(aPresShell, aContext),
@@ -261,7 +257,7 @@ nsSplitterFrame::nsSplitterFrame(nsIPresShell* aPresShell, nsStyleContext* aCont
 }
 
 void
-nsSplitterFrame::Destroy()
+nsSplitterFrame::DestroyFrom(nsIFrame* aDestructRoot)
 {
   if (mInner) {
     mInner->RemoveListener();
@@ -269,7 +265,7 @@ nsSplitterFrame::Destroy()
     mInner->Release();
     mInner = nsnull;
   }
-  nsBoxFrame::Destroy();
+  nsBoxFrame::DestroyFrom(aDestructRoot);
 }
 
 
@@ -342,7 +338,7 @@ nsSplitterFrame::Init(nsIContent*      aContent,
                           NS_LITERAL_STRING("vertical"), PR_FALSE);
         nsStyleContext* parentStyleContext = GetStyleContext()->GetParent();
         nsRefPtr<nsStyleContext> newContext = PresContext()->StyleSet()->
-          ResolveStyleFor(aContent, parentStyleContext);
+          ResolveStyleFor(aContent->AsElement(), parentStyleContext);
         SetStyleContextWithoutNotification(newContext);
       }
     }
@@ -351,9 +347,6 @@ nsSplitterFrame::Init(nsIContent*      aContent,
   nsresult  rv = nsBoxFrame::Init(aContent, aParent, aPrevInFlow);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  rv = nsHTMLContainerFrame::CreateViewForFrame(this, PR_TRUE);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   mInner->mState = nsSplitterFrameInner::Open;
   mInner->AddListener(PresContext());
   mInner->mParentBox = nsnull;
@@ -394,8 +387,9 @@ nsSplitterFrame::HandlePress(nsPresContext* aPresContext,
 
 NS_IMETHODIMP
 nsSplitterFrame::HandleMultiplePress(nsPresContext* aPresContext,
-                         nsGUIEvent *    aEvent,
-                         nsEventStatus*  aEventStatus)
+                                     nsGUIEvent *    aEvent,
+                                     nsEventStatus*  aEventStatus,
+                                     PRBool aControlHeld)
 {
   return NS_OK;
 }
@@ -471,7 +465,7 @@ nsSplitterFrameInner::MouseUp(nsPresContext* aPresContext, nsGUIEvent* aEvent)
   if (mDragging && mOuter) {
     AdjustChildren(aPresContext);
     AddListener(aPresContext);
-    mOuter->CaptureMouse(aPresContext, PR_FALSE);
+    nsIPresShell::SetCapturingContent(nsnull, 0); // XXXndeakin is this needed?
     mDragging = PR_FALSE;
     State newState = GetState(); 
     // if the state is dragging then make it Open.
@@ -542,13 +536,24 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
     PRBool supportsBefore = SupportsCollapseDirection(Before);
     PRBool supportsAfter = SupportsCollapseDirection(After);
 
+    const PRBool isRTL = mOuter->GetStyleVisibility()->mDirection == NS_STYLE_DIRECTION_RTL;
+    PRBool pastEnd = oldPos > 0 && oldPos > pos;
+    PRBool pastBegin = oldPos < 0 && oldPos < pos;
+    if (isRTL) {
+      // Swap the boundary checks in RTL mode
+      PRBool tmp = pastEnd;
+      pastEnd = pastBegin;
+      pastBegin = tmp;
+    }
+    const PRBool isCollapsedBefore = pastBegin && supportsBefore;
+    const PRBool isCollapsedAfter = pastEnd && supportsAfter;
+
     // if we are in a collapsed position
-    if ((oldPos > 0 && oldPos > pos && supportsAfter) ||
-        (oldPos < 0 && oldPos < pos && supportsBefore))
+    if (isCollapsedBefore || isCollapsedAfter)
     {
       // and we are not collapsed then collapse
       if (currentState == Dragging) {
-        if (oldPos > 0 && oldPos > pos)
+        if (pastEnd)
         {
           //printf("Collapse right\n");
           if (supportsAfter) 
@@ -562,7 +567,7 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
                            PR_TRUE);
           }
 
-        } else if (oldPos < 0 && oldPos < pos)
+        } else if (pastBegin)
         {
           //printf("Collapse left\n");
           if (supportsBefore)
@@ -585,20 +590,6 @@ nsSplitterFrameInner::MouseDrag(nsPresContext* aPresContext, nsGUIEvent* aEvent)
       AdjustChildren(aPresContext);
     }
 
-    // printf("----- resize ----- ");
-    /*
-      for (i=0; i < mChildInfosBeforeCount; i++) 
-        printf("before, index=%d, current=%d, changed=%d\n", mChildInfosBefore[i].index, mChildInfosBefore[i].current, mChildInfosBefore[i].changed);
-      for (i=0; i < mChildInfosAfterCount; i++) 
-        printf("after, index=%d, current=%d, changed=%d\n", mChildInfosAfter[i].index, mChildInfosAfter[i].current, mChildInfosAfter[i].changed);
-    */
-
-    /*
-      nsIPresShell *shell = aPresContext->PresShell();
-
-      shell->FrameNeedsReflow(mOuter, nsIPresShell::eStyleChange,
-                              NS_FRAME_IS_DIRTY);
-    */
     mDidDrag = PR_TRUE;
   }
 }
@@ -626,62 +617,13 @@ nsSplitterFrameInner::RemoveListener()
                              NS_GET_IID(nsIDOMMouseMotionListener));
 }
 
-/*
-nsresult
-nsSplitterFrameInner :: CaptureMouse(nsPresContext* aPresContext, PRBool aGrabMouseEvents)
-{
-  // get its view
-  nsIView* view = mOuter->GetView();
-  PRBool result;
-
-  if (view) {
-    nsIViewManager* viewMan = view->GetViewManager();
-    if (viewMan) {
-      // nsIWidget* widget = view->GetWidget();
-      if (aGrabMouseEvents) {
-        viewMan->GrabMouseEvents(view,result);
-        //  if (widget)
-        //   widget->CaptureMouse(PR_TRUE);
-      } else {
-        viewMan->GrabMouseEvents(nsnull,result);
-       // if (widget)
-         //  widget->CaptureMouse(PR_FALSE);
-      }
-    }
-  }
-
-  return NS_OK;
-}
-
-
-PRBool
-nsSplitterFrameInner :: IsMouseCaptured(nsPresContext* aPresContext)
-{
-    // get its view
-  nsIView* view = mOuter->GetView();
-  
-  if (view) {
-    nsIViewManager* viewMan = view->GetViewManager();
-
-    if (viewMan) {
-        nsIView* grabbingView;
-        viewMan->GetMouseEventGrabber(grabbingView);
-        if (grabbingView == view)
-          return PR_TRUE;
-    }
-  }
-
-  return PR_FALSE;
-}
-*/
-
 nsresult
 nsSplitterFrameInner::MouseUp(nsIDOMEvent* aMouseEvent)
 {  
   NS_ENSURE_TRUE(mOuter, NS_OK);
   mPressed = PR_FALSE;
 
-  mOuter->CaptureMouse(mOuter->PresContext(), PR_FALSE);
+  nsIPresShell::SetCapturingContent(nsnull, 0);
 
   return NS_OK;
 }
@@ -710,7 +652,7 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
 
   // get our index
   nsPresContext* outerPresContext = mOuter->PresContext();
-  nsFrameList siblingList(mParentBox->GetFirstChild(nsnull));
+  const nsFrameList& siblingList(mParentBox->GetChildList(nsnull));
   PRInt32 childIndex = siblingList.IndexOf(mOuter);
   // if it's 0 (or not found) then stop right here.
   // It might be not found if we're not in the parent's primary frame list.
@@ -842,26 +784,22 @@ nsSplitterFrameInner::MouseDown(nsIDOMEvent* aMouseEvent)
   if (resizeAfter == Grow)
      mChildInfosAfterCount = 0;
 
-  nsRect vr = mOuter->GetView()->GetBounds();
-
   PRInt32 c;
   nsPoint pt = nsLayoutUtils::GetDOMEventCoordinatesRelativeTo(mouseEvent,
                                                                mParentBox);
   if (isHorizontal) {
      c = pt.x;
      mSplitterPos = mOuter->mRect.x;
-     mSplitterViewPos = vr.x;
   } else {
      c = pt.y;
      mSplitterPos = mOuter->mRect.y;
-     mSplitterViewPos = vr.y;
   }
 
   mDragStart = c;
 
   //printf("Pressed mDragStart=%d\n",mDragStart);
 
-  mOuter->CaptureMouse(outerPresContext, PR_TRUE);
+  nsIPresShell::SetCapturingContent(mOuter->GetContent(), CAPTURE_IGNOREALLOWED);
 
   return NS_OK;
 }
@@ -947,8 +885,7 @@ nsSplitterFrameInner::UpdateState()
     // Find the splitter's immediate sibling.
     nsIFrame* splitterSibling;
     if (newState == CollapsedBefore || mState == CollapsedBefore) {
-      splitterSibling =
-        nsFrameList(mOuter->GetParent()->GetFirstChild(nsnull)).GetPrevSiblingFor(mOuter);
+      splitterSibling = mOuter->GetPrevSibling();
     } else {
       splitterSibling = mOuter->GetNextSibling();
     }
