@@ -44,9 +44,9 @@
 #include "nsIDOMXULElement.h"
 #include "nsIDocument.h"
 #include "nsGkAtoms.h"
-#include "nsIPresShell.h"
 #include "nsIFrame.h"
 #include "nsIPopupBoxObject.h"
+#include "nsMenuPopupFrame.h"
 #include "nsIServiceManager.h"
 #ifdef MOZ_XUL
 #include "nsIDOMNSDocument.h"
@@ -54,7 +54,6 @@
 #endif
 #include "nsGUIEvent.h"
 #include "nsIPrivateDOMEvent.h"
-#include "nsPresContext.h"
 #include "nsIScriptContext.h"
 #include "nsPIDOMWindow.h"
 #include "nsContentUtils.h"
@@ -158,6 +157,7 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
     return NS_OK;
 #endif
 
+#ifdef MOZ_XUL
   // check to see if the mouse left the targetNode, and if so,
   // hide the tooltip
   if (currentTooltip) {
@@ -166,26 +166,23 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aMouseEvent)
     aMouseEvent->GetTarget(getter_AddRefs(eventTarget));
     nsCOMPtr<nsIDOMNode> targetNode(do_QueryInterface(eventTarget));
 
-    // which node is our tooltip on?
-    nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(currentTooltip->GetDocument()));
-    if (!xulDoc)     // remotely possible someone could have 
-      return NS_OK;  // removed tooltip from dom while it was open
-    nsCOMPtr<nsIDOMNode> tooltipNode;
-    xulDoc->TrustedGetTooltipNode (getter_AddRefs(tooltipNode));
-
-    // if they're the same, the mouse left the node the tooltip appeared on,
-    // close the tooltip.
-    if (tooltipNode == targetNode) {
-      HideTooltip();
-#ifdef MOZ_XUL
-      // reset special tree tracking
-      if (mIsSourceTree) {
-        mLastTreeRow = -1;
-        mLastTreeCol = nsnull;
+    nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
+    if (pm) {
+      nsCOMPtr<nsIDOMNode> tooltipNode =
+        pm->GetLastTriggerTooltipNode(currentTooltip->GetCurrentDoc());
+      if (tooltipNode == targetNode) {
+        // if the target node is the current tooltip target node, the mouse
+        // left the node the tooltip appeared on, so close the tooltip.
+        HideTooltip();
+        // reset special tree tracking
+        if (mIsSourceTree) {
+          mLastTreeRow = -1;
+          mLastTreeCol = nsnull;
+        }
       }
-#endif
     }
   }
+#endif
 
   return NS_OK;
 }
@@ -224,10 +221,10 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
   mMouseScreenY = newMouseY;
   mCachedMouseEvent = aMouseEvent;
 
-  nsCOMPtr<nsIDOMEventTarget> eventTarget;
-  aMouseEvent->GetCurrentTarget(getter_AddRefs(eventTarget));
+  nsCOMPtr<nsIDOMEventTarget> currentTarget;
+  aMouseEvent->GetCurrentTarget(getter_AddRefs(currentTarget));
 
-  nsCOMPtr<nsIContent> sourceContent = do_QueryInterface(eventTarget);
+  nsCOMPtr<nsIContent> sourceContent = do_QueryInterface(currentTarget);
   mSourceNode = do_GetWeakReference(sourceContent);
 #ifdef MOZ_XUL
   mIsSourceTree = sourceContent->Tag() == nsGkAtoms::treechildren;
@@ -244,22 +241,28 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aMouseEvent)
   // showing and the tooltip hasn't been displayed since the mouse entered
   // the node, then start the timer to show the tooltip.
   if (!currentTooltip && !mTooltipShownOnce) {
-    // don't show tooltips attached to elements outside of a menu popup
-    // when hovering over an element inside it.
     nsCOMPtr<nsIDOMEventTarget> eventTarget;
     aMouseEvent->GetTarget(getter_AddRefs(eventTarget));
-    nsCOMPtr<nsIContent> targetContent = do_QueryInterface(eventTarget);
-    while (targetContent && targetContent != sourceContent) {
-      nsIAtom* tag = targetContent->Tag();
-      if (targetContent->GetNameSpaceID() == kNameSpaceID_XUL &&
-          (tag == nsGkAtoms::menupopup ||
-           tag == nsGkAtoms::panel ||
-           tag == nsGkAtoms::tooltip)) {
-        mSourceNode = nsnull;
-        return NS_OK;
-      }
 
-      targetContent = targetContent->GetParent();
+    // don't show tooltips attached to elements outside of a menu popup
+    // when hovering over an element inside it. The popupsinherittooltip
+    // attribute may be used to disable this behaviour, which is useful for
+    // large menu hierarchies such as bookmarks.
+    if (!sourceContent->AttrValueIs(kNameSpaceID_None, nsGkAtoms::popupsinherittooltip,
+                                    nsGkAtoms::_true, eCaseMatters)) {
+      nsCOMPtr<nsIContent> targetContent = do_QueryInterface(eventTarget);
+      while (targetContent && targetContent != sourceContent) {
+        nsIAtom* tag = targetContent->Tag();
+        if (targetContent->GetNameSpaceID() == kNameSpaceID_XUL &&
+            (tag == nsGkAtoms::menupopup ||
+             tag == nsGkAtoms::panel ||
+             tag == nsGkAtoms::tooltip)) {
+          mSourceNode = nsnull;
+          return NS_OK;
+        }
+
+        targetContent = targetContent->GetParent();
+      }
     }
 
     mTooltipTimer = do_CreateInstance("@mozilla.org/timer;1");
@@ -377,7 +380,7 @@ nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
   nsCOMPtr<nsIBoxObject> bx;
   nsIDocument* doc = sourceNode->GetDocument();
   if (doc) {
-    nsCOMPtr<nsIDOMElement> docElement = do_QueryInterface(doc->GetRootContent());
+    nsCOMPtr<nsIDOMElement> docElement = do_QueryInterface(doc->GetRootElement());
     if (docElement) {
       doc->GetBoxObjectFor(docElement, getter_AddRefs(bx));
     }
@@ -445,8 +448,6 @@ nsXULTooltipListener::ShowTooltip()
       }
 #endif
 
-      nsCOMPtr<nsIDOMNode> targetNode = do_QueryReferent(mTargetNode);
-      xulDoc->SetTooltipNode(targetNode);
       mCurrentTooltip = do_GetWeakReference(tooltipNode);
       LaunchTooltip();
       mTargetNode = nsnull;
@@ -604,12 +605,12 @@ nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
     return NS_ERROR_NULL_POINTER;
 
   // before we go on, make sure that target node still has a window
-  nsCOMPtr<nsIDocument> document = aTarget->GetDocument();
+  nsIDocument *document = aTarget->GetDocument();
   if (!document) {
     NS_WARNING("Unable to retrieve the tooltip node document.");
     return NS_ERROR_FAILURE;
   }
-  nsCOMPtr<nsPIDOMWindow> window = document->GetWindow();
+  nsPIDOMWindow *window = document->GetWindow();
   if (!window) {
     return NS_OK;
   }
@@ -625,7 +626,7 @@ nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
   aTarget->GetAttr(kNameSpaceID_None, nsGkAtoms::tooltiptext, tooltipText);
   if (!tooltipText.IsEmpty()) {
     // specifying tooltiptext means we will always use the default tooltip
-    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetPrimaryShell());
+    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetShell());
     NS_ENSURE_STATE(rootBox);
     *aTooltip = rootBox->GetDefaultTooltip();
     if (*aTooltip) {
@@ -646,20 +647,13 @@ nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
 
   if (!tooltipId.IsEmpty()) {
     // tooltip must be an id, use getElementById to find it
-    nsCOMPtr<nsIDOMDocument> domDocument =
-      do_QueryInterface(document);
-    if (!domDocument) {
-      return NS_ERROR_FAILURE;
-    }
-
-    nsCOMPtr<nsIDOMElement> tooltipEl;
-    domDocument->GetElementById(tooltipId, getter_AddRefs(tooltipEl));
+    nsCOMPtr<nsIContent> tooltipEl = document->GetElementById(tooltipId);
 
     if (tooltipEl) {
 #ifdef MOZ_XUL
       mNeedTitletip = PR_FALSE;
 #endif
-      CallQueryInterface(tooltipEl, aTooltip);
+      *aTooltip = tooltipEl.forget().get();
       return NS_OK;
     }
   }
@@ -667,7 +661,7 @@ nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
 #ifdef MOZ_XUL
   // titletips should just use the default tooltip
   if (mIsSourceTree && mNeedTitletip) {
-    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetPrimaryShell());
+    nsIRootBox* rootBox = nsIRootBox::GetRootBox(document->GetShell());
     NS_ENSURE_STATE(rootBox);
     NS_IF_ADDREF(*aTooltip = rootBox->GetDefaultTooltip());
   }
@@ -690,9 +684,7 @@ nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
   // Submenus can't be used as tooltips, see bug 288763.
   nsIContent* parent = tooltip->GetParent();
   if (parent) {
-    nsIDocument* doc = parent->GetCurrentDoc();
-    nsIPresShell* presShell = doc ? doc->GetPrimaryShell() : nsnull;
-    nsIFrame* frame = presShell ? presShell->GetPrimaryFrameFor(parent) : nsnull;
+    nsIFrame* frame = parent->GetPrimaryFrame();
     if (frame && frame->GetType() == nsGkAtoms::menuFrame) {
       NS_WARNING("Menu cannot be used as a tooltip");
       return NS_ERROR_FAILURE;
@@ -712,10 +704,6 @@ nsXULTooltipListener::DestroyTooltip()
     // clear out the tooltip node on the document
     nsCOMPtr<nsIDocument> doc = currentTooltip->GetDocument();
     if (doc) {
-      nsCOMPtr<nsIDOMXULDocument> xulDoc(do_QueryInterface(doc));
-      if (xulDoc)
-        xulDoc->SetTooltipNode(nsnull);
-
       // remove the mousedown and keydown listener from document
       nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(doc));
       evtTarget->RemoveEventListener(NS_LITERAL_STRING("DOMMouseScroll"), static_cast<nsIDOMMouseListener*>(this), PR_TRUE);
