@@ -52,7 +52,6 @@
 #include "nsIScriptSecurityManager.h"
 #include "nsIPrincipal.h"
 #include "nsIFileURL.h"
-#include "nsIJAR.h"
 
 static NS_DEFINE_CID(kZipReaderCID, NS_ZIPREADER_CID);
 
@@ -174,7 +173,8 @@ nsJARInputThunk::EnsureJarStream()
     }
 
     // ask the JarStream for the content length
-    mJarStream->Available((PRUint32 *) &mContentLength);
+    rv = mJarStream->Available((PRUint32 *) &mContentLength);
+    if (NS_FAILED(rv)) return rv;
 
     return NS_OK;
 }
@@ -496,14 +496,8 @@ nsJARChannel::GetOwner(nsISupports **result)
     if (!jarReader)
         return NS_ERROR_NOT_INITIALIZED;
 
-    nsCOMPtr<nsIJAR> jar = do_QueryInterface(jarReader, &rv);
-    if (NS_FAILED(rv)) {
-        NS_ERROR("nsIJAR not supported");
-        return rv;
-    }
-
     nsCOMPtr<nsIPrincipal> cert;
-    rv = jar->GetCertificatePrincipal(mJarEntry.get(), getter_AddRefs(cert));
+    rv = jarReader->GetCertificatePrincipal(mJarEntry.get(), getter_AddRefs(cert));
     if (NS_FAILED(rv)) return rv;
 
     if (cert) {
@@ -672,7 +666,8 @@ nsJARChannel::Open(nsIInputStream **stream)
 
     // force load the jar file now so GetContentLength will return a
     // meaningful value once we return.
-    mJarInput->EnsureJarStream();
+    rv = mJarInput->EnsureJarStream();
+    if (NS_FAILED(rv)) return rv;
 
     NS_ADDREF(*stream = mJarInput);
     return NS_OK;
@@ -913,8 +908,8 @@ nsJARChannel::OnDataAvailable(nsIRequest *req, nsISupports *ctx,
     // nsITransportEventSink implementation.
     // XXX do the 64-bit stuff for real
     if (mProgressSink && NS_SUCCEEDED(rv) && !(mLoadFlags & LOAD_BACKGROUND))
-        mProgressSink->OnProgress(this, nsnull, nsUint64(offset + count),
-                                  nsUint64(mContentLength));
+        mProgressSink->OnProgress(this, nsnull, PRUint64(offset + count),
+                                  PRUint64(mContentLength));
 
     return rv; // let the pump cancel on failure
 }
