@@ -22,6 +22,7 @@
  *
  * Contributor(s):
  *   Chris Saari <saari@netscape.com>
+ *   Bobby Holley <bobbyholley@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -80,8 +81,7 @@ mailing address.
 #include "nsIInputStream.h"
 #include "nsIComponentManager.h"
 #include "imgIContainerObserver.h"
-
-#include "imgILoad.h"
+#include "RasterImage.h"
 
 #include "gfxColor.h"
 #include "gfxPlatform.h"
@@ -122,6 +122,8 @@ nsGIFDecoder2::nsGIFDecoder2()
   , mLastFlushedPass(0)
   , mGIFOpen(PR_FALSE)
   , mSawTransparency(PR_FALSE)
+  , mError(PR_FALSE)
+  , mEnded(PR_FALSE)
 {
   // Clear out the structure, excluding the arrays
   memset(&mGIFStruct, 0, sizeof(mGIFStruct));
@@ -129,7 +131,6 @@ nsGIFDecoder2::nsGIFDecoder2()
 
 nsGIFDecoder2::~nsGIFDecoder2()
 {
-  Close();
 }
 
 //******************************************************************************
@@ -137,14 +138,25 @@ nsGIFDecoder2::~nsGIFDecoder2()
 //******************************************************************************
 
 //******************************************************************************
-/* void init (in imgILoad aLoad); */
-NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
+/* void init (in imgIContainer aImage,
+              in imgIDecoderObserver aObsever,
+              in unsigned long aFlags); */
+NS_IMETHODIMP nsGIFDecoder2::Init(imgIContainer *aImage,
+                                  imgIDecoderObserver *aObserver,
+                                  PRUint32 aFlags)
 {
-  mObserver = do_QueryInterface(aLoad);
+  NS_ABORT_IF_FALSE(aImage->GetType() == imgIContainer::TYPE_RASTER,
+                    "wrong type of imgIContainer for decoding into");
+
+  // Store parameters
+  mImageContainer = static_cast<mozilla::imagelib::RasterImage*>(aImage);
+  mObserver = aObserver;
+  mFlags = aFlags;
+
+  // Fire OnStartDecode at init time to support bug 512435
+  if (!(mFlags & imgIDecoder::DECODER_FLAG_HEADERONLY) && mObserver)
+    mObserver->OnStartDecode(nsnull);
 
-  mImageContainer = do_CreateInstance("@mozilla.org/image/container;2");
-  aLoad->SetImage(mImageContainer);
-  
   // Start with the version (GIF89a|GIF87a)
   mGIFStruct.state = gif_type;
   mGIFStruct.bytes_to_consume = 6;
@@ -159,14 +171,20 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
 
 //******************************************************************************
 /* void close (); */
-NS_IMETHODIMP nsGIFDecoder2::Close()
+NS_IMETHODIMP nsGIFDecoder2::Close(PRUint32 aFlags)
 {
-  if (mCurrentFrame == mGIFStruct.images_decoded)
-    EndImageFrame();
-  EndGIF();
+  // Send notifications if appropriate
+  if (!(mFlags & imgIDecoder::DECODER_FLAG_HEADERONLY) &&
+      !mError && !(aFlags & CLOSE_FLAG_DONTNOTIFY)) {
+    if (mCurrentFrame == mGIFStruct.images_decoded)
+      EndImageFrame();
+    EndGIF(/* aSuccess = */ PR_TRUE);
+  }
 
   PR_FREEIF(mGIFStruct.local_colormap);
 
+  mImageContainer = nsnull;
+
   return NS_OK;
 }
 
@@ -177,32 +195,13 @@ NS_IMETHODIMP nsGIFDecoder2::Flush()
     return NS_OK;
 }
 
-//******************************************************************************
-/* static callback from nsIInputStream::ReadSegments */
-static NS_METHOD ReadDataOut(nsIInputStream* in,
-                             void* closure,
-                             const char* fromRawSegment,
-                             PRUint32 toOffset,
-                             PRUint32 count,
-                             PRUint32 *writeCount)
-{
-  nsGIFDecoder2 *decoder = static_cast<nsGIFDecoder2*>(closure);
-  nsresult rv = decoder->ProcessData((unsigned char*)fromRawSegment, count, writeCount);
-  if (NS_FAILED(rv)) {
-    *writeCount = 0;
-    return rv;
-  }
-
-  return NS_OK;
-}
-
 // Push any new rows according to mCurrentPass/mLastFlushedPass and
 // mCurrentRow/mLastFlushedRow.  Note: caller is responsible for
 // updating mlastFlushed{Row,Pass}.
 nsresult
 nsGIFDecoder2::FlushImageData(PRUint32 fromRow, PRUint32 rows)
 {
-  nsIntRect r(0, fromRow, mGIFStruct.width, rows);
+  nsIntRect r(mGIFStruct.x_offset, mGIFStruct.y_offset + fromRow, mGIFStruct.width, rows);
 
   // Update image  
   nsresult rv = mImageContainer->FrameUpdated(mGIFStruct.images_decoded, r);
@@ -215,7 +214,6 @@ nsGIFDecoder2::FlushImageData(PRUint32 fromRow, PRUint32 rows)
   if (!mGIFStruct.images_decoded && mObserver) {
     PRUint32 imgCurFrame;
     mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
-    r.y += mGIFStruct.y_offset;
     mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
   }
   return NS_OK;
@@ -244,46 +242,52 @@ nsGIFDecoder2::FlushImageData()
 }
 
 //******************************************************************************
-nsresult nsGIFDecoder2::ProcessData(unsigned char *data, PRUint32 count, PRUint32 *_retval)
+/* void write (in string aBuffer, in PRUint32 aCount); */
+NS_IMETHODIMP
+nsGIFDecoder2::Write(const char *aBuffer, PRUint32 aCount)
 {
+  // Don't forgive previously flagged errors
+  if (mError)
+    return NS_ERROR_FAILURE;
+
   // Push the data to the GIF decoder
-  
-  nsresult rv = GifWrite(data, count);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv = GifWrite((const unsigned char *)aBuffer, aCount);
 
   // Flushing is only needed for first frame
-  if (!mGIFStruct.images_decoded) {
+  if (NS_SUCCEEDED(rv) && !mGIFStruct.images_decoded) {
     rv = FlushImageData();
+    NS_ENSURE_SUCCESS(rv, rv);
     mLastFlushedRow = mCurrentRow;
     mLastFlushedPass = mCurrentPass;
   }
 
-  *_retval = count;
+  // We do some fine-grained error control here. If we have at least one frame
+  // of an animated gif, we still want to display it (mostly for legacy reasons).
+  // libpr0n code is strict, so we have to lie and tell it we were successful. So
+  // if we have something to salvage, we send off final decode notifications, and
+  // pretend that we're decoded. Otherwise, we set mError.
+  if (NS_FAILED(rv)) {
 
-  return rv;
-}
-
-//******************************************************************************
-/* unsigned long writeFrom (in nsIInputStream inStr, in unsigned long count); */
-NS_IMETHODIMP nsGIFDecoder2::WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_retval)
-{
-  nsresult rv = inStr->ReadSegments(ReadDataOut, this,  count, _retval);
-
-  /* necko doesn't propagate the errors from ReadDataOut - take matters
-     into our own hands.  if we have at least one frame of an animated
-     gif, then return success so we keep displaying as much as possible. */
-  if (mGIFStruct.state == gif_error || mGIFStruct.state == gif_oom) {
+    // Determine if we want to salvage the situation
     PRUint32 numFrames = 0;
     if (mImageContainer)
       mImageContainer->GetNumFrames(&numFrames);
-    if (numFrames <= 1)
-      rv = NS_ERROR_FAILURE;
+
+    // If we're salvaging, send off notifications
+    // Note that we need to make sure that we have 2 frames, since that tells us
+    // that the first frame is complete (the second could be in any state).
+    if (numFrames > 1) {
+      EndGIF(/* aSuccess = */ PR_TRUE);
+    }
+
+    // Otherwise, set mError
+    else
+      mError = PR_TRUE;
   }
 
-  return rv;
+  return mError ? NS_ERROR_FAILURE : NS_OK;
 }
 
-
 //******************************************************************************
 // GIF decoder callback methods. Part of public API for GIF2
 //******************************************************************************
@@ -293,37 +297,41 @@ void nsGIFDecoder2::BeginGIF()
 {
   if (mGIFOpen)
     return;
-    
-  if (mObserver)
-    mObserver->OnStartDecode(nsnull);
 
-  mImageContainer->Init(mGIFStruct.screen_width, mGIFStruct.screen_height, mObserver);
+  mGIFOpen = PR_TRUE;
 
+  mImageContainer->SetSize(mGIFStruct.screen_width, mGIFStruct.screen_height);
   if (mObserver)
     mObserver->OnStartContainer(nsnull, mImageContainer);
 
-  mGIFOpen = PR_TRUE;
+  // If we're doing a header-only decode, we have what we came for
+  if (mFlags & imgIDecoder::DECODER_FLAG_HEADERONLY)
+    return;
 }
 
 //******************************************************************************
-void nsGIFDecoder2::EndGIF()
+void nsGIFDecoder2::EndGIF(PRBool aSuccess)
 {
-  if (!mGIFOpen)
+  if (mEnded)
     return;
 
+  if (aSuccess)
+    mImageContainer->DecodingComplete();
+
   if (mObserver) {
     mObserver->OnStopContainer(nsnull, mImageContainer);
-    mObserver->OnStopDecode(nsnull, NS_OK, nsnull);
+    mObserver->OnStopDecode(nsnull, aSuccess ? NS_OK : NS_ERROR_FAILURE,
+                            nsnull);
   }
-  
+
   mImageContainer->SetLoopCount(mGIFStruct.loop_count);
-  mImageContainer->DecodingComplete();
 
   mGIFOpen = PR_FALSE;
+  mEnded = PR_TRUE;
 }
 
 //******************************************************************************
-void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
+nsresult nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
 {
   if (!mGIFStruct.images_decoded) {
     // Send a onetime OnDataAvailable (Display Refresh) for the first frame
@@ -335,7 +343,10 @@ void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
       PRUint32 imgCurFrame;
       mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
       nsIntRect r(0, 0, imgWidth, mGIFStruct.y_offset);
-      mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
+      if (mObserver)
+        mObserver->OnDataAvailable(nsnull,
+                                   imgCurFrame == PRUint32(mGIFStruct.images_decoded),
+                                   &r);
     }
   }
 
@@ -355,8 +366,6 @@ void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
                                               mGIFStruct.width, mGIFStruct.height,
                                               format, aDepth, &mImageData, &imageDataLength,
                                               &mColormap, &mColormapSize);
-
-
   } else {
     // Regardless of depth of input, image is decoded into 24bit RGB
     rv = mImageContainer->AppendFrame(mGIFStruct.x_offset, mGIFStruct.y_offset,
@@ -365,7 +374,7 @@ void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
   }
 
   if (NS_FAILED(rv))
-    return;
+    return rv;
 
   mImageContainer->SetFrameDisposalMethod(mGIFStruct.images_decoded,
                                           mGIFStruct.disposal_method);
@@ -374,6 +383,7 @@ void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
     mObserver->OnStartFrame(nsnull, mGIFStruct.images_decoded);
 
   mCurrentFrame = mGIFStruct.images_decoded;
+  return NS_OK;
 }
 
 
@@ -392,10 +402,13 @@ void nsGIFDecoder2::EndImageFrame()
     if (realFrameHeight < mGIFStruct.screen_height) {
       PRUint32 imgCurFrame;
       mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
-      nsIntRect r(0, realFrameHeight, 
-                  mGIFStruct.screen_width, 
-				  mGIFStruct.screen_height - realFrameHeight);
-      mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
+      nsIntRect r(0, realFrameHeight,
+                  mGIFStruct.screen_width,
+                  mGIFStruct.screen_height - realFrameHeight);
+      if (mObserver)
+        mObserver->OnDataAvailable(nsnull,
+                                  imgCurFrame == PRUint32(mGIFStruct.images_decoded),
+                                  &r);
     }
     // This transparency check is only valid for first frame
     if (mGIFStruct.is_transparent && !mSawTransparency) {
@@ -421,10 +434,14 @@ void nsGIFDecoder2::EndImageFrame()
     // image frame decoded before we go off and try to display another frame.
     mImageContainer->SetFrameTimeout(mGIFStruct.images_decoded, mGIFStruct.delay_time);
     mImageContainer->EndFrameDecode(mGIFStruct.images_decoded);
-
-    mGIFStruct.images_decoded++;
   }
 
+  // Unconditionally increment images_decoded, because we unconditionally
+  // append frames in BeginImageFrame(). This ensures that images_decoded
+  // always refers to the frame in mImageContainer we're currently decoding,
+  // even if some of them weren't decoded properly and thus are blank.
+  mGIFStruct.images_decoded++;
+
   if (mObserver)
     mObserver->OnStopFrame(nsnull, curframe);
 
@@ -485,8 +502,16 @@ PRUint32 nsGIFDecoder2::OutputRow()
     PRUint8 *from = rowp + mGIFStruct.width;
     PRUint32 *to = ((PRUint32*)rowp) + mGIFStruct.width;
     PRUint32 *cmap = mColormap;
-    for (PRUint32 c = mGIFStruct.width; c > 0; c--) {
-      *--to = cmap[*--from];
+    if (mColorMask == 0xFF) {
+      for (PRUint32 c = mGIFStruct.width; c > 0; c--) {
+        *--to = cmap[*--from];
+      }
+    } else {
+      // Make sure that pixels within range of colormap.
+      PRUint8 mask = mColorMask;
+      for (PRUint32 c = mGIFStruct.width; c > 0; c--) {
+        *--to = cmap[(*--from) & mask];
+      }
     }
   
     // check for alpha (only for first frame)
@@ -1040,6 +1065,10 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
         }    
         // Create the image container with the right size.
         BeginGIF();
+
+        // If we were doing header-only, we're done
+        if (mFlags & imgIDecoder::DECODER_FLAG_HEADERONLY)
+          return NS_OK;
       }
 
       /* Work around more broken GIF files that have zero image
@@ -1059,15 +1088,14 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
       PRUint32 depth = mGIFStruct.global_colormap_depth;
       if (q[8] & 0x80)
         depth = (q[8]&0x07) + 1;
-      // Make sure the transparent pixel is within colormap space
       PRUint32 realDepth = depth;
       while (mGIFStruct.tpixel >= (1 << realDepth) && (realDepth < 8)) {
         realDepth++;
       } 
-      BeginImageFrame(realDepth);
-      
-      // handle allocation error
-      if (!mImageData) {
+      // Mask to limit the color values within the colormap
+      mColorMask = 0xFF >> (8 - realDepth);
+      nsresult rv = BeginImageFrame(realDepth);
+      if (NS_FAILED(rv) || !mImageData) {
         mGIFStruct.state = gif_error;
         break;
       }
@@ -1171,12 +1199,12 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
       break;
 
     case gif_done:
-      EndGIF();
+      EndGIF(/* aSuccess = */ PR_TRUE);
       return NS_OK;
       break;
 
     case gif_error:
-      EndGIF();
+      EndGIF(/* aSuccess = */ PR_FALSE);
       return NS_ERROR_FAILURE;
       break;
 
@@ -1192,7 +1220,7 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
 
   // if an error state is set but no data remains, code flow reaches here
   if (mGIFStruct.state == gif_error) {
-      EndGIF();
+      EndGIF(/* aSuccess = */ PR_FALSE);
       return NS_ERROR_FAILURE;
   }
   
