@@ -66,6 +66,10 @@
 #define _WIN32_WINNT 0x0600
 #endif
 
+#ifdef WINCE
+#define SHGetFileInfoW SHGetFileInfo
+#endif
+
 // we need windows.h to read out registry information...
 #include <windows.h>
 #include <shellapi.h>
@@ -222,11 +226,11 @@ nsresult nsIconChannel::ExtractIconInfoFromUrl(nsIFile ** aLocalFile, PRUint32 *
   iconURI->GetContentType(aContentType);
   iconURI->GetFileExtension(aFileExtension);
 
-  nsCOMPtr<nsIURI> fileURI;
-  rv = iconURI->GetIconFile(getter_AddRefs(fileURI));
-  if (NS_FAILED(rv) || !fileURI) return NS_OK;
+  nsCOMPtr<nsIURL> url;
+  rv = iconURI->GetIconURL(getter_AddRefs(url));
+  if (NS_FAILED(rv) || !url) return NS_OK;
 
-  nsCOMPtr<nsIFileURL>    fileURL = do_QueryInterface(fileURI, &rv);
+  nsCOMPtr<nsIFileURL> fileURL = do_QueryInterface(url, &rv);
   if (NS_FAILED(rv) || !fileURL) return NS_OK;
 
   nsCOMPtr<nsIFile> file;
@@ -298,12 +302,13 @@ static DWORD GetSpecialFolderIcon(nsIFile* aFile, int aFolder, SHFILEINFOW* aSFI
 static UINT GetSizeInfoFlag(PRUint32 aDesiredImageSize)
 {
   UINT infoFlag;
-#ifndef WINCE
-  // SHGFI_SHELLICONSIZE does not exist on windows mobile.
   if (aDesiredImageSize > 16)
+#ifndef WINCE
     infoFlag = SHGFI_SHELLICONSIZE;
-  else
+#else
+    infoFlag = SHGFI_LARGEICON;
 #endif
+  else
     infoFlag = SHGFI_SMALLICON;
 
   return infoFlag;
@@ -311,7 +316,7 @@ static UINT GetSizeInfoFlag(PRUint32 aDesiredImageSize)
 
 nsresult nsIconChannel::GetHIconFromFile(HICON *hIcon)
 {
-#ifdef WINCE
+#ifdef WINCE_WINDOWS_MOBILE
     // GetDIBits does not exist on windows mobile.
   return NS_ERROR_NOT_AVAILABLE;
 #else
@@ -336,8 +341,15 @@ nsresult nsIconChannel::GetHIconFromFile(HICON *hIcon)
     NS_ENSURE_SUCCESS(rv, rv);
 
     localFile->GetPath(filePath);
+#ifndef WINCE
     if (filePath.Length() < 2 || filePath[1] != ':')
       return NS_ERROR_MALFORMED_URI; // UNC
+#else
+    // WinCE paths don't have drive letters
+    if (filePath.Length() < 2 ||
+        filePath[0] != '\\' || filePath[1] == '\\')
+      return NS_ERROR_MALFORMED_URI; // UNC
+#endif
 
     if (filePath.Last() == ':')
       filePath.Append('\\');
@@ -368,12 +380,21 @@ nsresult nsIconChannel::GetHIconFromFile(HICON *hIcon)
     filePath = NS_LITERAL_STRING(".") + NS_ConvertUTF8toUTF16(defFileExt);
   }
 
+#ifndef WINCE
   // Is this the "Desktop" folder?
   DWORD shellResult = GetSpecialFolderIcon(localFile, CSIDL_DESKTOP, &sfi, infoFlags);
   if (!shellResult) {
     // Is this the "My Documents" folder?
     shellResult = GetSpecialFolderIcon(localFile, CSIDL_PERSONAL, &sfi, infoFlags);
   }
+#else
+  DWORD shellResult = 0;
+  // Fantastic. On WinCE, ::SHGetFileInfo (with a localFile) fails
+  // unless I also set another flag like this. We don't actually need
+  // the display name.
+  if (localFile)
+    infoFlags |= SHGFI_DISPLAYNAME;
+#endif
 
   // There are other "Special Folders" and Namespace entities that we are not 
   // fetching icons for, see: 
@@ -439,13 +460,88 @@ nsresult nsIconChannel::GetStockHIcon(nsIMozIconURI *aIconURI, HICON *hIcon)
 }
 #endif
 
+#ifdef WINCE
+int GetDIBits(HDC hdc,
+              HBITMAP hbmp,
+              UINT uStartScan,
+              UINT cScanLines,
+              LPVOID lpvBits,
+              LPBITMAPINFO lpbi,
+              UINT uUsage)
+{
+  // Enforce some assumptions this simplified implementation makes
+  if (!hdc || !hbmp || uStartScan != 0 ||
+      !lpbi || uUsage != DIB_RGB_COLORS ||
+      lpvBits == NULL && lpbi->bmiHeader.biSize != sizeof(BITMAPINFOHEADER))
+    return 0;
+
+  BITMAP bmpInfo;
+  if (!::GetObject(hbmp, sizeof(BITMAP), &bmpInfo))
+    return 0;
+
+  lpbi->bmiHeader.biWidth         = bmpInfo.bmWidth;
+  lpbi->bmiHeader.biHeight        = bmpInfo.bmHeight;
+  lpbi->bmiHeader.biPlanes        = bmpInfo.bmPlanes;
+  lpbi->bmiHeader.biBitCount      = bmpInfo.bmBitsPixel;
+  lpbi->bmiHeader.biCompression   = BI_RGB; // 0
+  lpbi->bmiHeader.biSizeImage     = bmpInfo.bmWidthBytes * bmpInfo.bmHeight;
+  lpbi->bmiHeader.biXPelsPerMeter = 0;
+  lpbi->bmiHeader.biYPelsPerMeter = 0;
+  lpbi->bmiHeader.biClrUsed       = 0;
+  lpbi->bmiHeader.biClrImportant  = 0;
+
+  if (lpbi->bmiHeader.biBitCount == 1) {
+    // Need to set this or else the mask is inverted.
+    lpbi->bmiHeader.biClrUsed       = 2;
+    lpbi->bmiHeader.biClrImportant  = 2;
+    lpbi->bmiColors[0].rgbRed = lpbi->bmiColors[0].rgbGreen =
+      lpbi->bmiColors[0].rgbBlue = lpbi->bmiColors[0].rgbReserved = 0;
+    lpbi->bmiColors[1].rgbRed = lpbi->bmiColors[1].rgbGreen =
+      lpbi->bmiColors[1].rgbBlue = lpbi->bmiColors[1].rgbReserved = 255;
+  }
+
+  if (lpvBits == NULL)
+    return bmpInfo.bmHeight;
+
+  // We just want to pull out the image bits, but Windows CE makes
+  // this stupidly difficult to do...
+  HBITMAP hTargetBitmap;
+  void *pBuffer; 
+  HDC someDC = ::GetDC(NULL);
+  hTargetBitmap = ::CreateDIBSection(someDC, lpbi, DIB_RGB_COLORS,
+                                     (void**)&pBuffer, NULL, 0);
+  ::ReleaseDC(NULL, someDC);
+
+  HDC memDc    = ::CreateCompatibleDC(NULL);
+  HDC targetDc = ::CreateCompatibleDC(NULL);
+  if (!memDc || !targetDc)
+    return 0;
+
+  HBITMAP hOldMemBitmap = (HBITMAP)::SelectObject(memDc, hbmp);
+  HBITMAP hOldTgtBitmap = (HBITMAP)::SelectObject(targetDc, hTargetBitmap);
+
+  // BitBlt into the target bitmap, then copy the bits into our buffer.
+  ::BitBlt(targetDc, 0, 0, bmpInfo.bmWidth, bmpInfo.bmHeight, memDc, 0, 0, SRCCOPY);
+  memcpy(lpvBits, pBuffer, lpbi->bmiHeader.biSizeImage);
+
+  // Cleanup
+  ::SelectObject(memDc, hOldMemBitmap);
+  ::SelectObject(targetDc, hOldTgtBitmap);
+  ::DeleteDC(memDc);
+  ::DeleteDC(targetDc); 
+  ::DeleteObject(hTargetBitmap); 
+
+  return bmpInfo.bmHeight;
+}
+#endif
+
 nsresult nsIconChannel::MakeInputStream(nsIInputStream** _retval, PRBool nonBlocking)
 {
   // Check whether the icon requested's a file icon or a stock icon
   nsresult rv = NS_ERROR_NOT_AVAILABLE;
 
   // GetDIBits does not exist on windows mobile.
-#ifndef WINCE
+#ifndef WINCE_WINDOWS_MOBILE
   HICON hIcon = NULL;
 
 #if MOZ_WINSDK_TARGETVER >= MOZ_NTDDI_LONGHORN
@@ -470,44 +566,72 @@ nsresult nsIconChannel::MakeInputStream(nsIInputStream** _retval, PRBool nonBloc
     {
       // we got the bitmaps, first find out their size
       HDC hDC = CreateCompatibleDC(NULL); // get a device context for the screen.
-      BITMAPINFO maskInfo = {{sizeof(BITMAPINFOHEADER)}};
-      if (GetDIBits(hDC, iconInfo.hbmMask, 0, 0, NULL, &maskInfo, DIB_RGB_COLORS) &&
+      BITMAPINFO maskInfo  = {{sizeof(BITMAPINFOHEADER)}};
+      BITMAPINFO colorInfo = {{sizeof(BITMAPINFOHEADER)}};
+      if (GetDIBits(hDC, iconInfo.hbmMask,  0, 0, NULL, &maskInfo,  DIB_RGB_COLORS) &&
+          GetDIBits(hDC, iconInfo.hbmColor, 0, 0, NULL, &colorInfo, DIB_RGB_COLORS) &&
+          maskInfo.bmiHeader.biHeight == colorInfo.bmiHeader.biHeight &&
+          maskInfo.bmiHeader.biWidth  == colorInfo.bmiHeader.biWidth  &&
+          colorInfo.bmiHeader.biBitCount > 8 &&
+          colorInfo.bmiHeader.biSizeImage > 0 &&
           maskInfo.bmiHeader.biSizeImage > 0) {
-        PRUint32 colorSize = maskInfo.bmiHeader.biWidth * maskInfo.bmiHeader.biHeight * 4;
-        PRUint32 iconSize = sizeof(ICONFILEHEADER) + sizeof(ICONENTRY) + sizeof(BITMAPINFOHEADER) + colorSize + maskInfo.bmiHeader.biSizeImage;
+
+        PRUint32 iconSize = sizeof(ICONFILEHEADER) +
+                            sizeof(ICONENTRY) +
+                            sizeof(BITMAPINFOHEADER) +
+                            colorInfo.bmiHeader.biSizeImage +
+                            maskInfo.bmiHeader.biSizeImage;
+
         char *buffer = new char[iconSize];
         if (!buffer)
           rv = NS_ERROR_OUT_OF_MEMORY;
         else {
-          // the data starts with an icon file header
-          ICONFILEHEADER *iconHeader = (ICONFILEHEADER *)buffer;
-          iconHeader->ifhReserved = 0;
-          iconHeader->ifhType = 1;
-          iconHeader->ifhCount = 1;
-          // followed by the single icon entry
-          ICONENTRY *iconEntry = (ICONENTRY *)(buffer + sizeof(ICONFILEHEADER));
-          iconEntry->ieWidth = maskInfo.bmiHeader.biWidth;
-          iconEntry->ieHeight = maskInfo.bmiHeader.biHeight;
-          iconEntry->ieColors = 0;
-          iconEntry->ieReserved = 0;
-          iconEntry->iePlanes = 1;
-          iconEntry->ieBitCount = 32;
-          iconEntry->ieSizeImage = sizeof(BITMAPINFOHEADER) + colorSize + maskInfo.bmiHeader.biSizeImage;
-          iconEntry->ieFileOffset = sizeof(ICONFILEHEADER) + sizeof(ICONENTRY);
-          // followed by the bitmap info header and the bits
-          LPBITMAPINFO lpBitmapInfo = (LPBITMAPINFO)(buffer + sizeof(ICONFILEHEADER) + sizeof(ICONENTRY));
-          memcpy(lpBitmapInfo, &maskInfo.bmiHeader, sizeof(BITMAPINFOHEADER));
-          if (GetDIBits(hDC, iconInfo.hbmMask, 0, maskInfo.bmiHeader.biHeight, buffer + sizeof(ICONFILEHEADER) + sizeof(ICONENTRY) + sizeof(BITMAPINFOHEADER) + colorSize, lpBitmapInfo, DIB_RGB_COLORS)) {
-            PRUint32 maskSize = lpBitmapInfo->bmiHeader.biSizeImage;
-            lpBitmapInfo->bmiHeader.biBitCount = 32;
-            lpBitmapInfo->bmiHeader.biSizeImage = colorSize;
-            lpBitmapInfo->bmiHeader.biClrUsed = 0;
-            lpBitmapInfo->bmiHeader.biClrImportant = 0;
-            if (GetDIBits(hDC, iconInfo.hbmColor, 0, maskInfo.bmiHeader.biHeight, buffer + sizeof(ICONFILEHEADER) + sizeof(ICONENTRY) + sizeof(BITMAPINFOHEADER), lpBitmapInfo, DIB_RGB_COLORS)) {
-              // doubling the height because icons have two bitmaps
-              lpBitmapInfo->bmiHeader.biHeight *= 2;
-              lpBitmapInfo->bmiHeader.biSizeImage += maskSize;
+          char *whereTo = buffer;
+          int howMuch;
 
+          // the data starts with an icon file header
+          ICONFILEHEADER iconHeader;
+          iconHeader.ifhReserved = 0;
+          iconHeader.ifhType = 1;
+          iconHeader.ifhCount = 1;
+          howMuch = sizeof(ICONFILEHEADER);
+          memcpy(whereTo, &iconHeader, howMuch);
+          whereTo += howMuch;
+
+          // followed by the single icon entry
+          ICONENTRY iconEntry;
+          iconEntry.ieWidth = colorInfo.bmiHeader.biWidth;
+          iconEntry.ieHeight = colorInfo.bmiHeader.biHeight;
+          iconEntry.ieColors = 0;
+          iconEntry.ieReserved = 0;
+          iconEntry.iePlanes = 1;
+          iconEntry.ieBitCount = colorInfo.bmiHeader.biBitCount;
+          iconEntry.ieSizeImage = sizeof(BITMAPINFOHEADER) +
+                                  colorInfo.bmiHeader.biSizeImage +
+                                  maskInfo.bmiHeader.biSizeImage;
+          iconEntry.ieFileOffset = sizeof(ICONFILEHEADER) + sizeof(ICONENTRY);
+          howMuch = sizeof(ICONENTRY);
+          memcpy(whereTo, &iconEntry, howMuch);
+          whereTo += howMuch;
+
+          // followed by the bitmap info header
+          // (doubling the height because icons have two bitmaps)
+          colorInfo.bmiHeader.biHeight *= 2;
+          colorInfo.bmiHeader.biSizeImage += maskInfo.bmiHeader.biSizeImage;
+          howMuch = sizeof(BITMAPINFOHEADER);
+          memcpy(whereTo, &colorInfo.bmiHeader, howMuch);
+          whereTo += howMuch;
+          colorInfo.bmiHeader.biHeight /= 2;
+          colorInfo.bmiHeader.biSizeImage -= maskInfo.bmiHeader.biSizeImage;
+
+          // followed by the bitmap data
+          if (GetDIBits(hDC, iconInfo.hbmColor, 0,
+                        colorInfo.bmiHeader.biHeight, whereTo,
+                        &colorInfo, DIB_RGB_COLORS)) {
+            whereTo += colorInfo.bmiHeader.biSizeImage;
+            if (GetDIBits(hDC, iconInfo.hbmMask, 0,
+                          maskInfo.bmiHeader.biHeight, whereTo,
+                          &maskInfo, DIB_RGB_COLORS)) {
               // Now, create a pipe and stuff our data into it
               nsCOMPtr<nsIInputStream> inStream;
               nsCOMPtr<nsIOutputStream> outStream;
@@ -534,6 +658,9 @@ nsresult nsIconChannel::MakeInputStream(nsIInputStream** _retval, PRBool nonBloc
     DestroyIcon(hIcon);
   } // if we got an hIcon
 
+  // If we didn't make a stream, then fail.
+  if (!*_retval && NS_SUCCEEDED(rv))
+    rv = NS_ERROR_NOT_AVAILABLE;
 #endif
   return rv;
 }
