@@ -15,7 +15,7 @@
  *
  * The Original Code is mozilla.org code.
  *
- * The Initial Developer of the Original Code is Mozilla Corporation.
+ * The Initial Developer of the Original Code is Mozilla Foundation.
  * Portions created by the Initial Developer are Copyright (C) 2009
  * the Initial Developer. All Rights Reserved.
  *
@@ -44,6 +44,7 @@
 #include "prenv.h"
 
 #include "gfxPlatform.h"
+#include "gfxUtils.h"
 
 static PRBool gDisableOptimize = PR_FALSE;
 
@@ -126,6 +127,11 @@ static PRBool ShouldUseImageSurfaces()
 #elif defined(USE_WIN_SURFACE)
   static const DWORD kGDIObjectsHighWaterMark = 7000;
 
+  if (gfxWindowsPlatform::GetPlatform()->GetRenderMode() ==
+      gfxWindowsPlatform::RENDER_DIRECT2D) {
+    return PR_TRUE;
+  }
+
   // at 7000 GDI objects, stop allocating normal images to make sure
   // we never hit the 10k hard limit.
   // GetCurrentProcess() just returns (HANDLE)-1, it's inlined afaik
@@ -152,10 +158,12 @@ imgFrame::imgFrame() :
   mBlendMethod(1), /* imgIContainer::kBlendOver */
   mSinglePixel(PR_FALSE),
   mNeverUseDeviceSurface(PR_FALSE),
-  mFormatChanged(PR_FALSE)
+  mFormatChanged(PR_FALSE),
+  mCompositingFailed(PR_FALSE)
 #ifdef USE_WIN_SURFACE
   , mIsDDBSurface(PR_FALSE)
 #endif
+  , mLocked(PR_FALSE)
 {
   static PRBool hasCheckedOptimize = PR_FALSE;
   if (!hasCheckedOptimize) {
@@ -204,7 +212,7 @@ nsresult imgFrame::Init(PRInt32 aX, PRInt32 aY, PRInt32 aWidth, PRInt32 aHeight,
   if (aPaletteDepth != 0) {
     // We're creating for a paletted image.
     if (aPaletteDepth > 8) {
-      NS_ERROR("This Depth is not supported\n");
+      NS_ERROR("This Depth is not supported");
       return NS_ERROR_FAILURE;
     }
 
@@ -368,10 +376,87 @@ nsresult imgFrame::Optimize()
   return NS_OK;
 }
 
-static PRBool                                                                                                       
-IsSafeImageTransformComponent(gfxFloat aValue)
+static void
+DoSingleColorFastPath(gfxContext*    aContext,
+                      const gfxRGBA& aSinglePixelColor,
+                      const gfxRect& aFill)
 {
-  return aValue >= -32768 && aValue <= 32767;
+  // if a == 0, it's a noop
+  if (aSinglePixelColor.a == 0.0)
+    return;
+
+  gfxContext::GraphicsOperator op = aContext->CurrentOperator();
+  if (op == gfxContext::OPERATOR_OVER && aSinglePixelColor.a == 1.0) {
+    aContext->SetOperator(gfxContext::OPERATOR_SOURCE);
+  }
+
+  aContext->SetDeviceColor(aSinglePixelColor);
+  aContext->NewPath();
+  aContext->Rectangle(aFill);
+  aContext->Fill();
+  aContext->SetOperator(op);
+  aContext->SetDeviceColor(gfxRGBA(0,0,0,0));
+}
+
+imgFrame::SurfaceWithFormat
+imgFrame::SurfaceForDrawing(PRBool             aDoPadding,
+                            PRBool             aDoPartialDecode,
+                            PRBool             aDoTile,
+                            const nsIntMargin& aPadding,
+                            gfxMatrix&         aUserSpaceToImageSpace,
+                            gfxRect&           aFill,
+                            gfxRect&           aSubimage,
+                            gfxRect&           aSourceRect,
+                            gfxRect&           aImageRect)
+{
+  gfxIntSize size(PRInt32(aImageRect.Width()), PRInt32(aImageRect.Height()));
+  if (!aDoPadding && !aDoPartialDecode) {
+    NS_ASSERTION(!mSinglePixel, "This should already have been handled");
+    return SurfaceWithFormat(new gfxSurfaceDrawable(ThebesSurface(), size), mFormat);
+  }
+
+  gfxRect available = gfxRect(mDecoded.x, mDecoded.y, mDecoded.width, mDecoded.height);
+
+  if (aDoTile || mSinglePixel) {
+    // Create a temporary surface.
+    // Give this surface an alpha channel because there are
+    // transparent pixels in the padding or undecoded area
+    gfxImageSurface::gfxImageFormat format = gfxASurface::ImageFormatARGB32;
+    nsRefPtr<gfxASurface> surface =
+      gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, format);
+    if (!surface || surface->CairoStatus())
+      return SurfaceWithFormat();
+
+    // Fill 'available' with whatever we've got
+    gfxContext tmpCtx(surface);
+    tmpCtx.SetOperator(gfxContext::OPERATOR_SOURCE);
+    if (mSinglePixel) {
+      tmpCtx.SetDeviceColor(mSinglePixelColor);
+    } else {
+      tmpCtx.SetSource(ThebesSurface(), gfxPoint(aPadding.left, aPadding.top));
+    }
+    tmpCtx.Rectangle(available);
+    tmpCtx.Fill();
+    return SurfaceWithFormat(new gfxSurfaceDrawable(surface, size), format);
+  }
+
+  // Not tiling, and we have a surface, so we can account for
+  // padding and/or a partial decode just by twiddling parameters.
+  // First, update our user-space fill rect.
+  aSourceRect = aSourceRect.Intersect(available);
+  gfxMatrix imageSpaceToUserSpace = aUserSpaceToImageSpace;
+  imageSpaceToUserSpace.Invert();
+  aFill = imageSpaceToUserSpace.Transform(aSourceRect);
+
+  aSubimage = aSubimage.Intersect(available) - gfxPoint(aPadding.left, aPadding.top);
+  aUserSpaceToImageSpace.Multiply(gfxMatrix().Translate(-gfxPoint(aPadding.left, aPadding.top)));
+  aSourceRect = aSourceRect - gfxPoint(aPadding.left, aPadding.top);
+  aImageRect = gfxRect(0, 0, mSize.width, mSize.height);
+
+  gfxIntSize availableSize(mDecoded.width, mDecoded.height);
+  return SurfaceWithFormat(new gfxSurfaceDrawable(ThebesSurface(),
+                                                  availableSize),
+                           mFormat);
 }
 
 void imgFrame::Draw(gfxContext *aContext, gfxPattern::GraphicsFilter aFilter,
@@ -384,268 +469,33 @@ void imgFrame::Draw(gfxContext *aContext, gfxPattern::GraphicsFilter aFilter,
 
   PRBool doPadding = aPadding != nsIntMargin(0,0,0,0);
   PRBool doPartialDecode = !ImageComplete();
-  gfxContext::GraphicsOperator op = aContext->CurrentOperator();
 
-  if (mSinglePixel && !doPadding && ImageComplete()) {
-    // Single-color fast path
-    // if a == 0, it's a noop
-    if (mSinglePixelColor.a == 0.0)
-      return;
-
-    if (op == gfxContext::OPERATOR_OVER && mSinglePixelColor.a == 1.0)
-      aContext->SetOperator(gfxContext::OPERATOR_SOURCE);
-
-    aContext->SetDeviceColor(mSinglePixelColor);
-    aContext->NewPath();
-    aContext->Rectangle(aFill);
-    aContext->Fill();
-    aContext->SetOperator(op);
-    aContext->SetDeviceColor(gfxRGBA(0,0,0,0));
+  if (mSinglePixel && !doPadding && !doPartialDecode) {
+    DoSingleColorFastPath(aContext, mSinglePixelColor, aFill);
     return;
   }
 
   gfxMatrix userSpaceToImageSpace = aUserSpaceToImageSpace;
   gfxRect sourceRect = userSpaceToImageSpace.Transform(aFill);
-  gfxRect imageRect(0, 0, mSize.width + aPadding.LeftRight(), mSize.height + aPadding.TopBottom());
+  gfxRect imageRect(0, 0, mSize.width + aPadding.LeftRight(),
+                    mSize.height + aPadding.TopBottom());
   gfxRect subimage(aSubimage.x, aSubimage.y, aSubimage.width, aSubimage.height);
   gfxRect fill = aFill;
-  nsRefPtr<gfxASurface> surface;
-  gfxImageSurface::gfxImageFormat format;
 
   NS_ASSERTION(!sourceRect.Intersect(subimage).IsEmpty(),
                "We must be allowed to sample *some* source pixels!");
 
   PRBool doTile = !imageRect.Contains(sourceRect);
-  if (doPadding || doPartialDecode) {
-    gfxRect available = gfxRect(mDecoded.x, mDecoded.y, mDecoded.width, mDecoded.height) +
-      gfxPoint(aPadding.left, aPadding.top);
+  SurfaceWithFormat surfaceResult =
+    SurfaceForDrawing(doPadding, doPartialDecode, doTile, aPadding,
+                      userSpaceToImageSpace, fill, subimage, sourceRect,
+                      imageRect);
 
-    if (!doTile && !mSinglePixel) {
-      // Not tiling, and we have a surface, so we can account for
-      // padding and/or a partial decode just by twiddling parameters.
-      // First, update our user-space fill rect.
-      sourceRect = sourceRect.Intersect(available);
-      gfxMatrix imageSpaceToUserSpace = userSpaceToImageSpace;
-      imageSpaceToUserSpace.Invert();
-      fill = imageSpaceToUserSpace.Transform(sourceRect);
-
-      surface = ThebesSurface();
-      format = mFormat;
-      subimage = subimage.Intersect(available) - gfxPoint(aPadding.left, aPadding.top);
-      userSpaceToImageSpace.Multiply(gfxMatrix().Translate(-gfxPoint(aPadding.left, aPadding.top)));
-      sourceRect = sourceRect - gfxPoint(aPadding.left, aPadding.top);
-      imageRect = gfxRect(0, 0, mSize.width, mSize.height);
-    } else {
-      // Create a temporary surface
-      gfxIntSize size(PRInt32(imageRect.Width()), PRInt32(imageRect.Height()));
-      // Give this surface an alpha channel because there are
-      // transparent pixels in the padding or undecoded area
-      format = gfxASurface::ImageFormatARGB32;
-      surface = gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, format);
-      if (!surface || surface->CairoStatus() != 0)
-        return;
-
-      // Fill 'available' with whatever we've got
-      gfxContext tmpCtx(surface);
-      tmpCtx.SetOperator(gfxContext::OPERATOR_SOURCE);
-      if (mSinglePixel) {
-        tmpCtx.SetDeviceColor(mSinglePixelColor);
-      } else {
-        tmpCtx.SetSource(ThebesSurface(), gfxPoint(aPadding.left, aPadding.top));
-      }
-      tmpCtx.Rectangle(available);
-      tmpCtx.Fill();
-    }
-  } else {
-    NS_ASSERTION(!mSinglePixel, "This should already have been handled");
-    surface = ThebesSurface();
-    format = mFormat;
-  }
-  // At this point, we've taken care of mSinglePixel images, images with
-  // aPadding, and partially-decoded images.
-
-  if (!AllowedImageSize(fill.size.width + 1, fill.size.height + 1)) {
-    NS_WARNING("Destination area too large, bailing out");
-    return;
-  }
-
-  // Compute device-space-to-image-space transform. We need to sanity-
-  // check it to work around a pixman bug :-(
-  // XXX should we only do this for certain surface types?
-  gfxFloat deviceX, deviceY;
-  nsRefPtr<gfxASurface> currentTarget =
-    aContext->CurrentSurface(&deviceX, &deviceY);
-  gfxMatrix currentMatrix = aContext->CurrentMatrix();
-  gfxMatrix deviceToUser = currentMatrix;
-  deviceToUser.Invert();
-  deviceToUser.Translate(-gfxPoint(-deviceX, -deviceY));
-  gfxMatrix deviceToImage = deviceToUser;
-  deviceToImage.Multiply(userSpaceToImageSpace);
-
-  PRBool pushedGroup = PR_FALSE;
-  if (currentTarget->GetType() != gfxASurface::SurfaceTypeQuartz) {
-    // BEGIN working around cairo/pixman bug (bug 364968)
-    // Quartz's limits for matrix are much larger than pixman
-      
-    // Our device-space-to-image-space transform may not be acceptable to pixman.
-    if (!IsSafeImageTransformComponent(deviceToImage.xx) ||
-        !IsSafeImageTransformComponent(deviceToImage.xy) ||
-        !IsSafeImageTransformComponent(deviceToImage.yx) ||
-        !IsSafeImageTransformComponent(deviceToImage.yy)) {
-      NS_WARNING("Scaling up too much, bailing out");
-      return;
-    }
-
-    if (!IsSafeImageTransformComponent(deviceToImage.x0) ||
-        !IsSafeImageTransformComponent(deviceToImage.y0)) {
-      // We'll push a group, which will hopefully reduce our transform's
-      // translation so it's in bounds
-      aContext->Save();
-
-      // Clip the rounded-out-to-device-pixels bounds of the
-      // transformed fill area. This is the area for the group we
-      // want to push.
-      aContext->IdentityMatrix();
-      gfxRect bounds = currentMatrix.TransformBounds(fill);
-      bounds.RoundOut();
-      aContext->Clip(bounds);
-      aContext->SetMatrix(currentMatrix);
-
-      aContext->PushGroup(gfxASurface::CONTENT_COLOR_ALPHA);
-      aContext->SetOperator(gfxContext::OPERATOR_OVER);
-      pushedGroup = PR_TRUE;
-    }
-    // END working around cairo/pixman bug (bug 364968)
-  }
-
-  nsRefPtr<gfxPattern> pattern = new gfxPattern(surface);
-  pattern->SetMatrix(userSpaceToImageSpace);
-
-  // OK now, the hard part left is to account for the subimage sampling
-  // restriction. If all the transforms involved are just integer
-  // translations, then we assume no resampling will occur so there's
-  // nothing to do.
-  // XXX if only we had source-clipping in cairo!
-  if (!currentMatrix.HasNonIntegerTranslation() &&
-      !userSpaceToImageSpace.HasNonIntegerTranslation()) {
-    if (doTile) {
-      pattern->SetExtend(gfxPattern::EXTEND_REPEAT);
-    }
-  } else {
-    if (doTile || !subimage.Contains(imageRect)) {
-      // EXTEND_PAD won't help us here; we have to create a temporary
-      // surface to hold the subimage of pixels we're allowed to
-      // sample
-
-      gfxRect userSpaceClipExtents = aContext->GetClipExtents();
-      // This isn't optimal --- if aContext has a rotation then GetClipExtents
-      // will have to do a bounding-box computation, and TransformBounds might
-      // too, so we could get a better result if we computed image space clip
-      // extents in one go --- but it doesn't really matter and this is easier
-      // to understand.
-      gfxRect imageSpaceClipExtents = userSpaceToImageSpace.TransformBounds(userSpaceClipExtents);
-      // Inflate by one pixel because bilinear filtering will sample at most
-      // one pixel beyond the computed image pixel coordinate.
-      imageSpaceClipExtents.Outset(1.0);
-
-      gfxRect needed = imageSpaceClipExtents.Intersect(sourceRect).Intersect(subimage);
-      needed.RoundOut();
-
-      // if 'needed' is empty, nothing will be drawn since aFill
-      // must be entirely outside the clip region, so it doesn't
-      // matter what we do here, but we should avoid trying to
-      // create a zero-size surface.
-      if (!needed.IsEmpty()) {
-        gfxIntSize size(PRInt32(needed.Width()), PRInt32(needed.Height()));
-        nsRefPtr<gfxASurface> temp =
-          gfxPlatform::GetPlatform()->CreateOffscreenSurface(size, format);
-        if (temp && temp->CairoStatus() == 0) {
-          gfxContext tmpCtx(temp);
-          tmpCtx.SetOperator(gfxContext::OPERATOR_SOURCE);
-          nsRefPtr<gfxPattern> tmpPattern = new gfxPattern(surface);
-          if (tmpPattern) {
-            tmpPattern->SetExtend(gfxPattern::EXTEND_REPEAT);
-            tmpPattern->SetMatrix(gfxMatrix().Translate(needed.pos));
-            tmpCtx.SetPattern(tmpPattern);
-            tmpCtx.Paint();
-            tmpPattern = new gfxPattern(temp);
-            if (tmpPattern) {
-              pattern.swap(tmpPattern);
-              pattern->SetMatrix(
-                  gfxMatrix(userSpaceToImageSpace).Multiply(gfxMatrix().Translate(-needed.pos)));
-            }
-          }
-        }
-      }
-    }
-
-    // In theory we can handle this using cairo's EXTEND_PAD,
-    // but implementation limitations mean we have to consult
-    // the surface type.
-    switch (currentTarget->GetType()) {
-      case gfxASurface::SurfaceTypeXlib:
-      case gfxASurface::SurfaceTypeXcb:
-      {
-        // See bug 324698.  This is a workaround for EXTEND_PAD not being
-        // implemented correctly on linux in the X server.
-        //
-        // Set the filter to CAIRO_FILTER_FAST --- otherwise,
-        // pixman's sampling will sample transparency for the outside edges and we'll
-        // get blurry edges.  CAIRO_EXTEND_PAD would also work here, if
-        // available
-        //
-        // But don't do this for simple downscales because it's horrible.
-        // Downscaling means that device-space coordinates are
-        // scaled *up* to find the image pixel coordinates.
-        //
-        // deviceToImage is slightly stale because up above we may
-        // have adjusted the pattern's matrix ... but the adjustment
-        // is only a translation so the scale factors in deviceToImage
-        // are still valid.
-        PRBool isDownscale =
-          deviceToImage.xx >= 1.0 && deviceToImage.yy >= 1.0 &&
-          deviceToImage.xy == 0.0 && deviceToImage.yx == 0.0;
-        if (!isDownscale) {
-          pattern->SetFilter(gfxPattern::FILTER_FAST);
-        }
-        break;
-      }
-
-      case gfxASurface::SurfaceTypeQuartz:
-      case gfxASurface::SurfaceTypeQuartzImage:
-        // Don't set EXTEND_PAD, Mac seems to be OK. Really?
-        pattern->SetFilter(aFilter);
-        break;
-
-      default:
-        // turn on EXTEND_PAD.
-        // This is what we really want for all surface types, if the
-        // implementation was universally good.
-        pattern->SetExtend(gfxPattern::EXTEND_PAD);
-        pattern->SetFilter(aFilter);
-        break;
-    }
-  }
-
-  if ((op == gfxContext::OPERATOR_OVER || pushedGroup) &&
-      format == gfxASurface::ImageFormatRGB24) {
-    aContext->SetOperator(gfxContext::OPERATOR_SOURCE);
-  }
-
-  // Phew! Now we can actually draw this image
-  aContext->NewPath();
-#ifdef MOZ_GFX_OPTIMIZE_MOBILE
-  pattern->SetFilter(gfxPattern::FILTER_FAST); 
-#endif
-  aContext->SetPattern(pattern);
-  aContext->Rectangle(fill);
-  aContext->Fill();
-
-  aContext->SetOperator(op);
-  if (pushedGroup) {
-    aContext->PopGroupToSource();
-    aContext->Paint();
-    aContext->Restore();
+  if (surfaceResult.IsValid()) {
+    gfxUtils::DrawPixelSnapped(aContext, surfaceResult.mDrawable,
+                               userSpaceToImageSpace,
+                               subimage, sourceRect, imageRect, fill,
+                               surfaceResult.mFormat, aFilter);
   }
 }
 
@@ -717,7 +567,7 @@ nsresult imgFrame::ImageUpdated(const nsIntRect &aUpdateRect)
 
   // clamp to bounds, in case someone sends a bogus updateRect (I'm looking at
   // you, gif decoder)
-  nsIntRect boundsRect(0, 0, mSize.width, mSize.height);
+  nsIntRect boundsRect(mOffset, mSize);
   mDecoded.IntersectRect(mDecoded, boundsRect);
 
 #ifdef XP_MACOSX
@@ -767,16 +617,18 @@ PRUint32 imgFrame::GetImageBytesPerRow() const
 {
   if (mImageSurface)
     return mImageSurface->Stride();
-  else
+
+  if (mPaletteDepth)
     return mSize.width;
+
+  NS_ERROR("GetImageBytesPerRow called with mImageSurface == null and mPaletteDepth == 0");
+
+  return 0;
 }
 
 PRUint32 imgFrame::GetImageDataLength() const
 {
-  if (mImageSurface)
-    return mImageSurface->Stride() * mSize.height;
-  else
-    return mSize.width * mSize.height;
+  return GetImageBytesPerRow() * mSize.height;
 }
 
 void imgFrame::GetImageData(PRUint8 **aData, PRUint32 *length) const
@@ -815,7 +667,13 @@ void imgFrame::GetPaletteData(PRUint32 **aPalette, PRUint32 *length) const
 nsresult imgFrame::LockImageData()
 {
   if (mPalettedImageData)
-    return NS_OK;
+    return NS_ERROR_NOT_AVAILABLE;
+
+  NS_ABORT_IF_FALSE(!mLocked, "Trying to lock already locked image data.");
+  if (mLocked) {
+    return NS_ERROR_FAILURE;
+  }
+  mLocked = PR_TRUE;
 
   if ((mOptSurface || mSinglePixel) && !mImageSurface) {
     // Recover the pixels
@@ -841,15 +699,43 @@ nsresult imgFrame::LockImageData()
 #endif
   }
 
+  // We might write to the bits in this image surface, so we need to make the
+  // surface ready for that.
+  if (mImageSurface)
+    mImageSurface->Flush();
+
+#ifdef USE_WIN_SURFACE
+  if (mWinSurface)
+    mWinSurface->Flush();
+#endif
+
   return NS_OK;
 }
 
 nsresult imgFrame::UnlockImageData()
 {
   if (mPalettedImageData)
-    return NS_OK;
+    return NS_ERROR_NOT_AVAILABLE;
+
+  NS_ABORT_IF_FALSE(mLocked, "Unlocking an unlocked image!");
+  if (!mLocked) {
+    return NS_ERROR_FAILURE;
+  }
+
+  mLocked = PR_FALSE;
+
+  // Assume we've been written to.
+  if (mImageSurface)
+    mImageSurface->MarkDirty();
+
+#ifdef USE_WIN_SURFACE
+  if (mWinSurface)
+    mWinSurface->MarkDirty();
+#endif
 
 #ifdef XP_MACOSX
+  // The quartz image surface (ab)uses the flush method to get the
+  // cairo_image_surface data into a CGImage, so we have to call Flush() here.
   if (mQuartzSurface)
     mQuartzSurface->Flush();
 #endif
@@ -904,7 +790,7 @@ void imgFrame::SetBlendMethod(PRInt32 aBlendMethod)
 
 PRBool imgFrame::ImageComplete() const
 {
-  return mDecoded == nsIntRect(0, 0, mSize.width, mSize.height);
+  return mDecoded == nsIntRect(mOffset, mSize);
 }
 
 // A hint from the image decoders that this image has no alpha, even
@@ -920,3 +806,51 @@ void imgFrame::SetHasNoAlpha()
       mFormatChanged = PR_TRUE;
   }
 }
+
+PRBool imgFrame::GetCompositingFailed() const
+{
+  return mCompositingFailed;
+}
+
+void imgFrame::SetCompositingFailed(PRBool val)
+{
+  mCompositingFailed = val;
+}
+
+PRUint32 imgFrame::EstimateMemoryUsed() const
+{
+  PRUint32 size = 0;
+
+  if (mSinglePixel) {
+    size += sizeof(gfxRGBA);
+  }
+
+  if (mPalettedImageData) {
+    size += GetImageDataLength() + PaletteDataLength();
+  }
+
+#ifdef USE_WIN_SURFACE
+  if (mWinSurface) {
+    size += mWinSurface->KnownMemoryUsed();
+  } else
+#endif
+#ifdef XP_MACOSX
+  if (mQuartzSurface) {
+    size += mSize.width * mSize.height * 4;
+  } else
+#endif
+  if (mImageSurface) {
+    size += mImageSurface->KnownMemoryUsed();
+  }
+
+  if (mOptSurface) {
+    size += mOptSurface->KnownMemoryUsed();
+  }
+
+  // fall back to pessimistic/approximate size
+  if (size == 0) {
+    size = mSize.width * mSize.height * 4;
+  }
+
+  return size;
+}
