@@ -35,23 +35,24 @@
 
 #include "nptest_platform.h"
 #include "npapi.h"
+#include <pthread.h>
 #include <gdk/gdk.h>
 #ifdef MOZ_X11
 #include <gdk/gdkx.h>
 #include <X11/extensions/shape.h>
 #endif
+#include <glib.h>
 #include <gtk/gtk.h>
+#include <unistd.h>
 
-/**
- * XXX In various places in this file we use GDK APIs to inspect the
- * window ancestors of the plugin. These APIs will not work properly if
- * this plugin is used in a browser that does not use GDK for all its
- * widgets. They would also fail for out-of-process plugins. These should
- * be fixed to use raw X APIs instead.
- */
+ using namespace std;
 
 struct _PlatformData {
+#ifdef MOZ_X11
   Display* display;
+  Visual* visual;
+  Colormap colormap;
+#endif
   GtkWidget* plug;
 };
 
@@ -76,8 +77,10 @@ pluginInstanceInit(InstanceData* instanceData)
   if (!instanceData->platformData)
     return NPERR_OUT_OF_MEMORY_ERROR;
 
-  instanceData->platformData->display = 0;
-  instanceData->platformData->plug = 0;
+  instanceData->platformData->display = NULL;
+  instanceData->platformData->visual = NULL;
+  instanceData->platformData->colormap = None;  
+  instanceData->platformData->plug = NULL;
 
   return NPERR_NO_ERROR;
 #else
@@ -105,7 +108,14 @@ pluginInstanceShutdown(InstanceData* instanceData)
   GtkWidget* plug = instanceData->platformData->plug;
   if (plug) {
     instanceData->platformData->plug = 0;
-    gtk_widget_destroy(plug);
+    if (instanceData->cleanupWidget) {
+      // Default/tidy behavior
+      gtk_widget_destroy(plug);
+    } else {
+      // Flash Player style: let the GtkPlug destroy itself on disconnect.
+      g_signal_handlers_disconnect_matched(plug, G_SIGNAL_MATCH_DATA, 0, 0,
+                                           NULL, NULL, instanceData);
+    }
   }
 
   NPN_MemFree(instanceData->platformData);
@@ -145,7 +155,8 @@ pluginDrawSolid(InstanceData* instanceData, GdkDrawable* gdkWindow,
 }
 
 static void
-pluginDrawWindow(InstanceData* instanceData, GdkDrawable* gdkWindow)
+pluginDrawWindow(InstanceData* instanceData, GdkDrawable* gdkWindow,
+                 const GdkRectangle& invalidRect)
 {
   NPWindow& window = instanceData->window;
   // When we have a widget, window.x/y are meaningless since our
@@ -157,7 +168,10 @@ pluginDrawWindow(InstanceData* instanceData, GdkDrawable* gdkWindow)
 
   if (instanceData->scriptableObject->drawMode == DM_SOLID_COLOR) {
     // drawing a solid color for reftests
-    pluginDrawSolid(instanceData, gdkWindow, x, y, width, height);
+    pluginDrawSolid(instanceData, gdkWindow,
+                    invalidRect.x, invalidRect.y,
+                    invalidRect.width, invalidRect.height);
+    notifyDidPaint(instanceData);
     return;
   }
 
@@ -203,6 +217,8 @@ pluginDrawWindow(InstanceData* instanceData, GdkDrawable* gdkWindow)
   g_object_unref(pangoTextLayout);
 
   g_object_unref(gdkContext);
+
+  notifyDidPaint(instanceData);
 }
 
 static gboolean
@@ -210,7 +226,27 @@ ExposeWidget(GtkWidget* widget, GdkEventExpose* event,
              gpointer user_data)
 {
   InstanceData* instanceData = static_cast<InstanceData*>(user_data);
-  pluginDrawWindow(instanceData, event->window);
+  pluginDrawWindow(instanceData, event->window, event->area);
+  return TRUE;
+}
+
+static gboolean
+MotionEvent(GtkWidget* widget, GdkEventMotion* event,
+            gpointer user_data)
+{
+  InstanceData* instanceData = static_cast<InstanceData*>(user_data);
+  instanceData->lastMouseX = event->x;
+  instanceData->lastMouseY = event->y;
+  return TRUE;
+}
+
+static gboolean
+ButtonEvent(GtkWidget* widget, GdkEventButton* event,
+            gpointer user_data)
+{
+  InstanceData* instanceData = static_cast<InstanceData*>(user_data);
+  instanceData->lastMouseX = event->x;
+  instanceData->lastMouseY = event->y;
   return TRUE;
 }
 
@@ -231,9 +267,13 @@ pluginDoSetWindow(InstanceData* instanceData, NPWindow* newWindow)
 {
   instanceData->window = *newWindow;
 
+#ifdef MOZ_X11
   NPSetWindowCallbackStruct *ws_info =
     static_cast<NPSetWindowCallbackStruct*>(newWindow->ws_info);
   instanceData->platformData->display = ws_info->display;
+  instanceData->platformData->visual = ws_info->visual;
+  instanceData->platformData->colormap = ws_info->colormap;
+#endif
 }
 
 void
@@ -252,13 +292,24 @@ pluginWidgetInit(InstanceData* instanceData, void* oldWindow)
   /* create a GtkPlug container */
   GtkWidget* plug = gtk_plug_new(nativeWinId);
 
+  // Test for bugs 539138 and 561308
+  if (!plug->window)
+    g_error("Plug has no window"); // aborts
+
   /* make sure the widget is capable of receiving focus */
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET(plug), GTK_CAN_FOCUS);
 
   /* all the events that our widget wants to receive */
-  gtk_widget_add_events(plug, GDK_EXPOSURE_MASK);
+  gtk_widget_add_events(plug, GDK_EXPOSURE_MASK | GDK_POINTER_MOTION_MASK |
+                              GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
   g_signal_connect(G_OBJECT(plug), "expose-event", G_CALLBACK(ExposeWidget),
                    instanceData);
+  g_signal_connect(G_OBJECT(plug), "motion_notify_event", G_CALLBACK(MotionEvent),
+                   instanceData);
+  g_signal_connect(G_OBJECT(plug), "button_press_event", G_CALLBACK(ButtonEvent),
+                   instanceData);
+  g_signal_connect(G_OBJECT(plug), "button_release_event", G_CALLBACK(ButtonEvent),
+                   instanceData);
   g_signal_connect(G_OBJECT(plug), "delete-event", G_CALLBACK(DeleteWidget),
                    instanceData);
   gtk_widget_show(plug);
@@ -271,20 +322,98 @@ int16_t
 pluginHandleEvent(InstanceData* instanceData, void* event)
 {
 #ifdef MOZ_X11
-  XEvent *nsEvent = (XEvent *)event;
+  XEvent* nsEvent = (XEvent*)event;
 
-  if (nsEvent->type != GraphicsExpose)
-    return 0;
+  switch (nsEvent->type) {
+  case GraphicsExpose: {
+    const XGraphicsExposeEvent& expose = nsEvent->xgraphicsexpose;
+    NPWindow& window = instanceData->window;
+    window.window = (void*)(expose.drawable);
 
-  XGraphicsExposeEvent *expose = &nsEvent->xgraphicsexpose;
-  instanceData->window.window = (void*)(expose->drawable);
+    GdkNativeWindow nativeWinId = reinterpret_cast<XID>(window.window);
 
-  GdkNativeWindow nativeWinId =
-    reinterpret_cast<XID>(instanceData->window.window);
-  GdkDrawable* gdkWindow = GDK_DRAWABLE(gdk_window_foreign_new(nativeWinId));  
-  pluginDrawWindow(instanceData, gdkWindow);
-  g_object_unref(gdkWindow);
+    GdkDisplay* gdkDisplay = gdk_x11_lookup_xdisplay(expose.display);
+    if (!gdkDisplay) {
+      g_warning("Display not opened by GDK");
+      return 0;
+    }
+    // gdk_pixmap_foreign_new() doesn't check whether a GdkPixmap already
+    // exists, so check first.
+    // https://bugzilla.gnome.org/show_bug.cgi?id=590690
+    GdkPixmap* gdkDrawable =
+      GDK_DRAWABLE(gdk_pixmap_lookup_for_display(gdkDisplay, nativeWinId));
+    // If there is no existing GdkPixmap or it doesn't have a colormap then
+    // create our own.
+    if (gdkDrawable) {
+      GdkColormap* gdkColormap = gdk_drawable_get_colormap(gdkDrawable);
+      if (!gdkColormap) {
+        g_warning("No GdkColormap on GdkPixmap");
+        return 0;
+      }
+      if (gdk_x11_colormap_get_xcolormap(gdkColormap)
+          != instanceData->platformData->colormap) {
+        g_warning("wrong Colormap");
+        return 0;
+      }
+      if (gdk_x11_visual_get_xvisual(gdk_colormap_get_visual(gdkColormap))
+          != instanceData->platformData->visual) {
+        g_warning("wrong Visual");
+        return 0;
+      }
+      g_object_ref(gdkDrawable);
+    } else {
+      gdkDrawable =
+        GDK_DRAWABLE(gdk_pixmap_foreign_new_for_display(gdkDisplay,
+                                                        nativeWinId));
+      VisualID visualID = instanceData->platformData->visual->visualid;
+      GdkVisual* gdkVisual =
+        gdk_x11_screen_lookup_visual(gdk_drawable_get_screen(gdkDrawable),
+                                     visualID);
+      GdkColormap* gdkColormap =
+        gdk_x11_colormap_foreign_new(gdkVisual,
+                                     instanceData->platformData->colormap);
+      gdk_drawable_set_colormap(gdkDrawable, gdkColormap);
+      g_object_unref(G_OBJECT(gdkColormap));
+    }
+
+    const NPRect& clip = window.clipRect;
+    if (expose.x < clip.left || expose.y < clip.top ||
+        expose.x + expose.width > clip.right ||
+        expose.y + expose.height > clip.bottom) {
+      g_warning("expose rectangle not in clip rectangle");
+      return 0;
+    }
+    if (expose.x < window.x || expose.y < window.y ||
+        expose.x + expose.width > window.x + int32_t(window.width) ||
+        expose.y + expose.height > window.y + int32_t(window.height)) {
+      g_warning("expose rectangle not in plugin rectangle");
+      return 0;
+    }      
+
+    GdkRectangle invalidRect =
+      { expose.x, expose.y, expose.width, expose.height };
+    pluginDrawWindow(instanceData, gdkDrawable, invalidRect);
+    g_object_unref(gdkDrawable);
+    break;
+  }
+  case MotionNotify: {
+    XMotionEvent* motion = &nsEvent->xmotion;
+    instanceData->lastMouseX = motion->x;
+    instanceData->lastMouseY = motion->y;
+    break;
+  }
+  case ButtonPress:
+  case ButtonRelease: {
+    XButtonEvent* button = &nsEvent->xbutton;
+    instanceData->lastMouseX = button->x;
+    instanceData->lastMouseY = button->y;
+    break;
+  }
+  default:
+    break;
+  }
 #endif
+
   return 0;
 }
 
@@ -502,3 +631,97 @@ int32_t pluginGetClipRegionRectEdge(InstanceData* instanceData,
   }
   return NPTEST_INT32_ERROR;
 }
+
+void pluginDoInternalConsistencyCheck(InstanceData* instanceData, string& error)
+{
+}
+
+string
+pluginGetClipboardText(InstanceData* instanceData)
+{
+  GtkClipboard* cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+  // XXX this is a BAD WAY to interact with GtkClipboard.  We use this
+  // deprecated interface only to test nested event loop handling.
+  gchar* text = gtk_clipboard_wait_for_text(cb);
+  string retText = text ? text : "";
+
+  g_free(text);
+
+  return retText;
+}
+
+//-----------------------------------------------------------------------------
+// NB: this test is quite gross in that it's not only
+// nondeterministic, but dependent on the guts of the nested glib
+// event loop handling code in PluginModule.  We first sleep long
+// enough to make sure that the "detection timer" will be pending when
+// we enter the nested glib loop, then similarly for the "process browser
+// events" timer.  Then we "schedule" the crasher thread to run at about the
+// same time we expect that the PluginModule "process browser events" task
+// will run.  If all goes well, the plugin process will crash and generate the
+// XPCOM "plugin crashed" task, and the browser will run that task while still
+// in the "process some events" loop.
+
+static void*
+CrasherThread(void* data)
+{
+  // Give the parent thread a chance to send the message.
+  usleep(200);
+
+  // Exit (without running atexit hooks) rather than crashing with a signal
+  // so as to make timing more reliable.  The process terminates immediately
+  // rather than waiting for a thread in the parent process to attach and
+  // generate a minidump.
+  _exit(1);
+
+  // not reached
+  return(NULL);
+}
+
+bool
+pluginCrashInNestedLoop(InstanceData* instanceData)
+{
+  // wait at least long enough for nested loop detector task to be pending ...
+  sleep(1);
+
+  // Run the nested loop detector by processing all events that are waiting.
+  bool found_event = false;
+  while (g_main_context_iteration(NULL, FALSE)) {
+    found_event = true;
+  }
+  if (!found_event) {
+    g_warning("DetectNestedEventLoop did not fire");
+    return true; // trigger a test failure
+  }
+
+  // wait at least long enough for the "process browser events" task to be
+  // pending ...
+  sleep(1);
+
+  // we'll be crashing soon, note that fact now to avoid messing with
+  // timing too much
+  NoteIntentionalCrash();
+
+  // schedule the crasher thread ...
+  pthread_t crasherThread;
+  if (0 != pthread_create(&crasherThread, NULL, CrasherThread, NULL)) {
+    g_warning("Failed to create thread");
+    return true; // trigger a test failure
+  }
+
+  // .. and hope it crashes at about the same time as the "process browser
+  // events" task (that should run in this loop) is being processed in the
+  // parent.
+  found_event = false;
+  while (g_main_context_iteration(NULL, FALSE)) {
+    found_event = true;
+  }
+  if (found_event) {
+    g_warning("Should have crashed in ProcessBrowserEvents");
+  } else {
+    g_warning("ProcessBrowserEvents did not fire");
+  }
+
+  // if we get here without crashing, then we'll trigger a test failure
+  return true;
+}
