@@ -53,9 +53,7 @@
 #include "nsIPrefBranch2.h"
 #include "nsServiceManagerUtils.h"
 
-#ifdef WINCE
-#include <windows.h>
-#endif
+#include "mozilla/FunctionTimer.h"
 
 #if defined(PR_LOGGING)
 PRLogModuleInfo *gSocketTransportLog = nsnull;
@@ -382,6 +380,8 @@ NS_IMPL_THREADSAFE_ISUPPORTS6(nsSocketTransportService,
 NS_IMETHODIMP
 nsSocketTransportService::Init()
 {
+    NS_TIME_FUNCTION;
+
     NS_ENSURE_TRUE(mLock, NS_ERROR_OUT_OF_MEMORY);
 
     if (!NS_IsMainThread()) {
@@ -412,6 +412,8 @@ nsSocketTransportService::Init()
             LOG(("running socket transport thread without a pollable event"));
         }
     }
+    
+    NS_TIME_FUNCTION_MARK("Created thread");
 
     nsCOMPtr<nsIThread> thread;
     nsresult rv = NS_NewThread(getter_AddRefs(thread), this);
@@ -428,6 +430,8 @@ nsSocketTransportService::Init()
         tmpPrefService->AddObserver(SEND_BUFFER_PREF, this, PR_FALSE);
     UpdatePrefs();
 
+    NS_TIME_FUNCTION_MARK("UpdatePrefs");
+
     mInitialized = PR_TRUE;
     return NS_OK;
 }
@@ -559,10 +563,6 @@ nsSocketTransportService::Run()
 
     gSocketThread = PR_GetCurrentThread();
 
-#ifdef WINCE
-    CeSetThreadPriority(GetCurrentThread(), 116);
-#endif
-
     // add thread event to poll list (mThreadEvent may be NULL)
     mPollList[0].fd = mThreadEvent;
     mPollList[0].in_flags = PR_POLL_READ;
