@@ -39,13 +39,19 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
+#include <limits.h>
 #include <stdio.h>
 
 #include "nsError.h"
 #include "nsMemory.h"
+#include "nsThreadUtils.h"
 #include "nsIClassInfoImpl.h"
 #include "nsIProgrammingLanguage.h"
+#include "Variant.h"
 
+#include "mozIStorageError.h"
+
+#include "mozStorageBindingParams.h"
 #include "mozStorageConnection.h"
 #include "mozStorageStatementJSHelper.h"
 #include "mozStoragePrivateHelpers.h"
@@ -55,6 +61,8 @@
 
 #include "prlog.h"
 
+#include "mozilla/FunctionTimer.h"
+
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gStorageLog;
 #endif
@@ -65,10 +73,13 @@ namespace storage {
 ////////////////////////////////////////////////////////////////////////////////
 //// nsIClassInfo
 
-NS_IMPL_CI_INTERFACE_GETTER2(
+NS_IMPL_CI_INTERFACE_GETTER5(
   Statement,
   mozIStorageStatement,
-  mozIStorageValueArray
+  mozIStorageBaseStatement,
+  mozIStorageBindingParams,
+  mozIStorageValueArray,
+  mozilla::storage::StorageBaseStatementInternal
 )
 
 class StatementClassInfo : public nsIClassInfo
@@ -147,7 +158,7 @@ static StatementClassInfo sStatementClassInfo;
 //// Statement
 
 Statement::Statement()
-: mDBConnection(nsnull)
+: StorageBaseStatementInternal()
 , mDBStatement(NULL)
 , mColumnNames()
 , mExecuting(false)
@@ -164,8 +175,7 @@ Statement::initialize(Connection *aDBConnection,
   sqlite3 *db = aDBConnection->GetNativeConnection();
   NS_ASSERTION(db, "We should never be called with a null sqlite3 database!");
 
-  int srv = ::sqlite3_prepare_v2(db, PromiseFlatCString(aSQLStatement).get(),
-                                 -1, &mDBStatement, NULL);
+  int srv = prepareStmt(db, PromiseFlatCString(aSQLStatement), &mDBStatement);
   if (srv != SQLITE_OK) {
 #ifdef PR_LOGGING
       PR_LOG(gStorageLog, PR_LOG_ERROR,
@@ -217,7 +227,7 @@ Statement::initialize(Connection *aDBConnection,
       // or @, all of which are valid characters for binding a parameter.
       // We will warn the consumer that they may not be safely using LIKE.
       NS_WARNING("Unsafe use of LIKE detected!  Please ensure that you "
-                 "are using mozIStorageConnection::escapeStringForLIKE "
+                 "are using mozIStorageStatement::escapeStringForLIKE "
                  "and that you are binding that result to the statement "
                  "to prevent SQL injection attacks.");
     }
@@ -231,27 +241,131 @@ Statement::initialize(Connection *aDBConnection,
   return NS_OK;
 }
 
+mozIStorageBindingParams *
+Statement::getParams()
+{
+  nsresult rv;
+
+  // If we do not have an array object yet, make it.
+  if (!mParamsArray) {
+    nsCOMPtr<mozIStorageBindingParamsArray> array;
+    rv = NewBindingParamsArray(getter_AddRefs(array));
+    NS_ENSURE_SUCCESS(rv, nsnull);
+
+    mParamsArray = static_cast<BindingParamsArray *>(array.get());
+  }
+
+  // If there isn't already any rows added, we'll have to add one to use.
+  if (mParamsArray->length() == 0) {
+    nsRefPtr<BindingParams> params(new BindingParams(mParamsArray, this));
+    NS_ENSURE_TRUE(params, nsnull);
+
+    rv = mParamsArray->AddParams(params);
+    NS_ENSURE_SUCCESS(rv, nsnull);
+
+    // We have to unlock our params because AddParams locks them.  This is safe
+    // because no reference to the params object was, or ever will be given out.
+    params->unlock(this);
+
+    // We also want to lock our array at this point - we don't want anything to
+    // be added to it.  Nothing has, or will ever get a reference to it, but we
+    // will get additional safety checks via assertions by doing this.
+    mParamsArray->lock();
+  }
+
+  return *mParamsArray->begin();
+}
+
 Statement::~Statement()
 {
-  (void)Finalize();
+  (void)internalFinalize(true);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+//// nsISupports
+
 NS_IMPL_THREADSAFE_ADDREF(Statement)
 NS_IMPL_THREADSAFE_RELEASE(Statement)
 
 NS_INTERFACE_MAP_BEGIN(Statement)
   NS_INTERFACE_MAP_ENTRY(mozIStorageStatement)
+  NS_INTERFACE_MAP_ENTRY(mozIStorageBaseStatement)
+  NS_INTERFACE_MAP_ENTRY(mozIStorageBindingParams)
   NS_INTERFACE_MAP_ENTRY(mozIStorageValueArray)
+  NS_INTERFACE_MAP_ENTRY(mozilla::storage::StorageBaseStatementInternal)
   if (aIID.Equals(NS_GET_IID(nsIClassInfo))) {
     foundInterface = static_cast<nsIClassInfo *>(&sStatementClassInfo);
   }
   else
-  NS_INTERFACE_MAP_ENTRY(nsISupports)
+  NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, mozIStorageStatement)
 NS_INTERFACE_MAP_END
 
+
+////////////////////////////////////////////////////////////////////////////////
+//// StorageBaseStatementInternal
+
+Connection *
+Statement::getOwner()
+{
+  return mDBConnection;
+}
+
+int
+Statement::getAsyncStatement(sqlite3_stmt **_stmt)
+{
+  // If we have no statement, we shouldn't be calling this method!
+  NS_ASSERTION(mDBStatement != NULL, "We have no statement to clone!");
+
+  // If we do not yet have a cached async statement, clone our statement now.
+  if (!mAsyncStatement) {
+    nsDependentCString sql(::sqlite3_sql(mDBStatement));
+    int rc = prepareStmt(mDBConnection->GetNativeConnection(), sql,
+                     &mAsyncStatement);
+    if (rc != SQLITE_OK) {
+      *_stmt = nsnull;
+      return rc;
+    }
+
+#ifdef PR_LOGGING
+    PR_LOG(gStorageLog, PR_LOG_NOTICE,
+           ("Cloned statement 0x%p to 0x%p", mDBStatement, mAsyncStatement));
+#endif
+  }
+
+  *_stmt = mAsyncStatement;
+  return SQLITE_OK;
+}
+
+nsresult
+Statement::getAsynchronousStatementData(StatementData &_data)
+{
+  if (!mDBStatement)
+    return NS_ERROR_UNEXPECTED;
+
+  sqlite3_stmt *stmt;
+  int rc = getAsyncStatement(&stmt);
+  if (rc != SQLITE_OK)
+    return convertResultCode(rc);
+
+  _data = StatementData(stmt, bindingParamsArray(), this);
+
+  return NS_OK;
+}
+
+already_AddRefed<mozIStorageBindingParams>
+Statement::newBindingParams(mozIStorageBindingParamsArray *aOwner)
+{
+  nsCOMPtr<mozIStorageBindingParams> params = new BindingParams(aOwner, this);
+  return params.forget();
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 //// mozIStorageStatement
 
+// proxy to StorageBaseStatementInternal using its define helper.
+MIXIN_IMPL_STORAGEBASESTATEMENTINTERNAL(Statement, (void)0;)
+
 NS_IMETHODIMP
 Statement::Clone(mozIStorageStatement **_statement)
 {
@@ -268,6 +382,12 @@ Statement::Clone(mozIStorageStatement **_statement)
 
 NS_IMETHODIMP
 Statement::Finalize()
+{
+  return internalFinalize(false);
+}
+
+nsresult
+Statement::internalFinalize(bool aDestructing)
 {
   if (!mDBStatement)
     return NS_OK;
@@ -280,6 +400,15 @@ Statement::Finalize()
   int srv = ::sqlite3_finalize(mDBStatement);
   mDBStatement = NULL;
 
+  if (mAsyncStatement) {
+    // If the destructor called us, there are no pending async statements (they
+    // hold a reference to us) and we can/must just kill the statement directly.
+    if (aDestructing)
+      internalAsyncFinalize();
+    else
+      asyncFinalize();
+  }
+
   // We are considered dead at this point, so any wrappers for row or params
   // need to lose their reference to us.
   if (mStatementParamsHolder) {
@@ -415,6 +544,7 @@ Statement::Reset()
   checkAndLogStatementPerformance(mDBStatement);
 #endif
 
+  mParamsArray = nsnull;
   (void)sqlite3_reset(mDBStatement);
   (void)sqlite3_clear_bindings(mDBStatement);
 
@@ -423,88 +553,6 @@ Statement::Reset()
   return NS_OK;
 }
 
-NS_IMETHODIMP
-Statement::BindUTF8StringParameter(PRUint32 aParamIndex,
-                                   const nsACString &aValue)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_text(mDBStatement, aParamIndex + 1,
-                                PromiseFlatCString(aValue).get(),
-                                aValue.Length(), SQLITE_TRANSIENT);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindStringParameter(PRUint32 aParamIndex,
-                               const nsAString &aValue)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_text16(mDBStatement, aParamIndex + 1,
-                                  PromiseFlatString(aValue).get(),
-                                  aValue.Length() * 2, SQLITE_TRANSIENT);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindDoubleParameter(PRUint32 aParamIndex,
-                               double aValue)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_double(mDBStatement, aParamIndex + 1, aValue);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindInt32Parameter(PRUint32 aParamIndex,
-                              PRInt32 aValue)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_int(mDBStatement, aParamIndex + 1, aValue);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindInt64Parameter(PRUint32 aParamIndex,
-                              PRInt64 aValue)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_int64(mDBStatement, aParamIndex + 1, aValue);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindNullParameter(PRUint32 aParamIndex)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_null(mDBStatement, aParamIndex + 1);
-  return convertResultCode(srv);
-}
-
-NS_IMETHODIMP
-Statement::BindBlobParameter(PRUint32 aParamIndex,
-                             const PRUint8 *aValue,
-                             PRUint32 aValueSize)
-{
-  if (!mDBStatement)
-    return NS_ERROR_NOT_INITIALIZED;
-
-  int srv = ::sqlite3_bind_blob(mDBStatement, aParamIndex + 1, aValue,
-                                aValueSize, SQLITE_TRANSIENT);
-  return convertResultCode(srv);
-}
-
 NS_IMETHODIMP
 Statement::BindParameters(mozIStorageBindingParamsArray *aParameters)
 {
@@ -515,23 +563,15 @@ Statement::BindParameters(mozIStorageBindingParamsArray *aParameters)
   if (array->getOwner() != this)
     return NS_ERROR_UNEXPECTED;
 
+  if (array->length() == 0)
+    return NS_ERROR_UNEXPECTED;
+
   mParamsArray = array;
   mParamsArray->lock();
 
   return NS_OK;
 }
 
-NS_IMETHODIMP
-Statement::NewBindingParamsArray(mozIStorageBindingParamsArray **_array)
-{
-  nsCOMPtr<mozIStorageBindingParamsArray> array =
-    new BindingParamsArray(this);
-  NS_ENSURE_TRUE(array, NS_ERROR_OUT_OF_MEMORY);
-
-  array.forget(_array);
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 Statement::Execute()
 {
@@ -540,9 +580,9 @@ Statement::Execute()
 
   PRBool ret;
   nsresult rv = ExecuteStep(&ret);
-  NS_ENSURE_SUCCESS(rv, rv);
+  nsresult rv2 = Reset();
 
-  return Reset();
+  return NS_FAILED(rv) ? rv : rv2;
 }
 
 NS_IMETHODIMP
@@ -551,7 +591,30 @@ Statement::ExecuteStep(PRBool *_moreResults)
   if (!mDBStatement)
     return NS_ERROR_NOT_INITIALIZED;
 
-  int srv = ::sqlite3_step(mDBStatement);
+  NS_TIME_FUNCTION_MIN_FMT(5, "mozIStorageStatement::ExecuteStep(%s) (0x%p)",
+                           mDBConnection->getFilename().get(), mDBStatement);
+
+  // Bind any parameters first before executing.
+  if (mParamsArray) {
+    // If we have more than one row of parameters to bind, they shouldn't be
+    // calling this method (and instead use executeAsync).
+    if (mParamsArray->length() != 1)
+      return NS_ERROR_UNEXPECTED;
+
+    BindingParamsArray::iterator row = mParamsArray->begin();
+    nsCOMPtr<IStorageBindingParamsInternal> bindingInternal = 
+      do_QueryInterface(*row);
+    nsCOMPtr<mozIStorageError> error = bindingInternal->bind(mDBStatement);
+    if (error) {
+      PRInt32 srv;
+      (void)error->GetResult(&srv);
+      return convertResultCode(srv);
+    }
+
+    // We have bound, so now we can clear our array.
+    mParamsArray = nsnull;
+  }
+  int srv = stepStmt(mDBStatement);
 
 #ifdef PR_LOGGING
   if (srv != SQLITE_ROW && srv != SQLITE_DONE) {
@@ -589,14 +652,6 @@ Statement::ExecuteStep(PRBool *_moreResults)
   return convertResultCode(srv);
 }
 
-nsresult
-Statement::ExecuteAsync(mozIStorageStatementCallback *aCallback,
-                        mozIStoragePendingStatement **_stmt)
-{
-  mozIStorageStatement *stmts[1] = {this};
-  return mDBConnection->ExecuteAsync(stmts, 1, aCallback, _stmt);
-}
-
 NS_IMETHODIMP
 Statement::GetState(PRInt32 *_state)
 {
@@ -610,25 +665,6 @@ Statement::GetState(PRInt32 *_state)
   return NS_OK;
 }
 
-NS_IMETHODIMP
-Statement::EscapeStringForLIKE(const nsAString &aValue,
-                               const PRUnichar aEscapeChar,
-                               nsAString &_escapedString)
-{
-  const PRUnichar MATCH_ALL('%');
-  const PRUnichar MATCH_ONE('_');
-
-  _escapedString.Truncate(0);
-
-  for (PRUint32 i = 0; i < aValue.Length(); i++) {
-    if (aValue[i] == aEscapeChar || aValue[i] == MATCH_ALL ||
-        aValue[i] == MATCH_ONE)
-      _escapedString += aEscapeChar;
-    _escapedString += aValue[i];
-  }
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 Statement::GetColumnDecltype(PRUint32 aParamIndex,
                              nsACString &_declType)
@@ -643,7 +679,7 @@ Statement::GetColumnDecltype(PRUint32 aParamIndex,
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-//// mozIStorageValueArray
+//// mozIStorageValueArray (now part of mozIStorageStatement too)
 
 NS_IMETHODIMP
 Statement::GetNumEntries(PRUint32 *_length)
@@ -667,19 +703,19 @@ Statement::GetTypeOfIndex(PRUint32 aIndex,
   int t = ::sqlite3_column_type(mDBStatement, aIndex);
   switch (t) {
     case SQLITE_INTEGER:
-      *_type = VALUE_TYPE_INTEGER;
+      *_type = mozIStorageStatement::VALUE_TYPE_INTEGER;
       break;
     case SQLITE_FLOAT:
-      *_type = VALUE_TYPE_FLOAT;
+      *_type = mozIStorageStatement::VALUE_TYPE_FLOAT;
       break;
     case SQLITE_TEXT:
-      *_type = VALUE_TYPE_TEXT;
+      *_type = mozIStorageStatement::VALUE_TYPE_TEXT;
       break;
     case SQLITE_BLOB:
-      *_type = VALUE_TYPE_BLOB;
+      *_type = mozIStorageStatement::VALUE_TYPE_BLOB;
       break;
     case SQLITE_NULL:
-      *_type = VALUE_TYPE_NULL;
+      *_type = mozIStorageStatement::VALUE_TYPE_NULL;
       break;
     default:
       return NS_ERROR_FAILURE;
@@ -746,8 +782,8 @@ Statement::GetUTF8String(PRUint32 aIndex,
   PRInt32 type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
   NS_ENSURE_SUCCESS(rv, rv);
-  if (type == VALUE_TYPE_NULL) {
-    // NULL columns should have IsVod set to distinguis them from an empty
+  if (type == mozIStorageStatement::VALUE_TYPE_NULL) {
+    // NULL columns should have IsVoid set to distinguish them from the empty
     // string.
     _value.Truncate(0);
     _value.SetIsVoid(PR_TRUE);
@@ -769,8 +805,8 @@ Statement::GetString(PRUint32 aIndex,
   PRInt32 type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
   NS_ENSURE_SUCCESS(rv, rv);
-  if (type == VALUE_TYPE_NULL) {
-    // NULL columns should have IsVod set to distinguis them from an empty
+  if (type == mozIStorageStatement::VALUE_TYPE_NULL) {
+    // NULL columns should have IsVoid set to distinguish them from the empty
     // string.
     _value.Truncate(0);
     _value.SetIsVoid(PR_TRUE);
@@ -853,9 +889,17 @@ Statement::GetIsNull(PRUint32 aIndex,
   PRInt32 type;
   nsresult rv = GetTypeOfIndex(aIndex, &type);
   NS_ENSURE_SUCCESS(rv, rv);
-  *_isNull = (type == VALUE_TYPE_NULL);
+  *_isNull = (type == mozIStorageStatement::VALUE_TYPE_NULL);
   return NS_OK;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+//// mozIStorageBindingParams
+
+BOILERPLATE_BIND_PROXIES(
+  Statement, 
+  if (!mDBStatement) return NS_ERROR_NOT_INITIALIZED;
+)
+
 } // namespace storage
 } // namespace mozilla
