@@ -67,6 +67,48 @@
 using std::string;
 using std::vector;
 
+#define IS_DELIM(m, c)          ((m)[(c) >> 3] & (1 << ((c) & 7)))
+#define SET_DELIM(m, c)         ((m)[(c) >> 3] |= (1 << ((c) & 7)))
+#define DELIM_TABLE_SIZE        32
+
+// Copied from nsCRT
+char* strtok2(char* string, const char* delims, char* *newStr)
+{
+  PR_ASSERT(string);
+  
+  char delimTable[DELIM_TABLE_SIZE];
+  PRUint32 i;
+  char* result;
+  char* str = string;
+  
+  for (i = 0; i < DELIM_TABLE_SIZE; i++)
+    delimTable[i] = '\0';
+  
+  for (i = 0; delims[i]; i++) {
+    SET_DELIM(delimTable, static_cast<PRUint8>(delims[i]));
+  }
+  
+  // skip to beginning
+  while (*str && IS_DELIM(delimTable, static_cast<PRUint8>(*str))) {
+    str++;
+  }
+  result = str;
+  
+  // fix up the end of the token
+  while (*str) {
+    if (IS_DELIM(delimTable, static_cast<PRUint8>(*str))) {
+      *str++ = '\0';
+      break;
+    }
+    str++;
+  }
+  *newStr = str;
+  
+  return str == result ? NULL : result;
+}
+
+
+
 enum client_auth_option {
   caNone = 0,
   caRequire = 1,
@@ -85,8 +127,20 @@ typedef struct {
   PRFileDesc* client_sock;
   PRNetAddr client_addr;
   server_info_t* server_info;
+  // the original host in the Host: header for this connection is
+  // stored here, for proxied connections
+  string original_host;
+  // true if no SSL should be used for this connection
+  bool http_proxy_only;
+  // true if this connection is for a WebSocket
+  bool iswebsocket;
 } connection_info_t;
 
+typedef struct {
+  string fullHost;
+  bool matched;
+} server_match_t;
+
 const PRInt32 BUF_SIZE = 16384;
 const PRInt32 BUF_MARGIN = 1024;
 const PRInt32 BUF_TOTAL = BUF_SIZE + BUF_MARGIN;
@@ -166,6 +220,7 @@ const PRUint32 DEFAULT_STACKSIZE = (512 * 1024);
 string nssconfigdir;
 vector<server_info_t> servers;
 PRNetAddr remote_addr;
+PRNetAddr websocket_server;
 PRThreadPool* threads = NULL;
 PRLock* shutdown_lock = NULL;
 PRCondVar* shutdown_condvar = NULL;
@@ -186,6 +241,14 @@ PR_CALLBACK PRIntn ClientAuthValueComparator(const void *v1, const void *v2)
     return -1;
 }
 
+static PRIntn match_hostname(PLHashEntry *he, PRIntn index, void* arg)
+{
+  server_match_t *match = (server_match_t*)arg;
+  if (match->fullHost.find((char*)he->key) != string::npos)
+    match->matched = true;
+  return HT_ENUMERATE_NEXT;
+}
+
 /*
  * Signal the main thread that the application should shut down.
  */
@@ -200,21 +263,36 @@ bool ReadConnectRequest(server_info_t* server_info,
     relayBuffer& buffer, PRInt32* result, string& certificate,
     client_auth_option* clientauth, string& host)
 {
-  if (buffer.present() < 4)
+  if (buffer.present() < 4) {
+    printf(" !! only %d bytes present in the buffer", (int)buffer.present());
     return false;
-  if (strncmp(buffer.buffertail-4, "\r\n\r\n", 4))
+  }
+  if (strncmp(buffer.buffertail-4, "\r\n\r\n", 4)) {
+    printf(" !! request is not tailed with CRLFCRLF but with %x %x %x %x", 
+           *(buffer.buffertail-4),
+           *(buffer.buffertail-3),
+           *(buffer.buffertail-2),
+           *(buffer.buffertail-1));
     return false;
+  }
+
+  printf(" parsing initial connect request, dump:\n%.*s\n", (int)buffer.present(), buffer.bufferhead);
 
   *result = 400;
 
   char* token;
-  token = strtok(buffer.bufferhead, " ");
-  if (!token) 
+  char* _caret;
+  token = strtok2(buffer.bufferhead, " ", &_caret);
+  if (!token) {
+    printf(" no space found");
     return true;
-  if (strcmp(token, "CONNECT")) 
+  }
+  if (strcmp(token, "CONNECT")) {
+    printf(" not CONNECT request but %s", token);
     return true;
+  }
 
-  token = strtok(NULL, " ");
+  token = strtok2(_caret, " ", &_caret);
   void* c = PL_HashTableLookup(server_info->host_cert_table, token);
   if (c)
     certificate = static_cast<char*>(c);
@@ -228,9 +306,11 @@ bool ReadConnectRequest(server_info_t* server_info,
   else
     *clientauth = caNone;
 
-  token = strtok(NULL, "/");
-  if (strcmp(token, "HTTP"))
+  token = strtok2(_caret, "/", &_caret);
+  if (strcmp(token, "HTTP")) {  
+    printf(" not tailed with HTTP but with %s", token);
     return true;
+  }
 
   *result = 200;
   return true;
@@ -282,6 +362,113 @@ bool ConfigureSSLServerSocket(PRFileDesc* socket, server_info_t* si, string &cer
   return true;
 }
 
+/**
+ * This function examines the buffer for a S5ec-WebSocket-Location: field, 
+ * and if it's present, it replaces the hostname in that field with the
+ * value in the server's original_host field.  This function works
+ * in the reverse direction as AdjustWebSocketHost(), replacing the real
+ * hostname of a response with the potentially fake hostname that is expected
+ * by the browser (e.g., mochi.test).
+ *
+ * @return true if the header was adjusted successfully, or not found, false
+ * if the header is present but the url is not, which should indicate
+ * that more data needs to be read from the socket
+ */
+bool AdjustWebSocketLocation(relayBuffer& buffer, connection_info_t *ci)
+{
+  assert(buffer.margin());
+  buffer.buffertail[1] = '\0';
+
+  char* wsloc = strstr(buffer.bufferhead, "Sec-WebSocket-Location:");
+  if (!wsloc)
+    return true;
+  // advance pointer to the start of the hostname
+  wsloc = strstr(wsloc, "ws://");
+  if (!wsloc)
+    return false;
+  wsloc += 5;
+  // find the end of the hostname
+  char* wslocend = strchr(wsloc + 1, '/');
+  if (!wslocend)
+    return false;
+  char *crlf = strstr(wsloc, "\r\n");
+  if (!crlf)
+    return false;
+  if (ci->original_host.empty())
+    return true;
+
+  int diff = ci->original_host.length() - (wslocend-wsloc);
+  if (diff > 0)
+    assert(size_t(diff) <= buffer.margin());
+  memmove(wslocend + diff, wslocend, buffer.buffertail - wsloc - diff);
+  buffer.buffertail += diff;
+
+  memcpy(wsloc, ci->original_host.c_str(), ci->original_host.length());
+  return true;
+}
+
+/**
+ * This function examines the buffer for a Host: field, and if it's present,
+ * it replaces the hostname in that field with the hostname in the server's
+ * remote_addr field.  This is needed because proxy requests may be coming
+ * from mochitest with fake hosts, like mochi.test, and these need to be
+ * replaced with the host that the destination server is actually running
+ * on.
+ */
+bool AdjustWebSocketHost(relayBuffer& buffer, connection_info_t *ci)
+{
+  const char HEADER_UPGRADE[] = "Upgrade:";
+  const char HEADER_HOST[] = "Host:";
+
+  PRNetAddr inet_addr = (websocket_server.inet.port ? websocket_server :
+    remote_addr);
+
+  assert(buffer.margin());
+
+  // Cannot use strnchr so add a null char at the end. There is always some
+  // space left because we preserve a margin.
+  buffer.buffertail[1] = '\0';
+
+  // Verify this is a WebSocket header.
+  char* h1 = strstr(buffer.bufferhead, HEADER_UPGRADE);
+  if (!h1)
+    return false;
+  h1 += strlen(HEADER_UPGRADE);
+  h1 += strspn(h1, " \t");
+  char* h2 = strstr(h1, "WebSocket\r\n");
+  if (!h2)
+    return false;
+
+  char* host = strstr(buffer.bufferhead, HEADER_HOST);
+  if (!host)
+    return false;
+  // advance pointer to beginning of hostname
+  host += strlen(HEADER_HOST);
+  host += strspn(host, " \t");
+
+  char* endhost = strstr(host, "\r\n");
+  if (!endhost)
+    return false;
+
+  // Save the original host, so we can use it later on responses from the
+  // server.
+  ci->original_host.assign(host, endhost-host);
+
+  char newhost[40];
+  PR_NetAddrToString(&inet_addr, newhost, sizeof(newhost));
+  assert(strlen(newhost) < sizeof(newhost) - 7);
+  sprintf(newhost, "%s:%d", newhost, PR_ntohs(inet_addr.inet.port));
+
+  int diff = strlen(newhost) - (endhost-host);
+  if (diff > 0)
+    assert(size_t(diff) <= buffer.margin());
+  memmove(endhost + diff, endhost, buffer.buffertail - host - diff);
+  buffer.buffertail += diff;
+
+  memcpy(host, newhost, strlen(newhost));
+  return true;
+}
+
 /**
  * This function prefixes Request-URI path with a full scheme-host-port
  * string.
@@ -293,6 +480,7 @@ bool AdjustRequestURI(relayBuffer& buffer, string *host)
   // Cannot use strnchr so add a null char at the end. There is always some space left
   // because we preserve a margin.
   buffer.buffertail[1] = '\0';
+  printf(" incoming request to adjust:\n%s\n", buffer.bufferhead);
 
   char *token, *path;
   path = strchr(buffer.bufferhead, ' ') + 1;
@@ -345,7 +533,7 @@ bool ConnectSocket(PRFileDesc *fd, const PRNetAddr *addr, PRIntervalTime timeout
 void HandleConnection(void* data)
 {
   connection_info_t* ci = static_cast<connection_info_t*>(data);
-  PRIntervalTime connect_timeout = PR_SecondsToInterval(2);
+  PRIntervalTime connect_timeout = PR_SecondsToInterval(30);
 
   AutoFD other_sock(PR_NewTCPSocket());
   bool client_done = false;
@@ -357,6 +545,10 @@ void HandleConnection(void* data)
   client_auth_option clientAuth;
   string fullHost;
 
+  printf("SSLTUNNEL(%p): incoming connection csock(0)=%p, ssock(1)=%p\n",
+         static_cast<void*>(data),
+         static_cast<void*>(ci->client_sock),
+         static_cast<void*>(other_sock));
   if (other_sock) 
   {
     PRInt32 numberOfSockets = 1;
@@ -365,7 +557,8 @@ void HandleConnection(void* data)
 
     if (!do_http_proxy)
     {
-      if (!ConfigureSSLServerSocket(ci->client_sock, ci->server_info, certificateToUse, caNone))
+      if (!ci->http_proxy_only && 
+          !ConfigureSSLServerSocket(ci->client_sock, ci->server_info, certificateToUse, caNone))
         client_error = true;
       else if (!ConnectSocket(other_sock, &remote_addr, connect_timeout))
         client_error = true;
@@ -378,21 +571,34 @@ void HandleConnection(void* data)
       {ci->client_sock, PR_POLL_READ, 0},
       {other_sock, PR_POLL_READ, 0}
     };
+    PRBool socketErrorState[2] = {PR_FALSE, PR_FALSE};
 
     while (!((client_error||client_done) && buffers[0].empty() && buffers[1].empty()))
     {
       sockets[0].in_flags |= PR_POLL_EXCEPT;
       sockets[1].in_flags |= PR_POLL_EXCEPT;
+      printf("SSLTUNNEL(%p): polling flags csock(0)=%c%c, ssock(1)=%c%c\n",
+             static_cast<void*>(data),
+             sockets[0].in_flags & PR_POLL_READ  ? 'R' : '-',
+             sockets[0].in_flags & PR_POLL_WRITE ? 'W' : '-',
+             sockets[1].in_flags & PR_POLL_READ  ? 'R' : '-',
+             sockets[1].in_flags & PR_POLL_WRITE ? 'W' : '-');
       PRInt32 pollStatus = PR_Poll(sockets, numberOfSockets, PR_MillisecondsToInterval(1000));
       if (pollStatus < 0)
       {
+        printf("SSLTUNNEL(%p): pollStatus=%d, exiting\n",
+               static_cast<void*>(data), pollStatus);
         client_error = true;
         break;
       }
 
       if (pollStatus == 0)
+      {
         // timeout
+        printf("SSLTUNNEL(%p): poll timeout, looping\n",
+               static_cast<void*>(data));
         continue;
+      }
 
       for (PRInt32 s = 0; s < numberOfSockets; ++s)
       {
@@ -402,42 +608,105 @@ void HandleConnection(void* data)
         PRInt16 &in_flags2 = sockets[s2].in_flags;
         sockets[s].out_flags = 0;
 
-        if (out_flags & PR_POLL_EXCEPT)
+        printf("SSLTUNNEL(%p): %csock(%d)=%p out_flags=%d",
+               static_cast<void*>(data),
+               s == 0 ? 'c' : 's',
+               s,
+               static_cast<void*>(sockets[s].fd),
+               out_flags);
+        if (out_flags & (PR_POLL_EXCEPT | PR_POLL_ERR | PR_POLL_HUP))
         {
+          printf(" :exception\n");
           client_error = true;
+          socketErrorState[s] = PR_TRUE;
+          // We got a fatal error state on the socket. Clear the output buffer
+          // for this socket to break the main loop, we will never more be able
+          // to send those data anyway.
+          buffers[s2].bufferhead = buffers[s2].buffertail = buffers[s2].buffer;
           continue;
-        } // PR_POLL_EXCEPT handling
+        } // PR_POLL_EXCEPT, PR_POLL_ERR, PR_POLL_HUP handling
+
+        if (out_flags & PR_POLL_READ && !buffers[s].free())
+        {
+           printf(" no place in read buffer but got read flag, dropping it now!");
+           in_flags &= ~PR_POLL_READ;
+        }
 
         if (out_flags & PR_POLL_READ && buffers[s].free())
         {
+          printf(" :reading");
           PRInt32 bytesRead = PR_Recv(sockets[s].fd, buffers[s].buffertail, 
               buffers[s].free(), 0, PR_INTERVAL_NO_TIMEOUT);
 
           if (bytesRead == 0)
           {
+            printf(" socket gracefully closed");
             client_done = true;
             in_flags &= ~PR_POLL_READ;
           }
           else if (bytesRead < 0)
           {
             if (PR_GetError() != PR_WOULD_BLOCK_ERROR)
+            {
+              printf(" error=%d", PR_GetError());
+              // We are in error state, indicate that the connection was 
+              // not closed gracefully
               client_error = true;
+              socketErrorState[s] = PR_TRUE;
+              // Wipe out our send buffer, we cannot send it anyway.
+              buffers[s2].bufferhead = buffers[s2].buffertail = buffers[s2].buffer;
+            }
+            else
+              printf(" would block");
           }
           else
           {
+            // If the other socket is in error state (unable to send/receive)
+            // throw this data away and continue loop
+            if (socketErrorState[s2])
+            {
+              printf(" have read but other socket is in error state\n");
+              continue;
+            }
+
             buffers[s].buffertail += bytesRead;
+            printf(", read %d bytes", bytesRead);
 
             // We have to accept and handle the initial CONNECT request here
             PRInt32 response;
             if (!connect_accepted && ReadConnectRequest(ci->server_info, buffers[s],
                 &response, certificateToUse, &clientAuth, fullHost))
             {
+              // Mark this as a proxy-only connection (no SSL) if the CONNECT
+              // request didn't come for port 443 or from any of the server's
+              // cert or clientauth hostnames.
+              if (fullHost.find(":443") == string::npos)
+              {
+                server_match_t match;
+                match.fullHost = fullHost;
+                match.matched = false;
+                PL_HashTableEnumerateEntries(ci->server_info->host_cert_table, 
+                                             match_hostname, 
+                                             &match);
+                PL_HashTableEnumerateEntries(ci->server_info->host_clientauth_table, 
+                                             match_hostname, 
+                                             &match);
+                ci->http_proxy_only = !match.matched;
+              }
+              else
+              {
+                ci->http_proxy_only = false;
+              }
+
               // Clean the request as it would be read
               buffers[s].bufferhead = buffers[s].buffertail = buffers[s].buffer;
+              in_flags |= PR_POLL_WRITE;
+              connect_accepted = true;
 
               // Store response to the oposite buffer
               if (response != 200)
               {
+                printf(" could not read the connect request, closing connection with %d", response);
                 client_done = true;
                 sprintf(buffers[s2].buffer, "HTTP/1.1 %d ERROR\r\nConnection: close\r\n\r\n", response);
                 buffers[s2].buffertail = buffers[s2].buffer + strlen(buffers[s2].buffer);
@@ -447,73 +716,126 @@ void HandleConnection(void* data)
               strcpy(buffers[s2].buffer, "HTTP/1.1 200 Connected\r\nConnection: keep-alive\r\n\r\n");
               buffers[s2].buffertail = buffers[s2].buffer + strlen(buffers[s2].buffer);
 
-              if (!ConnectSocket(other_sock, &remote_addr, connect_timeout))
-              {
-                client_error = true;
-                break;
-              }
-
+              printf(" accepted CONNECT request, connected to the server, sending OK to the client\n");
               // Send the response to the client socket
-              in_flags |= PR_POLL_WRITE;
-              connect_accepted = true;
               break;
             } // end of CONNECT handling
 
-            if (!buffers[s].free()) // Do not poll for read when the buffer is full
+            if (!buffers[s].free())
+            {
+              // Do not poll for read when the buffer is full
+              printf(" no place in our read buffer, stop reading");
               in_flags &= ~PR_POLL_READ;
+            }
 
             if (ssl_updated)
             {
-              if (s == 0 && expect_request_start)
-                expect_request_start = !AdjustRequestURI(buffers[s], &fullHost);
+              if (s == 0 && expect_request_start) 
+              {
+                if (!strstr(buffers[s].bufferhead, "\r\n\r\n"))
+                {
+                  // We haven't received the complete header yet, so wait.
+                  continue;
+                }
+                else
+                {
+                  ci->iswebsocket = AdjustWebSocketHost(buffers[s], ci);
+                  expect_request_start = !(ci->iswebsocket || 
+                                           AdjustRequestURI(buffers[s], &fullHost));
+                  PRNetAddr* addr = &remote_addr;
+                  if (ci->iswebsocket && websocket_server.inet.port)
+                    addr = &websocket_server;
+                  if (!ConnectSocket(other_sock, addr, connect_timeout))
+                  {
+                    printf(" could not open connection to the real server\n");
+                    client_error = true;
+                    break;
+                  }
+                  printf("\n connected to remote server\n");
+                  numberOfSockets = 2;
+                }
+              }
+              else if (s == 1 && ci->iswebsocket)
+              {
+                if (!AdjustWebSocketLocation(buffers[s], ci))
+                  continue;
+              }
 
               in_flags2 |= PR_POLL_WRITE;
+              printf(" telling the other socket to write");
             }
+            else
+              printf(" we have something for the other socket to write, but ssl has not been administered on it");
           }
         } // PR_POLL_READ handling
 
         if (out_flags & PR_POLL_WRITE)
         {
+          printf(" :writing");
           PRInt32 bytesWrite = PR_Send(sockets[s].fd, buffers[s2].bufferhead, 
               buffers[s2].present(), 0, PR_INTERVAL_NO_TIMEOUT);
 
           if (bytesWrite < 0)
           {
-            if (PR_GetError() != PR_WOULD_BLOCK_ERROR)
+            if (PR_GetError() != PR_WOULD_BLOCK_ERROR) {
+              printf(" error=%d", PR_GetError());
               client_error = true;
+              socketErrorState[s] = PR_TRUE;
+              // We got a fatal error while writting the buffer. Clear it to break
+              // the main loop, we will never more be able to send it.
+              buffers[s2].bufferhead = buffers[s2].buffertail = buffers[s2].buffer;
+            }
+            else
+              printf(" would block");
           }
           else
           {
+            printf(", written %d bytes", bytesWrite);
+            buffers[s2].buffertail[1] = '\0';
+            printf(" dump:\n%.*s\n", bytesWrite, buffers[s2].bufferhead);
+            
             buffers[s2].bufferhead += bytesWrite;
             if (buffers[s2].present())
-              in_flags |= PR_POLL_WRITE;              
+            {
+              printf(" still have to write %d bytes", (int)buffers[s2].present());
+              in_flags |= PR_POLL_WRITE;
+            }              
             else
             {
               if (!ssl_updated)
               {
+                printf(" proxy response sent to the client");
                 // Proxy response has just been writen, update to ssl
                 ssl_updated = true;
-                if (!ConfigureSSLServerSocket(ci->client_sock, ci->server_info, certificateToUse, clientAuth))
+                if (!ci->http_proxy_only && 
+                    !ConfigureSSLServerSocket(ci->client_sock, ci->server_info, certificateToUse, clientAuth))
                 {
+                  printf(" but failed to config server socket\n");
                   client_error = true;
                   break;
                 }
 
-                numberOfSockets = 2;
+                printf(" client socket updated to SSL");
               } // sslUpdate
 
-              in_flags &= ~PR_POLL_WRITE;              
+              printf(" dropping our write flag and setting other socket read flag");
+              in_flags &= ~PR_POLL_WRITE;
               in_flags2 |= PR_POLL_READ;
               buffers[s2].compact();
             }
           }
         } // PR_POLL_WRITE handling
+        printf("\n"); // end the log
       } // for...
     } // while, poll
   }
   else
     client_error = true;
 
+  printf("SSLTUNNEL(%p): exiting root function for csock=%p, ssock=%p\n",
+         static_cast<void*>(data),
+         static_cast<void*>(ci->client_sock),
+         static_cast<void*>(other_sock));
   if (!client_error)
     PR_Shutdown(ci->client_sock, PR_SHUTDOWN_SEND);
   PR_Close(ci->client_sock);
@@ -539,6 +861,13 @@ void StartServer(void* data)
     return;
   }
 
+  // In case the socket is still open in the TIME_WAIT state from a previous
+  // instance of ssltunnel we ask to reuse the port.
+  PRSocketOptionData socket_option;
+  socket_option.option = PR_SockOpt_Reuseaddr;
+  socket_option.value.reuse_addr = PR_TRUE;
+  PR_SetSocketOption(listen_socket, &socket_option);
+
   PRNetAddr server_addr;
   PR_InitializeNetAddr(PR_IpAddrAny, si->listen_port, &server_addr);
   if (PR_Bind(listen_socket, &server_addr) != PR_SUCCESS) {
@@ -603,27 +932,45 @@ int processConfigLine(char* configLine)
   if (*configLine == 0 || *configLine == '#')
     return 0;
 
-  char* keyword = strtok(configLine, ":");
+  char* _caret;
+  char* keyword = strtok2(configLine, ":", &_caret);
 
   // Configure usage of http/ssl tunneling proxy behavior
   if (!strcmp(keyword, "httpproxy"))
   {
-    char* value = strtok(NULL, ":");
+    char* value = strtok2(_caret, ":", &_caret);
     if (!strcmp(value, "1"))
       do_http_proxy = true;
 
     return 0;
   }
 
+  if (!strcmp(keyword, "websocketserver"))
+  {
+    char* ipstring = strtok2(_caret, ":", &_caret);
+    if (PR_StringToNetAddr(ipstring, &websocket_server) != PR_SUCCESS) {
+      fprintf(stderr, "Invalid IP address in proxy config: %s\n", ipstring);
+      return 1;
+    }
+    char* remoteport = strtok2(_caret, ":", &_caret);
+    int port = atoi(remoteport);
+    if (port <= 0) {
+      fprintf(stderr, "Invalid remote port in proxy config: %s\n", remoteport);
+      return 1;
+    }
+    websocket_server.inet.port = PR_htons(port);
+    return 0;
+  }
+
   // Configure the forward address of the target server
   if (!strcmp(keyword, "forward"))
   {
-    char* ipstring = strtok(NULL, ":");
+    char* ipstring = strtok2(_caret, ":", &_caret);
     if (PR_StringToNetAddr(ipstring, &remote_addr) != PR_SUCCESS) {
       fprintf(stderr, "Invalid remote IP address: %s\n", ipstring);
       return 1;
     }
-    char* serverportstring = strtok(NULL, ":");
+    char* serverportstring = strtok2(_caret, ":", &_caret);
     int port = atoi(serverportstring);
     if (port <= 0) {
       fprintf(stderr, "Invalid remote port: %s\n", serverportstring);
@@ -637,16 +984,16 @@ int processConfigLine(char* configLine)
   // Configure all listen sockets and port+certificate bindings
   if (!strcmp(keyword, "listen"))
   {
-    char* hostname = strtok(NULL, ":");
+    char* hostname = strtok2(_caret, ":", &_caret);
     char* hostportstring = NULL;
     if (strcmp(hostname, "*"))
     {
       any_host_spec_config = true;
-      hostportstring = strtok(NULL, ":");
+      hostportstring = strtok2(_caret, ":", &_caret);
     }
 
-    char* serverportstring = strtok(NULL, ":");
-    char* certnick = strtok(NULL, ":");
+    char* serverportstring = strtok2(_caret, ":", &_caret);
+    char* certnick = strtok2(_caret, ":", &_caret);
 
     int port = atoi(serverportstring);
     if (port <= 0) {
@@ -695,9 +1042,9 @@ int processConfigLine(char* configLine)
   
   if (!strcmp(keyword, "clientauth"))
   {
-    char* hostname = strtok(NULL, ":");
-    char* hostportstring = strtok(NULL, ":");
-    char* serverportstring = strtok(NULL, ":");
+    char* hostname = strtok2(_caret, ":", &_caret);
+    char* hostportstring = strtok2(_caret, ":", &_caret);
+    char* serverportstring = strtok2(_caret, ":", &_caret);
 
     int port = atoi(serverportstring);
     if (port <= 0) {
@@ -707,7 +1054,7 @@ int processConfigLine(char* configLine)
 
     if (server_info_t* existingServer = findServerInfo(port))
     {
-      char* authoptionstring = strtok(NULL, ":");
+      char* authoptionstring = strtok2(_caret, ":", &_caret);
       client_auth_option* authoption = new client_auth_option;
       if (!authoption) {
         fprintf(stderr, "Out of memory");
@@ -756,7 +1103,7 @@ int processConfigLine(char* configLine)
   // Configure the NSS certificate database directory
   if (!strcmp(keyword, "certdbdir"))
   {
-    nssconfigdir = strtok(NULL, "\n");
+    nssconfigdir = strtok2(_caret, "\n", &_caret);
     return 0;
   }
 
@@ -828,6 +1175,8 @@ int main(int argc, char** argv)
   else
     configFilePath = argv[1];
 
+  memset(&websocket_server, 0, sizeof(PRNetAddr));
+
   if (parseConfigFile(configFilePath)) {
     fprintf(stderr, "Error: config file \"%s\" missing or formating incorrect\n"
       "Specify path to the config file as parameter to ssltunnel or \n"
@@ -854,7 +1203,10 @@ int main(int argc, char** argv)
       "       # in httpproxy mode and only after the 'listen' option has been\n"
       "       # specified. You also have to specify the tunnel listen port.\n"
       "       clientauth:requesting-client-cert.host.com:443:4443:request\n"
-      "       clientauth:requiring-client-cert.host.com:443:4443:require\n",
+      "       clientauth:requiring-client-cert.host.com:443:4443:require\n"
+      "       # Proxy WebSocket traffic to the server at 127.0.0.1:9999,\n"
+      "       # instead of the server specified in the 'forward' option.\n"
+      "       websocketserver:127.0.0.1:9999\n",
       configFilePath);
     return 1;
   }
