@@ -49,9 +49,10 @@
 #include "nsInterfaceHashtable.h"
 #include "nsDataHashtable.h"
 #include "nsCycleCollectionParticipant.h"
+#include "mozilla/storage.h"
+#include "Helpers.h"
 
 class nsNavHistory;
-class nsIWritablePropertyBag;
 class nsNavHistoryQuery;
 class nsNavHistoryQueryOptions;
 
@@ -104,22 +105,13 @@ private:
   NS_IMETHOD OnClearHistory();                                          \
   NS_IMETHOD OnPageChanged(nsIURI *aURI, PRUint32 aWhat,                \
                            const nsAString &aValue);                    \
-  NS_IMETHOD OnPageExpired(nsIURI* aURI, PRTime aVisitTime,             \
-                           PRBool aWholeEntry);
-
-#define NS_DECL_EXTENDED_BOOKMARK_OBSERVER                              \
-  NS_IMETHOD OnItemAdded(PRInt64 aItemId, PRInt64 aFolder,              \
-                         PRInt32 aIndex, PRUint16 aItemType);
+  NS_IMETHOD OnDeleteVisits(nsIURI* aURI, PRTime aVisitTime);
 
 // nsNavHistoryResult
 //
 //    nsNavHistory creates this object and fills in mChildren (by getting
 //    it through GetTopLevel()). Then FilledAllResults() is called to finish
 //    object initialization.
-//
-//    This object implements nsITreeView so you can just set it to a tree
-//    view and it will work. This object also observes the necessary history
-//    and bookmark events to keep itself up-to-date.
 
 #define NS_NAVHISTORYRESULT_IID \
   { 0x455d1d40, 0x1b9b, 0x40e6, { 0xa6, 0x41, 0x8b, 0xb7, 0xe8, 0x82, 0x23, 0x87 } }
@@ -136,14 +128,8 @@ public:
                                    nsNavHistoryContainerResultNode* aRoot,
                                    nsNavHistoryResult** result);
 
-  // the tree viewer can go faster if it can bypass XPCOM
-  friend class nsNavHistoryResultTreeViewer;
-
   NS_DECLARE_STATIC_IID_ACCESSOR(NS_NAVHISTORYRESULT_IID)
 
-  nsresult PropertyBagFor(nsISupports* aObject,
-                          nsIWritablePropertyBag** aBag);
-
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_NSINAVHISTORYRESULT
   NS_DECL_BOOKMARK_HISTORY_OBSERVER
@@ -157,10 +143,6 @@ public:
   void RemoveAllBookmarksObserver(nsNavHistoryQueryResultNode* aNode);
   void StopObserving();
 
-  // returns the view. NOT-ADDREFED. May be NULL if there is no view
-  nsINavHistoryResultViewer* GetView() const
-    { return mView; }
-
 public:
   // two-stage init, use NewHistoryResult to construct
   nsNavHistoryResult(nsNavHistoryContainerResultNode* mRoot);
@@ -185,11 +167,6 @@ public:
   // The sorting annotation to be used for in SORT_BY_ANNOTATION_* modes
   nsCString mSortingAnnotation;
 
-  nsCOMPtr<nsINavHistoryResultViewer> mView;
-
-  // property bags for all result nodes, see PropertyBagFor
-  nsInterfaceHashtable<nsISupportsHashKey, nsIWritablePropertyBag> mPropertyBags;
-
   // node observers
   PRBool mIsHistoryObserver;
   PRBool mIsBookmarkFolderObserver;
@@ -209,6 +186,9 @@ public:
   void InvalidateTree();
   
   PRBool mBatchInProgress;
+
+  nsMaybeWeakPtrArray<nsINavHistoryResultObserver> mObservers;
+  PRBool mSuppressNotifications;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsNavHistoryResult, NS_NAVHISTORYRESULT_IID)
@@ -235,10 +215,6 @@ NS_DEFINE_STATIC_IID_ACCESSOR(nsNavHistoryResult, NS_NAVHISTORYRESULT_IID)
     { *aTime = mTime; return NS_OK; } \
   NS_IMETHOD GetIndentLevel(PRInt32* aIndentLevel) \
     { *aIndentLevel = mIndentLevel; return NS_OK; } \
-  NS_IMETHOD GetViewIndex(PRInt32* aViewIndex) \
-    { *aViewIndex = mViewIndex; return NS_OK; } \
-  NS_IMETHOD SetViewIndex(PRInt32 aViewIndex) \
-    { mViewIndex = aViewIndex; return NS_OK; } \
   NS_IMETHOD GetBookmarkIndex(PRInt32* aIndex) \
     { *aIndex = mBookmarkIndex; return NS_OK; } \
   NS_IMETHOD GetDateAdded(PRTime* aDateAdded) \
@@ -262,16 +238,14 @@ NS_DEFINE_STATIC_IID_ACCESSOR(nsNavHistoryResult, NS_NAVHISTORYRESULT_IID)
 // buffer.)
 #define NS_FORWARD_COMMON_RESULTNODE_TO_BASE_NO_GETITEMMID \
   NS_IMPLEMENT_SIMPLE_RESULTNODE_NO_GETITEMMID \
-  NS_IMETHOD GetIcon(nsIURI** aIcon) \
+  NS_IMETHOD GetIcon(nsACString& aIcon) \
     { return nsNavHistoryResultNode::GetIcon(aIcon); } \
   NS_IMETHOD GetParent(nsINavHistoryContainerResultNode** aParent) \
     { return nsNavHistoryResultNode::GetParent(aParent); } \
   NS_IMETHOD GetParentResult(nsINavHistoryResult** aResult) \
     { return nsNavHistoryResultNode::GetParentResult(aResult); } \
-  NS_IMETHOD GetPropertyBag(nsIWritablePropertyBag** aBag) \
-    { return nsNavHistoryResultNode::GetPropertyBag(aBag); } \
   NS_IMETHOD GetTags(nsAString& aTags) \
-    { return nsNavHistoryResultNode::GetTags(aTags); } \
+    { return nsNavHistoryResultNode::GetTags(aTags); }
 
 #define NS_FORWARD_COMMON_RESULTNODE_TO_BASE \
   NS_FORWARD_COMMON_RESULTNODE_TO_BASE_NO_GETITEMMID \
@@ -292,10 +266,9 @@ public:
   NS_DECL_CYCLE_COLLECTION_CLASS(nsNavHistoryResultNode)
 
   NS_IMPLEMENT_SIMPLE_RESULTNODE
-  NS_IMETHOD GetIcon(nsIURI** aIcon);
+  NS_IMETHOD GetIcon(nsACString& aIcon);
   NS_IMETHOD GetParent(nsINavHistoryContainerResultNode** aParent);
   NS_IMETHOD GetParentResult(nsINavHistoryResult** aResult);
-  NS_IMETHOD GetPropertyBag(nsIWritablePropertyBag** aBag);
   NS_IMETHOD GetType(PRUint32* type)
     { *type = nsNavHistoryResultNode::RESULT_TYPE_URI; return NS_OK; }
   NS_IMETHOD GetUri(nsACString& aURI)
@@ -309,7 +282,9 @@ public:
   NS_IMETHOD OnItemChanged(PRInt64 aItemId,
                            const nsACString &aProperty,
                            PRBool aIsAnnotationProperty,
-                           const nsACString &aValue);
+                           const nsACString &aValue,
+                           PRTime aNewLastModified,
+                           PRUint16 aItemType);
 
 public:
 
@@ -402,20 +377,13 @@ public:
   nsCString mFaviconURI;
   PRInt32 mBookmarkIndex;
   PRInt64 mItemId;
+  PRInt64 mFolderId;
   PRTime mDateAdded;
   PRTime mLastModified;
 
   // The indent level of this node. The root node will have a value of -1.  The
   // root's children will have a value of 0, and so on.
   PRInt32 mIndentLevel;
-
-  // Value used by the view for whatever it wants. For the built-in tree view,
-  // this is the index into the result's mVisibleElements list of this element.
-  // This is -1 if it is invalid. For items, >= 0 can be used to determine if
-  // the node is visible in the list or not. For folders, call IsVisible, since
-  // they can be the root node which is not itself visible, but its children
-  // are.
-  PRInt32 mViewIndex;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsNavHistoryResultNode, NS_NAVHISTORYRESULTNODE_IID)
@@ -490,6 +458,8 @@ public:
 // derived classes each provide their own implementation of has children and
 // forward the rest to us using this macro
 #define NS_FORWARD_CONTAINERNODE_EXCEPT_HASCHILDREN_AND_READONLY \
+  NS_IMETHOD GetState(PRUint16* _state) \
+    { return nsNavHistoryContainerResultNode::GetState(_state); } \
   NS_IMETHOD GetContainerOpen(PRBool *aContainerOpen) \
     { return nsNavHistoryContainerResultNode::GetContainerOpen(aContainerOpen); } \
   NS_IMETHOD SetContainerOpen(PRBool aContainerOpen) \
@@ -498,6 +468,13 @@ public:
     { return nsNavHistoryContainerResultNode::GetChildCount(aChildCount); } \
   NS_IMETHOD GetChild(PRUint32 index, nsINavHistoryResultNode **_retval) \
     { return nsNavHistoryContainerResultNode::GetChild(index, _retval); } \
+  NS_IMETHOD GetChildIndex(nsINavHistoryResultNode* aNode, PRUint32* _retval) \
+    { return nsNavHistoryContainerResultNode::GetChildIndex(aNode, _retval); } \
+  NS_IMETHOD FindNodeByDetails(const nsACString& aURIString, PRTime aTime, \
+                               PRInt64 aItemId, PRBool aRecursive, \
+                               nsINavHistoryResultNode** _retval) \
+    { return nsNavHistoryContainerResultNode::FindNodeByDetails(aURIString, aTime, aItemId, \
+                                                                aRecursive, _retval); } \
   NS_IMETHOD GetDynamicContainerType(nsACString& aDynamicContainerType) \
     { return nsNavHistoryContainerResultNode::GetDynamicContainerType(aDynamicContainerType); } \
   NS_IMETHOD AppendURINode(const nsACString& aURI, const nsACString& aTitle, PRUint32 aAccessCount, PRTime aTime, const nsACString& aIconURI, nsINavHistoryResultNode **_retval) \
@@ -555,25 +532,27 @@ public:
 
   PRBool AreChildrenVisible();
 
-  // overridded by descendents to populate
+  // Overridded by descendents to populate.
   virtual nsresult OpenContainer();
-  nsresult CloseContainer(PRBool aUpdateView = PR_TRUE);
+  nsresult CloseContainer(PRBool aSuppressNotifications = PR_FALSE);
 
-  // this points to the result that owns this container. All containers have
+  virtual nsresult OpenContainerAsync();
+
+  // This points to the result that owns this container. All containers have
   // their result pointer set so we can quickly get to the result without having
   // to walk the tree. Yet, this also saves us from storing a million pointers
   // for every leaf node to the result.
   nsRefPtr<nsNavHistoryResult> mResult;
 
-  // for example, RESULT_TYPE_QUERY. Query and Folder results override GetType
+  // For example, RESULT_TYPE_QUERY. Query and Folder results override GetType
   // so this is not used, but is still kept in sync.
   PRUint32 mContainerType;
 
-  // when there are children, this stores the open state in the tree
-  // this is set to the default in the constructor
+  // When there are children, this stores the open state in the tree
+  // this is set to the default in the constructor.
   PRBool mExpanded;
 
-  // Filled in by the result type generator in nsNavHistory
+  // Filled in by the result type generator in nsNavHistory.
   nsCOMArray<nsNavHistoryResultNode> mChildren;
 
   PRBool mChildrenReadOnly;
@@ -584,9 +563,9 @@ public:
   nsCString mDynamicContainerType;
 
   void FillStats();
-  void ReverseUpdateStats(PRInt32 aAccessCountChange);
+  nsresult ReverseUpdateStats(PRInt32 aAccessCountChange);
 
-  // sorting
+  // Sorting methods.
   typedef nsCOMArray<nsNavHistoryResultNode>::nsCOMArrayComparatorFunc SortComparator;
   virtual PRUint16 GetSortType();
   virtual void GetSortingAnnotation(nsACString& aSortingAnnotation);
@@ -650,8 +629,6 @@ public:
   }
   nsNavHistoryResultNode* FindChildURI(const nsACString& aSpec,
                                        PRUint32* aNodeIndex);
-  nsNavHistoryFolderResultNode* FindChildFolder(PRInt64 aFolderId,
-                                                PRUint32* aNodeIndex);
   nsNavHistoryContainerResultNode* FindChildContainerByName(const nsACString& aTitle,
                                                             PRUint32* aNodeIndex);
   // returns the index of the given node, -1 if not found
@@ -672,12 +649,24 @@ public:
                          nsNavHistoryContainerResultNode* aContainer,
                          const nsCString& aSpec,
                          nsCOMArray<nsNavHistoryResultNode>* aMatches);
-  void UpdateURIs(PRBool aRecursive, PRBool aOnlyOne, PRBool aUpdateSort,
-                  const nsCString& aSpec,
-                  void (*aCallback)(nsNavHistoryResultNode*,void*),
-                  void* aClosure);
+  nsresult UpdateURIs(PRBool aRecursive, PRBool aOnlyOne, PRBool aUpdateSort,
+                      const nsCString& aSpec,
+                      nsresult (*aCallback)(nsNavHistoryResultNode*,void*, nsNavHistoryResult*),
+                      void* aClosure);
   nsresult ChangeTitles(nsIURI* aURI, const nsACString& aNewTitle,
                         PRBool aRecursive, PRBool aOnlyOne);
+
+protected:
+
+  enum AsyncCanceledState {
+    NOT_CANCELED, CANCELED, CANCELED_RESTART_NEEDED
+  };
+
+  void CancelAsyncOpen(PRBool aRestart);
+  nsresult NotifyOnStateChange(PRUint16 aOldState);
+
+  nsCOMPtr<mozIStoragePendingStatement> mAsyncPendingStmt;
+  PRBool mAsyncCanceledState;
 };
 
 NS_DEFINE_STATIC_IID_ACCESSOR(nsNavHistoryContainerResultNode,
@@ -725,7 +714,6 @@ public:
   virtual nsresult OpenContainer();
 
   NS_DECL_BOOKMARK_HISTORY_OBSERVER
-  NS_DECL_EXTENDED_BOOKMARK_OBSERVER
   virtual void OnRemoving();
 
 public:
@@ -766,7 +754,8 @@ public:
 //    of the folder in sync with the bookmark service.
 
 class nsNavHistoryFolderResultNode : public nsNavHistoryContainerResultNode,
-                                     public nsINavHistoryQueryResultNode
+                                     public nsINavHistoryQueryResultNode,
+                                     public mozilla::places::AsyncStatementCallback
 {
 public:
   nsNavHistoryFolderResultNode(const nsACString& aTitle,
@@ -795,14 +784,15 @@ public:
 
   virtual nsresult OpenContainer();
 
+  virtual nsresult OpenContainerAsync();
+  NS_DECL_ASYNCSTATEMENTCALLBACK
+
   // This object implements a bookmark observer interface without deriving from
   // the bookmark observers. This is called from the result's actual observer
   // and it knows all observers are FolderResultNodes
   NS_DECL_NSINAVBOOKMARKOBSERVER
-  NS_DECL_EXTENDED_BOOKMARK_OBSERVER
 
   virtual void OnRemoving();
-public:
 
   // this indicates whether the folder contents are valid, they don't go away
   // after the container is closed until a notification comes in
@@ -824,7 +814,12 @@ public:
 
 private:
 
+  nsresult OnChildrenFilled();
+  void EnsureRegisteredAsFolderObserver();
+  nsresult FillChildrenAsync();
+
   PRBool mIsRegisteredFolderObserver;
+  PRInt32 mAsyncBookmarkIndex;
 };
 
 // nsNavHistorySeparatorResultNode
