@@ -42,7 +42,7 @@
 #include "nsCOMPtr.h"
 #include "nsMemory.h"
 #include "nsIServiceManager.h"
-#include "nsIGenericFactory.h"
+#include "mozilla/ModuleUtils.h"
 #include "nsIWebBrowserChrome.h"
 #include "nsCURILoader.h"
 #include "nsNetUtil.h"
@@ -70,8 +70,6 @@
 #include "nsIDOMNSHTMLDocument.h"
 #include "nsIDOMHTMLElement.h"
 #include "nsIEventStateManager.h"
-#include "nsIViewManager.h"
-#include "nsIScrollableView.h"
 #include "nsIDocument.h"
 #include "nsISelection.h"
 #include "nsISelectElement.h"
@@ -713,7 +711,7 @@ nsTypeAheadFind::GetSearchContainers(nsISupports *aContainer,
   }
 
   if (!rootContent)
-    rootContent = doc->GetRootContent();
+    rootContent = doc->GetRootElement();
  
   nsCOMPtr<nsIDOMNode> rootNode(do_QueryInterface(rootContent));
 
@@ -799,7 +797,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
   }
   origContent = startContent;
 
-  if (startContent->IsNodeOfType(nsINode::eELEMENT)) {
+  if (startContent->IsElement()) {
     nsIContent *childContent = startContent->GetChildAt(startOffset);
     if (childContent) {
       startContent = childContent;
@@ -831,7 +829,7 @@ nsTypeAheadFind::RangeStartsInsideLink(nsIDOMRange *aRange,
     // Keep testing while startContent is equal to something,
     // eventually we'll run out of ancestors
 
-    if (startContent->IsNodeOfType(nsINode::eHTML)) {
+    if (startContent->IsHTML()) {
       nsCOMPtr<nsILink> link(do_QueryInterface(startContent));
       if (link) {
         // Check to see if inside HTML link
@@ -987,9 +985,9 @@ nsTypeAheadFind::Find(const nsAString& aSearchString, PRBool aLinksOnly,
     // If false, we will scan from start of selection
     isFirstVisiblePreferred = !atEnd && !mCaretBrowsingOn && isSelectionCollapsed;
     if (isFirstVisiblePreferred) {
-      // Get focused content from esm. If it's null, the document is focused.
-      // If not, make sure the selection is in sync with the focus, so we can 
-      // start our search from there.
+      // Get the focused content. If there is a focused node, ensure the
+      // selection is at that point. Otherwise, we will just want to start
+      // from the caret position or the beginning of the document.
       nsPresContext* presContext = presShell->GetPresContext();
       NS_ENSURE_TRUE(presContext, NS_OK);
 
@@ -1002,8 +1000,17 @@ nsTypeAheadFind::Find(const nsAString& aSearchString, PRBool aLinksOnly,
 
       nsCOMPtr<nsIFocusManager> fm = do_GetService(FOCUSMANAGER_CONTRACTID);
       if (fm) {
-        fm->MoveCaretToFocus(window);
-        isFirstVisiblePreferred = PR_FALSE;
+        nsCOMPtr<nsIDOMElement> focusedElement;
+        nsCOMPtr<nsIDOMWindow> focusedWindow;
+        fm->GetFocusedElementForWindow(window, PR_FALSE, getter_AddRefs(focusedWindow),
+                                       getter_AddRefs(focusedElement));
+        // If the root element is focused, then it's actually the document
+        // that has the focus, so ignore this.
+        if (focusedElement &&
+            !SameCOMIdentity(focusedElement, document->GetRootElement())) {
+          fm->MoveCaretToFocus(window);
+          isFirstVisiblePreferred = PR_FALSE;
+        }
       }
     }
   }
@@ -1087,7 +1094,7 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   if (!content)
     return PR_FALSE;
 
-  nsIFrame *frame = aPresShell->GetPrimaryFrameFor(content);
+  nsIFrame *frame = content->GetPrimaryFrame();
   if (!frame)    
     return PR_FALSE;  // No frame! Not visible then.
 
@@ -1122,34 +1129,21 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
 
   // Set up the variables we need, return true if we can't get at them all
   const PRUint16 kMinPixels  = 12;
-  PRUint16 minPixels = nsPresContext::CSSPixelsToAppUnits(kMinPixels);
-
-  nsIViewManager* viewManager = aPresShell->GetViewManager();
-  if (!viewManager)
-    return PR_TRUE;
+  nscoord minDistance = nsPresContext::CSSPixelsToAppUnits(kMinPixels);
 
   // Get the bounds of the current frame, relative to the current view.
   // We don't use the more accurate AccGetBounds, because that is
   // more expensive and the STATE_OFFSCREEN flag that this is used
   // for only needs to be a rough indicator
-  nsIView *containingView = nsnull;
-  nsPoint frameOffset;
   nsRectVisibility rectVisibility = nsRectVisibility_kAboveViewport;
 
-  if (!aGetTopVisibleLeaf) {
-    nsRect relFrameRect = frame->GetRect();
-    frame->GetOffsetFromView(frameOffset, &containingView);
-    if (!containingView)      
-      return PR_FALSE;  // no view -- not visible    
+  if (!aGetTopVisibleLeaf && !frame->GetRect().IsEmpty()) {
+    rectVisibility =
+      aPresShell->GetRectVisibility(frame,
+                                    nsRect(nsPoint(0,0), frame->GetSize()),
+                                    minDistance);
 
-    relFrameRect.x = frameOffset.x;
-    relFrameRect.y = frameOffset.y;
-
-    viewManager->GetRectVisibility(containingView, relFrameRect,
-                                   minPixels, &rectVisibility);
-
-    if (rectVisibility != nsRectVisibility_kAboveViewport &&
-        rectVisibility != nsRectVisibility_kZeroAreaRect) {
+    if (rectVisibility != nsRectVisibility_kAboveViewport) {
       return PR_TRUE;
     }
   }
@@ -1171,19 +1165,17 @@ nsTypeAheadFind::IsRangeVisible(nsIPresShell *aPresShell,
   if (!frameTraversal)
     return PR_FALSE;
 
-  while (rectVisibility == nsRectVisibility_kAboveViewport || rectVisibility == nsRectVisibility_kZeroAreaRect) {
+  while (rectVisibility == nsRectVisibility_kAboveViewport) {
     frameTraversal->Next();
     frame = frameTraversal->CurrentItem();
     if (!frame)
       return PR_FALSE;
 
-    nsRect relFrameRect = frame->GetRect();
-    frame->GetOffsetFromView(frameOffset, &containingView);
-    if (containingView) {
-      relFrameRect.x = frameOffset.x;
-      relFrameRect.y = frameOffset.y;
-      viewManager->GetRectVisibility(containingView, relFrameRect,
-                                     minPixels, &rectVisibility);
+    if (!frame->GetRect().IsEmpty()) {
+      rectVisibility =
+        aPresShell->GetRectVisibility(frame,
+                                      nsRect(nsPoint(0,0), frame->GetSize()),
+                                      minDistance);
     }
   }
 
