@@ -47,6 +47,7 @@
 #include "nsIJSContextStack.h"
 #include "nsIDirectoryEnumerator.h"
 #include "nsILocalFile.h"
+#include "nsIObserver.h"
 #include "nsIObserverService.h"
 #include "nsIProfileChangeStatus.h"
 #include "nsISimpleEnumerator.h"
@@ -64,6 +65,8 @@
 #include "nsArrayEnumerator.h"
 #include "nsEnumeratorUtils.h"
 #include "nsReadableUtils.h"
+#include "mozilla/Services.h"
+#include "mozilla/Omnijar.h"
 
 #include <stdlib.h>
 
@@ -100,11 +103,20 @@
 
 #define PREF_OVERRIDE_DIRNAME "preferences"
 
+static already_AddRefed<nsILocalFile>
+CloneAndAppend(nsIFile* aFile, const char* name)
+{
+  nsCOMPtr<nsIFile> file;
+  aFile->Clone(getter_AddRefs(file));
+  nsCOMPtr<nsILocalFile> lfile = do_QueryInterface(file);
+  lfile->AppendNative(nsDependentCString(name));
+  return lfile.forget();
+}
+
 nsXREDirProvider* gDirServiceProvider = nsnull;
 
 nsXREDirProvider::nsXREDirProvider() :
-  mProfileNotified(PR_FALSE),
-  mExtensionsLoaded(PR_FALSE)
+  mProfileNotified(PR_FALSE)
 {
   gDirServiceProvider = this;
 }
@@ -119,6 +131,7 @@ nsXREDirProvider::Initialize(nsIFile *aXULAppDir,
                              nsILocalFile *aGREDir,
                              nsIDirectoryServiceProvider* aAppProvider)
 {
+  NS_ENSURE_ARG(aXULAppDir);
   NS_ENSURE_ARG(aGREDir);
 
   mAppProvider = aAppProvider;
@@ -135,6 +148,7 @@ nsXREDirProvider::Initialize(nsIFile *aXULAppDir,
     }
   }
 
+  LoadAppBundleDirs();
   return NS_OK;
 }
 
@@ -252,10 +266,10 @@ nsXREDirProvider::GetFile(const char* aProperty, PRBool* aPersistent,
     rv = GetUserAppDataDirectory((nsILocalFile**)(nsIFile**) getter_AddRefs(file));
   }
   else if (!strcmp(aProperty, XRE_UPDATE_ROOT_DIR)) {
-#ifdef XP_WIN
+#if defined(XP_WIN) && !defined(WINCE)
     rv = GetUpdateRootDir(getter_AddRefs(file));
 #else
-    // Only supported on Windows, so just immediately fail.
+    // Only supported on Windows other than WINCE, so just immediately fail.
     return NS_ERROR_FAILURE;
 #endif
   }
@@ -330,13 +344,7 @@ nsXREDirProvider::GetFile(const char* aProperty, PRBool* aPersistent,
   else if (NS_SUCCEEDED(GetProfileStartupDir(getter_AddRefs(file)))) {
     // We need to allow component, xpt, and chrome registration to
     // occur prior to the profile-after-change notification.
-    if (!strcmp(aProperty, NS_XPCOM_COMPONENT_REGISTRY_FILE)) {
-      rv = file->AppendNative(NS_LITERAL_CSTRING("compreg.dat"));
-    }
-    else if (!strcmp(aProperty, NS_XPCOM_XPTI_REGISTRY_FILE)) {
-      rv = file->AppendNative(NS_LITERAL_CSTRING("xpti.dat"));
-    }
-    else if (!strcmp(aProperty, NS_APP_USER_CHROME_DIR)) {
+    if (!strcmp(aProperty, NS_APP_USER_CHROME_DIR)) {
       rv = file->AppendNative(NS_LITERAL_CSTRING("chrome"));
     }
   }
@@ -409,37 +417,6 @@ nsXREDirProvider::GetFile(const char* aProperty, PRBool* aPersistent,
   return NS_OK;
 }
 
-static void
-LoadPlatformDirectory(nsIFile* aBundleDirectory,
-                      nsCOMArray<nsIFile> &aDirectories)
-{
-  nsCOMPtr<nsIFile> platformDir;
-  nsresult rv = aBundleDirectory->Clone(getter_AddRefs(platformDir));
-  if (NS_FAILED(rv))
-    return;
-
-  platformDir->AppendNative(NS_LITERAL_CSTRING("platform"));
-
-#ifdef TARGET_OS_ABI
-  nsCOMPtr<nsIFile> platformABIDir;
-  rv = platformDir->Clone(getter_AddRefs(platformABIDir));
-  if (NS_FAILED(rv))
-    return;
-#endif
-
-  platformDir->AppendNative(NS_LITERAL_CSTRING(OS_TARGET));
-
-  PRBool exists;
-  if (NS_SUCCEEDED(platformDir->Exists(&exists)) && exists)
-    aDirectories.AppendObject(platformDir);
-
-#ifdef TARGET_OS_ABI
-  platformABIDir->AppendNative(NS_LITERAL_CSTRING(TARGET_OS_ABI));
-  if (NS_SUCCEEDED(platformABIDir->Exists(&exists)) && exists)
-    aDirectories.AppendObject(platformABIDir);
-#endif
-}
-
 static void
 LoadAppDirIntoArray(nsIFile* aXULAppDir,
                     const char *const *aAppendList,
@@ -525,7 +502,8 @@ nsXREDirProvider::GetFiles(const char* aProperty, nsISimpleEnumerator** aResult)
 static void
 LoadExtensionDirectories(nsINIParser &parser,
                          const char *aSection,
-                         nsCOMArray<nsIFile> &aDirectories)
+                         nsCOMArray<nsIFile> &aDirectories,
+                         NSLocationType aType)
 {
   nsresult rv;
   PRInt32 i = 0;
@@ -547,26 +525,17 @@ LoadExtensionDirectories(nsINIParser &parser,
       continue;
 
     aDirectories.AppendObject(dir);
-    LoadPlatformDirectory(dir, aDirectories);
+
+    nsCOMPtr<nsILocalFile> manifest =
+      CloneAndAppend(dir, "chrome.manifest");
+    XRE_AddManifestLocation(aType, manifest);
   }
   while (PR_TRUE);
 }
 
 void
-nsXREDirProvider::LoadBundleDirectories()
+nsXREDirProvider::LoadExtensionBundleDirectories()
 {
-  if (mExtensionsLoaded)
-    return;
-
-  mExtensionsLoaded = PR_TRUE;
-
-  // first load distribution/bundles
-  if (mXULAppDir) {
-    LoadPlatformDirectory(mXULAppDir, mAppBundleDirectories);
-
-    LoadAppBundleDirs();
-  }
-
   if (mProfileDir && !gSafeMode) {
     nsCOMPtr<nsIFile> extensionsINI;
     mProfileDir->Clone(getter_AddRefs(extensionsINI));
@@ -585,17 +554,16 @@ nsXREDirProvider::LoadBundleDirectories()
     if (NS_FAILED(rv))
       return;
 
-    LoadExtensionDirectories(parser, "ExtensionDirs", mExtensionDirectories);
-    LoadExtensionDirectories(parser, "ThemeDirs", mThemeDirectories);
+    LoadExtensionDirectories(parser, "ExtensionDirs", mExtensionDirectories,
+                             NS_COMPONENT_LOCATION);
+    LoadExtensionDirectories(parser, "ThemeDirs", mThemeDirectories,
+                             NS_SKIN_LOCATION);
   }
 }
 
 void
 nsXREDirProvider::LoadAppBundleDirs()
 {
-  if (!mXULAppDir)
-    return;
-
   nsCOMPtr<nsIFile> dir;
   nsresult rv = mXULAppDir->Clone(getter_AddRefs(dir));
   if (NS_FAILED(rv))
@@ -604,10 +572,6 @@ nsXREDirProvider::LoadAppBundleDirs()
   dir->AppendNative(NS_LITERAL_CSTRING("distribution"));
   dir->AppendNative(NS_LITERAL_CSTRING("bundles"));
 
-  PRBool exists;
-  if (NS_FAILED(dir->Exists(&exists)) || !exists)
-    return;
-
   nsCOMPtr<nsISimpleEnumerator> e;
   rv = dir->GetDirectoryEntries(getter_AddRefs(e));
   if (NS_FAILED(rv))
@@ -620,7 +584,10 @@ nsXREDirProvider::LoadAppBundleDirs()
   nsCOMPtr<nsIFile> subdir;
   while (NS_SUCCEEDED(files->GetNextFile(getter_AddRefs(subdir))) && subdir) {
     mAppBundleDirectories.AppendObject(subdir);
-    LoadPlatformDirectory(subdir, mAppBundleDirectories);
+
+    nsCOMPtr<nsILocalFile> manifest =
+      CloneAndAppend(subdir, "chrome.manifest");
+    XRE_AddManifestLocation(NS_COMPONENT_LOCATION, manifest);
   }
 }
 
@@ -653,7 +620,6 @@ nsXREDirProvider::GetFilesInternal(const char* aProperty,
 
     static const char *const kAppendNothing[] = { nsnull };
 
-    LoadBundleDirectories();
     LoadDirsIntoArray(mAppBundleDirectories,
                       kAppendNothing, directories);
     LoadDirsIntoArray(mExtensionDirectories,
@@ -661,23 +627,9 @@ nsXREDirProvider::GetFilesInternal(const char* aProperty,
 
     rv = NS_NewArrayEnumerator(aResult, directories);
   }
-  else if (!strcmp(aProperty, NS_XPCOM_COMPONENT_DIR_LIST)) {
-    static const char *const kAppendCompDir[] = { "components", nsnull };
-    nsCOMArray<nsIFile> directories;
-
-    LoadBundleDirectories();
-    LoadDirsIntoArray(mAppBundleDirectories,
-                      kAppendCompDir, directories);
-    LoadDirsIntoArray(mExtensionDirectories,
-                      kAppendCompDir, directories);
-
-    rv = NS_NewArrayEnumerator(aResult, directories);
-  }
   else if (!strcmp(aProperty, NS_APP_PREFS_DEFAULTS_DIR_LIST)) {
     nsCOMArray<nsIFile> directories;
 
-    LoadBundleDirectories();
-
     LoadAppDirIntoArray(mXULAppDir, kAppendPrefDir, directories);
     LoadDirsIntoArray(mAppBundleDirectories,
                       kAppendPrefDir, directories);
@@ -687,7 +639,6 @@ nsXREDirProvider::GetFilesInternal(const char* aProperty,
   else if (!strcmp(aProperty, NS_EXT_PREFS_DEFAULTS_DIR_LIST)) {
     nsCOMArray<nsIFile> directories;
 
-    LoadBundleDirectories();
     LoadDirsIntoArray(mExtensionDirectories,
                       kAppendPrefDir, directories);
 
@@ -703,49 +654,12 @@ nsXREDirProvider::GetFilesInternal(const char* aProperty,
 
     rv = NS_NewArrayEnumerator(aResult, directories);
   }
-  else if (!strcmp(aProperty, NS_CHROME_MANIFESTS_FILE_LIST)) {
-    nsCOMArray<nsIFile> manifests;
-
-    nsCOMPtr<nsIFile> manifest;
-    mGREDir->Clone(getter_AddRefs(manifest));
-    manifest->AppendNative(NS_LITERAL_CSTRING("chrome"));
-    manifests.AppendObject(manifest);
-
-    if (mXULAppDir) {
-      nsCOMPtr<nsIFile> file;
-      mXULAppDir->Clone(getter_AddRefs(file));
-      file->AppendNative(NS_LITERAL_CSTRING("chrome"));
-      PRBool exists;
-      if (NS_SUCCEEDED(file->Exists(&exists)) && exists)
-        manifests.AppendObject(file);
-    }
-
-    LoadBundleDirectories();
-    LoadDirsIntoArray(mAppBundleDirectories,
-                      kAppendChromeManifests,
-                      manifests);
-    LoadDirsIntoArray(mExtensionDirectories,
-                      kAppendChromeManifests,
-                      manifests);
-
-    rv = NS_NewArrayEnumerator(aResult, manifests);
-  }
-  else if (!strcmp(aProperty, NS_SKIN_MANIFESTS_FILE_LIST)) {
-    nsCOMArray<nsIFile> manifests;
-
-    LoadBundleDirectories();
-    LoadDirsIntoArray(mThemeDirectories,
-                      kAppendChromeManifests, manifests);
-
-    rv = NS_NewArrayEnumerator(aResult, manifests);
-  }
   else if (!strcmp(aProperty, NS_APP_CHROME_DIR_LIST)) {
     // NS_APP_CHROME_DIR_LIST is only used to get default (native) icons
     // for OS window decoration.
 
     static const char *const kAppendChromeDir[] = { "chrome", nsnull };
     nsCOMArray<nsIFile> directories;
-    LoadBundleDirectories();
     LoadAppDirIntoArray(mXULAppDir,
                         kAppendChromeDir,
                         directories);
@@ -764,7 +678,6 @@ nsXREDirProvider::GetFilesInternal(const char* aProperty,
 
     // The root dirserviceprovider does quite a bit for us: we're mainly
     // interested in xulapp and extension-provided plugins.
-    LoadBundleDirectories();
     LoadDirsIntoArray(mAppBundleDirectories,
                       kAppendPlugins,
                       directories);
@@ -803,20 +716,33 @@ NS_IMETHODIMP
 nsXREDirProvider::DoStartup()
 {
   if (!mProfileNotified) {
-    nsCOMPtr<nsIObserverService> obsSvc
-      (do_GetService("@mozilla.org/observer-service;1"));
+    nsCOMPtr<nsIObserverService> obsSvc =
+      mozilla::services::GetObserverService();
     if (!obsSvc) return NS_ERROR_FAILURE;
 
     mProfileNotified = PR_TRUE;
 
     static const PRUnichar kStartup[] = {'s','t','a','r','t','u','p','\0'};
     obsSvc->NotifyObservers(nsnull, "profile-do-change", kStartup);
+    // Init the Extension Manager
+    nsCOMPtr<nsIObserver> em = do_GetService("@mozilla.org/addons/integration;1");
+    if (em) {
+      em->Observe(nsnull, "addons-startup", nsnull);
+    } else {
+      NS_WARNING("Failed to create Addons Manager.");
+    }
+
+    LoadExtensionBundleDirectories();
+
+    obsSvc->NotifyObservers(nsnull, "load-extension-defaults", nsnull);
     obsSvc->NotifyObservers(nsnull, "profile-after-change", kStartup);
 
     // Any component that has registered for the profile-after-change category
     // should also be created at this time.
     (void)NS_CreateServicesFromCategory("profile-after-change", nsnull,
                                         "profile-after-change");
+
+    obsSvc->NotifyObservers(nsnull, "profile-initial-state", nsnull);
   }
   return NS_OK;
 }
@@ -851,15 +777,15 @@ void
 nsXREDirProvider::DoShutdown()
 {
   if (mProfileNotified) {
-    nsCOMPtr<nsIObserverService> obssvc
-      (do_GetService("@mozilla.org/observer-service;1"));
-    NS_ASSERTION(obssvc, "No observer service?");
-    if (obssvc) {
+    nsCOMPtr<nsIObserverService> obsSvc =
+      mozilla::services::GetObserverService();
+    NS_ASSERTION(obsSvc, "No observer service?");
+    if (obsSvc) {
       nsCOMPtr<nsIProfileChangeStatus> cs = new ProfileChangeStatusImpl();
       static const PRUnichar kShutdownPersist[] =
         {'s','h','u','t','d','o','w','n','-','p','e','r','s','i','s','t','\0'};
-      obssvc->NotifyObservers(cs, "profile-change-net-teardown", kShutdownPersist);
-      obssvc->NotifyObservers(cs, "profile-change-teardown", kShutdownPersist);
+      obsSvc->NotifyObservers(cs, "profile-change-net-teardown", kShutdownPersist);
+      obsSvc->NotifyObservers(cs, "profile-change-teardown", kShutdownPersist);
 
       // Phase 2c: Now that things are torn down, force JS GC so that things which depend on
       // resources which are about to go away in "profile-before-change" are destroyed first.
@@ -874,7 +800,7 @@ nsXREDirProvider::DoShutdown()
       }
 
       // Phase 3: Notify observers of a profile change
-      obssvc->NotifyObservers(cs, "profile-before-change", kShutdownPersist);
+      obsSvc->NotifyObservers(cs, "profile-before-change", kShutdownPersist);
     }
     mProfileNotified = PR_FALSE;
   }
@@ -927,6 +853,60 @@ GetShellFolderPath(int folder, nsAString& _retval)
   return rv;
 }
 
+#ifndef WINCE
+/**
+ * Provides a fallback for getting the path to APPDATA or LOCALAPPDATA by
+ * querying the registry when the call to SHGetSpecialFolderLocation or
+ * SHGetPathFromIDListW is unable to provide these paths (Bug 513958).
+ */
+static nsresult
+GetRegWindowsAppDataFolder(PRBool aLocal, nsAString& _retval)
+{
+  HKEY key;
+  NS_NAMED_LITERAL_STRING(keyName,
+  "Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders");
+  DWORD res = ::RegOpenKeyExW(HKEY_CURRENT_USER, keyName.get(), 0, KEY_READ,
+                              &key);
+  if (res != ERROR_SUCCESS) {
+    _retval.SetLength(0);
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  DWORD type, size;
+  res = RegQueryValueExW(key, (aLocal ? L"Local AppData" : L"AppData"), NULL,
+                         &type, NULL, &size);
+  // The call to RegQueryValueExW must succeed, the type must be REG_SZ, the
+  // buffer size must not equal 0, and the buffer size be a multiple of 2.
+  if (res != ERROR_SUCCESS || type != REG_SZ || size == 0 || size % 2 != 0) {
+    ::RegCloseKey(key);
+    _retval.SetLength(0);
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  // |size| includes room for the terminating null character
+  DWORD resultLen = size / 2 - 1;
+
+  _retval.SetLength(resultLen);
+  nsAString::iterator begin;
+  _retval.BeginWriting(begin);
+  if (begin.size_forward() != resultLen) {
+    ::RegCloseKey(key);
+    _retval.SetLength(0);
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  res = RegQueryValueExW(key, (aLocal ? L"Local AppData" : L"AppData"), NULL,
+                         NULL, (LPBYTE) begin.get(), &size);
+  ::RegCloseKey(key);
+  if (res != ERROR_SUCCESS) {
+    _retval.SetLength(0);
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  return NS_OK;
+}
+#endif
+
 nsresult
 nsXREDirProvider::GetUpdateRootDir(nsIFile* *aResult)
 {
@@ -1059,10 +1039,23 @@ nsXREDirProvider::GetUserDataDirectoryHome(nsILocalFile** aFile, PRBool aLocal)
   localDir = do_QueryInterface(dirFileMac, &rv);
 #elif defined(XP_WIN)
   nsString path;
-  if (aLocal)
+  if (aLocal) {
     rv = GetShellFolderPath(CSIDL_LOCAL_APPDATA, path);
-  if (!aLocal || NS_FAILED(rv))
+#ifndef WINCE
+    if (NS_FAILED(rv))
+      rv = GetRegWindowsAppDataFolder(aLocal, path);
+#endif
+  }
+  if (!aLocal || NS_FAILED(rv)) {
     rv = GetShellFolderPath(CSIDL_APPDATA, path);
+#ifndef WINCE
+    if (NS_FAILED(rv)) {
+      if (!aLocal)
+        rv = GetRegWindowsAppDataFolder(aLocal, path);
+    }
+#endif
+
+  }
   NS_ENSURE_SUCCESS(rv, rv);
 
   rv = NS_NewLocalFile(path, PR_TRUE, getter_AddRefs(localDir));
@@ -1101,6 +1094,13 @@ nsXREDirProvider::GetUserDataDirectoryHome(nsILocalFile** aFile, PRBool aLocal)
 
   rv = NS_NewNativeLocalFile(nsDependentCString(appDir), PR_TRUE,
                              getter_AddRefs(localDir));
+#elif defined(ANDROID)
+  // used for setting the patch to our profile
+  // XXX: investigate putting the profile somewhere else
+  const char* homeDir = "/data/data/org.mozilla." MOZ_APP_NAME;
+
+  rv = NS_NewNativeLocalFile(nsDependentCString(homeDir), PR_TRUE,
+                             getter_AddRefs(localDir));
 #elif defined(XP_UNIX)
   const char* homeDir = getenv("HOME");
   if (!homeDir || !*homeDir)
@@ -1175,9 +1175,6 @@ nsXREDirProvider::GetSystemExtensionsDirectory(nsILocalFile** aFile)
   NS_ENSURE_SUCCESS(rv, rv);
 #endif
 
-  rv = EnsureDirectoryExists(localDir);
-  NS_ENSURE_SUCCESS(rv, rv);
-
   NS_ADDREF(*aFile = localDir);
   return NS_OK;
 }
@@ -1342,6 +1339,12 @@ nsXREDirProvider::AppendProfilePath(nsIFile* aFile)
   }
   NS_ENSURE_SUCCESS(rv, rv);
 
+#elif defined(ANDROID)
+  // The directory used for storing profiles
+  // The parent of this directory is set in GetUserDataDirectoryHome
+  // XXX: handle gAppData->profile properly
+  rv = aFile->AppendNative(nsDependentCString("mozilla"));
+  NS_ENSURE_SUCCESS(rv, rv);
 #elif defined(XP_UNIX)
   // Make it hidden (i.e. using the ".")
   nsCAutoString folder(".");
