@@ -491,6 +491,23 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
         aMetric = !nsUXThemeData::IsAppThemed();
 #else
         aMetric = 0;
+#endif
+        break;
+    case eMetric_TouchEnabled:
+        aMetric = 0;
+#ifndef WINCE
+        PRInt32 touchCapabilities;
+        touchCapabilities = ::GetSystemMetrics(SM_DIGITIZER);
+        if ((touchCapabilities & NID_READY) && 
+           (touchCapabilities & (NID_EXTERNAL_TOUCH | NID_INTEGRATED_TOUCH))) {
+            aMetric = 1;
+        }
+#elif defined(WINCE_WINDOWS_MOBILE)
+        WCHAR platformType[MAX_PATH];
+        SystemParametersInfo(SPI_GETPLATFORMTYPE, sizeof(platformType),
+                             platformType, 0);
+        if (!wcscmp(platformType, L"PocketPC"))
+            aMetric = 1;
 #endif
         break;
     case eMetric_WindowsDefaultTheme:
@@ -510,8 +527,6 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
           if (hresult == S_OK && nsWindow::GetWindowsVersion() <= WIN7_VERSION) {
             LPCWSTR defThemes[] = {
               L"luna.msstyles",
-              L"royale.msstyles",
-              L"zune.msstyles",
               L"aero.msstyles"
             };
 
@@ -534,6 +549,7 @@ NS_IMETHODIMP nsLookAndFeel::GetMetric(const nsMetricID aID, PRInt32 & aMetric)
         }
         break;
     case eMetric_MacGraphiteTheme:
+    case eMetric_MaemoClassic:
         aMetric = 0;
         res = NS_ERROR_NOT_IMPLEMENTED;
         break;
