@@ -46,11 +46,13 @@
 #include "nsAppDirectoryServiceDefs.h"
 #include "nsISimpleEnumerator.h"
 #include "nsIContent.h"
-
-#ifdef DEBUG
 #include "nsIServiceManager.h"
 #include "nsIPrefService.h"
 #include "nsIPrefBranch2.h"
+#include "BasicLayers.h"
+#include "LayerManagerOGL.h"
+
+#ifdef DEBUG
 #include "nsIObserver.h"
 
 static void debug_RegisterPrefCallbacks();
@@ -62,6 +64,8 @@ static PRBool debug_InSecureKeyboardInputMode = PR_FALSE;
 static PRInt32 gNumWidgets;
 #endif
 
+using namespace mozilla::layers;
+
 nsIContent* nsBaseWidget::mLastRollup = nsnull;
 
 // nsBaseWidget
@@ -91,19 +95,22 @@ nsAutoRollup::~nsAutoRollup()
 
 nsBaseWidget::nsBaseWidget()
 : mClientData(nsnull)
+, mViewWrapperPtr(nsnull)
 , mEventCallback(nsnull)
+, mViewCallback(nsnull)
 , mContext(nsnull)
 , mToolkit(nsnull)
-, mEventListener(nsnull)
 , mCursor(eCursor_standard)
 , mWindowType(eWindowType_child)
 , mBorderStyle(eBorderStyle_none)
 , mOnDestroyCalled(PR_FALSE)
+, mUseAcceleratedRendering(PR_FALSE)
 , mBounds(0,0,0,0)
 , mOriginalBounds(nsnull)
 , mClipRectCount(0)
 , mZIndex(0)
 , mSizeMode(nsSizeMode_Normal)
+, mPopupLevel(ePopupLevelTop)
 {
 #ifdef NOISY_WIDGET_LEAKS
   gNumWidgets++;
@@ -123,6 +130,11 @@ nsBaseWidget::nsBaseWidget()
 //-------------------------------------------------------------------------
 nsBaseWidget::~nsBaseWidget()
 {
+  if (mLayerManager &&
+      mLayerManager->GetBackendType() == LayerManager::LAYERS_BASIC) {
+    static_cast<BasicLayerManager*>(mLayerManager.get())->ClearRetainerWidget();
+  }
+
 #ifdef NOISY_WIDGET_LEAKS
   gNumWidgets--;
   printf("WIDGETS- = %d\n", gNumWidgets);
@@ -201,7 +213,9 @@ void nsBaseWidget::BaseCreate(nsIWidget *aParent,
   }
 
   if (nsnull != aInitData) {
-    PreCreateWidget(aInitData);
+    mWindowType = aInitData->mWindowType;
+    mBorderStyle = aInitData->mBorderStyle;
+    mPopupLevel = aInitData->mPopupLevel;
   }
 
   if (aParent) {
@@ -214,11 +228,6 @@ NS_IMETHODIMP nsBaseWidget::CaptureMouse(PRBool aCapture)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsBaseWidget::Validate()
-{
-  return NS_OK;
-}
-
 //-------------------------------------------------------------------------
 //
 // Accessor functions to get/set the client data
@@ -237,6 +246,46 @@ NS_IMETHODIMP nsBaseWidget::SetClientData(void* aClientData)
   return NS_OK;
 }
 
+// Attach a view to our widget which we'll send events to. 
+NS_IMETHODIMP
+nsBaseWidget::AttachViewToTopLevel(EVENT_CALLBACK aViewEventFunction,
+                                   nsIDeviceContext *aContext)
+{
+  NS_ASSERTION((mWindowType == eWindowType_toplevel), "Can't attach to child?");
+
+  mViewCallback = aViewEventFunction;
+
+  if (aContext) {
+    if (mContext) {
+      NS_IF_RELEASE(mContext);
+    }
+    mContext = aContext;
+    NS_ADDREF(mContext);
+  }
+
+  return NS_OK;
+}
+
+ViewWrapper* nsBaseWidget::GetAttachedViewPtr()
+ {
+   return mViewWrapperPtr;
+ }
+ 
+NS_IMETHODIMP nsBaseWidget::SetAttachedViewPtr(ViewWrapper* aViewWrapper)
+ {
+   mViewWrapperPtr = aViewWrapper;
+   return NS_OK;
+ }
+
+NS_METHOD nsBaseWidget::ResizeClient(PRInt32 aX,
+                                     PRInt32 aY,
+                                     PRInt32 aWidth,
+                                     PRInt32 aHeight,
+                                     PRBool aRepaint)
+{
+  return Resize(aX, aY, aWidth, aHeight, aRepaint);
+}
+
 //-------------------------------------------------------------------------
 //
 // Close this nsBaseWidget
@@ -251,8 +300,6 @@ NS_METHOD nsBaseWidget::Destroy()
   if (parent) {
     parent->RemoveChild(this);
   }
-  // disconnect listeners.
-  NS_IF_RELEASE(mEventListener);
 
   return NS_OK;
 }
@@ -284,16 +331,12 @@ nsIWidget* nsBaseWidget::GetParent(void)
 // Get this nsBaseWidget top level widget
 //
 //-------------------------------------------------------------------------
-nsIWidget* nsBaseWidget::GetTopLevelWidget(PRInt32* aLevelsUp)
+nsIWidget* nsBaseWidget::GetTopLevelWidget()
 {
   nsIWidget *topLevelWidget = nsnull, *widget = this;
-  if (aLevelsUp)
-    *aLevelsUp = -1;
   while (widget) {
     topLevelWidget = widget;
     widget = widget->GetParent();
-    if (aLevelsUp)
-      ++*aLevelsUp;
   }
   return topLevelWidget;
 }
@@ -542,12 +585,6 @@ NS_IMETHODIMP nsBaseWidget::GetWindowType(nsWindowType& aWindowType)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsBaseWidget::SetWindowType(nsWindowType aWindowType) 
-{
-  mWindowType = aWindowType;
-  return NS_OK;
-}
-
 //-------------------------------------------------------------------------
 //
 // Window transparency methods
@@ -646,40 +683,61 @@ NS_IMETHODIMP nsBaseWidget::MakeFullScreen(PRBool aFullScreen)
   return NS_OK;
 }
 
-//-------------------------------------------------------------------------
-//
-// Create a rendering context from this nsBaseWidget
-//
-//-------------------------------------------------------------------------
-nsIRenderingContext* nsBaseWidget::GetRenderingContext()
+nsBaseWidget::AutoLayerManagerSetup::AutoLayerManagerSetup(
+    nsBaseWidget* aWidget, gfxContext* aTarget,
+    BasicLayerManager::BufferMode aDoubleBuffering)
+  : mWidget(aWidget)
 {
-  nsresult                      rv;
-  nsCOMPtr<nsIRenderingContext> renderingCtx;
-
-  if (mOnDestroyCalled)
-    return nsnull;
-
-  rv = mContext->CreateRenderingContextInstance(*getter_AddRefs(renderingCtx));
-  if (NS_SUCCEEDED(rv)) {
-    gfxASurface* surface = GetThebesSurface();
-    NS_ENSURE_TRUE(surface, nsnull);
-    rv = renderingCtx->Init(mContext, surface);
-    if (NS_SUCCEEDED(rv)) {
-      nsIRenderingContext *ret = renderingCtx;
-      /* Increment object refcount that the |ret| object is still a valid one
-       * after we leave this function... */
-      NS_ADDREF(ret);
-      return ret;
-    }
-    else {
-      NS_WARNING("GetRenderingContext: nsIRenderingContext::Init() failed.");
-    }  
+  BasicLayerManager* manager =
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager());
+  if (manager) {
+    NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
+      "AutoLayerManagerSetup instantiated for non-basic layer backend!");
+    manager->SetDefaultTarget(aTarget, aDoubleBuffering);
   }
-  else {
-    NS_WARNING("GetRenderingContext: Cannot create RenderingContext.");
-  }  
-  
-  return nsnull;
+}
+
+nsBaseWidget::AutoLayerManagerSetup::~AutoLayerManagerSetup()
+{
+  BasicLayerManager* manager =
+    static_cast<BasicLayerManager*>(mWidget->GetLayerManager());
+  if (manager) {
+    NS_ASSERTION(manager->GetBackendType() == LayerManager::LAYERS_BASIC,
+      "AutoLayerManagerSetup instantiated for non-basic layer backend!");
+    manager->SetDefaultTarget(nsnull, BasicLayerManager::BUFFER_NONE);
+  }
+}
+
+LayerManager* nsBaseWidget::GetLayerManager()
+{
+  if (!mLayerManager) {
+    nsCOMPtr<nsIPrefBranch2> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+
+    PRBool allowAcceleration = PR_TRUE;
+    if (prefs) {
+      prefs->GetBoolPref("mozilla.widget.accelerated-layers",
+                         &allowAcceleration);
+    }
+
+    if (mUseAcceleratedRendering && allowAcceleration) {
+      nsRefPtr<LayerManagerOGL> layerManager =
+        new mozilla::layers::LayerManagerOGL(this);
+      /**
+       * XXX - On several OSes initialization is expected to fail for now.
+       * If we'd get a none-basic layer manager they'd crash. This is ok though
+       * since on those platforms it will fail. Anyone implementing new
+       * platforms on LayerManagerOGL should ensure their widget is able to
+       * deal with it though!
+       */
+      if (layerManager->Initialize()) {
+        mLayerManager = layerManager;
+      }
+    }
+    if (!mLayerManager) {
+      mLayerManager = new BasicLayerManager(this);
+    }
+  }
+  return mLayerManager;
 }
 
 //-------------------------------------------------------------------------
@@ -733,25 +791,11 @@ NS_METHOD nsBaseWidget::SetWindowClass(const nsAString& xulWinType)
   return NS_ERROR_NOT_IMPLEMENTED;
 }
 
-NS_METHOD nsBaseWidget::SetBorderStyle(nsBorderStyle aBorderStyle)
-{
-  mBorderStyle = aBorderStyle;
-  return NS_OK;
-}
-
-
-/**
-* Sets the event listener for a widget
-*
-**/
-NS_METHOD nsBaseWidget::AddEventListener(nsIEventListener * aListener)
-{
-  NS_PRECONDITION(mEventListener == nsnull, "Null event listener");
-  NS_IF_RELEASE(mEventListener);
-  NS_ADDREF(aListener);
-  mEventListener = aListener;
-  return NS_OK;
-}
+//-------------------------------------------------------------------------
+//
+// Bounds
+//
+//-------------------------------------------------------------------------
 
 /**
 * If the implementation of nsWindow supports borders this method MUST be overridden
@@ -782,18 +826,29 @@ NS_METHOD nsBaseWidget::GetScreenBounds(nsIntRect &aRect)
   return GetBounds(aRect);
 }
 
-/**
-* 
-*
-**/
+nsIntPoint nsBaseWidget::GetClientOffset()
+{
+  return nsIntPoint(0, 0);
+}
+
 NS_METHOD nsBaseWidget::SetBounds(const nsIntRect &aRect)
 {
   mBounds = aRect;
 
   return NS_OK;
 }
- 
 
+NS_IMETHODIMP
+nsBaseWidget::GetNonClientMargins(nsIntMargin &margins)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+ 
+NS_IMETHODIMP
+nsBaseWidget::SetNonClientMargins(nsIntMargin &margins)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
 
 NS_METHOD nsBaseWidget::EnableDragDrop(PRBool aEnable)
 {
@@ -854,6 +909,76 @@ nsBaseWidget::ShowsResizeIndicator(nsIntRect* aResizerRect)
   return PR_FALSE;
 }
 
+NS_IMETHODIMP
+nsBaseWidget::SetAcceleratedRendering(PRBool aEnabled)
+{
+  if (mUseAcceleratedRendering == aEnabled) {
+    return NS_OK;
+  }
+  mUseAcceleratedRendering = aEnabled;
+  mLayerManager = NULL;
+  return NS_OK;
+}
+
+PRBool
+nsBaseWidget::GetAcceleratedRendering()
+{
+  return mUseAcceleratedRendering;
+}
+
+NS_METHOD nsBaseWidget::RegisterTouchWindow()
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_METHOD nsBaseWidget::UnregisterTouchWindow()
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
+
+NS_IMETHODIMP
+nsBaseWidget::OverrideSystemMouseScrollSpeed(PRInt32 aOriginalDelta,
+                                             PRBool aIsHorizontal,
+                                             PRInt32 &aOverriddenDelta)
+{
+  aOverriddenDelta = aOriginalDelta;
+
+  nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+  NS_ENSURE_TRUE(prefs, NS_ERROR_FAILURE);
+  nsCOMPtr<nsIPrefBranch> prefBranch;
+  nsresult rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ENSURE_TRUE(prefBranch, NS_ERROR_FAILURE);
+
+  PRBool isOverrideEnabled;
+  const char* kPrefNameOverrideEnabled =
+    "mousewheel.system_scroll_override_on_root_content.enabled";
+  rv = prefBranch->GetBoolPref(kPrefNameOverrideEnabled, &isOverrideEnabled);
+  if (NS_FAILED(rv) || !isOverrideEnabled) {
+    return NS_OK;
+  }
+
+  PRInt32 iFactor;
+  nsCAutoString factorPrefName(
+    "mousewheel.system_scroll_override_on_root_content.");
+  if (aIsHorizontal) {
+    factorPrefName.AppendLiteral("horizontal.");
+  } else {
+    factorPrefName.AppendLiteral("vertical.");
+  }
+  factorPrefName.AppendLiteral("factor");
+  rv = prefBranch->GetIntPref(factorPrefName.get(), &iFactor);
+  // The pref value must be larger than 100, otherwise, we don't override the
+  // delta value.
+  if (NS_FAILED(rv) || iFactor <= 100) {
+    return NS_OK;
+  }
+  double factor = (double)iFactor / 100;
+  aOverriddenDelta = PRInt32(NS_round((double)aOriginalDelta * factor));
+
+  return NS_OK;
+}
+
 
 /**
  * Modifies aFile to point at an icon file with the given name and suffix.  The
@@ -927,6 +1052,12 @@ nsBaseWidget::BeginResizeDrag(nsGUIEvent* aEvent, PRInt32 aHorizontal, PRInt32 a
 {
   return NS_ERROR_NOT_IMPLEMENTED;
 }
+
+NS_IMETHODIMP
+nsBaseWidget::BeginMoveDrag(nsMouseEvent* aEvent)
+{
+  return NS_ERROR_NOT_IMPLEMENTED;
+}
  
 #ifdef DEBUG
 //////////////////////////////////////////////////////////////
@@ -952,7 +1083,6 @@ case _value: eventName.AssignWithConversion(_name) ; break
   switch(aGuiEvent->message)
   {
     _ASSIGN_eventName(NS_BLUR_CONTENT,"NS_BLUR_CONTENT");
-    _ASSIGN_eventName(NS_CONTROL_CHANGE,"NS_CONTROL_CHANGE");
     _ASSIGN_eventName(NS_CREATE,"NS_CREATE");
     _ASSIGN_eventName(NS_DESTROY,"NS_DESTROY");
     _ASSIGN_eventName(NS_DRAGDROP_GESTURE,"NS_DND_GESTURE");
@@ -971,7 +1101,6 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_KEY_DOWN,"NS_KEY_DOWN");
     _ASSIGN_eventName(NS_KEY_PRESS,"NS_KEY_PRESS");
     _ASSIGN_eventName(NS_KEY_UP,"NS_KEY_UP");
-    _ASSIGN_eventName(NS_MENU_SELECTED,"NS_MENU_SELECTED");
     _ASSIGN_eventName(NS_MOUSE_ENTER,"NS_MOUSE_ENTER");
     _ASSIGN_eventName(NS_MOUSE_EXIT,"NS_MOUSE_EXIT");
     _ASSIGN_eventName(NS_MOUSE_BUTTON_DOWN,"NS_MOUSE_BUTTON_DOWN");
@@ -981,8 +1110,10 @@ case _value: eventName.AssignWithConversion(_name) ; break
     _ASSIGN_eventName(NS_MOUSE_MOVE,"NS_MOUSE_MOVE");
     _ASSIGN_eventName(NS_MOVE,"NS_MOVE");
     _ASSIGN_eventName(NS_LOAD,"NS_LOAD");
+    _ASSIGN_eventName(NS_POPSTATE,"NS_POPSTATE");
     _ASSIGN_eventName(NS_PAGE_UNLOAD,"NS_PAGE_UNLOAD");
     _ASSIGN_eventName(NS_HASHCHANGE,"NS_HASHCHANGE");
+    _ASSIGN_eventName(NS_READYSTATECHANGE,"NS_READYSTATECHANGE");
     _ASSIGN_eventName(NS_PAINT,"NS_PAINT");
     _ASSIGN_eventName(NS_XUL_BROADCAST, "NS_XUL_BROADCAST");
     _ASSIGN_eventName(NS_XUL_COMMAND_UPDATE, "NS_XUL_COMMAND_UPDATE");
@@ -1213,20 +1344,6 @@ nsBaseWidget::debug_DumpPaintEvent(FILE *                aFileOut,
           aWidgetName.get(),
           (void *) aWindowID);
   
-  if (aPaintEvent->rect) 
-  {
-    fprintf(aFileOut,
-            "%3d,%-3d %3d,%-3d",
-            aPaintEvent->rect->x, 
-            aPaintEvent->rect->y,
-            aPaintEvent->rect->width, 
-            aPaintEvent->rect->height);
-  }
-  else
-  {
-    fprintf(aFileOut,"none");
-  }
-  
   fprintf(aFileOut,"\n");
 }
 //////////////////////////////////////////////////////////////
