@@ -138,12 +138,14 @@
 #include "prtime.h"
 #include "nsPrintfCString.h"
 #include "nsTArray.h"
+#include "mozilla/FunctionTimer.h"
 #include "nsIObserverService.h"
 #include "nsIConsoleService.h"
 #include "nsServiceManagerUtils.h"
 #include "nsThreadUtils.h"
 #include "nsTPtrArray.h"
 #include "nsTArray.h"
+#include "mozilla/Services.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -152,6 +154,8 @@
 #include <process.h>
 #endif
 
+//#define COLLECT_TIME_DEBUG
+
 #ifdef DEBUG_CC
 #define IF_DEBUG_CC_PARAM(_p) , _p
 #define IF_DEBUG_CC_ONLY_PARAM(_p) _p
@@ -749,10 +753,12 @@ public:
             if (!(PRUword(e->mObject) & PRUword(1))) {
                 // This is a real entry (rather than something on the
                 // free list).
-                nsXPCOMCycleCollectionParticipant *cp;
-                ToParticipant(e->mObject, &cp);
+                if (e->mObject) {
+                    nsXPCOMCycleCollectionParticipant *cp;
+                    ToParticipant(e->mObject, &cp);
 
-                cp->UnmarkPurple(e->mObject);
+                    cp->UnmarkPurple(e->mObject);
+                }
 
                 if (--mCount == 0)
                     break;
@@ -893,7 +899,7 @@ nsPurpleBuffer::SelectPointers(GCGraphBuilder &aBuilder)
             if (!(PRUword(e->mObject) & PRUword(1))) {
                 // This is a real entry (rather than something on the
                 // free list).
-                if (AddPurpleRoot(aBuilder, e->mObject)) {
+                if (!e->mObject || AddPurpleRoot(aBuilder, e->mObject)) {
 #ifdef DEBUG_CC
                     mNormalObjects.RemoveEntry(e->mObject);
 #endif
@@ -926,7 +932,8 @@ nsPurpleBuffer::SelectPointers(GCGraphBuilder &aBuilder)
 struct nsCycleCollectionXPCOMRuntime : 
     public nsCycleCollectionLanguageRuntime 
 {
-    nsresult BeginCycleCollection(nsCycleCollectionTraversalCallback &cb) 
+    nsresult BeginCycleCollection(nsCycleCollectionTraversalCallback &cb,
+                                  bool explainLiveExpectedGarbage)
     {
         return NS_OK;
     }
@@ -938,6 +945,10 @@ struct nsCycleCollectionXPCOMRuntime :
 
     inline nsCycleCollectionParticipant *ToParticipant(void *p);
 
+    void CommenceShutdown()
+    {
+    }
+
 #ifdef DEBUG_CC
     virtual void PrintAllReferencesTo(void *p) {}
 #endif
@@ -949,6 +960,7 @@ struct nsCycleCollector
     PRBool mScanInProgress;
     PRBool mFollowupCollection;
     PRUint32 mCollectedObjects;
+    PRBool mFirstCollection;
 
     nsCycleCollectionLanguageRuntime *mRuntimes[nsIProgrammingLanguage::MAX+1];
     nsCycleCollectionXPCOMRuntime mXPCOMRuntime;
@@ -1015,18 +1027,27 @@ struct nsCycleCollector
 };
 
 
+/**
+ * GraphWalker is templatized over a Visitor class that must provide
+ * the following two methods:
+ *
+ * PRBool ShouldVisitNode(PtrInfo const *pi);
+ * void VisitNode(PtrInfo *pi);
+ */
+template <class Visitor>
 class GraphWalker
 {
 private:
+    Visitor mVisitor;
+
     void DoWalk(nsDeque &aQueue);
 
 public:
     void Walk(PtrInfo *s0);
     void WalkFromRoots(GCGraph &aGraph);
-
-    // Provided by concrete walker subtypes.
-    virtual PRBool ShouldVisitNode(PtrInfo const *pi) = 0;
-    virtual void VisitNode(PtrInfo *pi) = 0;
+    // copy-constructing the visitor should be cheap, and less
+    // indirection than using a reference
+    GraphWalker(const Visitor aVisitor) : mVisitor(aVisitor) {}
 };
 
 
@@ -1118,7 +1139,7 @@ Fault(const char *msg, const void *ptr=nsnull)
     // Report to observers off an event so we don't run JS under GC
     // (which is where we might be right now).
     nsCOMPtr<nsIRunnable> ev = new CCRunnableFaultReport(str);
-    NS_DispatchToCurrentThread(ev);
+    NS_DispatchToMainThread(ev);
 }
 
 #ifdef DEBUG_CC
@@ -1152,7 +1173,15 @@ Fault(const char *msg, PtrInfo *pi)
 }
 #endif
 
-
+static inline void
+AbortIfOffMainThreadIfCheckFast()
+{
+#if defined(XP_WIN) || defined(NS_TLS)
+    if (!NS_IsMainThread()) {
+        NS_RUNTIMEABORT("Main-thread-only object used off the main thread");
+    }
+#endif
+}
 
 static nsISupports *
 canonicalize(nsISupports *in)
@@ -1188,16 +1217,18 @@ nsCycleCollectionXPCOMRuntime::ToParticipant(void *p)
 }
 
 
+template <class Visitor>
 void
-GraphWalker::Walk(PtrInfo *s0)
+GraphWalker<Visitor>::Walk(PtrInfo *s0)
 {
     nsDeque queue;
     queue.Push(s0);
     DoWalk(queue);
 }
 
+template <class Visitor>
 void
-GraphWalker::WalkFromRoots(GCGraph& aGraph)
+GraphWalker<Visitor>::WalkFromRoots(GCGraph& aGraph)
 {
     nsDeque queue;
     NodePool::Enumerator etor(aGraph.mNodes);
@@ -1207,16 +1238,17 @@ GraphWalker::WalkFromRoots(GCGraph& aGraph)
     DoWalk(queue);
 }
 
+template <class Visitor>
 void
-GraphWalker::DoWalk(nsDeque &aQueue)
+GraphWalker<Visitor>::DoWalk(nsDeque &aQueue)
 {
     // Use a aQueue to match the breadth-first traversal used when we
     // built the graph, for hopefully-better locality.
     while (aQueue.GetSize() > 0) {
         PtrInfo *pi = static_cast<PtrInfo*>(aQueue.PopFront());
 
-        if (this->ShouldVisitNode(pi)) {
-            this->VisitNode(pi);
+        if (mVisitor.ShouldVisitNode(pi)) {
+            mVisitor.VisitNode(pi);
             for (EdgePool::Iterator child = pi->mFirstChild,
                                 child_end = pi->mLastChild;
                  child != child_end; ++child) {
@@ -1277,6 +1309,7 @@ public:
     GCGraphBuilder(GCGraph &aGraph,
                    nsCycleCollectionLanguageRuntime **aRuntimes);
     ~GCGraphBuilder();
+    bool Initialized();
 
     PRUint32 Count() const { return mPtrToNodeMap.entryCount; }
 
@@ -1330,6 +1363,12 @@ GCGraphBuilder::~GCGraphBuilder()
         PL_DHashTableFinish(&mPtrToNodeMap);
 }
 
+bool
+GCGraphBuilder::Initialized()
+{
+    return !!mPtrToNodeMap.ops;
+}
+
 PtrInfo*
 GCGraphBuilder::AddNode(void *s, nsCycleCollectionParticipant *aParticipant
                         IF_DEBUG_CC_PARAM(PRUint32 aLangID)
@@ -1571,7 +1610,7 @@ nsPurpleBuffer::NoteAll(GCGraphBuilder &builder)
         for (nsPurpleBufferEntry *e = b->mEntries,
                               *eEnd = e + NS_ARRAY_LENGTH(b->mEntries);
             e != eEnd; ++e) {
-            if (!(PRUword(e->mObject) & PRUword(1))) {
+            if (!(PRUword(e->mObject) & PRUword(1)) && e->mObject) {
                 builder.NoteXPCOMRoot(e->mObject);
             }
         }
@@ -1604,9 +1643,10 @@ nsCycleCollector::MarkRoots(GCGraphBuilder &builder)
 ////////////////////////////////////////////////////////////////////////
 
 
-struct ScanBlackWalker : public GraphWalker
+struct ScanBlackVisitor
 {
-    ScanBlackWalker(PRUint32 &aWhiteNodeCount) : mWhiteNodeCount(aWhiteNodeCount)
+    ScanBlackVisitor(PRUint32 &aWhiteNodeCount)
+        : mWhiteNodeCount(aWhiteNodeCount)
     {
     }
 
@@ -1629,9 +1669,9 @@ struct ScanBlackWalker : public GraphWalker
 };
 
 
-struct scanWalker : public GraphWalker
+struct scanVisitor
 {
-    scanWalker(PRUint32 &aWhiteNodeCount) : mWhiteNodeCount(aWhiteNodeCount)
+    scanVisitor(PRUint32 &aWhiteNodeCount) : mWhiteNodeCount(aWhiteNodeCount)
     {
     }
 
@@ -1652,9 +1692,9 @@ struct scanWalker : public GraphWalker
             sCollector->mStats.mSetColorWhite++;
 #endif
         } else {
-            ScanBlackWalker(mWhiteNodeCount).Walk(pi);
+            GraphWalker<ScanBlackVisitor>(ScanBlackVisitor(mWhiteNodeCount)).Walk(pi);
             NS_ASSERTION(pi->mColor == black,
-                         "Why didn't ScanBlackWalker make pi black?");
+                         "Why didn't ScanBlackVisitor make pi black?");
         }
     }
 
@@ -1669,7 +1709,7 @@ nsCycleCollector::ScanRoots()
     // On the assumption that most nodes will be black, it's
     // probably faster to use a GraphWalker than a
     // NodePool::Enumerator.
-    scanWalker(mWhiteNodeCount).WalkFromRoots(mGraph); 
+    GraphWalker<scanVisitor>(scanVisitor(mWhiteNodeCount)).WalkFromRoots(mGraph); 
 
 #ifdef DEBUG_CC
     // Sanity check: scan should have colored all grey nodes black or
@@ -1975,6 +2015,7 @@ nsCycleCollector::nsCycleCollector() :
     mCollectionInProgress(PR_FALSE),
     mScanInProgress(PR_FALSE),
     mCollectedObjects(0),
+    mFirstCollection(PR_TRUE),
     mWhiteNodes(nsnull),
     mWhiteNodeCount(0),
 #ifdef DEBUG_CC
@@ -2201,6 +2242,8 @@ nsCycleCollector_isScanSafe(nsISupports *s)
 PRBool
 nsCycleCollector::Suspect(nsISupports *n)
 {
+    AbortIfOffMainThreadIfCheckFast();
+
     // Re-entering ::Suspect during collection used to be a fault, but
     // we are canonicalizing nsISupports pointers using QI, so we will
     // see some spurious refcount traffic here. 
@@ -2210,7 +2253,6 @@ nsCycleCollector::Suspect(nsISupports *n)
 
     NS_ASSERTION(nsCycleCollector_isScanSafe(n),
                  "suspected a non-scansafe pointer");
-    NS_ASSERTION(NS_IsMainThread(), "trying to suspect from non-main thread");
 
     if (mParams.mDoNothing)
         return PR_FALSE;
@@ -2240,6 +2282,8 @@ nsCycleCollector::Suspect(nsISupports *n)
 PRBool
 nsCycleCollector::Forget(nsISupports *n)
 {
+    AbortIfOffMainThreadIfCheckFast();
+
     // Re-entering ::Forget during collection used to be a fault, but
     // we are canonicalizing nsISupports pointers using QI, so we will
     // see some spurious refcount traffic here. 
@@ -2247,8 +2291,6 @@ nsCycleCollector::Forget(nsISupports *n)
     if (mScanInProgress)
         return PR_FALSE;
 
-    NS_ASSERTION(NS_IsMainThread(), "trying to forget from non-main thread");
-    
     if (mParams.mDoNothing)
         return PR_TRUE; // it's as good as forgotten
 
@@ -2274,6 +2316,8 @@ nsCycleCollector::Forget(nsISupports *n)
 nsPurpleBufferEntry*
 nsCycleCollector::Suspect2(nsISupports *n)
 {
+    AbortIfOffMainThreadIfCheckFast();
+
     // Re-entering ::Suspect during collection used to be a fault, but
     // we are canonicalizing nsISupports pointers using QI, so we will
     // see some spurious refcount traffic here. 
@@ -2283,7 +2327,6 @@ nsCycleCollector::Suspect2(nsISupports *n)
 
     NS_ASSERTION(nsCycleCollector_isScanSafe(n),
                  "suspected a non-scansafe pointer");
-    NS_ASSERTION(NS_IsMainThread(), "trying to suspect from non-main thread");
 
     if (mParams.mDoNothing)
         return nsnull;
@@ -2314,6 +2357,8 @@ nsCycleCollector::Suspect2(nsISupports *n)
 PRBool
 nsCycleCollector::Forget2(nsPurpleBufferEntry *e)
 {
+    AbortIfOffMainThreadIfCheckFast();
+
     // Re-entering ::Forget during collection used to be a fault, but
     // we are canonicalizing nsISupports pointers using QI, so we will
     // see some spurious refcount traffic here. 
@@ -2321,8 +2366,6 @@ nsCycleCollector::Forget2(nsPurpleBufferEntry *e)
     if (mScanInProgress)
         return PR_FALSE;
 
-    NS_ASSERTION(NS_IsMainThread(), "trying to forget from non-main thread");
-    
 #ifdef DEBUG_CC
     mStats.mForgetNode++;
 
@@ -2384,6 +2427,8 @@ nsCycleCollector::Collect(PRUint32 aTryCollections)
     if (mCollectionInProgress)
         return 0;
 
+    NS_TIME_FUNCTION;
+
 #ifdef COLLECT_TIME_DEBUG
     printf("cc: Starting nsCycleCollector::Collect(%d)\n", aTryCollections);
     PRTime start = PR_Now();
@@ -2392,10 +2437,9 @@ nsCycleCollector::Collect(PRUint32 aTryCollections)
     mCollectionInProgress = PR_TRUE;
 
     nsCOMPtr<nsIObserverService> obs =
-      do_GetService("@mozilla.org/observer-service;1");
-    if (obs) {
+        mozilla::services::GetObserverService();
+    if (obs)
         obs->NotifyObservers(nsnull, "cycle-collector-begin", nsnull);
-    }
 
     mFollowupCollection = PR_FALSE;
     mCollectedObjects = 0;
@@ -2404,14 +2448,27 @@ nsCycleCollector::Collect(PRUint32 aTryCollections)
 
     PRUint32 totalCollections = 0;
     while (aTryCollections > totalCollections) {
-        PRBool collected;
-        if (mRuntimes[nsIProgrammingLanguage::JAVASCRIPT]) {
-            collected = static_cast<nsCycleCollectionJSRuntime*>
+        // The cycle collector uses the mark bitmap to discover what JS objects
+        // were reachable only from XPConnect roots that might participate in
+        // cycles. If this is the first cycle collection after startup force
+        // a garbage collection, otherwise the GC might not have run yet and
+        // the bitmap is invalid.
+        // Also force a JS GC if we are doing our infamous shutdown dance
+        // (aTryCollections > 1).
+        if ((mFirstCollection || aTryCollections > 1) &&
+            mRuntimes[nsIProgrammingLanguage::JAVASCRIPT]) {
+#ifdef COLLECT_TIME_DEBUG
+            PRTime start = PR_Now();
+#endif
+            static_cast<nsCycleCollectionJSRuntime*>
                 (mRuntimes[nsIProgrammingLanguage::JAVASCRIPT])->Collect();
+            mFirstCollection = PR_FALSE;
+#ifdef COLLECT_TIME_DEBUG
+            printf("cc: GC() took %lldms\n", (PR_Now() - start) / PR_USEC_PER_MSEC);
+#endif
         }
-        else {
-            collected = BeginCollection() && FinishCollection();
-        }
+
+        PRBool collected = BeginCollection() && FinishCollection();
 
 #ifdef DEBUG_CC
         // We wait until after FinishCollection to check the white nodes because
@@ -2470,13 +2527,15 @@ nsCycleCollector::BeginCollection()
         return PR_FALSE;
 
     GCGraphBuilder builder(mGraph, mRuntimes);
+    if (!builder.Initialized())
+        return PR_FALSE;
 
 #ifdef COLLECT_TIME_DEBUG
     PRTime now = PR_Now();
 #endif
     for (PRUint32 i = 0; i <= nsIProgrammingLanguage::MAX; ++i) {
         if (mRuntimes[i])
-            mRuntimes[i]->BeginCycleCollection(builder);
+            mRuntimes[i]->BeginCycleCollection(builder, false);
     }
 
 #ifdef COLLECT_TIME_DEBUG
@@ -2628,6 +2687,11 @@ nsCycleCollector::Shutdown()
     // Here we want to run a final collection and then permanently
     // disable the collector because the program is shutting down.
 
+    for (PRUint32 i = 0; i <= nsIProgrammingLanguage::MAX; ++i) {
+        if (mRuntimes[i])
+            mRuntimes[i]->CommenceShutdown();
+    }
+
     Collect(SHUTDOWN_COLLECTIONS(mParams));
 
 #ifdef DEBUG_CC
@@ -2656,16 +2720,16 @@ AddExpectedGarbage(nsVoidPtrHashKey *p, void *arg)
     return PL_DHASH_NEXT;
 }
 
-struct SetSCCWalker : public GraphWalker
+struct SetSCCVisitor
 {
-    SetSCCWalker(PRUint32 aIndex) : mIndex(aIndex) {}
+    SetSCCVisitor(PRUint32 aIndex) : mIndex(aIndex) {}
     PRBool ShouldVisitNode(PtrInfo const *pi) { return pi->mSCCIndex == 0; }
     void VisitNode(PtrInfo *pi) { pi->mSCCIndex = mIndex; }
 private:
     PRUint32 mIndex;
 };
 
-struct SetNonRootGreyWalker : public GraphWalker
+struct SetNonRootGreyVisitor
 {
     PRBool ShouldVisitNode(PtrInfo const *pi) { return pi->mColor == white; }
     void VisitNode(PtrInfo *pi) { pi->mColor = grey; }
@@ -2720,7 +2784,7 @@ nsCycleCollector::ExplainLiveExpectedGarbage()
 
         for (PRUint32 i = 0; i <= nsIProgrammingLanguage::MAX; ++i) {
             if (mRuntimes[i])
-                mRuntimes[i]->BeginCycleCollection(builder);
+                mRuntimes[i]->BeginCycleCollection(builder, true);
         }
 
         // But just for extra information, add entries from the purple
@@ -2901,7 +2965,7 @@ nsCycleCollector::ExplainLiveExpectedGarbage()
                     PRUint32 currentSCC = 1;
 
                     while (DFSPostOrder.GetSize() > 0) {
-                        SetSCCWalker(currentSCC).Walk((PtrInfo*)DFSPostOrder.PopFront());
+                        GraphWalker<SetSCCVisitor>(SetSCCVisitor(currentSCC)).Walk((PtrInfo*)DFSPostOrder.PopFront());
                         ++currentSCC;
                     }
                 }
@@ -2918,7 +2982,7 @@ nsCycleCollector::ExplainLiveExpectedGarbage()
                                             child_end = pi->mLastChild;
                              child != child_end; ++child) {
                             if ((*child)->mSCCIndex != pi->mSCCIndex) {
-                                SetNonRootGreyWalker().Walk(*child);
+                                GraphWalker<SetNonRootGreyVisitor>(SetNonRootGreyVisitor()).Walk(*child);
                             }
                         }
                     }
@@ -3026,13 +3090,17 @@ nsCycleCollector::DestroyReversedEdges()
 void
 nsCycleCollector::ShouldBeFreed(nsISupports *n)
 {
-    mExpectedGarbage.PutEntry(n);
+    if (n) {
+        mExpectedGarbage.PutEntry(n);
+    }
 }
 
 void
 nsCycleCollector::WasFreed(nsISupports *n)
 {
-    mExpectedGarbage.RemoveEntry(n);
+    if (n) {
+        mExpectedGarbage.RemoveEntry(n);
+    }
 }
 #endif
 
@@ -3106,18 +3174,6 @@ nsCycleCollector_suspectedCount()
     return sCollector ? sCollector->SuspectedCount() : 0;
 }
 
-PRBool 
-nsCycleCollector_beginCollection()
-{
-    return sCollector ? sCollector->BeginCollection() : PR_FALSE;
-}
-
-PRBool 
-nsCycleCollector_finishCollection()
-{
-    return sCollector ? sCollector->FinishCollection() : PR_FALSE;
-}
-
 nsresult 
 nsCycleCollector_startup()
 {
