@@ -312,16 +312,6 @@ static nsresult ConvertWinError(DWORD winErr)
     return rv;
 }
 
-// definition of INVALID_SET_FILE_POINTER from VC.NET header files
-// it doesn't appear to be defined by VC6
-#ifndef INVALID_SET_FILE_POINTER
-# define INVALID_SET_FILE_POINTER ((DWORD)-1)
-#endif
-// same goes for INVALID_FILE_ATTRIBUTES
-#ifndef INVALID_FILE_ATTRIBUTES
-# define INVALID_FILE_ATTRIBUTES ((DWORD)-1)
-#endif
-
 // as suggested in the MSDN documentation on SetFilePointer
 static __int64 
 MyFileSeek64(HANDLE aHandle, __int64 aDistance, DWORD aMoveMethod)
@@ -425,7 +415,12 @@ OpenFile(const nsAFlatString &name, PRIntn osflags, PRIntn mode,
     }
 
     if (osflags & nsILocalFile::DELETE_ON_CLOSE) {
+#ifdef WINCE
+        NS_ASSERTION(!(osflags & nsILocalFile::DELETE_ON_CLOSE), "DELETE_ON_CLOSE is not supported on wince");
+        return NS_ERROR_NOT_AVAILABLE;
+#else
       flag6 |= FILE_FLAG_DELETE_ON_CLOSE;
+#endif
     }
 
     HANDLE file = ::CreateFileW(name.get(), access,
@@ -769,7 +764,7 @@ nsLocalFile::nsLocalFile()
 {
 }
 
-NS_METHOD
+nsresult
 nsLocalFile::nsLocalFileConstructor(nsISupports* outer, const nsIID& aIID, void* *aInstancePtr)
 {
     NS_ENSURE_ARG_POINTER(aInstancePtr);
@@ -1107,7 +1102,6 @@ nsLocalFile::AppendInternal(const nsAFlatString &node, PRBool multipleComponents
         || node.EqualsASCII(".."))                              // can't be ..
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
 
-#ifndef WINCE  // who cares?
     if (multipleComponents)
     {
         // can't contain .. as a path component. Ensure that the valid components
@@ -1134,7 +1128,6 @@ nsLocalFile::AppendInternal(const nsAFlatString &node, PRBool multipleComponents
     // single components can't contain '\'
     else if (node.FindChar(L'\\') != kNotFound)
         return NS_ERROR_FILE_UNRECOGNIZED_PATH;
-#endif
 
     MakeDirty();
     
@@ -1149,7 +1142,6 @@ nsLocalFile::AppendInternal(const nsAFlatString &node, PRBool multipleComponents
 NS_IMETHODIMP
 nsLocalFile::Normalize()
 {
-#ifndef WINCE
     // XXX See bug 187957 comment 18 for possible problems with this implementation.
     
     if (mWorkingPath.IsEmpty())
@@ -1197,13 +1189,17 @@ nsLocalFile::Normalize()
          * manages to eject the drive between our call to _getdrives() and
          * our *calls* to _wgetdcwd.
          */
+#ifdef WINCE
+        // no concept of a cwd on wince, let alone a cwd per drive
+        pcwd = L"\\";
+#else
         if (!((1 << (drive - 1)) & _getdrives()))
             return NS_ERROR_FILE_INVALID_PATH;
         if (!_wgetdcwd(drive, pcwd, MAX_PATH))
             pcwd = _wgetdcwd(drive, 0, 0);
         if (!pcwd)
             return NS_ERROR_OUT_OF_MEMORY;
-
+#endif
         nsAutoString currentDir(pcwd);
         if (pcwd != cwd)
             free(pcwd);
@@ -1301,9 +1297,6 @@ nsLocalFile::Normalize()
     } 
 
     MakeDirty();
-#else // WINCE
-    // WINCE FIX
-#endif 
     return NS_OK;
 }
 
@@ -1477,10 +1470,23 @@ nsLocalFile::CopySingleFile(nsIFile *sourceFile, nsIFile *destParent,
         copyOK = ::CopyFileW(filePath.get(), destPath.get(), PR_TRUE);
     else {
 #ifndef WINCE
-        copyOK = ::MoveFileExW(filePath.get(), destPath.get(),
-                               MOVEFILE_REPLACE_EXISTING |
-                               MOVEFILE_COPY_ALLOWED |
-                               MOVEFILE_WRITE_THROUGH);
+        DWORD status;
+        if (FileEncryptionStatusW(filePath.get(), &status)
+            && status == FILE_IS_ENCRYPTED)
+        {
+            copyOK = CopyFileExW(filePath.get(), destPath.get(), NULL, NULL, NULL,
+                                 COPY_FILE_ALLOW_DECRYPTED_DESTINATION);
+
+            if (copyOK)
+                DeleteFileW(filePath.get());
+        }
+        else
+        {
+            copyOK = ::MoveFileExW(filePath.get(), destPath.get(),
+                                   MOVEFILE_REPLACE_EXISTING |
+                                   MOVEFILE_COPY_ALLOWED |
+                                   MOVEFILE_WRITE_THROUGH);
+        }
 #else
         DeleteFile(destPath.get());
         copyOK = :: MoveFileW(filePath.get(), destPath.get());
@@ -1550,7 +1556,7 @@ nsLocalFile::CopyMove(nsIFile *aParentDir, const nsAString &newName, PRBool foll
     {
         PRBool isDir;
         newParentDir->IsDirectory(&isDir);
-        if (isDir == PR_FALSE)
+        if (!isDir)
         {
             if (followSymlinks)
             {
@@ -1970,12 +1976,14 @@ nsLocalFile::SetLastModifiedTimeOfLink(PRInt64 aLastModifiedTime)
 nsresult
 nsLocalFile::SetModDate(PRInt64 aLastModifiedTime, const PRUnichar *filePath)
 {
+    // The FILE_FLAG_BACKUP_SEMANTICS is required in order to change the
+    // modification time for directories.
     HANDLE file = ::CreateFileW(filePath,          // pointer to name of the file
                                 GENERIC_WRITE,     // access (write) mode
                                 0,                 // share mode
                                 NULL,              // pointer to security attributes
                                 OPEN_EXISTING,     // how to create
-                                0,                 // file attributes
+                                FILE_FLAG_BACKUP_SEMANTICS,  // file attributes
                                 NULL);
 
     if (file == INVALID_HANDLE_VALUE)
@@ -2203,7 +2211,6 @@ nsLocalFile::GetDiskSpaceAvailable(PRInt64 *aDiskSpaceAvailable)
     // Check we are correctly initialized.
     CHECK_mWorkingPath();
 
-#ifndef WINCE
     NS_ENSURE_ARG(aDiskSpaceAvailable);
 
     ResolveAndStat();
@@ -2215,8 +2222,6 @@ nsLocalFile::GetDiskSpaceAvailable(PRInt64 *aDiskSpaceAvailable)
         *aDiskSpaceAvailable = liFreeBytesAvailableToCaller.QuadPart;
         return NS_OK;
     }
-#endif
-    // WINCE FIX
     *aDiskSpaceAvailable = 0;
     return NS_OK;
 }
@@ -2751,7 +2756,6 @@ nsLocalFile::SetFileAttributesWin(PRUint32 aAttribs)
 NS_IMETHODIMP
 nsLocalFile::Reveal()
 {
-#ifndef WINCE
     // make sure mResolvedPath is set
     nsresult rv = ResolveAndStat();
     if (NS_FAILED(rv) && rv != NS_ERROR_FILE_NOT_FOUND)
@@ -2764,8 +2768,11 @@ nsLocalFile::Reveal()
     nsAutoString explorerPath;
     rv = winDir->GetPath(explorerPath);  
     NS_ENSURE_SUCCESS(rv, rv);
+#ifndef WINCE
     explorerPath.Append(L"\\explorer.exe");
-    
+#else
+    explorerPath.Append(L"\\fexplorer.exe");
+#endif
     // Always open a new window for files because Win2K doesn't appear to select
     // the file if a window showing that folder was already open. If the resolved 
     // path is a directory then instead of opening the parent and selecting it, 
@@ -2776,15 +2783,25 @@ nsLocalFile::Reveal()
     explorerParams.Append(L'\"');
     explorerParams.Append(mResolvedPath);
     explorerParams.Append(L'\"');
-    
+#ifdef WINCE
+    SHELLEXECUTEINFO seinfo;
+    memset(&seinfo, 0, sizeof(seinfo));
+    seinfo.cbSize = sizeof(SHELLEXECUTEINFO);
+    seinfo.fMask  = NULL;
+    seinfo.hwnd   = NULL;
+    seinfo.lpVerb =  L"open";
+    seinfo.lpFile = explorerPath.get();
+    seinfo.lpParameters =  explorerParams.get();
+    seinfo.lpDirectory  = NULL;
+    seinfo.nShow  = SW_SHOWNORMAL;
+    if (!ShellExecuteEx(&seinfo))
+        return NS_ERROR_FAILURE;
+#else    
     if (::ShellExecuteW(NULL, L"open", explorerPath.get(), explorerParams.get(),
                         NULL, SW_SHOWNORMAL) <= (HINSTANCE) 32)
         return NS_ERROR_FAILURE;
-    
+#endif    
     return NS_OK;
-#else
-    return NS_ERROR_NOT_AVAILABLE;
-#endif
 }
 #ifdef WINCE 
 #ifndef UNICODE
@@ -2957,7 +2974,7 @@ nsLocalFile::GetNativePath(nsACString &_retval)
 NS_IMETHODIMP
 nsLocalFile::GetNativeCanonicalPath(nsACString &aResult)
 {
-    NS_WARNING("This method is lossy. Use GetCanoincailPath !");
+    NS_WARNING("This method is lossy. Use GetCanonicalPath !");
     EnsureShortPath();
     NS_CopyUnicodeToNative(mShortWorkingPath, aResult);
     return NS_OK;
@@ -3050,7 +3067,7 @@ nsLocalFile::EnsureShortPath()
     WCHAR thisshort[MAX_PATH];
     DWORD thisr = ::GetShortPathNameW(mWorkingPath.get(), thisshort,
                                       sizeof(thisshort));
-    // If an error occured (thisr == 0) thisshort is uninitialized memory!
+    // If an error occurred (thisr == 0) thisshort is uninitialized memory!
     if (thisr != 0 && thisr < sizeof(thisshort))
         mShortWorkingPath.Assign(thisshort);
     else
